@@ -1,12 +1,12 @@
-(* omega_Zprime.ml --
+(* omega_Zprime_VM.ml --
 
-   Copyright (C) 1999-2014 by
+   Copyright (C) 1999-2018 by
 
        Wolfgang Kilian <kilian@physik.uni-siegen.de>
        Thorsten Ohl <ohl@physik.uni-wuerzburg.de>
        Juergen Reuter <juergen.reuter@desy.de>
        with contributions from
-       Christian Speckner <cnspeckn@googlemail.com>
+       cf. main AUTHORS file
 
    WHIZARD is free software; you can redistribute it and/or modify it
    under the terms of the GNU General Public License as published by
