@@ -1,6 +1,6 @@
 (* tree2.mli --
 
-   Copyright (C) 1999-2017 by
+   Copyright (C) 1999-2018 by
 
        Wolfgang Kilian <kilian@physik.uni-siegen.de>
        Thorsten Ohl <ohl@physik.uni-wuerzburg.de>
@@ -29,7 +29,10 @@ type ('n, 'e) t
 val cons : ('e * 'n * ('n, 'e) t list) list -> ('n, 'e) t
 val leaf : 'n -> ('n, 'e) t
 
+val is_singleton : ('n, 'e) t -> bool
 val to_string : ('n -> string) -> ('e -> string) -> ('n, 'e) t -> string
+val to_channel :
+  out_channel -> ('n -> string) -> ('e -> string) -> ('n, 'e) t -> unit
 
 (*i
  *  Local Variables:
