@@ -10,6 +10,7 @@ subroutine pyinit (frame, beam, target, win)
   write (0, "(A)")  "**************************************************************"
   stop  
 end subroutine pyinit
+
 subroutine pygive (chin)
   character chin*(*)
   write (0, "(A)")  "**************************************************************"
@@ -17,18 +18,21 @@ subroutine pygive (chin)
   write (0, "(A)")  "**************************************************************"
   stop  
 end subroutine pygive
+
 subroutine pyevnt()
   write (0, "(A)")  "**************************************************************"
   write (0, "(A)")  "*** Error: PYTHIA6 has not been enabled, WHIZARD terminates ***"
   write (0, "(A)")  "**************************************************************"
   stop  
 end subroutine pyevnt
+
 subroutine pyexec()
   write (0, "(A)")  "**************************************************************"
   write (0, "(A)")  "*** Error: PYTHIA6 has not been enabled, WHIZARD terminates ***"
   write (0, "(A)")  "**************************************************************"
   stop   
 end subroutine pyexec
+
 function pyp(I,J)
   integer, intent(in) :: i,j
   double precision :: pyp
@@ -37,12 +41,22 @@ function pyp(I,J)
   write (0, "(A)")  "**************************************************************"
   stop  
 end function pyp
+
+subroutine pystat (mstat)
+  integer, intent(in) :: mstat
+  write (0, "(A)")  "**************************************************************"
+  write (0, "(A)")  "*** Error: PYTHIA6 has not been enabled, WHIZARD terminates ***"
+  write (0, "(A)")  "**************************************************************"
+  stop  
+end subroutine pystat
+
 subroutine pystop ()
   write (0, "(A)")  "**************************************************************"
   write (0, "(A)")  "*** Error: PYTHIA6 has not been enabled, WHIZARD terminates ***"
   write (0, "(A)")  "**************************************************************"
   stop  
 end subroutine pystop
+
 subroutine pyrobo (imi, ima, the, phi, bex, bey, bez)
   integer, intent(in) :: imi, ima
   double precision, intent(in) :: the, phi, bex, bey, bez
@@ -51,6 +65,7 @@ subroutine pyrobo (imi, ima, the, phi, bex, bey, bez)
   write (0, "(A)")  "**************************************************************"
   stop  
 end subroutine pyrobo
+
 subroutine pyedit (medit)
   integer, intent(in) :: medit
   write (0, "(A)")  "**************************************************************"
@@ -58,6 +73,7 @@ subroutine pyedit (medit)
   write (0, "(A)")  "**************************************************************"
   stop  
 end subroutine pyedit
+
 subroutine pyhepc (mconv)
   integer, intent(in) :: mconv
   write (0, "(A)")  "**************************************************************"
@@ -65,3 +81,11 @@ subroutine pyhepc (mconv)
   write (0, "(A)")  "**************************************************************"
   stop  
 end subroutine pyhepc
+
+function pyr (idummy)
+  integer, intent(in) :: idummy
+  write (0, "(A)")  "**************************************************************"
+  write (0, "(A)")  "*** Error: PYTHIA6 has not been enabled, WHIZARD terminates ***"
+  write (0, "(A)")  "**************************************************************"
+  stop  
+end function pyr
