@@ -1,19 +1,12 @@
-! WHIZARD 2.4.1 Mar 24 2017
+! WHIZARD 2.6.4 Aug 23 2018
 !
-! Copyright (C) 1999-2017 by
+! Copyright (C) 1999-2018 by
 !     Wolfgang Kilian <kilian@physik.uni-siegen.de>
 !     Thorsten Ohl <ohl@physik.uni-wuerzburg.de>
 !     Juergen Reuter <juergen.reuter@desy.de>
 !
 !     with contributions from
-!     Fabian Bach <fabian.bach@t-online.de>
-!     Bijan Chokoufe <bijan.chokoufe@desy.de>
-!     Christian Speckner <cnspeckn@googlemail.com>
-!     So Young Shim <soyoung.shim@desy.de>
-!     Florian Staub <florian.staub@cern.ch>
-!     Christian Weiss <christian.weiss@desy.de>
-!     and Hans-Werner Boschmann, Felix Braam,
-!     Sebastian Schmidt, So-young Shim, Daniel Wiesler
+!     cf. main AUTHORS file
 !
 ! WHIZARD is free software; you can redistribute it and/or modify it
 ! under the terms of the GNU General Public License as published by
@@ -129,6 +122,7 @@ contains
     logical :: beam_events_warn_eof
     type(string_t) :: beam_events_dir, beam_events_file
     logical :: escan_normalize
+    integer :: i
     lhapdf_photon_sets = [var_str ("DOG0.LHgrid"), var_str ("DOG1.LHgrid"), &
          var_str ("DGG.LHgrid"), var_str ("LACG.LHgrid"), &
          var_str ("GSG0.LHgrid"), var_str ("GSG1.LHgrid"), &
@@ -382,7 +376,9 @@ contains
     if (allocated (data)) then
        allocate (pdg_out (size (pdg_prc, 1)))
        call data%get_pdg_out (pdg_out)
-       pdg_in(i_beam) = pdg_out
+       do i = 1, size (i_beam)
+          pdg_in(i_beam(i)) = pdg_out(i)
+       end do
     end if
   end subroutine dispatch_sf_data
 
@@ -485,7 +481,7 @@ contains
        call msg_fatal ("QCD alpha: calculation mode is ambiguous")
     case (1)
        if (fixed) then
-          call msg_fatal ("QCD alpha: use '?alpha_s_is_fixed = false' for " // &
+          call msg_fatal ("QCD alpha: use '?alphas_is_fixed = false' for " // &
                "running alphas")
        else if (from_mz) then
           allocate (alpha_qcd_from_scale_t :: qcd%alpha)
@@ -499,20 +495,21 @@ contains
        call msg_message ("QCD alpha: using a running strong coupling")
     end select
     call init_alpha ()
+    qcd%n_f = var_list%get_ival (var_str ("alphas_nf"))
   contains
     subroutine unpack_variables ()
-      fixed = var_list%get_lval (var_str ("?alpha_s_is_fixed"))
-      from_mz = var_list%get_lval (var_str ("?alpha_s_from_mz"))
+      fixed = var_list%get_lval (var_str ("?alphas_is_fixed"))
+      from_mz = var_list%get_lval (var_str ("?alphas_from_mz"))
       from_pdf_builtin = &
-           var_list%get_lval (var_str ("?alpha_s_from_pdf_builtin"))
+           var_list%get_lval (var_str ("?alphas_from_pdf_builtin"))
       from_lhapdf = &
-           var_list%get_lval (var_str ("?alpha_s_from_lhapdf"))
+           var_list%get_lval (var_str ("?alphas_from_lhapdf"))
       from_lambda_qcd = &
-           var_list%get_lval (var_str ("?alpha_s_from_lambda_qcd"))
+           var_list%get_lval (var_str ("?alphas_from_lambda_qcd"))
       pdfset = var_list%get_sval (var_str ("$pdf_builtin_set"))
       lambda = var_list%get_rval (var_str ("lambda_qcd"))
-      nf = var_list%get_ival (var_str ("alpha_s_nf"))
-      order = var_list%get_ival (var_str ("alpha_s_order"))
+      nf = var_list%get_ival (var_str ("alphas_nf"))
+      order = var_list%get_ival (var_str ("alphas_order"))
       lhapdf_dir = var_list%get_sval (var_str ("$lhapdf_dir"))
       lhapdf_file = var_list%get_sval (var_str ("$lhapdf_file"))
       lhapdf_member = var_list%get_ival (var_str ("lhapdf_member"))
