@@ -1,19 +1,12 @@
-! WHIZARD 2.4.1 Mar 24 2017
+! WHIZARD 2.6.4 Aug 23 2018
 !
-! Copyright (C) 1999-2017 by
+! Copyright (C) 1999-2018 by
 !     Wolfgang Kilian <kilian@physik.uni-siegen.de>
 !     Thorsten Ohl <ohl@physik.uni-wuerzburg.de>
 !     Juergen Reuter <juergen.reuter@desy.de>
 !
 !     with contributions from
-!     Fabian Bach <fabian.bach@t-online.de>
-!     Bijan Chokoufe <bijan.chokoufe@desy.de>
-!     Christian Speckner <cnspeckn@googlemail.com>
-!     So Young Shim <soyoung.shim@desy.de>
-!     Florian Staub <florian.staub@cern.ch>
-!     Christian Weiss <christian.weiss@desy.de>
-!     and Hans-Werner Boschmann, Felix Braam,
-!     Sebastian Schmidt, So-young Shim, Daniel Wiesler
+!     cf. main AUTHORS file
 !
 ! WHIZARD is free software; you can redistribute it and/or modify it
 ! under the terms of the GNU General Public License as published by
@@ -43,6 +36,7 @@ module sf_lhapdf
   use system_dependencies, only: LHAPDF5_AVAILABLE
   use system_dependencies, only: LHAPDF6_AVAILABLE
   use diagnostics
+  use physics_defs, only: n_beam_gluon_offset
   use physics_defs, only: PROTON, PHOTON, PIPLUS, GLUON
   use physics_defs, only: HADRON_REMNANT_SINGLET
   use physics_defs, only: HADRON_REMNANT_TRIPLET
@@ -322,22 +316,22 @@ contains
     end function dataset_member_exists
   end subroutine lhapdf_initialize
 
-  subroutine lhapdf_complete_kinematics (sf_int, x, f, r, rb, map)
+  subroutine lhapdf_complete_kinematics (sf_int, x, xb, f, r, rb, map)
     class(lhapdf_t), intent(inout) :: sf_int
     real(default), dimension(:), intent(out) :: x
+    real(default), dimension(:), intent(out) :: xb
     real(default), intent(out) :: f
     real(default), dimension(:), intent(in) :: r
     real(default), dimension(:), intent(in) :: rb
     logical, intent(in) :: map
-    real(default) :: xb1
     if (map) then
        call msg_fatal ("LHAPDF: map flag not supported")
     else
        x(1) = r(1)
+       xb(1)= rb(1)
        f = 1
     end if
-    xb1 = 1 - x(1)
-    call sf_int%split_momentum (x, xb1)
+    call sf_int%split_momentum (x, xb)
     select case (sf_int%status)
     case (SF_DONE_KINEMATICS)
        sf_int%x = x(1)
@@ -347,27 +341,26 @@ contains
     end select
   end subroutine lhapdf_complete_kinematics
 
-  subroutine lhapdf_inverse_kinematics (sf_int, x, f, r, rb, map, set_momenta)
+  subroutine lhapdf_inverse_kinematics (sf_int, x, xb, f, r, rb, map, set_momenta)
     class(lhapdf_t), intent(inout) :: sf_int
     real(default), dimension(:), intent(in) :: x
+    real(default), dimension(:), intent(in) :: xb
     real(default), intent(out) :: f
     real(default), dimension(:), intent(out) :: r
     real(default), dimension(:), intent(out) :: rb
     logical, intent(in) :: map
     logical, intent(in), optional :: set_momenta
-    real(default) :: xb1
     logical :: set_mom
     set_mom = .false.;  if (present (set_momenta))  set_mom = set_momenta
     if (map) then
        call msg_fatal ("LHAPDF: map flag not supported")
     else
        r(1) = x(1)
+       rb(1)= xb(1)
        f = 1
     end if
-    xb1 = 1 - x(1)
-    rb = 1 - r
     if (set_mom) then
-       call sf_int%split_momentum (x, xb1)
+       call sf_int%split_momentum (x, xb)
        select case (sf_int%status)
        case (SF_DONE_KINEMATICS)
           sf_int%x = x(1)
@@ -521,7 +514,7 @@ contains
     allocate (lhapdf_t :: sf_int)
   end subroutine lhapdf_data_allocate_sf_int
 
-  function lhapdf_data_get_pdf_set (data) result (pdf_set)
+  elemental function lhapdf_data_get_pdf_set (data) result (pdf_set)
     class(lhapdf_data_t), intent(in) :: data
     integer :: pdf_set
     pdf_set = data%set
@@ -609,24 +602,47 @@ contains
     end select
   end subroutine lhapdf_init
 
-  subroutine lhapdf_apply (sf_int, scale)
+  subroutine lhapdf_apply (sf_int, scale, rescale, i_sub, fill_sub)
     class(lhapdf_t), intent(inout) :: sf_int
     real(default), intent(in) :: scale
+    class(sf_rescale_t), intent(in), optional :: rescale
+    integer, intent(in), optional :: i_sub
+    logical, intent(in), optional :: fill_sub
     real(default) :: x, s
     double precision :: xx, qq, ss
     double precision, dimension(-6:6) :: ff
     double precision :: fphot
     complex(default), dimension(:), allocatable :: fc
+    integer :: i, i_sub_opt, j_sub
+    logical :: fill_sub_opt
     external :: evolvePDFM, evolvePDFpM
+    i_sub_opt = 0; if (present (i_sub)) i_sub_opt = i_sub
+    fill_sub_opt = .false.; if (present (fill_sub)) fill_sub_opt = fill_sub
+    if (present (rescale) .and. fill_sub_opt) then
+       call msg_bug ("[lhapdf_apply] &
+            & sf_rescale and fill_sub option are mutually exclusive.")
+    end if
+    if (i_sub_opt > 0 .and. fill_sub_opt) then
+       call msg_bug ("[lhapdf_apply] &
+            & i_sub and fill_sub options are mutually exclusive.")
+    end if
     associate (data => sf_int%data)
       sf_int%q = scale
       x = sf_int%x
+      if (present (rescale))  call rescale%apply (x)
       s = sf_int%s
       xx = x
-      qq = min (data% qmax, scale)
-      qq = max (data% qmin, qq)
+      if (debug2_active (D_BEAMS)) then
+         call msg_debug2 (D_BEAMS, "lhapdf_apply")
+         call msg_debug2 (D_BEAMS, "rescale: ", present(rescale))
+         call msg_debug2 (D_BEAMS, "i_sub: ", i_sub_opt)
+         call msg_debug2 (D_BEAMS, "fill_sub: ", fill_sub_opt)
+         call msg_debug2 (D_BEAMS, "x: ", x)
+      end if
+      qq = min (data%qmax, scale)
+      qq = max (data%qmin, qq)
       if (.not. data% photon) then
-         if (data% invert) then
+         if (data%invert) then
             if (data%has_photon) then
                if (LHAPDF5_AVAILABLE) then
                   call evolvePDFphotonM &
@@ -684,7 +700,22 @@ contains
          fc = max (pack (ff / x, data%mask), 0._default)
       end if
     end associate
-    call sf_int%set_matrix_element (fc)
+    if (debug_active (D_BEAMS)) print *, 'Set pdfs: ', real (fc)
+    if (present (rescale) .and. i_sub_opt > 0) then
+       call sf_int%set_matrix_element (fc, [(i_sub_opt * size(fc) + i, i = 1, size(fc))])
+       if (rescale%has_gluons ()) then
+          j_sub = i_sub_opt + n_beam_gluon_offset
+          call sf_int%set_matrix_element (&
+               spread (fc(7), 1, size(fc)), [(j_sub * size(fc) + i, i = 1, size(fc))])
+       end if
+    else
+       call sf_int%set_matrix_element (fc, [(i, i = 1, size(fc))])
+    end if
+    if(fill_sub_opt) then
+       do j_sub = 1, sf_int%get_n_sub ()
+          call sf_int%set_matrix_element (fc, [(j_sub * size(fc) + i, i = 1, size(fc))])
+       end do
+    end if
     sf_int%status = SF_EVALUATED
   end subroutine lhapdf_apply
 
