@@ -1,19 +1,12 @@
-! WHIZARD 2.4.1 Mar 24 2017
+! WHIZARD 2.6.4 Aug 23 2018
 !
-! Copyright (C) 1999-2017 by
+! Copyright (C) 1999-2018 by
 !     Wolfgang Kilian <kilian@physik.uni-siegen.de>
 !     Thorsten Ohl <ohl@physik.uni-wuerzburg.de>
 !     Juergen Reuter <juergen.reuter@desy.de>
 !
 !     with contributions from
-!     Fabian Bach <fabian.bach@t-online.de>
-!     Bijan Chokoufe <bijan.chokoufe@desy.de>
-!     Christian Speckner <cnspeckn@googlemail.com>
-!     So Young Shim <soyoung.shim@desy.de>
-!     Florian Staub <florian.staub@cern.ch>
-!     Christian Weiss <christian.weiss@desy.de>
-!     and Hans-Werner Boschmann, Felix Braam,
-!     Sebastian Schmidt, So-young Shim, Daniel Wiesler
+!     cf. main AUTHORS file
 !
 ! WHIZARD is free software; you can redistribute it and/or modify it
 ! under the terms of the GNU General Public License as published by
@@ -60,7 +53,6 @@ module sf_mappings
   public :: sf_ei_mapping_t
   public :: sf_eir_mapping_t
   public :: sf_eio_mapping_t
-  public :: log_prec
   public :: map_on_shell
   public :: map_on_shell_inverse
   public :: map_on_shell_single
@@ -1544,19 +1536,6 @@ contains
     end if
   end subroutine inverse_prec_y
 
-  function log_prec (x, xb) result (lx)
-    real(default), intent(in) :: x, xb
-    real(default) :: a1, a2, a3, lx
-    a1 = xb
-    a2 = a1 * xb / 2
-    a3 = a2 * xb * 2 / 3
-    if (abs (a3) < epsilon (a3)) then
-       lx = - a1 - a2 - a3
-    else
-       lx = log (x)
-    end if
-  end function log_prec
-
   subroutine map_on_shell (r, factor, p, lm2, x_free)
     real(default), dimension(2), intent(out) :: r
     real(default), intent(out) :: factor
