@@ -1,19 +1,12 @@
-! WHIZARD 2.4.1 Mar 24 2017
+! WHIZARD 2.6.4 Aug 23 2018
 !
-! Copyright (C) 1999-2017 by
+! Copyright (C) 1999-2018 by
 !     Wolfgang Kilian <kilian@physik.uni-siegen.de>
 !     Thorsten Ohl <ohl@physik.uni-wuerzburg.de>
 !     Juergen Reuter <juergen.reuter@desy.de>
 !
 !     with contributions from
-!     Fabian Bach <fabian.bach@t-online.de>
-!     Bijan Chokoufe <bijan.chokoufe@desy.de>
-!     Christian Speckner <cnspeckn@googlemail.com>
-!     So Young Shim <soyoung.shim@desy.de>
-!     Florian Staub <florian.staub@cern.ch>
-!     Christian Weiss <christian.weiss@desy.de>
-!     and Hans-Werner Boschmann, Felix Braam,
-!     Sebastian Schmidt, So-young Shim, Daniel Wiesler
+!     cf. main AUTHORS file
 !
 ! WHIZARD is free software; you can redistribute it and/or modify it
 ! under the terms of the GNU General Public License as published by
@@ -84,8 +77,8 @@ module blha_olp_interfaces
   public :: olp_print_parameter
   public :: blha_result_array_size
   public :: parameter_error_message
-  public :: blha_cc_fill_diag
-  public :: blha_cc_fill_offdiag
+  public :: blha_color_c_fill_diag
+  public :: blha_color_c_fill_offdiag
   public :: blha_loop_positions
 
   integer, parameter, public :: OLP_PARAMETER_LIMIT = 10
@@ -105,7 +98,8 @@ module blha_olp_interfaces
     integer :: I_REAL = 1
     integer :: I_LOOP = 2
     integer :: I_SUB = 3
-    logical, dimension(0:3) :: compute_component
+    integer :: I_DGLAP = 4
+    logical, dimension(0:4) :: compute_component
     logical :: include_polarizations = .false.
     logical :: switch_off_muon_yukawas = .false.
     logical :: use_internal_color_correlations = .true.
@@ -118,6 +112,7 @@ module blha_olp_interfaces
     procedure :: set_real_trees => blha_template_set_real_trees
     procedure :: set_loop => blha_template_set_loop
     procedure :: set_subtraction => blha_template_set_subtraction
+    procedure :: set_dglap => blha_template_set_dglap
     procedure :: set_internal_color_correlations &
        => blha_template_set_internal_color_correlations
     procedure :: get_internal_color_correlations &
@@ -126,6 +121,7 @@ module blha_olp_interfaces
     procedure :: compute_real_trees => blha_template_compute_real_trees
     procedure :: compute_loop => blha_template_compute_loop
     procedure :: compute_subtraction => blha_template_compute_subtraction
+    procedure :: compute_dglap => blha_template_compute_dglap
     procedure :: check => blha_template_check
     procedure :: reset => blha_template_reset
   end type blha_template_t
@@ -134,10 +130,9 @@ module blha_olp_interfaces
     integer :: n_particles
     integer :: n_hel
     integer :: n_proc
-    integer, dimension(:), allocatable :: i_tree, i_sc, i_cc
-    integer, dimension(:), allocatable :: i_virt
-    integer, dimension(:,:), allocatable :: i_hel
-    integer, dimension(:), allocatable :: i_whizard_to_i_olc
+    integer, dimension(:, :), allocatable :: i_tree, i_spin_c, i_color_c
+    integer, dimension(:, :), allocatable :: i_virt
+    integer, dimension(:, :), allocatable :: i_hel
     logical, dimension(3) :: ew_parameter_mask
     integer :: sqme_tree_pos
   contains
@@ -156,8 +151,8 @@ module blha_olp_interfaces
     procedure :: init_ew_parameters => prc_blha_init_ew_parameters
     procedure :: compute_sqme_virt => prc_blha_compute_sqme_virt
     procedure :: compute_sqme => prc_blha_compute_sqme
-    procedure :: compute_sqme_cc_raw => prc_blha_compute_sqme_cc_raw
-    procedure :: compute_sqme_cc => prc_blha_compute_sqme_cc
+    procedure :: compute_sqme_color_c_raw => prc_blha_compute_sqme_color_c_raw
+    procedure :: compute_sqme_color_c => prc_blha_compute_sqme_color_c
     generic :: get_beam_helicities => get_beam_helicities_single
     generic :: get_beam_helicities => get_beam_helicities_array
     procedure :: get_beam_helicities_single => prc_blha_get_beam_helicities_single
@@ -165,14 +160,6 @@ module blha_olp_interfaces
     procedure :: includes_polarization => prc_blha_includes_polarization
     procedure(prc_blha_init_driver), deferred :: &
         init_driver
-    procedure :: reset_helicity_list => prc_blha_reset_helicity_list
-    procedure :: set_helicity_list_trivial => prc_blha_set_helicity_list_trivial
-    procedure :: set_helicity_list => prc_blha_set_helicity_list
-    generic :: get_helicity_list => get_helicity_list_all
-    generic :: get_helicity_list => get_helicity_list_single
-    procedure :: get_helicity_list_all => prc_blha_get_helicity_list_all
-    procedure :: get_helicity_list_single => prc_blha_get_helicity_list_single
-    procedure :: warmup_helicities => prc_blha_warmup_helicities
   end type prc_blha_t
 
   type, abstract, extends (user_defined_driver_t) :: blha_driver_t
@@ -406,9 +393,9 @@ contains
           rsize = 1
        case (BLHA_AMP_LOOP)
           rsize = 4
-       case (BLHA_AMP_CC)
+       case (BLHA_AMP_COLOR_C)
           rsize = n_part * (n_part - 1) / 2
-       case (BLHA_AMP_SC)
+       case (BLHA_AMP_SPIN_C)
           rsize = 2 * n_part**2
        case default
           rsize = 0
@@ -467,6 +454,11 @@ contains
     template%compute_component (template%I_SUB) = .true.
   end subroutine blha_template_set_subtraction
 
+  subroutine blha_template_set_dglap (template)
+    class(blha_template_t), intent(inout) :: template
+    template%compute_component (template%I_DGLAP) = .true.
+  end subroutine blha_template_set_dglap
+
   subroutine blha_template_set_internal_color_correlations (template)
     class(blha_template_t), intent(inout) :: template
     template%use_internal_color_correlations = .true.
@@ -503,6 +495,12 @@ contains
     val = template%compute_component (template%I_SUB)
   end function blha_template_compute_subtraction
 
+  pure function blha_template_compute_dglap (template) result (val)
+    class(blha_template_t), intent(in) :: template
+    logical :: val
+    val = template%compute_component (template%I_DGLAP)
+  end function blha_template_compute_dglap
+
   function blha_template_check (template) result (val)
     class(blha_template_t), intent(in) :: template
     logical :: val
@@ -634,10 +632,11 @@ contains
   end subroutine blha_driver_load
 
   subroutine blha_driver_read_contract_file (driver, flavors, &
-       amp_type, flv_index, label, helicities)
+       amp_type, flv_index, hel_index, label, helicities)
     class(blha_driver_t), intent(inout) :: driver
     integer, intent(in), dimension(:,:) :: flavors
-    integer, intent(out), dimension(:), allocatable :: amp_type, flv_index, label
+    integer, intent(out), dimension(:), allocatable :: amp_type, &
+         flv_index, hel_index, label
     integer, intent(out), dimension(:,:) :: helicities
     integer :: unit, filestat
     character(len=LEN_MAX_FLAVOR_STRING) :: rd_line
@@ -652,8 +651,8 @@ contains
     integer, parameter :: list_incr = 50
     integer :: n_found
     allocate (amp_type (N_MAX_FLAVORS), flv_index (N_MAX_FLAVORS), &
-           label (N_MAX_FLAVORS))
-    amp_type = -1; flv_index = -1; label = -1
+           hel_index (N_MAX_FLAVORS), label (N_MAX_FLAVORS))
+    amp_type = -1; flv_index = -1; hel_index = -1; label = -1
     helicities = 0
     n_in = size (helicities, dim = 2)
     n_entries = size (flavors, 1) + 2
@@ -661,7 +660,7 @@ contains
     open (unit, file = char (driver%contract_file), status="old")
     read_flavor = .false.
     label_count = 1
-    i_hel = 0
+    i_hel = 1
     n_found = 0
     give_warning = .false.
     do
@@ -670,7 +669,7 @@ contains
          exit
       else
          if (rd_line(1:13) == 'AmplitudeType') then
-            if (i_hel > 2 * n_in - 1) i_hel = 0
+            if (i_hel > 2 * n_in) i_hel = 1
             i_next = find_next_word_index (rd_line, 13)
             if (label_count > size (amp_type)) &
                  call extend_integer_array (amp_type, list_incr)
@@ -679,10 +678,10 @@ contains
             else if (rd_line(i_next : i_next + 4) == 'Tree') then
                amp_type(label_count) = BLHA_AMP_TREE
             else if (rd_line(i_next : i_next + 6) == 'ccTree') then
-               amp_type(label_count) = BLHA_AMP_CC
+               amp_type(label_count) = BLHA_AMP_COLOR_C
             else if (rd_line(i_next : i_next + 6) == 'scTree' .or. &
                  rd_line(i_next : i_next + 14) == 'sctree_polvect') then
-               amp_type(label_count) = BLHA_AMP_SC
+               amp_type(label_count) = BLHA_AMP_SPIN_C
             else
                call msg_fatal ("AmplitudeType present but AmpType not known!")
             end if
@@ -703,7 +702,10 @@ contains
                       label(label_count) = i_array (n_entries)
                       if (label_count > size (flv_index)) &
                            call extend_integer_array (flv_index, list_incr)
-                      flv_index (label_count) = i_flv + i_hel
+                      flv_index (label_count) = i_flv
+                      if (label_count > size (hel_index)) &
+                           call extend_integer_array (hel_index, list_incr)
+                      hel_index (label_count) = i_hel
                       if (driver%include_polarizations) then
                          helicities (label(label_count), :) = hel_array (1:n_in)
                          i_hel = i_hel + 1
@@ -986,43 +988,46 @@ contains
   subroutine prc_blha_read_contract_file (object, flavors)
     class(prc_blha_t), intent(inout) :: object
     integer, intent(in), dimension(:,:) :: flavors
-    integer, dimension(:), allocatable :: amp_type, flv_index, label
+    integer, dimension(:), allocatable :: amp_type, flv_index, hel_index, label
     integer, dimension(:,:), allocatable :: helicities
-    integer :: i_proc
+    integer :: i_proc, i_hel
     allocate (helicities (N_MAX_FLAVORS, object%data%n_in))
     select type (driver => object%driver)
     class is (blha_driver_t)
        call driver%read_contract_file (flavors, amp_type, flv_index, &
-            label, helicities)
+            hel_index, label, helicities)
     end select
     object%n_proc = count (amp_type >= 0)
     do i_proc = 1, object%n_proc
        if (amp_type (i_proc) < 0) exit
+       if (hel_index(i_proc) < 0 .and. object%includes_polarization ()) &
+               call msg_bug ("Object includes polarization, but helicity index is undefined.")
+       i_hel = hel_index (i_proc)
        select case (amp_type (i_proc))
        case (BLHA_AMP_TREE)
           if (allocated (object%i_tree)) then
-             object%i_tree(flv_index(i_proc)) = label(i_proc)
+             object%i_tree(flv_index(i_proc), i_hel) = label(i_proc)
           else
              call msg_fatal ("Tree matrix element present, &
                   &but neither Born nor real indices are allocated!")
           end if
-       case (BLHA_AMP_CC)
-          if (allocated (object%i_cc)) then
-             object%i_cc(flv_index(i_proc)) = label(i_proc)
+       case (BLHA_AMP_COLOR_C)
+          if (allocated (object%i_color_c)) then
+             object%i_color_c(flv_index(i_proc), i_hel) = label(i_proc)
           else
              call msg_fatal ("Color-correlated matrix element present, &
                   &but cc-indices are not allocated!")
           end if
-       case (BLHA_AMP_SC)
-          if (allocated (object%i_sc)) then
-             object%i_sc(flv_index(i_proc)) = label(i_proc)
+       case (BLHA_AMP_SPIN_C)
+          if (allocated (object%i_spin_c)) then
+             object%i_spin_c(flv_index(i_proc), i_hel) = label(i_proc)
           else
              call msg_fatal ("Spin-correlated matrix element present, &
                   &but sc-indices are not allocated!")
           end if
        case (BLHA_AMP_LOOP)
           if (allocated (object%i_virt)) then
-             object%i_virt(flv_index(i_proc)) = label(i_proc)
+             object%i_virt(flv_index(i_proc), i_hel) = label(i_proc)
           else
              call msg_fatal ("Loop matrix element present, &
                   &but virt-indices are not allocated!")
@@ -1078,33 +1083,41 @@ contains
     object%n_hel = n_hel
     if (blha_template%compute_loop ()) then
        if (blha_template%include_polarizations) then
-          allocate (object%i_virt (n_flv * n_hel), &
-               object%i_cc (n_flv * n_hel))
+          allocate (object%i_virt (n_flv, n_hel), &
+               object%i_color_c (n_flv, n_hel))
           if (blha_template%use_internal_color_correlations) then
              allocate (object%i_hel (n_flv * n_in * n_hel * 2, n_in))
           else
              allocate (object%i_hel (n_flv * n_in * n_hel, n_in))
           end if
        else
-          allocate (object%i_virt (n_flv), object%i_cc (n_flv))
+          allocate (object%i_virt (n_flv, 1), object%i_color_c (n_flv, 1))
        end if
        object%i_virt = 0
-       object%i_cc = 0
+       object%i_color_c = 0
     else if (blha_template%compute_subtraction ()) then
        if (blha_template%include_polarizations) then
-          allocate (object%i_tree (n_flv * n_hel), &
-               object%i_cc (n_flv * n_hel), &
-               object%i_sc (n_flv * n_hel))
+          allocate (object%i_tree (n_flv, n_hel), &
+               object%i_color_c (n_flv, n_hel), &
+               object%i_spin_c (n_flv, n_hel), &
+               object%i_hel (3 * (n_flv * n_hel * n_in), n_in))
+          object%i_hel = 0
        else
-          allocate (object%i_tree (n_flv), object%i_cc (n_flv) , &
-               object%i_sc (n_flv))
+          allocate (object%i_tree (n_flv, 1), object%i_color_c (n_flv, 1) , &
+               object%i_spin_c (n_flv, 1))
        end if
        object%i_tree = 0
-       object%i_cc = 0
-       object%i_sc = 0
-    else if (blha_template%compute_real_trees () .or. blha_template%compute_born ()) then
-       if (blha_template%include_polarizations) allocate (object%i_hel (n_hel, n_in))
-       allocate (object%i_tree (n_flv * n_hel))
+       object%i_color_c = 0
+       object%i_spin_c = 0
+    else if (blha_template%compute_real_trees () .or. blha_template%compute_born () &
+           .or. blha_template%compute_dglap ()) then
+       if (blha_template%include_polarizations) then
+          allocate (object%i_tree (n_flv, n_hel))
+          allocate (object%i_hel (n_flv * n_hel * n_in, n_in))
+          object%i_hel = 0
+       else
+          allocate (object%i_tree (n_flv, 1))
+       end if
        object%i_tree = 0
     end if
 
@@ -1176,9 +1189,9 @@ contains
   end subroutine prc_blha_init_ew_parameters
 
   subroutine prc_blha_compute_sqme_virt (object, &
-       i_flv, p, ren_scale, sqme, bad_point)
+       i_flv, i_hel, p, ren_scale, sqme, bad_point)
     class(prc_blha_t), intent(in) :: object
-    integer, intent(in) :: i_flv
+    integer, intent(in) :: i_flv, i_hel
     type(vector4_t), dimension(:), intent(in) :: p
     real(default), intent(in) :: ren_scale
     real(default), dimension(4), intent(out) :: sqme
@@ -1189,11 +1202,11 @@ contains
     real(double) :: acc_dble
     real(default) :: acc
     real(default) :: alpha_s
-    if (object%i_virt(i_flv) > 0) then
+    if (object%i_virt(i_flv, i_hel) > 0) then
        allocate (r (blha_result_array_size (object%n_particles, BLHA_AMP_LOOP)))
        call msg_debug2 (D_VIRTUAL, "prc_blha_compute_sqme_virt")
        call msg_debug2 (D_VIRTUAL, "i_flv", i_flv)
-       call msg_debug2 (D_VIRTUAL, "object%i_virt(i_flv)", object%i_virt(i_flv))
+       call msg_debug2 (D_VIRTUAL, "object%i_virt(i_flv, i_hel)", object%i_virt(i_flv, i_hel))
        if (debug2_active (D_VIRTUAL)) then
            call msg_debug2 (D_VIRTUAL, "use momenta: ")
            call vector4_write_set (p, show_mass = .true., &
@@ -1207,7 +1220,7 @@ contains
        select type (driver => object%driver)
        class is (blha_driver_t)
           call driver%set_alpha_s (alpha_s)
-          call driver%blha_olp_eval2 (object%i_virt(i_flv), mom, mu_dble, r, acc_dble)
+          call driver%blha_olp_eval2 (object%i_virt(i_flv, i_hel), mom, mu_dble, r, acc_dble)
        end select
        acc = acc_dble
        sqme = r(1:4)
@@ -1218,10 +1231,10 @@ contains
     end if
   end subroutine prc_blha_compute_sqme_virt
 
-  subroutine prc_blha_compute_sqme (object, i_flv, p, &
+  subroutine prc_blha_compute_sqme (object, i_flv, i_hel, p, &
       ren_scale, sqme, bad_point)
     class(prc_blha_t), intent(in) :: object
-    integer, intent(in) :: i_flv
+    integer, intent(in) :: i_flv, i_hel
     type(vector4_t), intent(in), dimension(:) :: p
     real(default), intent(in) :: ren_scale
     real(default), intent(out) :: sqme
@@ -1230,7 +1243,7 @@ contains
     real(double), dimension(OLP_RESULTS_LIMIT) :: r
     real(double) :: mu_dble, acc_dble
     real(default) :: acc, alpha_s
-    if (object%i_tree(i_flv) > 0) then
+    if (object%i_tree(i_flv, i_hel) > 0) then
        mom = object%create_momentum_array (p)
        if (vanishes (ren_scale)) &
             call msg_fatal ("prc_blha_compute_sqme: ren_scale vanishes")
@@ -1239,7 +1252,7 @@ contains
        select type (driver => object%driver)
        class is (blha_driver_t)
           call driver%set_alpha_s (alpha_s)
-          call driver%blha_olp_eval2 (object%i_tree(i_flv), mom, &
+          call driver%blha_olp_eval2 (object%i_tree(i_flv, i_hel), mom, &
                mu_dble, r, acc_dble)
           sqme = r(object%sqme_tree_pos)
        end select
@@ -1251,43 +1264,51 @@ contains
     end if
   end subroutine prc_blha_compute_sqme
 
-  subroutine blha_cc_fill_diag (sqme_born, flavors, sqme_cc)
+  subroutine blha_color_c_fill_diag (sqme_born, flavors, sqme_color_c)
      real(default), intent(in) :: sqme_born
      integer, intent(in), dimension(:) :: flavors
-     real(default), intent(inout), dimension(:,:) :: sqme_cc
+     real(default), intent(inout), dimension(:,:) :: sqme_color_c
      integer :: i
      do i = 1, size (flavors)
         if (is_quark (flavors(i))) then
-           sqme_cc (i, i) = -cf * sqme_born
+           sqme_color_c (i, i) = -cf * sqme_born
         else if (is_gluon (flavors(i))) then
-           sqme_cc (i, i) = -ca * sqme_born
+           sqme_color_c (i, i) = -ca * sqme_born
         else
-           sqme_cc (i, i) = zero
+           sqme_color_c (i, i) = zero
         end if
      end do
-  end subroutine blha_cc_fill_diag
+  end subroutine blha_color_c_fill_diag
 
-  subroutine blha_cc_fill_offdiag (n, r, sqme_cc)
+  subroutine blha_color_c_fill_offdiag (n, r, sqme_color_c, offset, n_flv)
     integer, intent(in) :: n
     real(default), intent(in), dimension(:) :: r
-    real(default), intent(inout), dimension(:,:) :: sqme_cc
-    integer :: i, j, im1, jm1, pos
+    real(default), intent(inout), dimension(:,:) :: sqme_color_c
+    integer, intent(in), optional :: offset, n_flv
+    integer :: i, j, pos, incr
+    if (present (offset)) then
+       incr = offset
+    else
+       incr = 0
+    end if
+    pos = 0
     do j = 1, n
        do i = 1, j
           if (i /= j) then
-             im1 = i - 1; jm1 = j - 1
-             pos = im1 + jm1 * (jm1 - 1) / 2 + 1
-             sqme_cc (i, j) = -r (pos)
+             pos = (j - 1) * (j - 2) / 2 + i
+             if (present (n_flv))  incr = incr + n_flv - 1
+             if (present (offset))  pos = pos + incr
+             sqme_color_c (i, j) = -r (pos)
+             sqme_color_c (j, i) = sqme_color_c (i, j)
           end if
-          sqme_cc (j, i) = sqme_cc (i, j)
        end do
     end do
-  end subroutine blha_cc_fill_offdiag
+  end subroutine blha_color_c_fill_offdiag
 
-  subroutine prc_blha_compute_sqme_cc_raw &
-     (object, i_flv, p, ren_scale, rr, bad_point)
+  subroutine prc_blha_compute_sqme_color_c_raw &
+     (object, i_flv, i_hel, p, ren_scale, rr, bad_point)
     class(prc_blha_t), intent(in) :: object
-    integer, intent(in) :: i_flv
+    integer, intent(in) :: i_flv, i_hel
     type(vector4_t), intent(in), dimension(:) :: p
     real(default), intent(in) :: ren_scale
     real(default), intent(out), dimension(:) :: rr
@@ -1296,35 +1317,35 @@ contains
     real(double), dimension(size(rr)) :: r
     real(default) :: alpha_s, acc
     real(double) :: mu_dble, acc_dble
-    if (object%i_cc(i_flv) > 0) then
+    if (object%i_color_c(i_flv, i_hel) > 0) then
        mom = object%create_momentum_array (p)
        if (vanishes (ren_scale)) &
-          call msg_fatal ("prc_blha_compute_sqme_cc: ren_scale vanishes")
+          call msg_fatal ("prc_blha_compute_sqme_color_c: ren_scale vanishes")
        mu_dble = dble(ren_scale)
        alpha_s = object%qcd%alpha%get (ren_scale)
 
        select type (driver => object%driver)
        class is (blha_driver_t)
           call driver%set_alpha_s (alpha_s)
-          call driver%blha_olp_eval2 (object%i_cc(i_flv), &
+          call driver%blha_olp_eval2 (object%i_color_c(i_flv, i_hel), &
                mom, mu_dble, r, acc_dble)
        end select
        rr = r
        acc = acc_dble
        bad_point = acc > object%maximum_accuracy
-       if (object%includes_polarization ()) rr = object%n_hel * rr
+       if (object%includes_polarization ())  rr = object%n_hel * rr
     else
        rr = zero
     end if
-  end subroutine prc_blha_compute_sqme_cc_raw
+  end subroutine prc_blha_compute_sqme_color_c_raw
 
-  subroutine prc_blha_compute_sqme_cc &
-         (object, i_flv, p, ren_scale, born_cc, bad_point, born_out)
+  subroutine prc_blha_compute_sqme_color_c &
+         (object, i_flv, i_hel, p, ren_scale, born_color_c, bad_point, born_out)
     class(prc_blha_t), intent(inout) :: object
-    integer, intent(in) :: i_flv
+    integer, intent(in) :: i_flv, i_hel
     type(vector4_t), intent(in), dimension(:) :: p
     real(default), intent(in) :: ren_scale
-    real(default), intent(inout), dimension(:,:) :: born_cc
+    real(default), intent(inout), dimension(:,:) :: born_color_c
     real(default), intent(out), optional :: born_out
     logical, intent(out) :: bad_point
     real(default), dimension(:), allocatable :: r
@@ -1332,27 +1353,24 @@ contains
     real(default) :: born
     integer, dimension(:), allocatable :: flavors
     allocate (r (blha_result_array_size &
-         (size(born_cc, dim=1), BLHA_AMP_CC)))
-    call object%compute_sqme_cc_raw (i_flv, p, ren_scale, r, bad_point)
+         (size(born_color_c, dim=1), BLHA_AMP_COLOR_C)))
+    call object%compute_sqme_color_c_raw (i_flv, i_hel, p, ren_scale, r, bad_point)
 
     select type (driver => object%driver)
     class is (blha_driver_t)
        if (allocated (object%i_tree)) then
-          call object%compute_sqme (i_flv, p, ren_scale, born, bad_point2)
+          call object%compute_sqme (i_flv, i_hel, p, ren_scale, born, bad_point2)
        else
           born = zero
        end if
        if (present (born_out)) born_out = born
     end select
-    call blha_cc_fill_offdiag (object%n_particles, r, born_cc)
-    !!! !!! !!! Workaround for ifort 16.0 missing default allocate-on-assignment
-    allocate (flavors (object%get_n_flvs (i_flv)))
-    !!! !!! !!! End of workaround
+    call blha_color_c_fill_offdiag (object%n_particles, r, born_color_c)
     flavors = object%get_flv_state (i_flv)
-    call blha_cc_fill_diag (born, flavors, born_cc)
+    call blha_color_c_fill_diag (born, flavors, born_color_c)
 
     bad_point = bad_point .or. bad_point2
-  end subroutine prc_blha_compute_sqme_cc
+  end subroutine prc_blha_compute_sqme_color_c
 
   function prc_blha_get_beam_helicities_single (object, i, invert_second) result (hel)
     integer, dimension(:), allocatable :: hel
@@ -1386,12 +1404,6 @@ contains
     end do
   end function prc_blha_get_beam_helicities_array
 
-  subroutine prc_blha_reset_helicity_list (object)
-    class(prc_blha_t), intent(inout) :: object
-    if (allocated (object%i_whizard_to_i_olc)) &
-         deallocate (object%i_whizard_to_i_olc)
-  end subroutine prc_blha_reset_helicity_list
-
   recursive function blha_loop_positions (i_flv, n_sub) result (index)
     integer :: index
     integer, intent(in) :: i_flv, n_sub
@@ -1403,76 +1415,6 @@ contains
     end if
   end function blha_loop_positions
 
-  subroutine prc_blha_set_helicity_list_trivial (object, n_flv, n_sub)
-    class(prc_blha_t), intent(inout) :: object
-    integer, intent(in) :: n_flv, n_sub
-    integer :: i_flv
-    if (allocated (object%i_whizard_to_i_olc)) &
-         deallocate (object%i_whizard_to_i_olc)
-    allocate (object%i_whizard_to_i_olc (n_flv))
-    do i_flv = 1, n_flv
-       object%i_whizard_to_i_olc (i_flv) = blha_loop_positions (i_flv, n_sub)
-    end do
-  end subroutine prc_blha_set_helicity_list_trivial
-
-  subroutine prc_blha_set_helicity_list (object, &
-     helicities)
-    class(prc_blha_t), intent(inout) :: object
-    integer, intent(in), dimension(:,:) :: helicities
-    integer, dimension(:, :), allocatable :: hel_olc
-    integer :: n1, n2
-    integer :: i, j
-    n1 = size (helicities, dim=1)
-    n2 = size (object%get_beam_helicities (), dim=1)
-    if (allocated (object%i_whizard_to_i_olc)) &
-         deallocate (object%i_whizard_to_i_olc)
-    allocate (object%i_whizard_to_i_olc (n2))
-    allocate (hel_olc (n2, 2))
-    hel_olc = object%get_beam_helicities (invert_second = .true.)
-    do i = 1, n1
-       do j = 1, n2
-          if (all (helicities (i, :) == hel_olc (j, :))) &
-             object%i_whizard_to_i_olc (j) = i
-       end do
-    end do
-    deallocate (hel_olc)
-  end subroutine prc_blha_set_helicity_list
-
-  function prc_blha_get_helicity_list_all (object) result (i_out)
-    integer, dimension(:), allocatable :: i_out
-    class(prc_blha_t), intent(in) :: object
-    allocate (i_out (size (object%i_whizard_to_i_olc)))
-    i_out = object%i_whizard_to_i_olc
-  end function prc_blha_get_helicity_list_all
-
-  function prc_blha_get_helicity_list_single (object, i) result (i_out)
-    integer :: i_out
-    class(prc_blha_t), intent(in) :: object
-    integer, intent(in) :: i
-    i_out = object%i_whizard_to_i_olc (i)
-  end function prc_blha_get_helicity_list_single
-
-  subroutine prc_blha_warmup_helicities (object, p)
-    class(prc_blha_t), intent(inout) :: object
-    type(vector4_t), intent(in), dimension(:) :: p
-    real(double), dimension(5 * object%n_particles) :: mom
-    real(double), dimension(:), allocatable :: r
-    real(default) :: alpha_s
-    real(double) :: ren_scale, acc
-    integer :: i_hel
-    allocate (r (blha_result_array_size (object%n_particles, BLHA_AMP_LOOP)))
-    alpha_s = 0.1178_double
-    ren_scale = 200._double
-    mom = object%create_momentum_array (p)
-    select type (driver => object%driver)
-    class is (blha_driver_t)
-       call driver%set_alpha_s (alpha_s)
-       do i_hel = 1, object%n_hel
-          call driver%blha_olp_eval2 (i_hel, mom, ren_scale, r, acc)
-       end do
-    end select
-  end subroutine prc_blha_warmup_helicities
-
 
 end module blha_olp_interfaces
 
