@@ -1,19 +1,12 @@
-! WHIZARD 2.4.1 Mar 24 2017
+! WHIZARD 2.6.4 Aug 23 2018
 !
-! Copyright (C) 1999-2017 by
+! Copyright (C) 1999-2018 by
 !     Wolfgang Kilian <kilian@physik.uni-siegen.de>
 !     Thorsten Ohl <ohl@physik.uni-wuerzburg.de>
 !     Juergen Reuter <juergen.reuter@desy.de>
 !
 !     with contributions from
-!     Fabian Bach <fabian.bach@t-online.de>
-!     Bijan Chokoufe <bijan.chokoufe@desy.de>
-!     Christian Speckner <cnspeckn@googlemail.com>
-!     So Young Shim <soyoung.shim@desy.de>
-!     Florian Staub <florian.staub@cern.ch>
-!     Christian Weiss <christian.weiss@desy.de>
-!     and Hans-Werner Boschmann, Felix Braam,
-!     Sebastian Schmidt, So-young Shim, Daniel Wiesler
+!     cf. main AUTHORS file
 !
 ! WHIZARD is free software; you can redistribute it and/or modify it
 ! under the terms of the GNU General Public License as published by
@@ -62,7 +55,7 @@ contains
     integer :: n_in, n_out
     integer :: alpha_power, alphas_power
     integer, dimension(:,:), allocatable :: flv_born, flv_real
-    type(string_t) :: proc_id, method
+    type(string_t) :: proc_id, method, correction_type
     type(os_data_t) :: os_data
     type(model_list_t) :: model_list
     type(var_list_t) :: var_list
@@ -107,6 +100,7 @@ contains
     write (u, "(A)") "* Mode: GoSam"
 
     method = var_str ("gosam")
+    correction_type = var_str ("QCD")
     call var_list%append_string (var_str ("$born_me_method"), method)
     call var_list%append_string (var_str ("$real_tree_me_method"), method)
     call var_list%append_string (var_str ("$loop_me_method"), method)
@@ -114,6 +108,7 @@ contains
     call blha_master%set_ew_scheme (var_str ("GF"))
     call blha_master%set_methods (.true., var_list)
     call blha_master%allocate_config_files ()
+    call blha_master%set_correction_type (correction_type)
     call blha_master%generate (proc_id, model, n_in, &
          alpha_power, alphas_power, flv_born, flv_real)
 
@@ -125,12 +120,14 @@ contains
     openloops_phs_tolerance = 7
 
     method = var_str ("openloops")
+    correction_type = var_str ("QCD")
     call var_list%append_string (var_str ("$born_me_method"), method)
     call var_list%append_string (var_str ("$real_tree_me_method"), method)
     call var_list%append_string (var_str ("$loop_me_method"), method)
     call var_list%append_string (var_str ("$correlation_me_method"), method)
     call blha_master%set_methods (.true., var_list)
     call blha_master%allocate_config_files ()
+    call blha_master%set_correction_type (correction_type)
     call blha_master%generate (proc_id, model, n_in, &
          alpha_power, alphas_power, flv_born, flv_real)
 
