@@ -1,19 +1,12 @@
-! WHIZARD 2.4.1 Mar 24 2017
+! WHIZARD 2.6.4 Aug 23 2018
 !
-! Copyright (C) 1999-2017 by
+! Copyright (C) 1999-2018 by
 !     Wolfgang Kilian <kilian@physik.uni-siegen.de>
 !     Thorsten Ohl <ohl@physik.uni-wuerzburg.de>
 !     Juergen Reuter <juergen.reuter@desy.de>
 !
 !     with contributions from
-!     Fabian Bach <fabian.bach@t-online.de>
-!     Bijan Chokoufe <bijan.chokoufe@desy.de>
-!     Christian Speckner <cnspeckn@googlemail.com>
-!     So Young Shim <soyoung.shim@desy.de>
-!     Florian Staub <florian.staub@cern.ch>
-!     Christian Weiss <christian.weiss@desy.de>
-!     and Hans-Werner Boschmann, Felix Braam,
-!     Sebastian Schmidt, So-young Shim, Daniel Wiesler
+!     cf. main AUTHORS file
 !
 ! WHIZARD is free software; you can redistribute it and/or modify it
 ! under the terms of the GNU General Public License as published by
@@ -45,7 +38,6 @@ module sorting
   public :: sort_abs
   public :: order
   public :: order_abs
-  public :: concat
 
   interface sort
      module procedure sort_int
@@ -74,11 +66,6 @@ module sorting
      module procedure merge_int_abs
   end interface
 
-  interface concat
-     module procedure concat_int
-     module procedure concat_real
-  end interface
-
 
 contains
 
@@ -105,10 +92,6 @@ contains
     integer, dimension(size(val)) :: idx
     integer :: n, i, s, b1, b2, e1, e2
     n = size (idx)
-    !!! !!! !!! Work around ifort 16.0.(0/1) compiler bug
-    !forall (i = 1:n)
-       !idx(i) = i
-    !end forall
     do i = 1, n
        idx(i) = i
     end do
@@ -129,10 +112,6 @@ contains
     integer, dimension(size(val)) :: idx
     integer :: n, i, s, b1, b2, e1, e2
     n = size (idx)
-    !!! !!! !!! Work around ifort 16.0.(0/1) compiler bug
-    !forall (i = 1:n)
-       !idx(i) = i
-    !end forall
     do i = 1, n
        idx(i) = i
     end do
@@ -153,10 +132,6 @@ contains
     integer, dimension(size(val)) :: idx
     integer :: n, i, s, b1, b2, e1, e2
     n = size (idx)
-    !!! !!! !!! Work around ifort 16.0.(0/1) compiler bug
-    !forall (i = 1:n)
-       !idx(i) = i
-    !end forall
     do i = 1, n
        idx(i) = i
     end do
@@ -252,19 +227,5 @@ contains
     res = tmp
   end subroutine merge_int_abs
 
-  function concat_int (val1, val2) result (val12)
-    integer, dimension(:), intent(in) :: val1, val2
-    integer, dimension(size(val1)+size(val2)) :: val12
-    val12(:size(val1)) = val1
-    val12(size(val1)+1:) = val2
-  end function concat_int
-
-  function concat_real (val1, val2) result (val12)
-    real(default), dimension(:), intent(in) :: val1, val2
-    integer, dimension(size(val1)+size(val2)) :: val12
-    val12(:size(val1)) = val1
-    val12(size(val1)+1:) = val2
-  end function concat_real
-
 
 end module sorting
