@@ -1,19 +1,12 @@
-! WHIZARD 2.4.1 Mar 24 2017
+! WHIZARD 2.6.4 Aug 23 2018
 !
-! Copyright (C) 1999-2017 by
+! Copyright (C) 1999-2018 by
 !     Wolfgang Kilian <kilian@physik.uni-siegen.de>
 !     Thorsten Ohl <ohl@physik.uni-wuerzburg.de>
 !     Juergen Reuter <juergen.reuter@desy.de>
 !
 !     with contributions from
-!     Fabian Bach <fabian.bach@t-online.de>
-!     Bijan Chokoufe <bijan.chokoufe@desy.de>
-!     Christian Speckner <cnspeckn@googlemail.com>
-!     So Young Shim <soyoung.shim@desy.de>
-!     Florian Staub <florian.staub@cern.ch>
-!     Christian Weiss <christian.weiss@desy.de>
-!     and Hans-Werner Boschmann, Felix Braam,
-!     Sebastian Schmidt, So-young Shim, Daniel Wiesler
+!     cf. main AUTHORS file
 !
 ! WHIZARD is free software; you can redistribute it and/or modify it
 ! under the terms of the GNU General Public License as published by
@@ -104,6 +97,7 @@ contains
     end select
     call eio%init_out (sample, data)
     call event%generate (1, [0._default, 0._default])
+    call event%set_index (42)
     call event%evaluate_expressions ()
 
     call eio%output (event, i_prc = 1)
@@ -189,6 +183,7 @@ contains
     end select
     call eio%init_out (sample, data)
     call event%generate (1, [0._default, 0._default])
+    call event%set_index (42)
     call event%evaluate_expressions ()
 
     call eio%output (event, i_prc = 1)
@@ -274,6 +269,7 @@ contains
     end select
     call eio%init_out (sample, data)
     call event%generate (1, [0._default, 0._default])
+    call event%increment_index ()
     call event%evaluate_expressions ()
 
     call eio%output (event, i_prc = 1)
@@ -359,6 +355,7 @@ contains
     end select
     call eio%init_out (sample, data)
     call event%generate (1, [0._default, 0._default])
+    call event%increment_index ()
     call event%evaluate_expressions ()
 
     call eio%output (event, i_prc = 1)
@@ -444,6 +441,7 @@ contains
     end select
     call eio%init_out (sample, data)
     call event%generate (1, [0._default, 0._default])
+    call event%increment_index ()
     call event%evaluate_expressions ()
 
     call eio%output (event, i_prc = 1)
@@ -529,6 +527,7 @@ contains
     end select
     call eio%init_out (sample, data)
     call event%generate (1, [0._default, 0._default])
+    call event%increment_index ()
     call event%evaluate_expressions ()
 
     call eio%output (event, i_prc = 1)
@@ -614,6 +613,7 @@ contains
     end select
     call eio%init_out (sample, data)
     call event%generate (1, [0._default, 0._default])
+    call event%increment_index ()
     call event%evaluate_expressions ()
 
     call eio%output (event, i_prc = 1)
@@ -699,6 +699,7 @@ contains
     end select
     call eio%init_out (sample, data)
     call event%generate (1, [0._default, 0._default])
+    call event%increment_index ()
     call event%evaluate_expressions ()
 
     call eio%output (event, i_prc = 1)
@@ -784,6 +785,7 @@ contains
     end select
     call eio%init_out (sample, data)
     call event%generate (1, [0._default, 0._default])
+    call event%increment_index ()
     call event%evaluate_expressions ()
 
     call eio%output (event, i_prc = 1)
@@ -869,6 +871,7 @@ contains
     end select
     call eio%init_out (sample, data)
     call event%generate (1, [0._default, 0._default])
+    call event%increment_index ()
     call event%evaluate_expressions ()
 
     call eio%output (event, i_prc = 1)
