@@ -1,19 +1,12 @@
-! WHIZARD 2.4.1 Mar 24 2017
+! WHIZARD 2.6.4 Aug 23 2018
 !
-! Copyright (C) 1999-2017 by
+! Copyright (C) 1999-2018 by
 !     Wolfgang Kilian <kilian@physik.uni-siegen.de>
 !     Thorsten Ohl <ohl@physik.uni-wuerzburg.de>
 !     Juergen Reuter <juergen.reuter@desy.de>
 !
 !     with contributions from
-!     Fabian Bach <fabian.bach@t-online.de>
-!     Bijan Chokoufe <bijan.chokoufe@desy.de>
-!     Christian Speckner <cnspeckn@googlemail.com>
-!     So Young Shim <soyoung.shim@desy.de>
-!     Florian Staub <florian.staub@cern.ch>
-!     Christian Weiss <christian.weiss@desy.de>
-!     and Hans-Werner Boschmann, Felix Braam,
-!     Sebastian Schmidt, So-young Shim, Daniel Wiesler
+!     cf. main AUTHORS file
 !
 ! WHIZARD is free software; you can redistribute it and/or modify it
 ! under the terms of the GNU General Public License as published by
@@ -79,7 +72,7 @@ module eio_dump
 contains
 
   subroutine eio_dump_set_parameters (eio, extension, &
-       pacify, weights, compressed, summary, screen)
+       pacify, weights, compressed, summary, screen, unit)
     class(eio_dump_t), intent(inout) :: eio
     type(string_t), intent(in), optional :: extension
     logical, intent(in), optional :: pacify
@@ -87,11 +80,13 @@ contains
     logical, intent(in), optional :: compressed
     logical, intent(in), optional :: summary
     logical, intent(in), optional :: screen
+    integer, intent(in), optional :: unit
     if (present (pacify))  eio%pacify = pacify
     if (present (weights))  eio%weights = weights
     if (present (compressed))  eio%compressed = compressed
     if (present (summary))  eio%summary = summary
     if (present (screen))  eio%screen = screen
+    if (present (unit))  eio%unit = unit
     eio%extension = "pset.dat"
     if (present (extension))  eio%extension = extension
   end subroutine eio_dump_set_parameters
@@ -104,7 +99,7 @@ contains
     write (u, "(1x,A)")  "Dump event stream:"
     if (object%writing) then
        write (u, "(3x,A,L1)") "Screen output     = ", object%screen
-       write (u, "(3x,A,A)")  "Writing to file   = ", char (object%filename)
+       write (u, "(3x,A,A,A)")  "Writing to file   = '", char (object%filename), "'"
        write (u, "(3x,A,L1)") "Reduced I/O prec. = ", object%pacify
        write (u, "(3x,A,L1)") "Show weights/sqme = ", object%weights
        write (u, "(3x,A,L1)") "Compressed        = ", object%compressed
@@ -122,10 +117,12 @@ contains
        object%screen = .false.
     end if
     if (object%writing) then
-       write (msg_buffer, "(A,A,A)")  "Events: closing event dump file '", &
-            char (object%filename), "'"
-       call msg_message ()
-       close (object%unit)
+       if (object%filename /= "") then
+          write (msg_buffer, "(A,A,A)")  "Events: closing event dump file '", &
+               char (object%filename), "'"
+          call msg_message ()
+          close (object%unit)
+       end if
        object%writing = .false.
     end if
   end subroutine eio_dump_final
@@ -141,7 +138,10 @@ contains
     else
        eio%extension = "pset.dat"
     end if
-    if (sample /= "") then
+    if (sample == "" .and. eio%unit /= 0) then
+       eio%filename = ""
+       eio%writing = .true.
+    else if (sample /= "") then
        eio%filename = sample // "." // eio%extension
        eio%unit = free_unit ()
        write (msg_buffer, "(A,A,A)")  "Events: writing to event dump file '", &
@@ -182,9 +182,12 @@ contains
     integer, intent(in) :: i_prc
     logical, intent(in), optional :: reading, passed, pacify
     character(len=7) :: fmt
-    integer :: i
     eio%count = eio%count + 1
-    call pac_fmt (fmt, FMT_19, FMT_16, pacify)
+    if (present (pacify)) then
+       call pac_fmt (fmt, FMT_19, FMT_16, pacify)
+    else
+       call pac_fmt (fmt, FMT_19, FMT_16, eio%pacify)
+    end if
     if (eio%writing)  call dump (eio%unit)
     if (eio%screen) then
        call dump (output_unit)
@@ -193,8 +196,22 @@ contains
   contains
     subroutine dump (u)
       integer, intent(in) :: u
+      integer :: i
       call write_separator (u, 2)
-      write (u, "(1x,A,I0)")  "Event #", eio%count
+      write (u, "(1x,A,I0)", advance="no")  "Event"
+      if (event%has_index ()) then
+         write (u, "(1x,'#',I0)")  event%get_index ()
+      else
+         write (u, *)
+      end if
+      call write_separator (u, 2)
+      write (u, "(1x,A,1x,I0)")  "count  =", eio%count
+      if (present (passed)) then
+         write (u, "(1x,A,1x,L1)")  "passed =", passed
+      else
+         write (u, "(1x,A)")  "passed = [N/A]"
+      end if
+      write (u, "(1x,A,1x,I0)")  "prc id =", i_prc
       if (eio%weights) then
          call write_separator (u)
          if (event%sqme_ref_known) then
@@ -203,7 +220,7 @@ contains
          else
             write (u, "(1x,A)")  "sqme (ref)    = [undefined]"
          end if
-         if (event%sqme_ref_known) then
+         if (event%sqme_prc_known) then
             write (u, "(1x,A," // fmt // ")")  "sqme (prc)   = ", &
                  event%sqme_prc
          else
