@@ -1,19 +1,12 @@
-! WHIZARD 2.4.1 Mar 24 2017
+! WHIZARD 2.6.4 Aug 23 2018
 !
-! Copyright (C) 1999-2017 by
+! Copyright (C) 1999-2018 by
 !     Wolfgang Kilian <kilian@physik.uni-siegen.de>
 !     Thorsten Ohl <ohl@physik.uni-wuerzburg.de>
 !     Juergen Reuter <juergen.reuter@desy.de>
 !
 !     with contributions from
-!     Fabian Bach <fabian.bach@t-online.de>
-!     Bijan Chokoufe <bijan.chokoufe@desy.de>
-!     Christian Speckner <cnspeckn@googlemail.com>
-!     So Young Shim <soyoung.shim@desy.de>
-!     Florian Staub <florian.staub@cern.ch>
-!     Christian Weiss <christian.weiss@desy.de>
-!     and Hans-Werner Boschmann, Felix Braam,
-!     Sebastian Schmidt, So-young Shim, Daniel Wiesler
+!     cf. main AUTHORS file
 !
 ! WHIZARD is free software; you can redistribute it and/or modify it
 ! under the terms of the GNU General Public License as published by
@@ -57,10 +50,11 @@ contains
     integer, intent(in) :: u
     class(generic_event_t), pointer :: event
     class(eio_t), allocatable :: eio
+    integer :: i_prc
     integer :: u_file
 
     write (u, "(A)")  "* Test output: eio_dump_1"
-    write (u, "(A)")  "*   Purpose: generate an event and dump to output"
+    write (u, "(A)")  "*   Purpose: generate events and write essentials to output"
     write (u, "(A)")
 
     write (u, "(A)")  "* Initialize test process"
@@ -68,21 +62,29 @@ contains
     call eio_prepare_test (event, unweighted = .false.)
 
     write (u, "(A)")
-    write (u, "(A)")  "* Generate and write an event"
+    write (u, "(A)")  "* Generate and write three events (two passed)"
     write (u, "(A)")
 
     allocate (eio_dump_t :: eio)
     select type (eio)
     type is (eio_dump_t)
-       eio%unit = u
-       eio%writing = .true.
-       eio%weights = .true.
+       call eio%set_parameters (unit = u, weights = .true., pacify = .true.)
     end select
 
+    i_prc = 42
+    
     call eio%init_out (var_str (""))
-    call event%generate (1, [0._default, 0._default])
 
-    call eio%output (event, i_prc = 42)
+    call event%generate (1, [0._default, 0._default])
+    call eio%output (event, i_prc = i_prc)
+
+    call event%generate (1, [0.1_default, 0._default])
+    call event%set_index (99)
+    call eio%output (event, i_prc = i_prc, passed = .false.)
+
+    call event%generate (1, [0.2_default, 0._default])
+    call event%increment_index ()
+    call eio%output (event, i_prc = i_prc, passed = .true.)
 
     write (u, "(A)")
     write (u, "(A)")  "* Contents of eio_dump object"
