@@ -1,19 +1,12 @@
-! WHIZARD 2.4.1 Mar 24 2017
+! WHIZARD 2.6.4 Aug 23 2018
 !
-! Copyright (C) 1999-2017 by
+! Copyright (C) 1999-2018 by
 !     Wolfgang Kilian <kilian@physik.uni-siegen.de>
 !     Thorsten Ohl <ohl@physik.uni-wuerzburg.de>
 !     Juergen Reuter <juergen.reuter@desy.de>
 !
 !     with contributions from
-!     Fabian Bach <fabian.bach@t-online.de>
-!     Bijan Chokoufe <bijan.chokoufe@desy.de>
-!     Christian Speckner <cnspeckn@googlemail.com>
-!     So Young Shim <soyoung.shim@desy.de>
-!     Florian Staub <florian.staub@cern.ch>
-!     Christian Weiss <christian.weiss@desy.de>
-!     and Hans-Werner Boschmann, Felix Braam,
-!     Sebastian Schmidt, So-young Shim, Daniel Wiesler
+!     cf. main AUTHORS file
 !
 ! WHIZARD is free software; you can redistribute it and/or modify it
 ! under the terms of the GNU General Public License as published by
@@ -35,7 +28,6 @@
 
 module event_base
 
-  use system_defs, only: LF
   use kinds, only: default
   use kinds, only: i64
   use iso_varying_string, string_t => varying_string
@@ -121,9 +113,13 @@ module event_base
      procedure (generic_event_generate), deferred :: generate
      procedure (generic_event_set_hard_particle_set), deferred :: &
           set_hard_particle_set
+     procedure (generic_event_set_index), deferred :: set_index
+     procedure (generic_event_handler), deferred :: reset_index
+     procedure (generic_event_increment_index), deferred :: increment_index
      procedure (generic_event_handler), deferred :: evaluate_expressions
      procedure (generic_event_select), deferred :: select
      procedure (generic_event_get_model_ptr), deferred :: get_model_ptr
+     procedure (generic_event_has_index), deferred :: has_index
      procedure (generic_event_get_index), deferred :: get_index
      procedure (generic_event_get_fac_scale), deferred :: get_fac_scale
      procedure (generic_event_get_alpha_s), deferred :: get_alpha_s
@@ -136,8 +132,8 @@ module event_base
           set_alpha_qcd_forced
      procedure (generic_event_set_scale_forced), deferred :: &
           set_scale_forced
-     procedure :: reset => generic_event_reset
-     procedure :: base_reset => generic_event_reset
+     procedure :: reset_contents => generic_event_reset_contents
+     procedure :: base_reset_contents => generic_event_reset_contents
      procedure :: pacify_particle_set => generic_event_pacify_particle_set
   end type generic_event_t
 
@@ -189,6 +185,14 @@ module event_base
      end subroutine generic_event_set_hard_particle_set
   end interface
 
+  abstract interface
+     subroutine generic_event_set_index (event, index)
+       import
+       class(generic_event_t), intent(inout) :: event
+       integer, intent(in) :: index
+     end subroutine generic_event_set_index
+  end interface
+
   abstract interface
      subroutine generic_event_handler (event)
        import
@@ -196,6 +200,14 @@ module event_base
      end subroutine generic_event_handler
   end interface
 
+  abstract interface
+     subroutine generic_event_increment_index (event, offset)
+       import
+       class(generic_event_t), intent(inout) :: event
+       integer, intent(in), optional :: offset
+     end subroutine generic_event_increment_index
+  end interface
+
   abstract interface
      subroutine generic_event_select (event,  i_mci, i_term, channel)
        import
@@ -212,6 +224,14 @@ module event_base
      end function generic_event_get_model_ptr
   end interface
 
+  abstract interface
+     function generic_event_has_index (event) result (flag)
+       import
+       class(generic_event_t), intent(in) :: event
+       logical :: flag
+     end function generic_event_has_index
+  end interface
+
   abstract interface
      function generic_event_get_index (event) result (index)
        import
@@ -554,7 +574,7 @@ contains
     end if
   end subroutine generic_event_set
 
-  subroutine generic_event_reset (event)
+  subroutine generic_event_reset_contents (event)
     class(generic_event_t), intent(inout) :: event
     call event%discard_particle_set ()
     event%sqme_ref_known = .false.
@@ -564,7 +584,7 @@ contains
     event%weight_prc_known = .false.
     event%weight_alt_known = .false.
     event%excess_prc_known = .false.
-  end subroutine generic_event_reset
+  end subroutine generic_event_reset_contents
 
   subroutine generic_event_pacify_particle_set (event)
     class(generic_event_t), intent(inout) :: event
