@@ -1,19 +1,12 @@
-! WHIZARD 2.4.1 Mar 24 2017
+! WHIZARD 2.6.4 Aug 23 2018
 !
-! Copyright (C) 1999-2017 by
+! Copyright (C) 1999-2018 by
 !     Wolfgang Kilian <kilian@physik.uni-siegen.de>
 !     Thorsten Ohl <ohl@physik.uni-wuerzburg.de>
 !     Juergen Reuter <juergen.reuter@desy.de>
 !
 !     with contributions from
-!     Fabian Bach <fabian.bach@t-online.de>
-!     Bijan Chokoufe <bijan.chokoufe@desy.de>
-!     Christian Speckner <cnspeckn@googlemail.com>
-!     So Young Shim <soyoung.shim@desy.de>
-!     Florian Staub <florian.staub@cern.ch>
-!     Christian Weiss <christian.weiss@desy.de>
-!     and Hans-Werner Boschmann, Felix Braam,
-!     Sebastian Schmidt, So-young Shim, Daniel Wiesler
+!     cf. main AUTHORS file
 !
 ! WHIZARD is free software; you can redistribute it and/or modify it
 ! under the terms of the GNU General Public License as published by
@@ -169,6 +162,10 @@ contains
        end if
        if (present (i_evt)) then
           call hepevt_set_event_parameters (i_evt = i_evt)
+       else if (event%has_index ()) then
+          call hepevt_set_event_parameters (i_evt = event%get_index ())
+       else
+          call hepevt_set_event_parameters (i_evt = 0)
        end if
     else
        call msg_bug ("HEPEVT: event incomplete")
@@ -244,16 +241,9 @@ contains
        call hepmc_event_add_vertex (evt, v(i))
     end do
     allocate (is_beam (n_tot))
-    !!! !!! !!! Workaround for ifort 16.0 standard-semantics bug
-    do i = 1, n_tot
-       is_beam(i) = particle_set%prt(i)%get_status () == PRT_BEAM
-    end do
-    !!! is_beam = particle_set%prt(1:n_tot)%get_status () == PRT_BEAM
+    is_beam = particle_set%prt(1:n_tot)%get_status () == PRT_BEAM
     if (.not. any (is_beam)) then
-       do i = 1, n_tot
-          is_beam(i) = particle_set%prt(i)%get_status () == PRT_INCOMING
-       end do
-       !!! is_beam = particle_set%prt(1:n_tot)%get_status () == PRT_INCOMING
+       is_beam = particle_set%prt(1:n_tot)%get_status () == PRT_INCOMING
     end if
     if (count (is_beam) == 2) then
        hbeam = pack (hprt, is_beam)
@@ -421,6 +411,7 @@ contains
     real(default) :: scale, alpha_qcd
     type(particle_set_t) :: particle_set
     model => event%get_model_ptr ()
+    call event%set_index (hepmc_event_get_event_index (hepmc_event))
     call hepmc_event_to_particle_set (particle_set, &
          hepmc_event, model, fallback_model, PRT_DEFINITE_HELICITY)
     call event%set_hard_particle_set (particle_set)
@@ -621,6 +612,7 @@ contains
     real(default) :: scale, alpha_qcd
     type(particle_set_t) :: particle_set
     model => event%get_model_ptr ()
+
     call lcio_event_to_particle_set (particle_set, &
          lcio_event, model, fallback_model, PRT_DEFINITE_HELICITY)
     call event%set_hard_particle_set (particle_set)
