@@ -1,19 +1,12 @@
-! WHIZARD 2.4.1 Mar 24 2017
+! WHIZARD 2.6.4 Aug 23 2018
 !
-! Copyright (C) 1999-2017 by
+! Copyright (C) 1999-2018 by
 !     Wolfgang Kilian <kilian@physik.uni-siegen.de>
 !     Thorsten Ohl <ohl@physik.uni-wuerzburg.de>
 !     Juergen Reuter <juergen.reuter@desy.de>
 !
 !     with contributions from
-!     Fabian Bach <fabian.bach@t-online.de>
-!     Bijan Chokoufe <bijan.chokoufe@desy.de>
-!     Christian Speckner <cnspeckn@googlemail.com>
-!     So Young Shim <soyoung.shim@desy.de>
-!     Florian Staub <florian.staub@cern.ch>
-!     Christian Weiss <christian.weiss@desy.de>
-!     and Hans-Werner Boschmann, Felix Braam,
-!     Sebastian Schmidt, So-young Shim, Daniel Wiesler
+!     cf. main AUTHORS file
 !
 ! WHIZARD is free software; you can redistribute it and/or modify it
 ! under the terms of the GNU General Public License as published by
@@ -60,6 +53,7 @@ module lcio_interface
   public :: show_lcio_event
   public :: write_lcio_event
   public :: lcio_event_final
+  public :: lcio_event_set_weight
   public :: lcio_event_set_alpha_qcd
   public :: lcio_event_set_scale
   public :: lcio_event_set_sqrts
@@ -98,6 +92,7 @@ module lcio_interface
   public :: lcio_open_file
   public :: lcio_reader_close
   public :: lcio_read_event
+  public :: lcio_event_get_event_index
   public :: lcio_event_get_process_id
   public :: lcio_event_get_n_tot
   public :: lcio_event_get_alphas
@@ -192,6 +187,13 @@ module lcio_interface
        character(c_char), dimension(*), intent(in) :: filename
      end subroutine lcio_event_to_file
   end interface
+  interface
+     subroutine lcio_set_weight (evt_obj, weight) bind(C)
+       import
+       type(c_ptr), value :: evt_obj
+       real(c_double), value :: weight
+     end subroutine lcio_set_weight
+  end interface
   interface
      subroutine lcio_set_alpha_qcd (evt_obj, alphas) bind(C)
        import
@@ -464,6 +466,13 @@ module lcio_interface
        type(c_ptr), value :: io_obj
      end function read_lcio_event
   end interface
+  interface
+     integer(c_int) function lcio_event_get_event_number (evt_obj) bind(C)
+       import
+       type(c_ptr), value :: evt_obj
+     end function lcio_event_get_event_number
+  end interface
+
   interface
      integer(c_int) function lcio_event_signal_process_id (evt_obj) bind(C)
        import
@@ -512,7 +521,7 @@ contains
     rid = 0; if (present (run_id))  rid = run_id
     runhdr%obj = new_lcio_run_header (rid)
     call run_header_set_simstring (runhdr%obj, &
-         "WHIZARD version:" // "2.4.1")
+         "WHIZARD version:" // "2.6.4")
   end subroutine lcio_run_header_init
 
   subroutine lcio_run_header_write (wrt, hdr)
@@ -552,6 +561,12 @@ contains
     call lcio_event_delete (evt%obj)
   end subroutine lcio_event_final
 
+  subroutine lcio_event_set_weight (evt, weight)
+    type(lcio_event_t), intent(inout) :: evt
+    real(default), intent(in) :: weight
+    call lcio_set_weight (evt%obj, real (weight, c_double))
+  end subroutine lcio_event_set_weight
+
   subroutine lcio_event_set_alpha_qcd (evt, alphas)
     type(lcio_event_t), intent(inout) :: evt
     real(default), intent(in) :: alphas
@@ -731,17 +746,8 @@ contains
   subroutine lcio_polarization_init_int (prt, hel)
     type(lcio_particle_t), intent(inout) :: prt
     integer, intent(in) :: hel
-    select case (hel)
-    case (1:)
-       call lcio_particle_set_spin (prt%obj, 1._c_double, &
-            0._c_double, 0._c_double)
-    case (:-1)
-       call lcio_particle_set_spin (prt%obj, 1._c_double, &
-            real (pi, c_double), 0._c_double)
-    case (0)
-       call lcio_particle_set_spin (prt%obj, 1._c_double, &
-            real (pi/2, c_double), 0._c_double)
-    end select
+    call lcio_particle_set_spin (prt%obj, 0._c_double, &
+         0._c_double, real (hel, c_double))
   end subroutine lcio_polarization_init_int
 
   subroutine lcio_particle_to_pol (prt, flv, pol)
@@ -861,6 +867,12 @@ contains
     ok = c_associated (evt%obj)
   end subroutine lcio_read_event
 
+  function lcio_event_get_event_index (evt) result (i_evt)
+    integer :: i_evt
+    type(lcio_event_t), intent(in) :: evt
+    i_evt = lcio_event_get_event_number (evt%obj)
+  end function lcio_event_get_event_index
+
   function lcio_event_get_process_id (evt) result (i_proc)
     integer :: i_proc
     type(lcio_event_t), intent(in) :: evt
