@@ -1,19 +1,12 @@
-! WHIZARD 2.4.1 Mar 24 2017
+! WHIZARD 2.6.4 Aug 23 2018
 !
-! Copyright (C) 1999-2017 by
+! Copyright (C) 1999-2018 by
 !     Wolfgang Kilian <kilian@physik.uni-siegen.de>
 !     Thorsten Ohl <ohl@physik.uni-wuerzburg.de>
 !     Juergen Reuter <juergen.reuter@desy.de>
 !
 !     with contributions from
-!     Fabian Bach <fabian.bach@t-online.de>
-!     Bijan Chokoufe <bijan.chokoufe@desy.de>
-!     Christian Speckner <cnspeckn@googlemail.com>
-!     So Young Shim <soyoung.shim@desy.de>
-!     Florian Staub <florian.staub@cern.ch>
-!     Christian Weiss <christian.weiss@desy.de>
-!     and Hans-Werner Boschmann, Felix Braam,
-!     Sebastian Schmidt, So-young Shim, Daniel Wiesler
+!     cf. main AUTHORS file
 !
 ! WHIZARD is free software; you can redistribute it and/or modify it
 ! under the terms of the GNU General Public License as published by
@@ -53,7 +46,6 @@ module nlo_data
 
   public :: fks_template_t
   public :: real_scales_t
-  public :: pdf_container_t
   public :: get_threshold_momenta
   public :: nlo_settings_t
 
@@ -71,11 +63,12 @@ module nlo_data
     real(default) :: fks_dij_exp2
     real(default) :: xi_min
     real(default) :: y_max
+    real(default) :: xi_cut, delta_zero, delta_i
     type(string_t), dimension(:), allocatable :: excluded_resonances
+    integer :: n_f
   contains
     procedure :: write => fks_template_write
-    procedure :: set_dij_exp => fks_template_set_dij_exp
-    procedure :: set_xi_and_y_bounds => fks_template_set_xi_and_y_bounds
+    procedure :: set_parameters => fks_template_set_parameters
     procedure :: set_mapping_type => fks_template_set_mapping_type
     procedure :: set_counter => fks_template_set_counter
   end type fks_template_t
@@ -89,10 +82,6 @@ module nlo_data
      real(default) :: ren_scale_born
   end type real_scales_t
 
-  type :: pdf_container_t
-     real(default), dimension(-6:6) :: f
-  end type pdf_container_t
-
   type :: nlo_settings_t
      logical :: use_internal_color_correlations = .true.
      logical :: use_internal_spin_correlations = .false.
@@ -111,6 +100,7 @@ module nlo_data
      logical :: virtual_resonance_aware_collinear = .true.
      logical :: use_born_scale = .true.
      logical :: cut_all_sqmes = .true.
+     type(string_t) :: nlo_correction_type
   contains
   procedure :: init => nlo_settings_init
     procedure :: write => nlo_settings_write
@@ -137,21 +127,28 @@ contains
     end select
     write (u,'(1x,A,ES4.3,ES4.3)') 'd_ij exponentials: ', &
        template%fks_dij_exp1, template%fks_dij_exp2
+    write (u, '(1x,A,ES4.3,ES4.3)') 'xi_cut: ', &
+       template%xi_cut
+    write (u, '(1x,A,ES4.3,ES4.3)') 'delta_zero: ', &
+       template%delta_zero
+    write (u, '(1x,A,ES4.3,ES4.3)') 'delta_i: ', &
+         template%delta_i
   end subroutine fks_template_write
 
-  subroutine fks_template_set_dij_exp (template, exp1, exp2)
+  subroutine fks_template_set_parameters (template, exp1, exp2, xi_min, &
+    y_max, xi_cut, delta_zero, delta_i)
     class(fks_template_t), intent(inout) :: template
     real(default), intent(in) :: exp1, exp2
+    real(default), intent(in) :: xi_min, y_max, &
+         xi_cut, delta_zero, delta_i
     template%fks_dij_exp1 = exp1
     template%fks_dij_exp2 = exp2
-  end subroutine fks_template_set_dij_exp
-
-  subroutine fks_template_set_xi_and_y_bounds (template, xi_min, y_max)
-    class(fks_template_t), intent(inout) :: template
-    real(default), intent(in) :: xi_min, y_max
     template%xi_min = xi_min
     template%y_max = y_max
-  end subroutine fks_template_set_xi_and_y_bounds
+    template%xi_cut = xi_cut
+    template%delta_zero = delta_zero
+    template%delta_i = delta_i
+  end subroutine fks_template_set_parameters
 
   subroutine fks_template_set_mapping_type (template, val)
     class(fks_template_t), intent(inout) :: template
@@ -180,6 +177,7 @@ contains
     type(string_t) :: color_method
     if (present (fks_template)) nlo_settings%fks_template = fks_template
     color_method = var_list%get_sval (var_str ('$correlation_me_method'))
+    if (color_method == "")  color_method = var_list%get_sval (var_str ('$method'))
     nlo_settings%use_internal_color_correlations = color_method == 'omega' &
            .or. color_method == 'threshold'
     nlo_settings%combined_integration = var_list%get_lval &
@@ -199,6 +197,7 @@ contains
          var_list%get_lval (var_str ("?nlo_use_born_scale"))
     nlo_settings%cut_all_sqmes = &
          var_list%get_lval (var_str ("?nlo_cut_all_sqmes"))
+    nlo_settings%nlo_correction_type = var_list%get_sval (var_str ('$nlo_correction_type'))
   contains
     subroutine setup_alr_selection ()
       type(string_t) :: alr_selection
