@@ -1,19 +1,12 @@
-! WHIZARD 2.4.1 Mar 24 2017
+! WHIZARD 2.6.4 Aug 23 2018
 !
-! Copyright (C) 1999-2017 by
+! Copyright (C) 1999-2018 by
 !     Wolfgang Kilian <kilian@physik.uni-siegen.de>
 !     Thorsten Ohl <ohl@physik.uni-wuerzburg.de>
 !     Juergen Reuter <juergen.reuter@desy.de>
 !
 !     with contributions from
-!     Fabian Bach <fabian.bach@t-online.de>
-!     Bijan Chokoufe <bijan.chokoufe@desy.de>
-!     Christian Speckner <cnspeckn@googlemail.com>
-!     So Young Shim <soyoung.shim@desy.de>
-!     Florian Staub <florian.staub@cern.ch>
-!     Christian Weiss <christian.weiss@desy.de>
-!     and Hans-Werner Boschmann, Felix Braam,
-!     Sebastian Schmidt, So-young Shim, Daniel Wiesler
+!     cf. main AUTHORS file
 !
 ! WHIZARD is free software; you can redistribute it and/or modify it
 ! under the terms of the GNU General Public License as published by
@@ -42,12 +35,12 @@ module virtual
   use diagnostics
   use pdg_arrays
   use models
+  use model_data, only: model_data_t
   use physics_defs
   use sm_physics
   use lorentz
   use flavors
-  use blha_olp_interfaces, only: blha_loop_positions
-  use nlo_data, only: get_threshold_momenta
+  use nlo_data, only: get_threshold_momenta, nlo_settings_t
   use nlo_data, only: ASSOCIATED_LEG_PAIR
   use fks_regions
 
@@ -59,54 +52,58 @@ module virtual
 
 
   type :: virtual_t
-    real(default), dimension(:,:), allocatable :: gamma_0, gamma_p, c_flv
-    real(default) :: ren_scale2, fac_scale, es_scale2
-    integer, dimension(:), allocatable :: n_is_neutrinos
-    integer :: n_in, n_legs, n_flv
-    logical :: bad_point = .false.
-    type(string_t) :: selection
-    integer :: factorization_mode = NO_FACTORIZATION
-    real(default), dimension(:,:,:), allocatable :: sqme_cc
-    logical :: use_internal_cc = .false.
-    integer, dimension(:), allocatable :: me_index
-    logical :: collinear_resonance_aware = .true.
-  contains
-    procedure :: init => virtual_init
-    procedure :: init_constants => virtual_init_constants
-    procedure :: set_ren_scale => virtual_set_ren_scale
-    procedure :: set_fac_scale => virtual_set_fac_scale
-    procedure :: set_ellis_sexton_scale => virtual_set_ellis_sexton_scale
-    procedure :: compute_n_sub => virtual_compute_n_sub
-    procedure :: evaluate => virtual_evaluate
-    procedure :: compute_eikonals => virtual_compute_eikonals
-    procedure :: compute_eikonals_threshold => virtual_compute_eikonals_threshold
-    procedure :: set_bad_point => virtual_set_bad_point
-    procedure :: evaluate_initial_state => virtual_evaluate_initial_state
-    procedure :: compute_collinear_contribution &
-       => virtual_compute_collinear_contribution
-    procedure :: compute_massive_self_eikonals => virtual_compute_massive_self_eikonals
-    procedure :: final => virtual_final
+     type(nlo_settings_t), pointer :: settings
+     real(default), dimension(:,:), allocatable :: gamma_0, gamma_p, c_flv
+     real(default) :: ren_scale2, fac_scale, es_scale2
+     integer, dimension(:), allocatable :: n_is_neutrinos
+     integer :: n_in, n_legs, n_flv
+     logical :: bad_point = .false.
+     type(string_t) :: selection
+     real(default), dimension(:), allocatable :: sqme_born
+     real(default), dimension(:), allocatable :: sqme_virt_fin
+     real(default), dimension(:,:,:), allocatable :: sqme_color_c
+     real(default), dimension(:,:,:), allocatable :: sqme_charge_c
+     logical :: has_pdfs = .false.
+   contains
+     procedure :: init => virtual_init
+     procedure :: init_constants => virtual_init_constants
+     procedure :: set_ren_scale => virtual_set_ren_scale
+     procedure :: set_fac_scale => virtual_set_fac_scale
+     procedure :: set_ellis_sexton_scale => virtual_set_ellis_sexton_scale
+     procedure :: evaluate => virtual_evaluate
+     procedure :: compute_eikonals => virtual_compute_eikonals
+     procedure :: compute_eikonals_threshold => virtual_compute_eikonals_threshold
+     procedure :: set_bad_point => virtual_set_bad_point
+     procedure :: evaluate_initial_state => virtual_evaluate_initial_state
+     procedure :: compute_collinear_contribution &
+        => virtual_compute_collinear_contribution
+     procedure :: compute_massive_self_eikonals => virtual_compute_massive_self_eikonals
+     procedure :: final => virtual_final
   end type virtual_t
 
 
 contains
 
- subroutine virtual_init (virt, flv_born, n_in, use_internal_cc, selection, resonance_aware)
+  subroutine virtual_init (virt, flv_born, n_in, settings, &
+      nlo_corr_type, model, has_pdfs)
     class(virtual_t), intent(inout) :: virt
     integer, intent(in), dimension(:,:) :: flv_born
     integer, intent(in) :: n_in
-    logical, intent(in) :: use_internal_cc
-    type(string_t), intent(in) :: selection
-    logical, intent(in) :: resonance_aware
-    integer :: n_sub
+    type(nlo_settings_t), intent(in), pointer :: settings
+    type(string_t), intent(in) :: nlo_corr_type
+    class(model_data_t), intent(in) :: model
+    logical, intent(in) :: has_pdfs
     integer :: i_flv
     virt%n_legs = size (flv_born, 1); virt%n_flv = size (flv_born, 2)
     virt%n_in = n_in
-    allocate (virt%sqme_cc (virt%n_legs, virt%n_legs, virt%n_flv))
+    allocate (virt%sqme_born (virt%n_flv))
+    allocate (virt%sqme_virt_fin (virt%n_flv))
+    allocate (virt%sqme_color_c (virt%n_legs, virt%n_legs, virt%n_flv))
+    allocate (virt%sqme_charge_c (virt%n_legs, virt%n_legs, virt%n_flv))
     allocate (virt%gamma_0 (virt%n_legs, virt%n_flv), &
        virt%gamma_p (virt%n_legs, virt%n_flv), &
        virt%c_flv (virt%n_legs, virt%n_flv))
-    call virt%init_constants (flv_born)
+    call virt%init_constants (flv_born, settings%fks_template%n_f, nlo_corr_type, model)
     allocate (virt%n_is_neutrinos (virt%n_flv))
     virt%n_is_neutrinos = 0
     do i_flv = 1, virt%n_flv
@@ -115,19 +112,14 @@ contains
        if (is_neutrino (flv_born(2, i_flv))) &
           virt%n_is_neutrinos(i_flv) = virt%n_is_neutrinos(i_flv) + 1
     end do
-    virt%use_internal_cc = use_internal_cc
-    allocate (virt%me_index (virt%n_flv))
-    n_sub = virt%compute_n_sub ()
-    do i_flv = 1, virt%n_flv
-       virt%me_index (i_flv) = blha_loop_positions (i_flv, n_sub)
-    end do
-    select case (char (selection))
+    select case (char (settings%virtual_selection))
     case ("Full", "OLP", "Subtraction")
-       virt%selection = selection
+       virt%selection = settings%virtual_selection
     case default
        call msg_fatal ('Virtual selection: Possible values are "Full", "OLP" or "Subtraction')
     end select
-    virt%collinear_resonance_aware = resonance_aware
+    virt%settings => settings
+    virt%has_pdfs = has_pdfs
   contains
 
     function is_neutrino (flv) result (neutrino)
@@ -138,22 +130,46 @@ contains
 
   end subroutine virtual_init
 
-  subroutine virtual_init_constants (virt, flv_born)
+  subroutine virtual_init_constants (virt, flv_born, nf_input, nlo_corr_type, model)
     class(virtual_t), intent(inout) :: virt
     integer, intent(in), dimension(:,:) :: flv_born
+    integer, intent(in) :: nf_input
+    type(string_t), intent(in) :: nlo_corr_type
+    class(model_data_t), intent(in) :: model
     integer :: i_part, i_flv
-    integer, parameter :: nf = 1
+    real(default) :: nf, CA_factor
+    real(default), dimension(:,:), allocatable :: CF_factor, TR_factor
+    type(flavor_t) :: flv
+    allocate (CF_factor (size (flv_born, 1), size (flv_born, 2)), &
+         TR_factor (size (flv_born, 1), size (flv_born, 2)))
+    if (nlo_corr_type == "QCD") then
+       CA_factor = CA; CF_factor = CF; TR_factor = TR
+       nf = real(nf_input, default)
+    else if (nlo_corr_type == "QED") then
+       CA_factor = zero
+       do i_flv = 1, size (flv_born, 2)
+          do i_part = 1, size (flv_born, 1)
+             call flv%init (flv_born(i_part, i_flv), model)
+             CF_factor(i_part, i_flv) = (flv%get_charge ())**2
+             TR_factor(i_part, i_flv) = (flv%get_charge ())**2
+          end do
+       end do
+       ! TODO vincent_r fixed nf needs replacement  !!! for testing only, needs dynamical treatment!
+       nf = real(4, default)
+    end if
     do i_flv = 1, size (flv_born, 2)
        do i_part = 1, size (flv_born, 1)
-          if (is_gluon (flv_born(i_part, i_flv))) then
-             virt%gamma_0(i_part, i_flv) = (11 * ca - 2 * nf) / 6
-             virt%gamma_p(i_part, i_flv) = (67.0 / 9 - 2 * pi**2 / 3) * ca &
-                - 23.0 / 18 * nf
-             virt%c_flv(i_part, i_flv) = ca
-          else if (is_quark (flv_born(i_part, i_flv))) then
-             virt%gamma_0(i_part, i_flv) = 1.5 * cf
-             virt%gamma_p(i_part, i_flv) = (6.5 - 2 * pi**2 / 3) * cf
-             virt%c_flv(i_part, i_flv) = cf
+          if (is_corresponding_vector (flv_born(i_part, i_flv), nlo_corr_type)) then
+             virt%gamma_0(i_part, i_flv) = 11._default / 6._default * CA_factor &
+                  - two / three * TR_factor(i_part, i_flv) * nf
+             virt%gamma_p(i_part, i_flv) = (67._default / 9._default &
+                  - two * pi**2 / three) * CA_factor &
+                  - 23._default / 9._default * TR_factor(i_part, i_flv) * nf
+             virt%c_flv(i_part, i_flv) = CA_factor
+          else if (is_corresponding_fermion (flv_born(i_part, i_flv), nlo_corr_type)) then
+             virt%gamma_0(i_part, i_flv) = 1.5_default * CF_factor(i_part, i_flv)
+             virt%gamma_p(i_part, i_flv) = (6.5_default - two * pi**2 / three) * CF_factor(i_part, i_flv)
+             virt%c_flv(i_part, i_flv) = CF_factor(i_part, i_flv)
           else
              virt%gamma_0(i_part, i_flv) = zero
              virt%gamma_p(i_part, i_flv) = zero
@@ -161,6 +177,29 @@ contains
           end if
        end do
     end do
+  contains
+    function is_corresponding_vector (pdg_nr, nlo_corr_type)
+      logical :: is_corresponding_vector
+      integer, intent(in) :: pdg_nr
+      type(string_t), intent(in) :: nlo_corr_type
+      is_corresponding_vector = .false.
+      if (nlo_corr_type == "QCD") then
+         is_corresponding_vector = is_gluon (pdg_nr)
+      else if (nlo_corr_type == "QED") then
+         is_corresponding_vector = is_photon (pdg_nr)
+      end if
+    end function is_corresponding_vector
+    function is_corresponding_fermion (pdg_nr, nlo_corr_type)
+      logical :: is_corresponding_fermion
+      integer, intent(in) :: pdg_nr
+      type(string_t), intent(in) :: nlo_corr_type
+      is_corresponding_fermion = .false.
+      if (nlo_corr_type == "QCD") then
+         is_corresponding_fermion = is_quark (pdg_nr)
+      else if (nlo_corr_type == "QED") then
+         is_corresponding_fermion = is_fermion (pdg_nr)
+      end if
+    end function is_corresponding_fermion
   end subroutine virtual_init_constants
 
   subroutine virtual_set_ren_scale (virt, p, ren_scale)
@@ -195,78 +234,63 @@ contains
     end if
   end subroutine virtual_set_ellis_sexton_scale
 
-  function virtual_compute_n_sub (virt) result (n_sub)
-    integer :: n_sub
-    class(virtual_t), intent(in) :: virt
-    n_sub = 1
-    if (.not. virt%use_internal_cc) &
-         n_sub = n_sub + virt%n_legs * (virt%n_legs - 1 ) / 2
-  end function virtual_compute_n_sub
-
-  subroutine virtual_evaluate (virt, reg_data, alpha_s, &
-         p_born, sqme, separate_alrs, sqme_virt)
+  subroutine virtual_evaluate (virt, reg_data, alpha_coupling, &
+         p_born, separate_alrs, sqme_virt)
     class(virtual_t), intent(inout) :: virt
     type(region_data_t), intent(in) :: reg_data
-    real(default), intent(in) :: alpha_s
+    real(default), intent(in) :: alpha_coupling
     type(vector4_t), intent(in), dimension(:)  :: p_born
-    real(default), intent(in), dimension(:) :: sqme
     logical, intent(in) :: separate_alrs
     real(default), dimension(:), intent(inout) :: sqme_virt
-    real(default) :: sqme_alr
     real(default) :: s, s_o_Q2
     real(default), dimension(reg_data%n_flv_born) :: QB, BI
     integer :: i_flv, ii_flv
     QB = zero; BI = zero
     if (virt%bad_point) return
     if (debug2_active (D_VIRTUAL)) then
-       print *, 'Compute virtual component using alpha_s = ', alpha_s
+       print *, 'Compute virtual component using alpha = ', alpha_coupling
        print *, 'Virtual selection: ', char (virt%selection)
        print *, 'virt%es_scale2 =    ', virt%es_scale2 !!! Debugging
     end if
     s = sum (p_born(1 : virt%n_in))**2
-    if (virt%factorization_mode == FACTORIZATION_THRESHOLD) &
+    if (virt%settings%factorization_mode == FACTORIZATION_THRESHOLD) &
          call set_s_for_threshold ()
-    s_o_Q2 = s / virt%es_scale2
+    s_o_Q2 = s / virt%es_scale2 * virt%settings%fks_template%xi_cut**2
     do i_flv = 1, reg_data%n_flv_born
-
        if (separate_alrs) then
           ii_flv = i_flv
        else
           ii_flv = 1
        end if
-
        if (virt%selection == var_str ("Full") .or. virt%selection == var_str ("OLP")) then
-          !!! A factor of alpha_s/twopi is assumed to be included in vfin
-          sqme_virt(ii_flv) = sqme_virt(ii_flv) + sqme(virt%me_index(i_flv))
+          !!! A factor of alpha_coupling/twopi is assumed to be included in vfin
+          sqme_virt(ii_flv) = sqme_virt(ii_flv) + virt%sqme_virt_fin(i_flv)
        end if
-
        if (virt%selection == var_str ("Full") .or. virt%selection == var_str ("Subtraction")) then
-          call virt%evaluate_initial_state (i_flv, sqrt(s), reg_data, sqme, QB)
-          call virt%compute_collinear_contribution (i_flv, p_born, sqrt(s), reg_data, sqme, QB)
-
-          select case (virt%factorization_mode)
+          call virt%evaluate_initial_state (i_flv, QB)
+          call virt%compute_collinear_contribution (i_flv, p_born, sqrt(s), reg_data, QB)
+          select case (virt%settings%factorization_mode)
           case (FACTORIZATION_THRESHOLD)
-             call virt%compute_eikonals_threshold (i_flv, p_born, s, s_o_Q2, reg_data, sqme, QB, BI)
+             call virt%compute_eikonals_threshold (i_flv, p_born, s_o_Q2, QB, BI)
           case default
-             call virt%compute_massive_self_eikonals (i_flv, p_born, s, reg_data, sqme, QB)
-             call virt%compute_eikonals (i_flv, p_born, s, s_o_Q2, reg_data, sqme, BI)
+             call virt%compute_massive_self_eikonals (i_flv, p_born, s_o_Q2, reg_data, QB)
+             call virt%compute_eikonals (i_flv, p_born, s_o_Q2, reg_data, BI)
           end select
-
           if (debug2_active (D_VIRTUAL)) then
              print *, 'Evaluate i_flv: ', i_flv
-             print *, 'Q * sqme_born: ', QB(i_flv)
-             print *, 'BI: ', BI(i_flv)
-             print *, 'vfin: ', sqme (virt%me_index(i_flv))
+             print *, 'sqme_born: ', virt%sqme_born (i_flv)
+             print *, 'Q * sqme_born: ', alpha_coupling / twopi * QB(i_flv)
+             print *, 'BI: ', alpha_coupling / twopi * BI(i_flv)
+             print *, 'vfin: ', virt%sqme_virt_fin (i_flv)
           end if
-          sqme_virt(ii_flv) = sqme_virt(ii_flv) + alpha_s / twopi * (QB(i_flv) + BI(i_flv))
+          sqme_virt(ii_flv) = &
+               sqme_virt(ii_flv) + alpha_coupling / twopi * (QB(i_flv) + BI(i_flv))
        end if
     end do
-
     if (debug2_active (D_VIRTUAL)) then
        call msg_debug2 (D_VIRTUAL, "virtual-subtracted matrix element(s): ")
        print *, sqme_virt
     end if
-
     do i_flv = 1, reg_data%n_flv_born
        if (virt%n_is_neutrinos(i_flv) > 0) &
             sqme_virt = sqme_virt * virt%n_is_neutrinos(i_flv) * two
@@ -278,79 +302,85 @@ contains
       mtop2 = m1s_to_mpole (sqrt(s))**2
       if (s < four * mtop2) s = four * mtop2
     end subroutine set_s_for_threshold
+
   end subroutine virtual_evaluate
 
   subroutine virtual_compute_eikonals (virtual, i_flv, &
-           p_born, s, s_o_Q2, reg_data, sqme, BI)
+           p_born, s_o_Q2, reg_data, BI)
     class(virtual_t), intent(inout) :: virtual
     integer, intent(in) :: i_flv
     type(vector4_t), intent(in), dimension(:)  :: p_born
-    real(default), intent(in) :: s, s_o_Q2
+    real(default), intent(in) :: s_o_Q2
     type(region_data_t), intent(in) :: reg_data
-    real(default), intent(in), dimension(:) :: sqme
     real(default), intent(inout), dimension(:) :: BI
-    integer :: i_born
-    integer :: i, j, k
+    integer :: i, j
     real(default) :: I_ij, BI_tmp
     BI_tmp = zero
-    i_born = virtual%me_index(i_flv) + 1
-    associate (flst_born => reg_data%flv_born(i_flv))
+    ! TODO vincent_r: Split the procedure into one computing QCD eikonals and one computing QED eikonals.
+    ! TODO vincent_r: In the best case, remove the dependency on reg_data completely.
+    associate (flst_born => reg_data%flv_born(i_flv), &
+            nlo_corr_type => reg_data%regions(1)%nlo_correction_type)
        do i = 1, virtual%n_legs
           do j = 1, virtual%n_legs
              if (i /= j) then
-                if (flst_born%colored(i) .and. flst_born%colored(j)) then
+                if (nlo_corr_type == "QCD") then
+                   if (flst_born%colored(i) .and. flst_born%colored(j)) then
+                      I_ij = compute_eikonal_factor (p_born, flst_born%massive, &
+                           i, j, s_o_Q2)
+                      BI_tmp = BI_tmp + virtual%sqme_color_c (i, j, i_flv) * I_ij
+                      if (debug2_active (D_VIRTUAL)) &
+                           print *, 'b_ij: ', i, j, virtual%sqme_color_c (i, j, i_flv), 'I_ij: ', I_ij
+                   end if
+                else if (nlo_corr_type == "QED") then
                    I_ij = compute_eikonal_factor (p_born, flst_born%massive, &
                         i, j, s_o_Q2)
-                   BI_tmp = BI_tmp + virtual%sqme_cc (i, j, i_flv) * I_ij
+                   BI_tmp = BI_tmp + virtual%sqme_charge_c (i, j, i_flv) * I_ij
                    if (debug2_active (D_VIRTUAL)) &
-                      print *, 'b_ij: ', virtual%sqme_cc (i, j, i_flv), 'I_ij: ', I_ij
+                        print *, 'b_ij: ', virtual%sqme_charge_c (i, j, i_flv), 'I_ij: ', I_ij
                 end if
+             else if (debug2_active (D_VIRTUAL)) then
+                print *, 'b_ij: ', i, j, virtual%sqme_color_c (i, j, i_flv), 'I_ij: ', I_ij
              end if
           end do
        end do
+       if (virtual%settings%use_internal_color_correlations .or. nlo_corr_type == "QED") &
+            BI_tmp = BI_tmp * virtual%sqme_born (i_flv)
     end associate
-    if (virtual%use_internal_cc) BI_tmp = BI_tmp * sqme (i_born)
     BI(i_flv) = BI(i_flv) + BI_tmp
   end subroutine virtual_compute_eikonals
 
   subroutine virtual_compute_eikonals_threshold (virtual, i_flv, &
-         p_born, s, s_o_Q2, reg_data, sqme, QB, BI)
+         p_born, s_o_Q2, QB, BI)
     class(virtual_t), intent(in) :: virtual
     integer, intent(in) :: i_flv
     type(vector4_t), intent(in), dimension(:) :: p_born
-    real(default), intent(in) :: s, s_o_Q2
-    type(region_data_t), intent(in) :: reg_data
-    real(default), intent(in), dimension(:) :: sqme
+    real(default), intent(in) :: s_o_Q2
     real(default), intent(inout), dimension(:) :: QB
     real(default), intent(inout), dimension(:) :: BI
     type(vector4_t), dimension(4) :: p_thr
-    integer :: i_born, leg
+    integer :: leg
     BI = zero; p_thr = get_threshold_momenta (p_born)
-    i_born = virtual%me_index(i_flv) + 1
-    call compute_massive_self_eikonals (sqme (i_born), QB(i_flv))
+    call compute_massive_self_eikonals (virtual%sqme_born(i_flv), QB(i_flv))
     do leg = 1, 2
-       BI(i_flv) = BI(i_flv) + evaluate_leg_pair (ASSOCIATED_LEG_PAIR(leg), i_flv, i_born)
+       BI(i_flv) = BI(i_flv) + evaluate_leg_pair (ASSOCIATED_LEG_PAIR(leg), i_flv)
     end do
   contains
-
     subroutine compute_massive_self_eikonals (sqme_born, QB)
       real(default), intent(in) :: sqme_born
       real(default), intent(inout) :: QB
-      real(default) :: term_1, term_2
       integer :: i
       call msg_debug2 (D_VIRTUAL, "compute_massive_self_eikonals")
       call msg_debug2 (D_VIRTUAL, "s_o_Q2", s_o_Q2)
       call msg_debug2 (D_VIRTUAL, "log (s_o_Q2)", log (s_o_Q2))
       do i = 1, 4
-         term_1 = log (s_o_Q2)
-         term_2 = 0.5_default * I_m_eps (p_thr(i))
-         QB = QB - (cf * (term_1 - term_2)) * sqme_born
+         QB = QB - (cf * (log (s_o_Q2) - 0.5_default * I_m_eps (p_thr(i)))) &
+              * sqme_born
       end do
     end subroutine compute_massive_self_eikonals
 
-    function evaluate_leg_pair (i_start, i_flv, i_born) result (b_ij_times_I)
+    function evaluate_leg_pair (i_start, i_flv) result (b_ij_times_I)
       real(default) :: b_ij_times_I
-      integer, intent(in) :: i_start, i_flv, i_born
+      integer, intent(in) :: i_start, i_flv
       real(default) :: I_ij
       integer :: i, j
       b_ij_times_I = zero
@@ -360,15 +390,16 @@ contains
                I_ij = compute_eikonal_factor &
                     (p_thr, [.true., .true., .true., .true.], i, j, s_o_Q2)
                b_ij_times_I = b_ij_times_I + &
-                    virtual%sqme_cc (i, j, i_flv) * I_ij
+                    virtual%sqme_color_c (i, j, i_flv) * I_ij
                if (debug2_active (D_VIRTUAL)) &
-                  print *, 'b_ij: ', virtual%sqme_cc (i, j, i_flv), 'I_ij: ', I_ij
+                  print *, 'b_ij: ', virtual%sqme_color_c (i, j, i_flv), 'I_ij: ', I_ij
             end if
          end do
       end do
-      if (virtual%use_internal_cc) b_ij_times_I = b_ij_times_I * sqme (i_born)
+      if (virtual%settings%use_internal_color_correlations) &
+           b_ij_times_I = b_ij_times_I * virtual%sqme_born (i_flv)
       if (debug2_active (D_VIRTUAL)) then
-         print *, 'internal color: ', virtual%use_internal_cc
+         print *, 'internal color: ', virtual%settings%use_internal_color_correlations
          print *, 'b_ij_times_I =    ', b_ij_times_I
          print *, 'QB           =    ', QB
       end if
@@ -381,36 +412,31 @@ contains
      virt%bad_point = value
   end subroutine virtual_set_bad_point
 
-  subroutine virtual_evaluate_initial_state (virt, i_flv, sqrts, reg_data, sqme, QB)
+  subroutine virtual_evaluate_initial_state (virt, i_flv, QB)
     class(virtual_t), intent(inout) :: virt
     integer, intent(in) :: i_flv
-    real(default), intent(in) :: sqrts
-    type(region_data_t), intent(in) :: reg_data
-    real(default), intent(in), dimension(:) :: sqme
     real(default), intent(inout), dimension(:) :: QB
-    integer :: i, i_sqme
+    integer :: i
     if (virt%n_in == 2) then
-       i_sqme = virt%me_index(i_flv) + 1
        do i = 1, virt%n_in
-          QB(i_flv) = QB(i_flv) - virt%gamma_0 (i, i_flv) * &
-               log(virt%fac_scale**2 / virt%es_scale2) * sqme (i_sqme)
+          QB(i_flv) = QB(i_flv) - (virt%gamma_0 (i, i_flv) + two * virt%c_flv(i, i_flv) &
+               * log (virt%settings%fks_template%xi_cut)) &
+               * log(virt%fac_scale**2 / virt%es_scale2) * virt%sqme_born (i_flv)
        end do
     end if
   end subroutine virtual_evaluate_initial_state
 
   subroutine virtual_compute_collinear_contribution (virt, i_flv, &
-           p_born, sqrts, reg_data, sqme, QB)
+           p_born, sqrts, reg_data, QB)
     class(virtual_t), intent(inout) :: virt
     integer, intent(in) :: i_flv
     type(vector4_t), dimension(:), intent(in) :: p_born
     real(default), intent(in) :: sqrts
     type(region_data_t), intent(in) :: reg_data
-    real(default), intent(in), dimension(:) :: sqme
     real(default), intent(inout), dimension(:) :: QB
     real(default) :: s1, s2, s3, s4, s5
-    integer :: alr, em, i_sqme
+    integer :: alr, em
     real(default) :: E_em, log_xi_max, E_tot2
-    logical :: massive
     logical, dimension(virt%n_flv, virt%n_legs) :: evaluated
     integer :: i_contr
     type(vector4_t) :: k_res
@@ -421,11 +447,9 @@ contains
        em = reg_data%regions(alr)%emitter
        if (em == 0) cycle
        if (evaluated(i_flv, em)) cycle
-       massive = reg_data%regions(alr)%flst_uborn%massive(em)
        !!! Collinear terms only for massless particles
-       if (massive) cycle
+       if (reg_data%regions(alr)%flst_uborn%massive(em)) cycle
        E_em = p_born(em)%p(0)
-       i_sqme = virt%me_index(i_flv) + 1
        if (allocated (reg_data%alr_contributors)) then
           i_contr = reg_data%alr_to_i_contributor (alr)
           k_res = get_resonance_momentum (p_born, reg_data%alr_contributors(i_contr)%c)
@@ -436,51 +460,53 @@ contains
           E_tot2 = sqrts**2
           log_xi_max = log (two * E_em / sqrts)
        end if
-       if (virt%collinear_resonance_aware) then
-          if (debug_active (D_VIRTUAL)) &
-              call msg_debug (D_VIRTUAL, "Using resonance-aware collinear subtraction")
-          s1 = virt%gamma_p(em, i_flv)
-          s2 = two * (log (sqrts / (two * E_em)) + log_xi_max) * &
-               (log (sqrts / (two * E_em)) + log_xi_max + log (virt%es_scale2 / sqrts**2)) &
-             * virt%c_flv(em, i_flv)
-          s3 = two * log_xi_max * &
-               (log_xi_max - log (virt%es_scale2 / E_tot2)) * virt%c_flv(em, i_flv)
-          s4 = (log (virt%es_scale2 / E_tot2) - two * log_xi_max) * virt%gamma_0(em, i_flv)
-          QB(i_flv) = QB(i_flv) + (s1 + s2 + s3 + s4) * sqme(i_sqme)
-       else
-          if (debug_active (D_VIRTUAL)) &
-              call msg_debug (D_VIRTUAL, "Using old-fashioned collinear subtraction")
-          s1 = virt%gamma_p(em, i_flv)
-          s2 = log (sqrts**2 / virt%es_scale2) * virt%gamma_0(em,i_flv)
-          s3 = log (sqrts**2 / virt%es_scale2) * two * virt%c_flv(em,i_flv) * &
-               log (two * E_em / sqrts)
-          s4 = two * virt%c_flv(em,i_flv) * log (two * E_em / sqrts)**2
-          s5 = two * virt%gamma_0(em,i_flv) * log (two * E_em / sqrts)
-          QB(i_flv) = QB(i_flv) + (s1 - s2 + s3 + s4 - s5) * sqme(i_sqme)
-       end if
+       ! TODO sbrass evaluate xi-cut formalism for resonance-aware FKS
+       associate (xi_cut => virt%settings%fks_template%xi_cut, delta_zero => virt%settings%fks_template%delta_zero)
+         if (virt%settings%virtual_resonance_aware_collinear) then
+            if (debug_active (D_VIRTUAL)) &
+                 call msg_debug (D_VIRTUAL, "Using resonance-aware collinear subtraction")
+            s1 = virt%gamma_p(em, i_flv)
+            s2 = two * (log (sqrts / (two * E_em)) + log_xi_max) * &
+                 (log (sqrts / (two * E_em)) + log_xi_max + log (virt%es_scale2 / sqrts**2)) &
+                 * virt%c_flv(em, i_flv)
+            s3 = two * log_xi_max * &
+                 (log_xi_max - log (virt%es_scale2 / E_tot2)) * virt%c_flv(em, i_flv)
+            s4 = (log (virt%es_scale2 / E_tot2) - two * log_xi_max) * virt%gamma_0(em, i_flv)
+            QB(i_flv) = QB(i_flv) + (s1 + s2 + s3 + s4) * virt%sqme_born(i_flv)
+         else
+            if (debug_active (D_VIRTUAL)) &
+                 call msg_debug (D_VIRTUAL, "Using old-fashioned collinear subtraction")
+            s1 = virt%gamma_p(em, i_flv)
+            s2 = log (delta_zero * sqrts**2 / (two * virt%es_scale2)) * virt%gamma_0(em,i_flv)
+            s3 = log (delta_zero * sqrts**2 / (two * virt%es_scale2)) * two * virt%c_flv(em,i_flv) * &
+                 log (two * E_em / (xi_cut * sqrts))
+            ! s4 = two * virt%c_flv(em,i_flv) * (log (two * E_em / sqrts)**2 - log (xi_cut)**2)
+            s4 = two * virt%c_flv(em,i_flv) * & ! a**2 - b**2 = (a - b) * (a + b), for better numerical performance
+                 (log (two * E_em / sqrts) + log (xi_cut)) * (log (two * E_em / sqrts) - log (xi_cut))
+            s5 = two * virt%gamma_0(em,i_flv) * log (two * E_em / sqrts)
+            QB(i_flv) = QB(i_flv) + (s1 - s2 + s3 + s4 - s5) * virt%sqme_born(i_flv)
+         end if
+       end associate
        evaluated(i_flv, em) = .true.
     end do
   end subroutine virtual_compute_collinear_contribution
 
   subroutine virtual_compute_massive_self_eikonals (virt, i_flv, &
-           p_born, s, reg_data, sqme, QB)
+           p_born, s_over_Q2, reg_data,  QB)
     class(virtual_t), intent(inout) :: virt
     integer, intent(in) :: i_flv
     type(vector4_t), intent(in), dimension(:) :: p_born
-    real(default), intent(in) :: s
+    real(default), intent(in) :: s_over_Q2
     type(region_data_t), intent(in) :: reg_data
-    real(default), intent(in), dimension(:) :: sqme
     real(default), intent(inout), dimension(:) :: QB
-    real(default) :: term1, term2
-    integer :: i, i_born
+    integer :: i
     logical :: massive
-    i_born = virt%me_index(i_flv) + 1
     do i = 1, virt%n_legs
        massive = reg_data%flv_born(i_flv)%massive(i)
        if (massive) then
-          term1 = log(s / virt%es_scale2)
-          term2 = 0.5_default * I_m_eps (p_born(i))
-          QB(i_flv) = QB(i_flv) - (virt%c_flv (i, i_flv) * (term1 - term2)) * sqme (i_born)
+          QB(i_flv) = QB(i_flv) - (virt%c_flv (i, i_flv) &
+               * (log (s_over_Q2) - 0.5_default * I_m_eps (p_born(i)))) &
+               * virt%sqme_born (i_flv)
        end if
     end do
   end subroutine virtual_compute_massive_self_eikonals
@@ -514,15 +540,15 @@ contains
     s1 = 0; s2 = 0; s3 = 0; s4 = 0; s5 = 0
     Ei = pi%p(0); Ej = pj%p(0)
     pij = pi * pj; Eij = Ei * Ej
-    s1 = 0.5 * log(s_o_Q2)**2
+    s1 = 0.5_default * log(s_o_Q2)**2
     s2 = log(s_o_Q2) * log(pij / (two * Eij))
     s3 = Li2 (pij / (two * Eij))
-    s4 = 0.5 * log (pij / (two * Eij))**2
-    arglog = one - pij / (2*Eij)
+    s4 = 0.5_default * log (pij / (two * Eij))**2
+    arglog = one - pij / (two * Eij)
     if (arglog > tiny_value) then
       s5 = log(arglog) * log(pij / (two * Eij))
     else
-      s5 = 0
+      s5 = zero
     end if
     I = s1 + s2 - s3 + s4 - s5
   end function compute_I00
