@@ -1,19 +1,12 @@
-! WHIZARD 2.4.1 Mar 24 2017
+! WHIZARD 2.6.4 Aug 23 2018
 !
-! Copyright (C) 1999-2017 by
+! Copyright (C) 1999-2018 by
 !     Wolfgang Kilian <kilian@physik.uni-siegen.de>
 !     Thorsten Ohl <ohl@physik.uni-wuerzburg.de>
 !     Juergen Reuter <juergen.reuter@desy.de>
 !
 !     with contributions from
-!     Fabian Bach <fabian.bach@t-online.de>
-!     Bijan Chokoufe <bijan.chokoufe@desy.de>
-!     Christian Speckner <cnspeckn@googlemail.com>
-!     So Young Shim <soyoung.shim@desy.de>
-!     Florian Staub <florian.staub@cern.ch>
-!     Christian Weiss <christian.weiss@desy.de>
-!     and Hans-Werner Boschmann, Felix Braam,
-!     Sebastian Schmidt, So-young Shim, Daniel Wiesler
+!     cf. main AUTHORS file
 !
 ! WHIZARD is free software; you can redistribute it and/or modify it
 ! under the terms of the GNU General Public License as published by
@@ -65,6 +58,8 @@ module prclib_interfaces
   public :: write_driver_code
   public :: prclib_unload_hook
   public :: prclib_reload_hook
+  public :: workspace_prefix
+  public :: workspace_path
 
   type, abstract :: prc_writer_t
      character(32) :: md5sum = ""
@@ -75,6 +70,8 @@ module prclib_interfaces
      procedure(write_feature_code), deferred :: write_interface
      procedure(write_code_os), deferred :: write_makefile_code
      procedure(write_code_file), deferred :: write_source_code
+     procedure(write_code_file), deferred :: before_compile
+     procedure(write_code_file), deferred :: after_compile
      procedure :: init_test => prc_writer_init_test
      procedure(write_code), deferred :: write_md5sum_call
      procedure(write_feature_code), deferred :: write_int_sub_call
@@ -117,6 +114,8 @@ module prclib_interfaces
      procedure :: write_wrappers => prclib_driver_record_write_wrappers
      procedure :: write_makefile_code => prclib_driver_record_write_makefile_code
      procedure :: write_source_code => prclib_driver_record_write_source_code
+     procedure :: before_compile => prclib_driver_record_before_compile
+     procedure :: after_compile => prclib_driver_record_after_compile
      procedure :: write_md5sum_call => prclib_driver_record_write_md5sum_call
      procedure :: write_int_sub_call => prclib_driver_record_write_int_sub_call
      procedure :: write_col_state_call => prclib_driver_record_write_col_state_call
@@ -238,12 +237,13 @@ module prclib_interfaces
   end interface
 
   abstract interface
-     subroutine write_code_os (writer, unit, id, os_data, testflag)
+     subroutine write_code_os (writer, unit, id, os_data, verbose, testflag)
        import
        class(prc_writer_t), intent(in) :: writer
        integer, intent(in) :: unit
        type(string_t), intent(in) :: id
        type(os_data_t), intent(in) :: os_data
+       logical, intent(in) :: verbose
        logical, intent(in), optional :: testflag
      end subroutine write_code_os
   end interface
@@ -466,12 +466,14 @@ contains
   end subroutine prclib_driver_record_write_wrappers
 
   subroutine prclib_driver_record_write_makefile_code &
-       (record, unit, os_data, testflag)
+       (record, unit, os_data, verbose, testflag)
     class(prclib_driver_record_t), intent(in) :: record
     integer, intent(in) :: unit
     type(os_data_t), intent(in) :: os_data
+    logical, intent(in) :: verbose
     logical, intent(in), optional :: testflag
-    call record%writer%write_makefile_code (unit, record%id, os_data, testflag)
+    call record%writer%write_makefile_code &
+         (unit, record%id, os_data, verbose, testflag)
   end subroutine prclib_driver_record_write_makefile_code
 
   subroutine prclib_driver_record_write_source_code (record)
@@ -479,6 +481,16 @@ contains
     call record%writer%write_source_code (record%id)
   end subroutine prclib_driver_record_write_source_code
 
+  subroutine prclib_driver_record_before_compile (record)
+    class(prclib_driver_record_t), intent(in) :: record
+    call record%writer%before_compile (record%id)
+  end subroutine prclib_driver_record_before_compile
+
+  subroutine prclib_driver_record_after_compile (record)
+    class(prclib_driver_record_t), intent(in) :: record
+    call record%writer%after_compile (record%id)
+  end subroutine prclib_driver_record_after_compile
+
   subroutine prclib_driver_write (object, unit, libpath)
     class(prclib_driver_t), intent(in) :: object
     integer, intent(in) :: unit
@@ -578,10 +590,11 @@ contains
     end do
   end subroutine prclib_driver_write_interfaces
 
-  subroutine prclib_driver_generate_makefile (driver, unit, os_data, testflag)
+  subroutine prclib_driver_generate_makefile (driver, unit, os_data, verbose, testflag)
     class(prclib_driver_t), intent(in) :: driver
     integer, intent(in) :: unit
     type(os_data_t), intent(in) :: os_data
+    logical, intent(in) :: verbose
     logical, intent(in), optional :: testflag
     integer :: i
     write (unit, "(A)")  "# WHIZARD: Makefile for process library '" &
@@ -622,9 +635,15 @@ contains
     write (unit, "(A)")  ""
     write (unit, "(A)")  "# Libtool"
     write (unit, "(A)")  "LIBTOOL = " // char (os_data%whizard_libtool)
-    write (unit, "(A)")  "FCOMPILE = $(LIBTOOL) --tag=FC --mode=compile"
-    write (unit, "(A)")  "CCOMPILE = $(LIBTOOL) --tag=CC --mode=compile"
-    write (unit, "(A)")  "LINK = $(LIBTOOL) --tag=FC --mode=link"
+    if (verbose) then
+       write (unit, "(A)")  "FCOMPILE = $(LIBTOOL) --tag=FC --mode=compile"
+       write (unit, "(A)")  "CCOMPILE = $(LIBTOOL) --tag=CC --mode=compile"
+       write (unit, "(A)")  "LINK = $(LIBTOOL) --tag=FC --mode=link"
+    else
+       write (unit, "(A)")  "FCOMPILE = @$(LIBTOOL) --silent --tag=FC --mode=compile"
+       write (unit, "(A)")  "CCOMPILE = @$(LIBTOOL) --silent --tag=CC --mode=compile"
+       write (unit, "(A)")  "LINK = @$(LIBTOOL) --silent --tag=FC --mode=link"
+    end if
     write (unit, "(A)")  ""
     write (unit, "(A)")  "# Compile commands (default)"
     write (unit, "(A)")  "LTFCOMPILE = $(FCOMPILE) $(FC) -c &
@@ -637,15 +656,21 @@ contains
     write (unit, "(A)")  ""
     write (unit, "(A)")  "# Matrix-element code files"
     do i = 1, size (driver%record)
-       call driver%record(i)%write_makefile_code (unit, os_data, testflag)
+       call driver%record(i)%write_makefile_code (unit, os_data, verbose, testflag)
     end do
     write (unit, "(A)")  ""
     write (unit, "(A)")  "# Library driver"
     write (unit, "(A)")  "$(BASE).lo: $(BASE).f90 $(OBJECTS)"
     write (unit, "(A)")  TAB // "$(LTFCOMPILE) $<"
+    if (.not. verbose) then
+       write (unit, "(A)")  TAB // '@echo  "  FC       " $@'
+    end if
     write (unit, "(A)")  ""
     write (unit, "(A)")  "# Library"
     write (unit, "(A)")  "$(BASE).la: $(BASE).lo $(OBJECTS)"
+    if (.not. verbose) then
+       write (unit, "(A)")  TAB // '@echo  "  FCLD     " $@'
+    end if
     write (unit, "(A)")  TAB // "$(LINK) $(FC) -module -rpath /dev/null &
          &$(FCFLAGS) $(LDFLAGS) -o $(BASE).la $^"
     write (unit, "(A)")  ""
@@ -664,16 +689,43 @@ contains
     write (unit, "(A)")  ""
     write (unit, "(A)")  "# Generic cleanup targets"
     write (unit, "(A)")  "clean-library:"
-    write (unit, "(A)")  TAB // "rm -f $(BASE).la"
+    if (verbose) then
+       write (unit, "(A)")  TAB // "rm -f $(BASE).la"
+    else
+       write (unit, "(A)")  TAB // '@echo  "  RM        $(BASE).la"'
+       write (unit, "(A)")  TAB // "@rm -f $(BASE).la"
+    end if
     write (unit, "(A)")  "clean-objects:"
-    write (unit, "(A)")  TAB // "rm -f $(BASE).lo $(BASE)_driver.mod &
-         &$(CLEAN_OBJECTS)"
+    if (verbose) then
+       write (unit, "(A)")  TAB // "rm -f $(BASE).lo $(BASE)_driver.mod &
+            &$(CLEAN_OBJECTS)"
+    else
+       write (unit, "(A)")  TAB // '@echo  "  RM        $(BASE).lo &
+            &$(BASE)_driver.mod $(CLEAN_OBJECTS)"'
+       write (unit, "(A)")  TAB // "@rm -f $(BASE).lo $(BASE)_driver.mod &
+            &$(CLEAN_OBJECTS)"
+    end if
     write (unit, "(A)")  "clean-source:"
-    write (unit, "(A)")  TAB // "rm -f $(CLEAN_SOURCES)"
+    if (verbose) then
+       write (unit, "(A)")  TAB // "rm -f $(CLEAN_SOURCES)"
+    else
+       write (unit, "(A)")  TAB // '@echo  "  RM        $(CLEAN_SOURCES)"'
+       write (unit, "(A)")  TAB // "@rm -f $(CLEAN_SOURCES)"
+    end if
     write (unit, "(A)")  "clean-driver:"
-    write (unit, "(A)")  TAB // "rm -f $(BASE).f90"
+    if (verbose) then
+       write (unit, "(A)")  TAB // "rm -f $(BASE).f90"
+    else
+       write (unit, "(A)")  TAB // '@echo  "  RM        $(BASE).f90"'
+       write (unit, "(A)")  TAB // "@rm -f $(BASE).f90"
+    end if
     write (unit, "(A)")  "clean-makefile:"
-    write (unit, "(A)")  TAB // "rm -f $(BASE).makefile"
+    if (verbose) then
+       write (unit, "(A)")  TAB // "rm -f $(BASE).makefile"
+    else
+       write (unit, "(A)")  TAB // '@echo  "  RM        $(BASE).makefile"'
+       write (unit, "(A)")  TAB // "@rm -f $(BASE).makefile"
+    end if
     write (unit, "(A)")  ".PHONY: clean-library clean-objects &
          &clean-source clean-driver clean-makefile"
     write (unit, "(A)")  ""
@@ -1529,129 +1581,219 @@ contains
     write (unit, "(3A)")  "end subroutine ", char (prefix), "get_fptr"
   end subroutine write_get_fptr_sub
 
-  subroutine prclib_driver_make_source (driver, os_data)
+  function workspace_prefix (workspace) result (prefix)
+    type(string_t), intent(in), optional :: workspace
+    type(string_t) :: prefix
+    if (present (workspace)) then
+       if (workspace /= "") then
+          prefix = workspace // "/"
+       else
+          prefix = ""
+       end if
+    else
+       prefix = ""
+    end if
+  end function workspace_prefix
+
+  function workspace_path (workspace) result (path)
+    type(string_t), intent(in), optional :: workspace
+    type(string_t) :: path
+    if (present (workspace)) then
+       if (workspace /= "") then
+          path = workspace
+       else
+          path = "."
+       end if
+    else
+       path = "."
+    end if
+  end function workspace_path
+
+  function workspace_cmd (workspace) result (cmd)
+    type(string_t), intent(in), optional :: workspace
+    type(string_t) :: cmd
+    if (present (workspace)) then
+       if (workspace /= "") then
+          cmd = "cd " // workspace // " && "
+       else
+          cmd = ""
+       end if
+    else
+       cmd = ""
+    end if
+  end function workspace_cmd
+  
+  subroutine prclib_driver_make_source (driver, os_data, workspace)
     class(prclib_driver_t), intent(in) :: driver
     type(os_data_t), intent(in) :: os_data
+    type(string_t), intent(in), optional :: workspace
     integer :: i
     do i = 1, driver%n_processes
        call driver%record(i)%write_source_code ()
     end do
-    call os_system_call ("make source " // os_data%makeflags &
+    call os_system_call ( &
+         workspace_cmd (workspace) &
+         // "make source " // os_data%makeflags &
          // " -f " // driver%basename // ".makefile")
   end subroutine prclib_driver_make_source
 
-  subroutine prclib_driver_make_compile (driver, os_data)
+  subroutine prclib_driver_make_compile (driver, os_data, workspace)
     class(prclib_driver_t), intent(in) :: driver
     type(os_data_t), intent(in) :: os_data
-    call os_system_call ("make compile " // os_data%makeflags &
+    type(string_t), intent(in), optional :: workspace
+    integer :: i
+    do i = 1, driver%n_processes
+       call driver%record(i)%before_compile ()
+    end do
+    call os_system_call ( &
+         workspace_cmd (workspace) &
+         // "make compile " // os_data%makeflags &
          // " -f " // driver%basename // ".makefile")
+    do i = 1, driver%n_processes
+       call driver%record(i)%after_compile ()
+    end do
   end subroutine prclib_driver_make_compile
 
-  subroutine prclib_driver_make_link (driver, os_data)
+  subroutine prclib_driver_make_link (driver, os_data, workspace)
     class(prclib_driver_t), intent(in) :: driver
     type(os_data_t), intent(in) :: os_data
-    call os_system_call ("make link " // os_data%makeflags &
+    type(string_t), intent(in), optional :: workspace
+    integer :: i
+    call os_system_call ( &
+         workspace_cmd (workspace) &
+         // "make link " // os_data%makeflags &
          // " -f " // driver%basename // ".makefile")
   end subroutine prclib_driver_make_link
 
-  subroutine prclib_driver_clean_library (driver, os_data)
+  subroutine prclib_driver_clean_library (driver, os_data, workspace)
     class(prclib_driver_t), intent(in) :: driver
     type(os_data_t), intent(in) :: os_data
+    type(string_t), intent(in), optional :: workspace
     if (driver%makefile_exists ()) then
-       call os_system_call ("make clean-library " // os_data%makeflags &
+       call os_system_call ( &
+         workspace_cmd (workspace) &
+         // "make clean-library " // os_data%makeflags &
             // " -f " // driver%basename // ".makefile")
     end if
   end subroutine prclib_driver_clean_library
 
-  subroutine prclib_driver_clean_objects (driver, os_data)
+  subroutine prclib_driver_clean_objects (driver, os_data, workspace)
     class(prclib_driver_t), intent(in) :: driver
     type(os_data_t), intent(in) :: os_data
+    type(string_t), intent(in), optional :: workspace
     if (driver%makefile_exists ()) then
-       call os_system_call ("make clean-objects " // os_data%makeflags &
+       call os_system_call ( &
+         workspace_cmd (workspace) &
+         // "make clean-objects " // os_data%makeflags &
             // " -f " // driver%basename // ".makefile")
     end if
   end subroutine prclib_driver_clean_objects
 
-  subroutine prclib_driver_clean_source (driver, os_data)
+  subroutine prclib_driver_clean_source (driver, os_data, workspace)
     class(prclib_driver_t), intent(in) :: driver
     type(os_data_t), intent(in) :: os_data
+    type(string_t), intent(in), optional :: workspace
     if (driver%makefile_exists ()) then
-       call os_system_call ("make clean-source " // os_data%makeflags &
+       call os_system_call ( &
+         workspace_cmd (workspace) &
+         // "make clean-source " // os_data%makeflags &
             // " -f " // driver%basename // ".makefile")
     end if
   end subroutine prclib_driver_clean_source
 
-  subroutine prclib_driver_clean_driver (driver, os_data)
+  subroutine prclib_driver_clean_driver (driver, os_data, workspace)
     class(prclib_driver_t), intent(in) :: driver
     type(os_data_t), intent(in) :: os_data
+    type(string_t), intent(in), optional :: workspace
     if (driver%makefile_exists ()) then
-       call os_system_call ("make clean-driver " // os_data%makeflags &
+       call os_system_call ( &
+         workspace_cmd (workspace) &
+         // "make clean-driver " // os_data%makeflags &
             // " -f " // driver%basename // ".makefile")
     end if
   end subroutine prclib_driver_clean_driver
 
-  subroutine prclib_driver_clean_makefile (driver, os_data)
+  subroutine prclib_driver_clean_makefile (driver, os_data, workspace)
     class(prclib_driver_t), intent(in) :: driver
     type(os_data_t), intent(in) :: os_data
+    type(string_t), intent(in), optional :: workspace
     if (driver%makefile_exists ()) then
-       call os_system_call ("make clean-makefile " // os_data%makeflags &
+       call os_system_call ( &
+         workspace_cmd (workspace) &
+         // "make clean-makefile " // os_data%makeflags &
             // " -f " // driver%basename // ".makefile")
     end if
   end subroutine prclib_driver_clean_makefile
 
-  subroutine prclib_driver_clean (driver, os_data)
+  subroutine prclib_driver_clean (driver, os_data, workspace)
     class(prclib_driver_t), intent(in) :: driver
     type(os_data_t), intent(in) :: os_data
+    type(string_t), intent(in), optional :: workspace
     if (driver%makefile_exists ()) then
-       call os_system_call ("make clean " // os_data%makeflags &
+       call os_system_call ( &
+         workspace_cmd (workspace) &
+         // "make clean " // os_data%makeflags &
             // " -f " // driver%basename // ".makefile")
     end if
   end subroutine prclib_driver_clean
 
-  subroutine prclib_driver_distclean (driver, os_data)
+  subroutine prclib_driver_distclean (driver, os_data, workspace)
     class(prclib_driver_t), intent(in) :: driver
     type(os_data_t), intent(in) :: os_data
+    type(string_t), intent(in), optional :: workspace
     if (driver%makefile_exists ()) then
-       call os_system_call ("make distclean " // os_data%makeflags &
+       call os_system_call ( &
+         workspace_cmd (workspace) &
+         // "make distclean " // os_data%makeflags &
             // " -f " // driver%basename // ".makefile")
     end if
   end subroutine prclib_driver_distclean
 
-  subroutine prclib_driver_clean_proc (driver, i, os_data)
+  subroutine prclib_driver_clean_proc (driver, i, os_data, workspace)
     class(prclib_driver_t), intent(in) :: driver
     integer, intent(in) :: i
     type(os_data_t), intent(in) :: os_data
+    type(string_t), intent(in), optional :: workspace
     type(string_t) :: id
     if (driver%makefile_exists ()) then
        id = driver%record(i)%id
-      call os_system_call ("make clean-" // driver%record(i)%id // " " &
+      call os_system_call ( &
+         workspace_cmd (workspace) &
+         // "make clean-" // driver%record(i)%id // " " &
            // os_data%makeflags &
            // " -f " // driver%basename // ".makefile")
     end if
   end subroutine prclib_driver_clean_proc
 
-  function prclib_driver_makefile_exists (driver) result (flag)
+  function prclib_driver_makefile_exists (driver, workspace) result (flag)
     class(prclib_driver_t), intent(in) :: driver
+    type(string_t), intent(in), optional :: workspace
     logical :: flag
-    inquire (file = char (driver%basename) // ".makefile", exist = flag)
+    inquire (file = char (workspace_prefix (workspace) &
+         &                // driver%basename) // ".makefile", &
+         exist = flag)
   end function prclib_driver_makefile_exists
 
-  subroutine prclib_driver_load (driver, os_data, noerror)
+  subroutine prclib_driver_load (driver, os_data, noerror, workspace)
     class(prclib_driver_t), intent(inout) :: driver
     type(os_data_t), intent(in) :: os_data
     logical, intent(in), optional :: noerror
+    type(string_t), intent(in), optional :: workspace
     type(c_funptr) :: c_fptr
     logical :: ignore
 
     ignore = .false.;  if (present (noerror))  ignore = noerror
 
-    driver%libname = os_get_dlname (driver%basename, os_data, noerror, noerror)
+    driver%libname = os_get_dlname ( &
+         workspace_prefix (workspace) // driver%basename, &
+         os_data, noerror, noerror)
     if (driver%libname == "")  return
     select type (driver)
     type is (prclib_driver_dynamic_t)
        if (.not. dlaccess_is_open (driver%dlaccess)) then
           call dlaccess_init &
-               (driver%dlaccess, var_str ("."), driver%libname, os_data)
+               (driver%dlaccess, workspace_path (workspace), &
+                driver%libname, os_data)
           if (.not. ignore)  call driver%check_dlerror ()
        end if
        driver%loaded = dlaccess_is_open (driver%dlaccess)
@@ -1777,15 +1919,16 @@ contains
     call driver%check_dlerror ()
   end function prclib_driver_dynamic_get_c_funptr
 
-  function prclib_driver_get_md5sum_makefile (driver) result (md5sum)
+  function prclib_driver_get_md5sum_makefile (driver, workspace) result (md5sum)
     class(prclib_driver_t), intent(in) :: driver
+    type(string_t), intent(in), optional :: workspace
     character(32) :: md5sum
     type(string_t) :: filename
     character(80) :: buffer
     logical :: exist
     integer :: u, iostat
     md5sum = ""
-    filename = driver%basename // ".makefile"
+    filename = workspace_prefix (workspace) // driver%basename // ".makefile"
     inquire (file = char (filename), exist = exist)
     if (exist) then
        u = free_unit ()
@@ -1805,15 +1948,16 @@ contains
     end if
   end function prclib_driver_get_md5sum_makefile
 
-  function prclib_driver_get_md5sum_driver (driver) result (md5sum)
+  function prclib_driver_get_md5sum_driver (driver, workspace) result (md5sum)
     class(prclib_driver_t), intent(in) :: driver
+    type(string_t), intent(in), optional :: workspace
     character(32) :: md5sum
     type(string_t) :: filename
     character(80) :: buffer
     logical :: exist
     integer :: u, iostat
     md5sum = ""
-    filename = driver%basename // ".f90"
+    filename = workspace_prefix (workspace) // driver%basename // ".f90"
     inquire (file = char (filename), exist = exist)
     if (exist) then
        u = free_unit ()
@@ -1833,9 +1977,11 @@ contains
     end if
   end function prclib_driver_get_md5sum_driver
 
-  function prclib_driver_get_md5sum_source (driver, i) result (md5sum)
+  function prclib_driver_get_md5sum_source &
+       (driver, i, workspace) result (md5sum)
     class(prclib_driver_t), intent(in) :: driver
     integer, intent(in) :: i
+    type(string_t), intent(in), optional :: workspace
     character(32) :: md5sum
     type(string_t) :: filename
     character(80) :: buffer
@@ -1843,7 +1989,7 @@ contains
     integer :: u, iostat
     md5sum = ""
 
-    filename = driver%record(i)%id // ".f90"
+    filename = workspace_prefix (workspace) // driver%record(i)%id // ".f90"
     inquire (file = char (filename), exist = exist)
     if (exist) then
        u = free_unit ()
