@@ -1,19 +1,12 @@
-! WHIZARD 2.4.1 Mar 24 2017
+! WHIZARD 2.6.4 Aug 23 2018
 !
-! Copyright (C) 1999-2017 by
+! Copyright (C) 1999-2018 by
 !     Wolfgang Kilian <kilian@physik.uni-siegen.de>
 !     Thorsten Ohl <ohl@physik.uni-wuerzburg.de>
 !     Juergen Reuter <juergen.reuter@desy.de>
 !
 !     with contributions from
-!     Fabian Bach <fabian.bach@t-online.de>
-!     Bijan Chokoufe <bijan.chokoufe@desy.de>
-!     Christian Speckner <cnspeckn@googlemail.com>
-!     So Young Shim <soyoung.shim@desy.de>
-!     Florian Staub <florian.staub@cern.ch>
-!     Christian Weiss <christian.weiss@desy.de>
-!     and Hans-Werner Boschmann, Felix Braam,
-!     Sebastian Schmidt, So-young Shim, Daniel Wiesler
+!     cf. main AUTHORS file
 !
 ! WHIZARD is free software; you can redistribute it and/or modify it
 ! under the terms of the GNU General Public License as published by
@@ -116,21 +109,23 @@ module process_libraries
      procedure :: get_n_tot => process_component_def_get_n_tot
      procedure :: get_prt_in => process_component_def_get_prt_in
      procedure :: get_prt_out => process_component_def_get_prt_out
+     procedure :: get_prt_spec_in => process_component_def_get_prt_spec_in
+     procedure :: get_prt_spec_out => process_component_def_get_prt_spec_out
      procedure :: get_pdg_in => process_component_def_get_pdg_in
      procedure :: get_md5sum => process_component_def_get_md5sum
      procedure :: get_nlo_type => process_component_def_get_nlo_type
      procedure :: get_associated_born &
-                  => process_component_def_get_associated_born
+          => process_component_def_get_associated_born
      procedure :: get_associated_real_fin &
-                  => process_component_def_get_associated_real_fin
+          => process_component_def_get_associated_real_fin
      procedure :: get_associated_real_sing &
-                  => process_component_def_get_associated_real_sing
+          => process_component_def_get_associated_real_sing
      procedure :: get_associated_subtraction &
-                  => process_component_def_get_associated_subtraction
+          => process_component_def_get_associated_subtraction
      procedure :: get_association_list &
-                  => process_component_def_get_association_list
+          => process_component_def_get_association_list
      procedure :: can_be_integrated &
-                  => process_component_def_can_be_integrated
+          => process_component_def_can_be_integrated
      procedure :: get_associated_real => process_component_def_get_associated_real
      procedure :: get_me_method => process_component_def_get_me_method
      procedure :: get_fixed_emitter => process_component_def_get_fixed_emitter
@@ -150,6 +145,7 @@ module process_libraries
      type(process_component_def_t), dimension(:), allocatable :: extra
      character(32) :: md5sum = ""
      logical :: nlo_process = .false.
+     logical :: requires_resonances = .false.
    contains
      procedure :: write => process_def_write
      procedure :: read => process_def_read
@@ -187,6 +183,8 @@ module process_libraries
      procedure :: append => process_def_list_append
      procedure :: get_n_processes => process_def_list_get_n_processes
      procedure :: get_process_id_list => process_def_list_get_process_id_list
+     procedure :: get_process_id_req_resonant => &
+          process_def_list_get_process_id_req_resonant
      procedure :: contains => process_def_list_contains
      procedure :: get_entry_index => process_def_list_get_entry_index
      procedure :: get_num_id => process_def_list_get_num_id
@@ -198,6 +196,7 @@ module process_libraries
      procedure :: get_component_list => process_def_list_get_component_list
      procedure :: get_component_description_list => &
           process_def_list_get_component_description_list
+     procedure :: req_resonant => process_def_list_req_resonant
      procedure :: get_nlo_process => process_def_list_get_nlo_process
   end type process_def_list_t
 
@@ -500,6 +499,20 @@ contains
     end do
   end subroutine process_component_def_get_prt_out
 
+  function process_component_def_get_prt_spec_in (component) result (prt)
+    class(process_component_def_t), intent(in) :: component
+    type(prt_spec_t), dimension(:), allocatable :: prt
+    allocate (prt (component%n_in))
+    prt(:) = component%prt_in(:)
+  end function process_component_def_get_prt_spec_in
+
+  function process_component_def_get_prt_spec_out (component) result (prt)
+    class(process_component_def_t), intent(in) :: component
+    type(prt_spec_t), dimension(:), allocatable :: prt
+    allocate (prt (component%n_out))
+    prt(:) = component%prt_out(:)
+  end function process_component_def_get_prt_spec_out
+
   subroutine process_component_def_get_pdg_in (component, model, pdg)
     class(process_component_def_t), intent(in) :: component
     class(model_data_t), intent(in), target :: model
@@ -621,6 +634,10 @@ contains
          object%n_initial
     write (unit, "(1x,A,I0)")  "Extra generated component(s)   = ", &
          object%n_extra
+    if (object%requires_resonances) then
+       ! This line has to matched with the reader below!
+       write (unit, "(1x,A,I0)")  "Resonant subprocesses required"
+    end if
     write (unit, "(1x,A,A,A)") "MD5 sum   = '", object%md5sum, "'"
     if (allocated (object%initial)) then
        do i = 1, size (object%initial)
@@ -674,6 +691,13 @@ contains
     read (buffer, *)  object%n_extra
 
     read (unit, "(A)")  buffer
+    if (buffer(1:9) == " Resonant") then
+       object%requires_resonances = .true.
+       read (unit, "(A)")  buffer
+    else
+       object%requires_resonances = .false.
+    end if
+
     call strip_equation_lhs (buffer)
     read (buffer(3:34), "(A32)")  object%md5sum
 
@@ -697,7 +721,11 @@ contains
     if (object%num_id /= 0) &
          write (unit, "(1x,'(',I0,')')", advance="no")  object%num_id
     if (object%model_name /= "") &
-         write (unit, "(1x,'[',A,']')")  char (object%model_name)
+         write (unit, "(1x,'[',A,']')", advance="no")  char (object%model_name)
+    if (object%requires_resonances) then
+       write (unit, "(1x,A)", advance="no")  "[+ resonant subprocesses]"
+    end if
+    write (unit, *)
     if (allocated (object%initial)) then
        do i = 1, size (object%initial)
           call object%initial(i)%show (unit)
@@ -711,7 +739,8 @@ contains
   end subroutine process_def_show
 
   subroutine process_def_init (def, id, &
-       model, model_name, n_in, n_components, num_id, nlo_process)
+       model, model_name, n_in, n_components, num_id, &
+       nlo_process, requires_resonances)
     class(process_def_t), intent(out) :: def
     type(string_t), intent(in), optional :: id
     class(model_data_t), intent(in), optional, target :: model
@@ -720,6 +749,7 @@ contains
     integer, intent(in), optional :: n_components
     integer, intent(in), optional :: num_id
     logical, intent(in), optional :: nlo_process
+    logical, intent(in), optional :: requires_resonances
     character(16) :: suffix
     integer :: i
     if (present (id)) then
@@ -746,7 +776,12 @@ contains
        def%n_initial = n_components
        allocate (def%initial (n_components))
     end if
-    if (present (nlo_process)) def%nlo_process = nlo_process
+    if (present (nlo_process)) then
+       def%nlo_process = nlo_process
+    end if
+    if (present (requires_resonances)) then
+       def%requires_resonances = requires_resonances
+    end if
     def%initial%initial = .true.
     def%initial%method     = ""
     do i = 1, def%n_initial
@@ -825,7 +860,7 @@ contains
               end if
            end if
            nlo_type_string = component_status (comp%nlo_type)
-           if (nlo_type_string /= "Born") then
+           if (nlo_type_string /= "born") then
              d = d // ", [" // nlo_type_string // "]"
            end if
          end associate
@@ -853,21 +888,29 @@ contains
   end subroutine process_def_set_coupling_powers
 
   subroutine process_def_set_associated_components (def, i, &
-         i_born, i_real, i_virt, i_sub, i_pdf, i_rfin)
+       i_list, remnant, real_finite, mismatch)
     class(process_def_t), intent(inout) :: def
+    logical, intent(in) :: remnant, real_finite, mismatch
     integer, intent(in) :: i
-    integer, intent(in) :: i_born, i_real
-    integer, intent(in) :: i_virt, i_sub
-    integer, intent(in), optional :: i_pdf, i_rfin
+    integer, dimension(:), intent(in) :: i_list
+    integer :: add_index
+    add_index = 0
     associate (comp => def%initial(i)%associated_components)
-       comp(ASSOCIATED_BORN) = i_born
-       comp(ASSOCIATED_REAL) = i_real
-       comp(ASSOCIATED_VIRT) = i_virt
-       comp(ASSOCIATED_SUB) = i_sub
-       if (present (i_pdf)) &
-          comp(ASSOCIATED_PDF) = i_pdf
-       if (present (i_rfin)) &
-          comp(ASSOCIATED_REAL_FIN) = i_rfin
+       comp(ASSOCIATED_BORN) = i_list(1)
+       comp(ASSOCIATED_REAL) = i_list(2)
+       comp(ASSOCIATED_VIRT) = i_list(3)
+       comp(ASSOCIATED_SUB) = i_list(4)
+       if (remnant) then
+          comp(ASSOCIATED_PDF) = i_list(5)
+          add_index = add_index + 1
+       end if
+       if (real_finite) then
+          comp(ASSOCIATED_REAL_FIN) = i_list(5+add_index)
+          add_index = add_index + 1
+       end if
+       if (mismatch) then
+          !!! incomplete
+       end if
     end associate
   end subroutine process_def_set_associated_components
 
@@ -1044,6 +1087,24 @@ contains
     end do
   end subroutine process_def_list_get_process_id_list
 
+  subroutine process_def_list_get_process_id_req_resonant (list, id)
+    class(process_def_list_t), intent(in) :: list
+    type(string_t), dimension(:), allocatable, intent(out) :: id
+    type(process_def_entry_t), pointer :: current
+    integer :: i
+    allocate (id (list%get_n_processes ()))
+    i = 0
+    current => list%first
+    do while (associated (current))
+       if (current%requires_resonances) then
+          i = i + 1
+          id(i) = current%id
+       end if
+       current => current%next
+    end do
+    id = id(1:i)
+  end subroutine process_def_list_get_process_id_req_resonant
+
   function process_def_list_contains (list, id) result (flag)
     logical :: flag
     class(process_def_list_t), intent(in) :: list
@@ -1221,6 +1282,21 @@ contains
     end do
   end subroutine process_def_list_get_component_description_list
 
+  function process_def_list_req_resonant (list, id) result (nlo)
+    class(process_def_list_t), intent(in) :: list
+    type(string_t), intent(in) :: id
+    logical :: nlo
+    type(process_def_entry_t), pointer :: current
+    current => list%first
+    do while (associated (current))
+      if (id == current%id) then
+        nlo = current%requires_resonances
+        return
+      end if
+      current => current%next
+    end do
+  end function process_def_list_req_resonant
+
   function process_def_list_get_nlo_process (list, id) result (nlo)
     class(process_def_list_t), intent(in) :: list
     type(string_t), intent(in) :: id
@@ -1476,18 +1552,20 @@ contains
     call lib%driver%set_md5sum (lib%md5sum)
   end subroutine process_library_compute_md5sum
 
-  subroutine process_library_write_makefile (lib, os_data, force, testflag)
+  subroutine process_library_write_makefile &
+       (lib, os_data, force, verbose, testflag, workspace)
     class(process_library_t), intent(inout) :: lib
     type(os_data_t), intent(in) :: os_data
-    logical, intent(in) :: force
+    logical, intent(in) :: force, verbose
     logical, intent(in), optional :: testflag
+    type(string_t), intent(in), optional :: workspace
     character(32) :: md5sum_file
     logical :: generate
     integer :: unit
     if (lib%external .and. .not. lib%static) then
        generate = .true.
        if (.not. force) then
-          md5sum_file = lib%driver%get_md5sum_makefile ()
+          md5sum_file = lib%driver%get_md5sum_makefile (workspace)
           if (lib%md5sum == md5sum_file) then
              call msg_message ("Process library '" // char (lib%basename) &
                   // "': keeping makefile")
@@ -1498,25 +1576,28 @@ contains
           call msg_message ("Process library '" // char (lib%basename) &
                // "': writing makefile")
           unit = free_unit ()
-          open (unit, file = char (lib%driver%basename // ".makefile"), &
+          open (unit, &
+               file = char (workspace_prefix (workspace) &
+               &            // lib%driver%basename // ".makefile"), &
                status="replace", action="write")
-          call lib%driver%generate_makefile (unit, os_data, testflag)
+          call lib%driver%generate_makefile (unit, os_data, verbose, testflag)
           close (unit)
        end if
        lib%makefile_exists = .true.
     end if
   end subroutine process_library_write_makefile
 
-  subroutine process_library_write_driver (lib, force)
+  subroutine process_library_write_driver (lib, force, workspace)
     class(process_library_t), intent(inout) :: lib
     logical, intent(in) :: force
+    type(string_t), intent(in), optional :: workspace
     character(32) :: md5sum_file
     logical :: generate
     integer :: unit
     if (lib%external .and. .not. lib%static) then
        generate = .true.
        if (.not. force) then
-          md5sum_file = lib%driver%get_md5sum_driver ()
+          md5sum_file = lib%driver%get_md5sum_driver (workspace)
           if (lib%md5sum == md5sum_file) then
              call msg_message ("Process library '" // char (lib%basename) &
                   // "': keeping driver")
@@ -1527,7 +1608,9 @@ contains
           call msg_message ("Process library '" // char (lib%basename) &
                // "': writing driver")
           unit = free_unit ()
-          open (unit, file = char (lib%driver%basename // ".f90"), &
+          open (unit, & 
+               file = char (workspace_prefix (workspace) &
+               &            // lib%driver%basename // ".f90"), &
                status="replace", action="write")
           call lib%driver%generate_driver_code (unit)
           close (unit)
@@ -1536,15 +1619,16 @@ contains
     end if
   end subroutine process_library_write_driver
 
-  subroutine process_library_update_status (lib, os_data)
+  subroutine process_library_update_status (lib, os_data, workspace)
     class(process_library_t), intent(inout) :: lib
     type(os_data_t), intent(in) :: os_data
+    type(string_t), intent(in), optional :: workspace
     character(32) :: md5sum_file
     integer :: i, i_external, i_component
     if (lib%external) then
        select case (lib%status)
        case (STAT_CONFIGURED:STAT_LINKED)
-          call lib%driver%load (os_data, noerror=.true.)
+          call lib%driver%load (os_data, noerror=.true., workspace=workspace)
        end select
        if (lib%driver%loaded) then
           md5sum_file = lib%driver%get_md5sum (0)
@@ -1582,7 +1666,8 @@ contains
                if (i_external /= 0) then
                   select case (entry%status)
                   case (STAT_CONFIGURED)
-                     md5sum_file = lib%driver%get_md5sum_source (i_external)
+                     md5sum_file = lib%driver%get_md5sum_source &
+                          (i_external, workspace)
                      if (entry%def%get_md5sum (i_component) == md5sum_file) then
                         entry%status = STAT_SOURCE
                      end if
@@ -1591,7 +1676,7 @@ contains
              end associate
           end do
           if (all (lib%entry%status >= STAT_SOURCE)) then
-             md5sum_file = lib%driver%get_md5sum_driver ()
+             md5sum_file = lib%driver%get_md5sum_driver (workspace)
              if (lib%md5sum == md5sum_file) then
                 lib%status = STAT_SOURCE
              end if
@@ -1600,10 +1685,12 @@ contains
     end if
   end subroutine process_library_update_status
 
-  subroutine process_library_make_source (lib, os_data, keep_old_source)
+  subroutine process_library_make_source &
+       (lib, os_data, keep_old_source, workspace)
     class(process_library_t), intent(inout) :: lib
     type(os_data_t), intent(in) :: os_data
     logical, intent(in), optional :: keep_old_source
+    type(string_t), intent(in), optional :: workspace
     logical :: keep_old
     integer :: i, i_external
     keep_old = .false.
@@ -1622,12 +1709,13 @@ contains
                   i_external = entry%i_external
                   if (i_external /= 0 &
                        .and. lib%entry(i)%status == STAT_CONFIGURED) then
-                     call lib%driver%clean_proc (i_external, os_data)
+                     call lib%driver%clean_proc &
+                          (i_external, os_data, workspace)
                   end if
                 end associate
                 if (signal_is_pending ())  return
              end do
-             call lib%driver%make_source (os_data)
+             call lib%driver%make_source (os_data, workspace)
           end if
           lib%status = STAT_SOURCE
           where (lib%entry%i_external /= 0 &
@@ -1639,21 +1727,23 @@ contains
     end if
   end subroutine process_library_make_source
 
-  subroutine process_library_make_compile (lib, os_data, keep_old_source)
+  subroutine process_library_make_compile &
+       (lib, os_data, keep_old_source, workspace)
     class(process_library_t), intent(inout) :: lib
     type(os_data_t), intent(in) :: os_data
     logical, intent(in), optional :: keep_old_source
+    type(string_t), intent(in), optional :: workspace
     if (lib%external .and. .not. lib%static) then
        select case (lib%status)
        case (STAT_CONFIGURED)
-          call lib%make_source (os_data, keep_old_source)
+          call lib%make_source (os_data, keep_old_source, workspace)
        end select
        if (signal_is_pending ())  return
        select case (lib%status)
        case (STAT_SOURCE)
           call msg_message ("Process library '" // char (lib%basename) &
                // "': compiling sources")
-          call lib%driver%make_compile (os_data)
+          call lib%driver%make_compile (os_data, workspace)
           where (lib%entry%i_external /= 0 &
                .and. lib%entry%status == STAT_SOURCE)
              lib%entry%status = STAT_COMPILED
@@ -1663,34 +1753,37 @@ contains
     end if
   end subroutine process_library_make_compile
 
-  subroutine process_library_make_link (lib, os_data, keep_old_source)
+  subroutine process_library_make_link &
+       (lib, os_data, keep_old_source, workspace)
     class(process_library_t), intent(inout) :: lib
     type(os_data_t), intent(in) :: os_data
     logical, intent(in), optional :: keep_old_source
+    type(string_t), intent(in), optional :: workspace
     if (lib%external .and. .not. lib%static) then
        select case (lib%status)
        case (STAT_CONFIGURED:STAT_SOURCE)
-          call lib%make_compile (os_data, keep_old_source)
+          call lib%make_compile (os_data, keep_old_source, workspace)
        end select
        if (signal_is_pending ())  return
        select case (lib%status)
        case (STAT_COMPILED)
           call msg_message ("Process library '" // char (lib%basename) &
                // "': linking")
-          call lib%driver%make_link (os_data)
+          call lib%driver%make_link (os_data, workspace)
           lib%entry%status = STAT_LINKED
           lib%status = STAT_LINKED
        end select
     end if
   end subroutine process_library_make_link
 
-  subroutine process_library_load (lib, os_data, keep_old_source)
+  subroutine process_library_load (lib, os_data, keep_old_source, workspace)
     class(process_library_t), intent(inout) :: lib
     type(os_data_t), intent(in) :: os_data
     logical, intent(in), optional :: keep_old_source
+    type(string_t), intent(in), optional :: workspace
     select case (lib%status)
     case (STAT_CONFIGURED:STAT_COMPILED)
-       call lib%make_link (os_data, keep_old_source)
+       call lib%make_link (os_data, keep_old_source, workspace)
     end select
     if (signal_is_pending ())  return
     select case (lib%status)
@@ -1698,7 +1791,7 @@ contains
        if (lib%external) then
           call msg_message ("Process library '" // char (lib%basename) &
                // "': loading")
-          call lib%driver%load (os_data)
+          call lib%driver%load (os_data, workspace=workspace)
           call lib%load_entries ()
        end if
        lib%entry%status = STAT_ACTIVE
@@ -1732,18 +1825,19 @@ contains
     end select
   end subroutine process_library_unload
 
-  subroutine process_library_clean (lib, os_data, distclean)
+  subroutine process_library_clean (lib, os_data, distclean, workspace)
     class(process_library_t), intent(inout) :: lib
     type(os_data_t), intent(in) :: os_data
     logical, intent(in) :: distclean
+    type(string_t), intent(in), optional :: workspace
     call lib%unload ()
     if (lib%external .and. .not. lib%static) then
        call msg_message ("Process library '" // char (lib%basename) &
             // "': removing old files")
        if (distclean) then
-          call lib%driver%distclean (os_data)
+          call lib%driver%distclean (os_data, workspace)
        else
-          call lib%driver%clean (os_data)
+          call lib%driver%clean (os_data, workspace)
        end if
     end if
     where (lib%entry%i_external /= 0)
