@@ -1,19 +1,12 @@
-! WHIZARD 2.4.1 Mar 24 2017
+! WHIZARD 2.6.4 Aug 23 2018
 !
-! Copyright (C) 1999-2017 by
+! Copyright (C) 1999-2018 by
 !     Wolfgang Kilian <kilian@physik.uni-siegen.de>
 !     Thorsten Ohl <ohl@physik.uni-wuerzburg.de>
 !     Juergen Reuter <juergen.reuter@desy.de>
 !
 !     with contributions from
-!     Fabian Bach <fabian.bach@t-online.de>
-!     Bijan Chokoufe <bijan.chokoufe@desy.de>
-!     Christian Speckner <cnspeckn@googlemail.com>
-!     So Young Shim <soyoung.shim@desy.de>
-!     Florian Staub <florian.staub@cern.ch>
-!     Christian Weiss <christian.weiss@desy.de>
-!     and Hans-Werner Boschmann, Felix Braam,
-!     Sebastian Schmidt, So-young Shim, Daniel Wiesler
+!     cf. main AUTHORS file
 !
 ! WHIZARD is free software; you can redistribute it and/or modify it
 ! under the terms of the GNU General Public License as published by
@@ -307,7 +300,7 @@ contains
 
     write (u, "(A)")  "* Write makefile (no-op)"
     write (u, "(A)")
-    call lib%write_makefile (os_data, force = .true.)
+    call lib%write_makefile (os_data, force = .true., verbose = .true.)
 
     write (u, "(A)")  "* Write driver source code (no-op)"
     write (u, "(A)")
@@ -380,7 +373,7 @@ contains
 
     write (u, "(A)")  "* Write makefile"
     write (u, "(A)")
-    call lib%write_makefile (os_data, force = .true.)
+    call lib%write_makefile (os_data, force = .true., verbose = .false.)
 
     write (u, "(A)")  "* Write driver source code"
     write (u, "(A)")
@@ -452,7 +445,7 @@ contains
 
     write (u, "(A)")  "* Write makefile"
     write (u, "(A)")
-    call lib%write_makefile (os_data, force = .true.)
+    call lib%write_makefile (os_data, force = .true., verbose = .false.)
 
     write (u, "(A)")  "* Write driver source code"
     write (u, "(A)")
@@ -670,7 +663,7 @@ contains
     write (u, "(A)")
     write (u, "(A)")  "* Write makefile"
     write (u, "(A)")
-    call lib%write_makefile (os_data, force = .true.)
+    call lib%write_makefile (os_data, force = .true., verbose = .false.)
 
     write (u, "(A)")  "* Update status"
     write (u, "(A)")
@@ -726,7 +719,7 @@ contains
     call lib%configure (os_data)
     call lib%compute_md5sum ()
     call lib%test_transfer_md5sum (2, 2, 1)
-    call lib%write_makefile (os_data, force = .false.)
+    call lib%write_makefile (os_data, force = .false., verbose = .false.)
     call lib%write_driver (force = .false.)
 
     write (u, "(1x,A,L1)")  "library loaded = ", lib%is_loaded ()
