@@ -1,19 +1,12 @@
-! WHIZARD 2.4.1 Mar 24 2017
+! WHIZARD 2.6.4 Aug 23 2018
 !
-! Copyright (C) 1999-2017 by
+! Copyright (C) 1999-2018 by
 !     Wolfgang Kilian <kilian@physik.uni-siegen.de>
 !     Thorsten Ohl <ohl@physik.uni-wuerzburg.de>
 !     Juergen Reuter <juergen.reuter@desy.de>
 !
 !     with contributions from
-!     Fabian Bach <fabian.bach@t-online.de>
-!     Bijan Chokoufe <bijan.chokoufe@desy.de>
-!     Christian Speckner <cnspeckn@googlemail.com>
-!     So Young Shim <soyoung.shim@desy.de>
-!     Florian Staub <florian.staub@cern.ch>
-!     Christian Weiss <christian.weiss@desy.de>
-!     and Hans-Werner Boschmann, Felix Braam,
-!     Sebastian Schmidt, So-young Shim, Daniel Wiesler
+!     cf. main AUTHORS file
 !
 ! WHIZARD is free software; you can redistribute it and/or modify it
 ! under the terms of the GNU General Public License as published by
@@ -59,15 +52,16 @@ module prc_omega
   private
 
   public :: omega_def_t
-  public :: omega_omega_def_t
-  public :: omega_ovm_def_t
   public :: omega_driver_t
   public :: omega_make_process_component
   public :: prc_omega_t
   public :: omega_state_t
 
-  type, abstract, extends (prc_core_def_t) :: omega_def_t
+  type, extends (prc_core_def_t) :: omega_def_t
+     logical :: ufo = .false.
+     logical :: ovm = .false.
    contains
+     procedure, nopass :: type_string => omega_def_type_string
      procedure :: init => omega_def_init
      procedure :: write => omega_def_write
      procedure :: read => omega_def_read
@@ -77,16 +71,6 @@ module prc_omega
      procedure :: connect => omega_def_connect
   end type omega_def_t
 
-  type, extends (omega_def_t) :: omega_omega_def_t
-   contains
-     procedure, nopass :: type_string => omega_omega_def_type_string
-  end type omega_omega_def_t
-
-  type, extends (omega_def_t) :: omega_ovm_def_t
-   contains
-     procedure, nopass :: type_string => omega_ovm_def_type_string
-  end type omega_ovm_def_t
-
   type, extends (prc_writer_f_module_t), abstract :: omega_writer_t
      type(string_t) :: model_name
      type(string_t) :: process_mode
@@ -97,6 +81,7 @@ module prc_omega
      logical :: diags = .false.
      logical :: diags_color = .false.
      logical :: complex_mass_scheme = .false.
+     logical :: write_phs_output = .false.
      type(string_t) :: extra_options
    contains
      procedure, nopass :: get_module_name => omega_writer_get_module_name
@@ -104,6 +89,8 @@ module prc_omega
      procedure :: init => omega_writer_init
      procedure :: write_makefile_code => omega_write_makefile_code
      procedure :: write_source_code => omega_write_source_code
+     procedure :: before_compile => omega_before_compile
+     procedure :: after_compile => omega_after_compile
      procedure, nopass :: get_procname => omega_writer_get_procname
      procedure :: write_interface => omega_write_interface
      procedure :: write_wrapper => omega_write_wrapper
@@ -114,6 +101,12 @@ module prc_omega
      procedure, nopass :: type_name => omega_omega_writer_type_name
   end type omega_omega_writer_t
 
+  type, extends (omega_omega_writer_t) :: omega_ufo_writer_t
+     type(string_t) :: ufo_path
+   contains
+     procedure, nopass :: type_name => omega_ufo_writer_type_name
+  end type omega_ufo_writer_t
+
   type, extends (omega_writer_t) :: omega_ovm_writer_t
    contains
      procedure, nopass :: type_name => omega_ovm_writer_type_name
@@ -214,46 +207,50 @@ module prc_omega
 
 contains
 
-  function omega_omega_def_type_string () result (string)
+  function omega_def_type_string () result (string)
     type(string_t) :: string
     string = "omega"
-  end function omega_omega_def_type_string
+  end function omega_def_type_string
 
-  function omega_ovm_def_type_string () result (string)
-    type(string_t) :: string
-    string = "ovm"
-  end function omega_ovm_def_type_string
-
-  subroutine omega_def_init (object, model_name, prt_in, prt_out, &
-       restrictions, cms_scheme, openmp_support, report_progress, &
-       extra_options, diags, diags_color)
+  subroutine omega_def_init (object, &
+       model_name, prt_in, prt_out, &
+       ovm, ufo, ufo_path, &
+       restrictions, cms_scheme, &
+       openmp_support, report_progress, write_phs_output, extra_options, diags, diags_color)
     class(omega_def_t), intent(out) :: object
     type(string_t), intent(in) :: model_name
     type(string_t), dimension(:), intent(in) :: prt_in
     type(string_t), dimension(:), intent(in) :: prt_out
-    logical, intent(in), optional :: cms_scheme
+    logical, intent(in) :: ovm
+    logical, intent(in) :: ufo
+    type(string_t), intent(in), optional :: ufo_path
     type(string_t), intent(in), optional :: restrictions
+    logical, intent(in), optional :: cms_scheme
     logical, intent(in), optional :: openmp_support
     logical, intent(in), optional :: report_progress
-    logical, intent(in), optional :: diags, diags_color
+    logical, intent(in), optional :: write_phs_output
     type(string_t), intent(in), optional :: extra_options
-    select type (object)
-    type is (omega_omega_def_t)
-       allocate (omega_omega_writer_t :: object%writer)
-       select type (writer => object%writer)
-       type is (omega_omega_writer_t)
-          call writer%init (model_name, prt_in, prt_out, &
-               restrictions, cms_scheme, openmp_support, &
-               report_progress, extra_options, diags, diags_color)
-       end select
-    type is (omega_ovm_def_t)
-       allocate (omega_ovm_writer_t :: object%writer)
-       select type (writer => object%writer)
-       type is (omega_ovm_writer_t)
-          call writer%init (model_name, prt_in, prt_out, &
-               restrictions, cms_scheme, openmp_support, &
-               report_progress, extra_options, diags, diags_color)
-       end select
+    logical, intent(in), optional :: diags, diags_color
+    object%ufo = ufo
+    object%ovm = ovm
+    if (object%ufo) then
+       if (object%ovm) then
+          call msg_fatal ("Omega process: OVM method does not support UFO model")
+       else
+          allocate (omega_ufo_writer_t :: object%writer)
+       end if
+    else
+       if (object%ovm) then
+          allocate (omega_ovm_writer_t :: object%writer)
+       else
+          allocate (omega_omega_writer_t :: object%writer)
+       end if
+    end if
+    select type (writer => object%writer)
+    class is (omega_writer_t)
+       call writer%init (model_name, prt_in, prt_out, &
+            ufo_path, restrictions, cms_scheme, &
+            openmp_support, report_progress, write_phs_output, extra_options, diags, diags_color)
     end select
   end subroutine omega_def_init
 
@@ -261,9 +258,7 @@ contains
     class(omega_def_t), intent(in) :: object
     integer, intent(in) :: unit
     select type (writer => object%writer)
-    type is (omega_omega_writer_t)
-       call writer%write (unit)
-    type is (omega_ovm_writer_t)
+    class is (omega_writer_t)
        call writer%write (unit)
     end select
   end subroutine omega_def_write
@@ -334,6 +329,11 @@ contains
     string = "omega"
   end function omega_omega_writer_type_name
 
+  function omega_ufo_writer_type_name () result (string)
+    type(string_t) :: string
+    string = "omega/UFO"
+  end function omega_ufo_writer_type_name
+
   function omega_ovm_writer_type_name () result (string)
     type(string_t) :: string
     string = "ovm"
@@ -358,6 +358,7 @@ contains
          '"' // char (object%restrictions) // '"'
     write (unit, "(5x,A,L1)")  "OpenMP support    = ", object%openmp_support
     write (unit, "(5x,A,L1)")  "Report progress   = ", object%report_progress
+    ! write (unit, "(5x,A,L1)")  "Write phs output  = ", object%write_phs_output
     write (unit, "(5x,A,A)")  "Extra options     = ", &
          '"' // char (object%extra_options) // '"'
     write (unit, "(5x,A,L1)")  "Write diagrams    = ", object%diags
@@ -367,35 +368,46 @@ contains
   end subroutine omega_writer_write
 
   subroutine omega_writer_init (writer, model_name, prt_in, prt_out, &
-       restrictions, cms_scheme, openmp_support, report_progress, &
-       extra_options, diags, diags_color)
+       ufo_path, restrictions, cms_scheme, &
+       openmp_support, report_progress, write_phs_output, extra_options, diags, diags_color)
     class(omega_writer_t), intent(out) :: writer
     type(string_t), intent(in) :: model_name
     type(string_t), dimension(:), intent(in) :: prt_in
     type(string_t), dimension(:), intent(in) :: prt_out
-    logical, intent(in), optional :: cms_scheme
+    type(string_t), intent(in), optional :: ufo_path
     type(string_t), intent(in), optional :: restrictions
+    logical, intent(in), optional :: cms_scheme
     logical, intent(in), optional :: openmp_support
     logical, intent(in), optional :: report_progress
-    logical, intent(in), optional :: diags, diags_color
+    logical, intent(in), optional :: write_phs_output
     type(string_t), intent(in), optional :: extra_options
+    logical, intent(in), optional :: diags, diags_color
     integer :: i
     writer%model_name = model_name
+    select type (writer)
+    type is (omega_ufo_writer_t)
+       if (present (ufo_path)) then
+          writer%ufo_path = ufo_path
+       else
+          call msg_fatal ("O'Mega: UFO model option is selected, but UFO model path is unset")
+       end if
+    end select
     if (present (restrictions)) then
        writer%restrictions = restrictions
     else
        writer%restrictions = ""
     end if
+    if (present (cms_scheme))  writer%complex_mass_scheme = cms_scheme
     if (present (openmp_support))  writer%openmp_support = openmp_support
     if (present (report_progress))  writer%report_progress = report_progress
-    if (present (cms_scheme))  writer%complex_mass_scheme = cms_scheme
-    if (present (diags))  writer%diags = diags
-    if (present (diags_color))  writer%diags_color = diags_color
+    if (present (write_phs_output)) writer%write_phs_output = write_phs_output
     if (present (extra_options)) then
        writer%extra_options = " " // extra_options
     else
        writer%extra_options = ""
     end if
+    if (present (diags))  writer%diags = diags
+    if (present (diags_color))  writer%diags_color = diags_color
     select case (size (prt_in))
     case (1);  writer%process_mode = " -decay"
     case (2);  writer%process_mode = " -scatter"
@@ -414,11 +426,12 @@ contains
     end associate
   end subroutine omega_writer_init
 
-  subroutine omega_write_makefile_code (writer, unit, id, os_data, testflag)
+  subroutine omega_write_makefile_code (writer, unit, id, os_data, verbose, testflag)
     class(omega_writer_t), intent(in) :: writer
     integer, intent(in) :: unit
     type(string_t), intent(in) :: id
     type(os_data_t), intent(in) :: os_data
+    logical, intent(in) :: verbose
     logical, intent(in), optional :: testflag
     type(string_t) :: omega_binary, omega_path
     type(string_t) :: restrictions_string
@@ -427,15 +440,19 @@ contains
     type(string_t) :: progress_string
     type(string_t) :: diagrams_string
     type(string_t) :: cms_string
+    type(string_t) :: write_phs_output_string
+    type(string_t) :: parameter_module
     logical :: escape_hyperref
     escape_hyperref = .false.
     if (present (testflag))  escape_hyperref = testflag
     select type (writer)
     type is (omega_omega_writer_t)
        omega_binary = "omega_" // writer%model_name // ".opt"
+    type is (omega_ufo_writer_t)
+       omega_binary = "omega_UFO.opt"
     type is (omega_ovm_writer_t)
        select case (char (writer%model_name))
-       case ("SM", "SM_CKM", "SM_Higgs", "2HDM", "2HDM_CKM", &
+       case ("SM", "SM_CKM", "SM_Higgs", "THDM", "THDM_CKM", &
              "HSExt", "QED", "QCD", "Zprime")
        case default
           call msg_fatal ("The model " // char (writer%model_name) &
@@ -444,6 +461,7 @@ contains
        omega_binary = "omega_" // writer%model_name // "_VM.opt"
     end select
     omega_path = os_data%whizard_omega_binpath // "/" // omega_binary
+    if (.not. verbose)  omega_path = "@" // omega_path
     if (writer%restrictions /= "") then
        restrictions_string = " -cascade '" // writer%restrictions // "'"
     else
@@ -480,6 +498,11 @@ contains
     else
        cms_string = ""
     end if
+    if (writer%write_phs_output) then
+       write_phs_output_string = " -phase_space " // char (id) // ".fds"
+    else
+       write_phs_output_string = ""
+    endif
     select case (char (writer%model_name))
     case ("SM_rx", "SSC", "NoH_rx", "AltH")
        kmatrix_string = " -target:kmatrix_2_write"
@@ -505,6 +528,9 @@ contains
     select type (writer)
     type is (omega_omega_writer_t)
        write (unit, "(5A)")  char (id), ".f90:"
+       if (.not. verbose) then
+          write (unit, "(5A)")  TAB // '@echo  "  OMEGA     ', trim (char (id)), '.f90"'
+       end if
        write (unit, "(99A)")  TAB, char (omega_path), &
             " -o ", char (id), ".f90", &
             " -target:whizard", &
@@ -517,7 +543,44 @@ contains
             char (kmatrix_string), &
             char (writer%process_mode), char (writer%process_string), &
             char (restrictions_string), char (diagrams_string), &
-            char (writer%extra_options)
+            char (writer%extra_options), char (write_phs_output_string)
+    type is (omega_ufo_writer_t)
+       parameter_module = char (id) // "_par_" // char (writer%model_name)
+       write (unit, "(5A)")  char (id), ".f90: ", char (parameter_module), ".lo"
+       if (.not. verbose) then
+          write (unit, "(5A)")  TAB // '@echo  "  OMEGA[UFO]', trim (char (id)), '.f90"'
+       end if
+       write (unit, "(99A)")  TAB, char (omega_path), &
+            " -o ", char (id), ".f90", &
+            " -model:UFO_dir ", &
+            char (writer%ufo_path), "/", char (writer%model_name), &
+            " -model:exec", &
+            " -target:whizard", &
+            " -target:parameter_module ", char (parameter_module), &
+            " -target:module opr_", char (id), &
+            " -target:md5sum '", writer%md5sum, "'", &
+            char (cms_string), &
+            char (openmp_string), &
+            char (progress_string), &
+            char (kmatrix_string), &
+            char (writer%process_mode), char (writer%process_string), &
+            char (restrictions_string), char (diagrams_string), &
+            char (writer%extra_options), char (write_phs_output_string)
+       write (unit, "(5A)") "SOURCES += ", char (parameter_module), ".f90"
+       write (unit, "(5A)") "OBJECTS += ", char (parameter_module), ".lo"
+       write (unit, "(5A)")  char (parameter_module), ".f90:"
+       write (unit, "(99A)")  TAB, char (omega_path), &
+            " -model:UFO_dir ", &
+            char (writer%ufo_path), "/", char (writer%model_name), &
+            " -model:exec", &
+            " -target:parameter_module ", char (parameter_module), &
+            " -params", &
+            " -o $@"
+       write (unit, "(5A)")  char (parameter_module), ".lo: ", char (parameter_module), ".f90"
+       if (.not. verbose) then
+          write (unit, "(5A)")  TAB // '@echo  "  FC       " $@'
+       end if
+       write (unit, "(5A)")  TAB, "$(LTFCOMPILE) $<"
     type is (omega_ovm_writer_t)
        write (unit, "(5A)")  char (id), ".hbc:"
        write (unit, "(99A)")  TAB, char (omega_path), &
@@ -526,8 +589,11 @@ contains
             char (cms_string), &
             char (writer%process_mode), char (writer%process_string), &
             char (restrictions_string), char (diagrams_string), &
-            char (writer%extra_options)
+            char (writer%extra_options), char (write_phs_output_string)
        write (unit, "(5A)")  char (id), ".f90:"
+       if (.not. verbose) then
+          write (unit, "(5A)")  TAB // '@echo  "  OVM       ', trim (char (id)), '.f90"'
+       end if
        write (unit, "(99A)")  TAB, char (omega_path), &
             " -o ", char (id), ".f90 -params", &
             " -target:whizard ", &
@@ -541,11 +607,24 @@ contains
     if (writer%diags .or. writer%diags_color) &
        write (unit, "(5A)")  char (id), "_diags.tex: ", char (id), ".f90"
     write (unit, "(5A)")  "clean-", char (id), ":"
-    write (unit, "(5A)")  TAB, "rm -f ", char (id), ".f90"
-    write (unit, "(5A)")  TAB, "rm -f opr_", char (id), ".mod"
-    write (unit, "(5A)")  TAB, "rm -f ", char (id), ".lo"
+    if (verbose) then
+       write (unit, "(5A)")  TAB, "rm -f ", char (id), ".f90"
+       write (unit, "(5A)")  TAB, "rm -f opr_", char (id), ".mod"
+       write (unit, "(5A)")  TAB, "rm -f ", char (id), ".lo"
+    else
+       write (unit, "(5A)")  TAB // '@echo  "  RM        ', &
+            trim (char (id)), '.f90,.mod,.lo"'
+       write (unit, "(5A)")  TAB, "@rm -f ", char (id), ".f90"
+       write (unit, "(5A)")  TAB, "@rm -f opr_", char (id), ".mod"
+       write (unit, "(5A)")  TAB, "@rm -f ", char (id), ".lo"
+    end if
     write (unit, "(5A)")  "CLEAN_SOURCES += ", char (id), ".f90"
     select type (writer)
+    type is (omega_ufo_writer_t)
+       write (unit, "(5A)")  "CLEAN_SOURCES += ", char (writer%model_name), ".mdl"
+       write (unit, "(5A)")  "CLEAN_SOURCES += ", char (parameter_module), ".f90"
+       write (unit, "(5A)")  "CLEAN_OBJECTS += ", char (parameter_module), ".mod"
+       write (unit, "(5A)")  "CLEAN_OBJECTS += ", char (parameter_module), ".lo"
     type is (omega_ovm_writer_t)
        write (unit, "(5A)")  "CLEAN_SOURCES += ", char (id), ".hbc"
     end select
@@ -583,24 +662,59 @@ contains
              write (unit, "(5A)")  char (id), "_diags.ps: ", char (id), "_diags.tex"
           end if
           if (escape_hyperref) then
-             write (unit, "(5A)")  TAB, "-cat ", char (id), "_diags.tex | \"
+             if (verbose) then
+                write (unit, "(5A)")  TAB, "-cat ", char (id), "_diags.tex | \"
+             else
+                write (unit, "(5A)")  TAB // '@echo  "  HYPERREF  ', &
+                     trim (char (id)) // '_diags.tex"'
+                write (unit, "(5A)")  TAB, "@cat ", char (id), "_diags.tex | \"
+             end if
              write (unit, "(5A)")  TAB, "   sed -e" // &
-                "'s/\\usepackage\[colorlinks\]{hyperref}.*/%\\usepackage" // &
-                "\[colorlinks\]{hyperref}/' > \"
+                  "'s/\\usepackage\[colorlinks\]{hyperref}.*/%\\usepackage" // &
+                  "\[colorlinks\]{hyperref}/' > \"
              write (unit, "(5A)")  TAB, "   ", char (id), "_diags.tex.tmp"
-             write (unit, "(5A)")  TAB, "mv -f ", char (id), "_diags.tex.tmp \"
+             if (verbose) then
+                write (unit, "(5A)")  TAB, "mv -f ", char (id), "_diags.tex.tmp \"
+             else
+                write (unit, "(5A)")  TAB, "@mv -f ", char (id), "_diags.tex.tmp \"
+             end if
              write (unit, "(5A)")  TAB, "   ", char (id), "_diags.tex"
           end if
-          write (unit, "(5A)")  TAB, "-TEXINPUTS=$(TEX_FLAGS) $(LATEX) " // &
-               char (id) // "_diags.tex"
-          write (unit, "(5A)")  TAB, "MPINPUTS=$(MP_FLAGS) $(MPOST) " // &
-               char (id) // "_diags-fmf.mp"
-          write (unit, "(5A)")  TAB, "TEXINPUTS=$(TEX_FLAGS) $(LATEX) " // &
-               char (id) // "_diags.tex"
-          write (unit, "(5A)")  TAB, "$(DVIPS) -o " // char (id) // "_diags.ps " // &
-               char (id) // "_diags.dvi"
+          if (verbose) then
+             write (unit, "(5A)")  TAB, "-TEXINPUTS=$(TEX_FLAGS) $(LATEX) " // &
+                  char (id) // "_diags.tex"
+             write (unit, "(5A)")  TAB, "MPINPUTS=$(MP_FLAGS) $(MPOST) " // &
+                  char (id) // "_diags-fmf.mp"
+             write (unit, "(5A)")  TAB, "TEXINPUTS=$(TEX_FLAGS) $(LATEX) " // &
+                  char (id) // "_diags.tex"
+             write (unit, "(5A)")  TAB, "$(DVIPS) -o " // char (id) // "_diags.ps " // &
+                  char (id) // "_diags.dvi"
+          else
+             write (unit, "(5A)")  TAB // '@echo  "  LATEX     ', &
+                  trim (char (id)) // '_diags.tex"'
+             write (unit, "(5A)")  TAB, "@TEXINPUTS=$(TEX_FLAGS) $(LATEX) " // &
+                  char (id) // "_diags.tex > /dev/null"
+             write (unit, "(5A)")  TAB // '@echo  "  METAPOST  ', &
+                  trim (char (id)) // '_diags-fmf.mp"'
+             write (unit, "(5A)")  TAB, "@MPINPUTS=$(MP_FLAGS) $(MPOST) " // &
+                  char (id) // "_diags-fmf.mp  > /dev/null"
+             write (unit, "(5A)")  TAB // '@echo  "  LATEX     ', &
+                  trim (char (id)) // '_diags.tex"'
+             write (unit, "(5A)")  TAB, "@TEXINPUTS=$(TEX_FLAGS) $(LATEX) " // &
+                  char (id) // "_diags.tex > /dev/null"
+             write (unit, "(5A)")  TAB // '@echo  "  DVIPS     ', &
+                  trim (char (id)) // '_diags.dvi"'
+             write (unit, "(5A)")  TAB, "@$(DVIPS) -q -o " // char (id) &
+                  // "_diags.ps " // char (id) // "_diags.dvi"
+          end if
           if (os_data%event_analysis_pdf) then
-             write (unit, "(5A)")  TAB, "$(PS2PDF) " // char (id) // "_diags.ps"
+             if (verbose) then
+                write (unit, "(5A)")  TAB, "$(PS2PDF) " // char (id) // "_diags.ps"
+             else
+                write (unit, "(5A)")  TAB // '@echo  "  PS2PDF    ', &
+                     trim (char (id)) // '_diags.ps"'
+                write (unit, "(5A)")  TAB, "@$(PS2PDF) " // char (id) // "_diags.ps"
+             end if
           end if
        end if
     end if
@@ -611,6 +725,16 @@ contains
     type(string_t), intent(in) :: id
   end subroutine omega_write_source_code
 
+  subroutine omega_before_compile (writer, id)
+    class(omega_writer_t), intent(in) :: writer
+    type(string_t), intent(in) :: id
+  end subroutine omega_before_compile
+  
+  subroutine omega_after_compile (writer, id)
+    class(omega_writer_t), intent(in) :: writer
+    type(string_t), intent(in) :: id
+  end subroutine omega_after_compile
+  
   function omega_writer_get_procname (feature) result (name)
     type(string_t) :: name
     type(string_t), intent(in) :: feature
@@ -773,26 +897,33 @@ contains
   end function omega_driver_type_name
 
   subroutine omega_make_process_component (entry, component_index, &
-         model_name, prt_in, prt_out, restrictions, openmp_support, &
-         cms_scheme, report_progress, extra_options, diags, diags_color)
+         model_name, prt_in, prt_out, &
+         ufo, ufo_path, restrictions, cms_scheme, &
+         openmp_support, report_progress, write_omega_output, extra_options, diags, diags_color)
     class(process_def_entry_t), intent(inout) :: entry
     integer, intent(in) :: component_index
     type(string_t), intent(in) :: model_name
     type(string_t), dimension(:), intent(in) :: prt_in
     type(string_t), dimension(:), intent(in) :: prt_out
-    logical, intent(in), optional :: cms_scheme
+    logical, intent(in), optional :: ufo
+    type(string_t), intent(in), optional :: ufo_path
     type(string_t), intent(in), optional :: restrictions
+    logical, intent(in), optional :: cms_scheme
     logical, intent(in), optional :: openmp_support
     logical, intent(in), optional :: report_progress
-    logical, intent(in), optional :: diags, diags_color
+    logical, intent(in), optional :: write_omega_output
     type(string_t), intent(in), optional :: extra_options
+    logical, intent(in), optional :: diags, diags_color
+    logical :: ufo_model
     class(prc_core_def_t), allocatable :: def
-    allocate (omega_omega_def_t :: def)
+    ufo_model = .false.;  if (present (ufo))  ufo_model = ufo    
+    allocate (omega_def_t :: def)
     select type (def)
-    type is (omega_omega_def_t)
+    class is (omega_def_t)
        call def%init (model_name, prt_in, prt_out, &
-            restrictions, cms_scheme, openmp_support, &
-            report_progress, extra_options, diags, diags_color)
+            .false., ufo_model, ufo_path, &
+            restrictions, cms_scheme, &
+            openmp_support, report_progress, write_omega_output, extra_options, diags, diags_color)
     end select
     call entry%process_def_t%import_component (component_index, &
          n_out = size (prt_out), &
