@@ -1,19 +1,12 @@
-! WHIZARD 2.4.1 Mar 24 2017
+! WHIZARD 2.6.4 Aug 23 2018
 !
-! Copyright (C) 1999-2017 by
+! Copyright (C) 1999-2018 by
 !     Wolfgang Kilian <kilian@physik.uni-siegen.de>
 !     Thorsten Ohl <ohl@physik.uni-wuerzburg.de>
 !     Juergen Reuter <juergen.reuter@desy.de>
 !
 !     with contributions from
-!     Fabian Bach <fabian.bach@t-online.de>
-!     Bijan Chokoufe <bijan.chokoufe@desy.de>
-!     Christian Speckner <cnspeckn@googlemail.com>
-!     So Young Shim <soyoung.shim@desy.de>
-!     Florian Staub <florian.staub@cern.ch>
-!     Christian Weiss <christian.weiss@desy.de>
-!     and Hans-Werner Boschmann, Felix Braam,
-!     Sebastian Schmidt, So-young Shim, Daniel Wiesler
+!     cf. main AUTHORS file
 !
 ! WHIZARD is free software; you can redistribute it and/or modify it
 ! under the terms of the GNU General Public License as published by
@@ -117,7 +110,7 @@ module prc_user_defined
     procedure :: get_flv_state => prc_user_defined_base_get_flv_state
     procedure :: compute_sqme => prc_user_defined_base_compute_sqme
     procedure :: compute_sqme_virt => prc_user_defined_base_compute_sqme_virt
-    procedure :: compute_sqme_cc => prc_user_defined_base_compute_sqme_cc
+    procedure :: compute_sqme_color_c => prc_user_defined_base_compute_sqme_color_c
     procedure :: compute_alpha_s => prc_user_defined_base_compute_alpha_s
     procedure :: get_alpha_s => prc_user_defined_base_get_alpha_s
     procedure :: is_allowed => prc_user_defined_base_is_allowed
@@ -130,11 +123,10 @@ module prc_user_defined
     procedure :: init_sf_handler_dummy => prc_user_defined_base_init_sf_handler_dummy
     procedure :: apply_structure_functions => prc_user_defined_base_apply_structure_functions
     procedure :: get_sf_value => prc_user_defined_base_get_sf_value
-    procedure :: get_helicity_list_base => prc_user_defined_base_get_helicity_list
     procedure(prc_user_defined_base_includes_polarization), deferred :: &
       includes_polarization
-    procedure(prc_user_defined_base_create_and_load_extra_libraries), deferred :: &
-      create_and_load_extra_libraries
+    procedure(prc_user_defined_base_create_and_load_extra_libraries), &
+         deferred :: create_and_load_extra_libraries
   end type prc_user_defined_base_t
 
   type, abstract, extends (prc_core_def_t) :: user_defined_def_t
@@ -152,6 +144,8 @@ module prc_user_defined
     type(string_t) :: process_mode
     type(string_t) :: process_string
     type(string_t) :: restrictions
+    integer :: n_in = 0
+    integer :: n_out = 0
     logical :: active = .true.
   contains
     procedure :: init => user_defined_writer_init
@@ -160,6 +154,8 @@ module prc_user_defined
     procedure :: write_wrapper => prc_user_defined_writer_write_wrapper
     procedure :: write_interface => prc_user_defined_writer_write_interface
     procedure :: write_source_code => prc_user_defined_writer_write_source_code
+    procedure :: before_compile => prc_user_defined_writer_before_compile
+    procedure :: after_compile => prc_user_defined_writer_after_compile
     procedure :: write_makefile_code => prc_user_defined_writer_write_makefile_code
     procedure :: base_write_makefile_code => prc_user_defined_writer_write_makefile_code
     procedure, nopass:: get_procname => prc_user_defined_writer_writer_get_procname
@@ -210,8 +206,8 @@ module prc_user_defined
   end interface
 
   abstract interface
-    subroutine prc_user_defined_base_create_and_load_extra_libraries ( &
-         core, flv_states, var_list, os_data, libname, model, i_core)
+    subroutine prc_user_defined_base_create_and_load_extra_libraries &
+         (core, flv_states, var_list, os_data, libname, model, i_core, is_nlo)
       import
       class(prc_user_defined_base_t), intent(inout) :: core
       integer, intent(in), dimension(:,:), allocatable :: flv_states
@@ -220,6 +216,7 @@ module prc_user_defined
       type(string_t), intent(in) :: libname
       type(model_data_t), intent(in), target :: model
       integer, intent(in) :: i_core
+      logical, intent(in) :: is_nlo
     end subroutine prc_user_defined_base_create_and_load_extra_libraries
   end interface
 
@@ -337,10 +334,10 @@ contains
     flv = object%data%flv_state (:,i_flv)
   end function prc_user_defined_base_get_flv_state
 
-  subroutine prc_user_defined_base_compute_sqme (object, i_flv, p, &
+  subroutine prc_user_defined_base_compute_sqme (object, i_flv, i_hel, p, &
          ren_scale, sqme, bad_point)
      class(prc_user_defined_base_t), intent(in) :: object
-     integer, intent(in) :: i_flv
+     integer, intent(in) :: i_flv, i_hel
      type(vector4_t), dimension(:), intent(in) :: p
      real(default), intent(in) :: ren_scale
      real(default), intent(out) :: sqme
@@ -349,10 +346,10 @@ contains
      bad_point = .false.
   end subroutine prc_user_defined_base_compute_sqme
 
-  subroutine prc_user_defined_base_compute_sqme_virt (object, i_flv, &
+  subroutine prc_user_defined_base_compute_sqme_virt (object, i_flv, i_hel, &
      p, ren_scale, sqme, bad_point)
     class(prc_user_defined_base_t), intent(in) :: object
-    integer, intent(in) :: i_flv
+    integer, intent(in) :: i_flv, i_hel
     type(vector4_t), dimension(:), intent(in) :: p
     real(default), intent(in) :: ren_scale
     logical, intent(out) :: bad_point
@@ -365,37 +362,38 @@ contains
     bad_point = .false.
   end subroutine prc_user_defined_base_compute_sqme_virt
 
-  subroutine prc_user_defined_base_compute_sqme_cc (object, i_flv, p, &
-     ren_scale, born_cc, bad_point, born_out)
+  subroutine prc_user_defined_base_compute_sqme_color_c (object, i_flv, i_hel, p, &
+     ren_scale, born_color_c, bad_point, born_out)
     class(prc_user_defined_base_t), intent(inout) :: object
-    integer, intent(in) :: i_flv
+    integer, intent(in) :: i_flv, i_hel
     type(vector4_t), intent(in), dimension(:) :: p
     real(default), intent(in) :: ren_scale
-    real(default), intent(inout), dimension(:,:) :: born_cc
+    real(default), intent(inout), dimension(:,:) :: born_color_c
     logical, intent(out) :: bad_point
     real(default), intent(out), optional :: born_out
-    call msg_debug2 (D_ME_METHODS, "prc_user_defined_base_compute_sqme_cc")
+    call msg_debug2 (D_ME_METHODS, "prc_user_defined_base_compute_sqme_color_c")
     if (size (p) == 4) then
        if (present (born_out)) then
           born_out = 0.0015_default
-          born_cc = zero
-          born_cc(3,3) = - CF * born_out
-          born_cc(4,4) = - CF * born_out
-          born_cc(3,4) = CF * born_out
-          born_cc(4,3) = born_cc(3,4)
+          born_color_c = zero
+          born_color_c(3,3) = - CF * born_out
+          born_color_c(4,4) = - CF * born_out
+          born_color_c(3,4) = CF * born_out
+          born_color_c(4,3) = born_color_c(3,4)
           bad_point = .false.
        end if
     else
        if (present (born_out)) born_out = zero
-       born_cc = zero
+       born_color_c = zero
     end if
-  end subroutine prc_user_defined_base_compute_sqme_cc
+  end subroutine prc_user_defined_base_compute_sqme_color_c
 
-  subroutine prc_user_defined_base_compute_alpha_s (object, core_state, fac_scale)
+  subroutine prc_user_defined_base_compute_alpha_s &
+       (object, core_state, ren_scale)
     class(prc_user_defined_base_t), intent(in) :: object
     class(user_defined_state_t), intent(inout) :: core_state
-    real(default), intent(in) :: fac_scale
-    core_state%alpha_qcd = object%qcd%alpha%get (fac_scale)
+    real(default), intent(in) :: ren_scale
+    core_state%alpha_qcd = object%qcd%alpha%get (ren_scale)
   end subroutine prc_user_defined_base_compute_alpha_s
 
   function prc_user_defined_base_get_alpha_s (object, core_state) result (alpha)
@@ -522,13 +520,6 @@ contains
     val = core%sf_handler%val
   end function prc_user_defined_base_get_sf_value
 
-  function prc_user_defined_base_get_helicity_list (object, i) result (i_out)
-    integer :: i_out
-    class(prc_user_defined_base_t), intent(in) :: object
-    integer, intent(in) :: i
-    i_out = i
-  end function prc_user_defined_base_get_helicity_list
-
   subroutine user_defined_def_set_active_writer (def, active)
     class(user_defined_def_t), intent(inout) :: def
     logical, intent(in) :: active
@@ -587,6 +578,8 @@ contains
     else
        writer%restrictions = ""
     end if
+    writer%n_in = size (prt_in)
+    writer%n_out = size (prt_out)
     select case (size (prt_in))
        case(1); writer%process_mode = " -decay"
        case(2); writer%process_mode = " -scatter"
@@ -752,19 +745,40 @@ contains
   subroutine prc_user_defined_writer_write_source_code (writer, id)
     class(prc_user_defined_writer_t), intent(in) :: writer
     type(string_t), intent(in) :: id
+    call msg_debug (D_ME_METHODS, &
+         "prc_user_defined_writer_write_source_code (no-op)")
     !!! This is a dummy
   end subroutine prc_user_defined_writer_write_source_code
 
-  subroutine prc_user_defined_writer_write_makefile_code (writer, unit, id, os_data, testflag)
+  subroutine prc_user_defined_writer_before_compile (writer, id)
+    class(prc_user_defined_writer_t), intent(in) :: writer
+    type(string_t), intent(in) :: id
+    call msg_debug (D_ME_METHODS, &
+         "prc_user_defined_writer_before_compile (no-op)")
+    !!! This is a dummy
+  end subroutine prc_user_defined_writer_before_compile
+  
+  subroutine prc_user_defined_writer_after_compile (writer, id)
+    class(prc_user_defined_writer_t), intent(in) :: writer
+    type(string_t), intent(in) :: id
+    call msg_debug (D_ME_METHODS, &
+         "prc_user_defined_writer_after_compile (no-op)")
+    !!! This is a dummy
+  end subroutine prc_user_defined_writer_after_compile
+  
+  subroutine prc_user_defined_writer_write_makefile_code &
+       (writer, unit, id, os_data, verbose, testflag)
     class(prc_user_defined_writer_t), intent(in) :: writer
     integer, intent(in) :: unit
     type(string_t), intent(in) :: id
     type(os_data_t), intent(in) :: os_data
+    logical, intent(in) :: verbose
     logical, intent(in), optional :: testflag
     type(string_t) :: omega_binary, omega_path
     type(string_t) :: restrictions_string
     omega_binary = "omega_" // writer%model_name // ".opt"
     omega_path = os_data%whizard_omega_binpath // "/" // omega_binary
+    if (.not. verbose)  omega_path = "@" // omega_path
     if (writer%restrictions /= "") then
        restrictions_string = " -cascade '" // writer%restrictions // "'"
     else
@@ -772,6 +786,9 @@ contains
     end if
     write (unit, "(5A)")  "OBJECTS += ", char (id), ".lo"
     write (unit, "(5A)")  char (id), ".f90:"
+    if (.not. verbose) then
+       write (unit, "(5A)")  TAB // '@echo  "  OMEGA     ', trim (char (id)), '.f90"'
+    end if
     write (unit, "(99A)")  TAB, char (omega_path), &
          " -o ", char (id), ".f90", &
          " -target:whizard", &
@@ -788,6 +805,9 @@ contains
     write (unit, "(5A)")  "CLEAN_OBJECTS += opr_", char (id), ".mod"
     write (unit, "(5A)")  "CLEAN_OBJECTS += ", char (id), ".lo"
     write (unit, "(5A)")  char (id), ".lo: ", char (id), ".f90"
+    if (.not. verbose) then
+       write (unit, "(5A)")  TAB // '@echo  "  FC       " $@'
+    end if    
     write (unit, "(5A)")  TAB, "$(LTFCOMPILE) $<"
 
   end subroutine prc_user_defined_writer_write_makefile_code
@@ -905,8 +925,8 @@ contains
     polarized = .false.
   end function prc_user_defined_test_includes_polarization
 
-  subroutine prc_user_defined_test_create_and_load_extra_libraries ( &
-         core, flv_states, var_list, os_data, libname, model, i_core)
+  subroutine prc_user_defined_test_create_and_load_extra_libraries &
+       (core, flv_states, var_list, os_data, libname, model, i_core, is_nlo)
     class(prc_user_defined_test_t), intent(inout) :: core
     integer, intent(in), dimension(:,:), allocatable :: flv_states
     type(var_list_t), intent(in) :: var_list
@@ -914,6 +934,7 @@ contains
     type(string_t), intent(in) :: libname
     type(model_data_t), intent(in), target :: model
     integer, intent(in) :: i_core
+    logical, intent(in) :: is_nlo
   end subroutine prc_user_defined_test_create_and_load_extra_libraries
 
 
