@@ -1,11 +1,11 @@
-! $Id: parameters.SM_higgs.f90,v 1.4 2006/06/16 13:31:48 kilian Exp $
+! parameters.SM_higgs.f90
 !
-! Copyright (C) 1999-2017 by 
+! Copyright (C) 1999-2018 by 
 !     Wolfgang Kilian <kilian@physik.uni-siegen.de>
 !     Thorsten Ohl <ohl@physik.uni-wuerzburg.de>
 !     Juergen Reuter <juergen.reuter@desy.de>
 !     with contributions from
-!     Christian Speckner <cnspeckn@googlemail.com>
+!     cf. main AUTHORS file
 !
 ! WHIZARD is free software; you can redistribute it and/or modify it
 ! under the terms of the GNU General Public License as published by 
@@ -42,7 +42,7 @@ module parameters_sm_higgs
   complex(default), public :: qlep, qup, qdwn, gcc, qw, &
        gzww, gwww, ghww, ghhww, ghzz, ghhzz, &
        ghbb, ghtt, ghcc, ghtautau, gh3, gh4, &
-       ghgaga, ghgaz, ghgg, ghmm, & 
+       ghgaga, ghgaz, ghgg, ghmm, ghee, &
        iqw, igzww, igwww, gw4, gzzww, gazww, gaaww
   real(default), public :: vev
   complex(default), dimension(2), public :: &
@@ -183,6 +183,7 @@ contains
     ghcc = - mass(4) / vev
     ghtautau = - mass(15) / vev
     ghmm = - mass(13) / vev
+    ghee = - mass(11) / vev
     gh3 = - 3 * mass(25)**2 / vev
     gh4 = - 3 * mass(25)**2 / vev**2
     !!! Color flow basis, divide by sqrt(2)
