@@ -1,19 +1,12 @@
-! WHIZARD 2.4.1 Mar 24 2017
+! WHIZARD 2.6.4 Aug 23 2018
 !
-! Copyright (C) 1999-2017 by
+! Copyright (C) 1999-2018 by
 !     Wolfgang Kilian <kilian@physik.uni-siegen.de>
 !     Thorsten Ohl <ohl@physik.uni-wuerzburg.de>
 !     Juergen Reuter <juergen.reuter@desy.de>
 !
 !     with contributions from
-!     Fabian Bach <fabian.bach@t-online.de>
-!     Bijan Chokoufe <bijan.chokoufe@desy.de>
-!     Christian Speckner <cnspeckn@googlemail.com>
-!     So Young Shim <soyoung.shim@desy.de>
-!     Florian Staub <florian.staub@cern.ch>
-!     Christian Weiss <christian.weiss@desy.de>
-!     and Hans-Werner Boschmann, Felix Braam,
-!     Sebastian Schmidt, So-young Shim, Daniel Wiesler
+!     cf. main AUTHORS file
 !
 ! WHIZARD is free software; you can redistribute it and/or modify it
 ! under the terms of the GNU General Public License as published by
@@ -72,6 +65,7 @@ module cascades
   public :: cascade_set_write_graph_format
   public :: cascade_set_write
   public :: cascade_set_generate
+  public :: phase_space_vanishes
   public :: assignment(=)
   public :: cascade_set_get_n_trees
   public :: cascade_set_get_resonance_histories
@@ -220,7 +214,7 @@ contains
     integer, intent(in), optional :: unit
     type(flavor_t) :: flv
     integer :: u, i
-2   format(3x,A,1x,I3,1x,A,1x,I7,1x,'!',1x,A)
+2   format(3x,A,1x,I3,1x,A,1x,I9,1x,'!',1x,A)
     u = given_output_unit (unit);  if (u < 0)  return
     call write_reduced (cascade%tree, u)
     write (u, "(A)")
@@ -533,11 +527,11 @@ contains
     type(cascade_t), intent(inout) :: cascade
     integer(i8), dimension(1) :: mold
     cascade%res_hash = hash (transfer &
-         (concat (sort (pack (cascade%tree_pdg, &
-                              cascade%tree_resonant)), &
-                  sort (pack (abs (cascade%tree_pdg), &
-                              cascade%tree_mapping == T_CHANNEL .or. &
-                              cascade%tree_mapping == U_CHANNEL))), &
+         ([sort (pack (cascade%tree_pdg, &
+                 cascade%tree_resonant)), &
+           sort (pack (abs (cascade%tree_pdg), &
+                 cascade%tree_mapping == T_CHANNEL .or. &
+                 cascade%tree_mapping == U_CHANNEL))], &
           mold))
   end subroutine cascade_assign_resonance_hash
 
@@ -826,7 +820,6 @@ contains
           fmt_head = fmt_head // ",1x,'  '"
        end if
     end do
-    !!! !!! !!! Workaround for standard-semantics ifort 16.0 bug
     do i = 1, n_tot
        fmt_proc(i) = fmt_proc(i) // ")"
     end do
@@ -2050,13 +2043,7 @@ contains
     n_prt = size (flv, 1)
     n_flv = size (flv, 2)
     allocate (mass (n_prt, n_flv), mass_in (n_flv), mass_out (n_flv))
-    !!! !!! !!! Workaround for ifort 16.0 standard-semantics bug
-    !!! mass = flv%get_mass ()
-    do i = 1, n_prt
-       do j = 1, n_flv
-          mass(i,j) = flv(i,j)%get_mass ()
-       end do
-    end do
+    mass = flv%get_mass ()
     mass_in = sum (mass(:n_in,:), 1)
     mass_out = sum (mass(n_in+1:,:), 1)
     if (any (mass_in > sqrts)) then
@@ -2146,6 +2133,7 @@ contains
           cascade => cascade%next
        end do
     end do
+    call res_hist_set%freeze ()
     call res_hist_set%to_array (res_hists)
   end subroutine cascade_set_get_resonance_histories
 
