@@ -1,19 +1,12 @@
-! WHIZARD 2.4.1 Mar 24 2017
+! WHIZARD 2.6.4 Aug 23 2018
 !
-! Copyright (C) 1999-2017 by
+! Copyright (C) 1999-2018 by
 !     Wolfgang Kilian <kilian@physik.uni-siegen.de>
 !     Thorsten Ohl <ohl@physik.uni-wuerzburg.de>
 !     Juergen Reuter <juergen.reuter@desy.de>
 !
 !     with contributions from
-!     Fabian Bach <fabian.bach@t-online.de>
-!     Bijan Chokoufe <bijan.chokoufe@desy.de>
-!     Christian Speckner <cnspeckn@googlemail.com>
-!     So Young Shim <soyoung.shim@desy.de>
-!     Florian Staub <florian.staub@cern.ch>
-!     Christian Weiss <christian.weiss@desy.de>
-!     and Hans-Werner Boschmann, Felix Braam,
-!     Sebastian Schmidt, So-young Shim, Daniel Wiesler
+!     cf. main AUTHORS file
 !
 ! WHIZARD is free software; you can redistribute it and/or modify it
 ! under the terms of the GNU General Public License as published by
@@ -157,7 +150,6 @@ module phs_base
      procedure (phs_config_write), deferred :: write
      procedure :: base_write => phs_config_write
      procedure :: init => phs_config_init
-     procedure :: set_component_index => phs_config_set_component_index
      procedure (phs_config_configure), deferred :: configure
      procedure :: set_sf_channel => phs_config_set_sf_channel
      procedure :: collect_channels => phs_config_collect_channels
@@ -244,7 +236,7 @@ module phs_base
   abstract interface
      subroutine phs_config_configure (phs_config, sqrts, &
           sqrts_fixed, cm_frame, azimuthal_dependence, rebuild, ignore_mismatch, &
-          nlo_type)
+          nlo_type, subdir)
        import
        class(phs_config_t), intent(inout) :: phs_config
        real(default), intent(in) :: sqrts
@@ -254,6 +246,7 @@ module phs_base
        logical, intent(in), optional :: rebuild
        logical, intent(in), optional :: ignore_mismatch
        integer, intent(in), optional :: nlo_type
+       type(string_t), intent(in), optional :: subdir
      end subroutine phs_config_configure
   end interface
 
@@ -600,22 +593,6 @@ contains
     phs_config%md5sum_process = data%md5sum
   end subroutine phs_config_init
 
-  subroutine phs_config_set_component_index (phs_config, index)
-    class(phs_config_t), intent(inout) :: phs_config
-    integer, intent(in) :: index
-    type(string_t), dimension(:), allocatable :: id
-    type(string_t) :: suffix
-    integer :: i, n
-    suffix = var_str ('i') // int2string (index)
-    call split_string (phs_config%id, var_str ('_'), id)
-    phs_config%id = var_str ('')
-    n = size (id) - 1
-    do i = 1, n
-       phs_config%id = phs_config%id // id(i) // var_str ('_')
-    end do
-    phs_config%id = phs_config%id // suffix
-  end subroutine phs_config_set_component_index
-
   subroutine phs_config_set_sf_channel (phs_config, sf_channel)
     class(phs_config_t), intent(inout) :: phs_config
     integer, dimension(:), intent(in) :: sf_channel
@@ -805,8 +782,7 @@ contains
     class(phs_t), intent(inout) :: phs
     type(vector4_t), dimension(:), intent(in) :: p
     type(vector4_t) :: p0, p1
-    type(lorentz_transformation_t) :: lt0, lt0_inv, lt_inv
-    !!! !!! !!! Workaround for standard-semantics ifort 16.0 bug
+    type(lorentz_transformation_t) :: lt0
     integer :: i
     phs%p = p
     if (phs%config%cm_frame) then
@@ -825,18 +801,10 @@ contains
        case (1)
           phs%lt_cm_to_lab = lt0
        case (2)
-          !!! !!! !!! Workaround for standard-semantics ifort 16.0 bug
-          lt0_inv = inverse (lt0)
-          p1 = lt0_inv * p(1)
-          !!! p1 = inverse (lt0) * p(1)
+          p1 = inverse (lt0) * p(1)
           phs%lt_cm_to_lab = lt0 * rotation_to_2nd (3, space_part (p1))
        end select
-       !!! !!! !!! Workaround for standard-semantics ifort 16.0 bug
-       lt_inv = inverse (phs%lt_cm_to_lab)
-       do i = 1, size (p)
-          phs%p(i) = lt_inv * p(i)
-       end do
-       !!! phs%p = inverse (phs%lt_cm_to_lab) * p
+       phs%p = inverse (phs%lt_cm_to_lab) * p
     end if
     phs%p_defined = .true.
   end subroutine phs_set_incoming_momenta
@@ -844,18 +812,12 @@ contains
   subroutine phs_set_outgoing_momenta (phs, q)
     class(phs_t), intent(inout) :: phs
     type(vector4_t), dimension(:), intent(in) :: q
-    !!! !!! !!! Workaround for standard-semantics ifort 16.0 bug
-    type(lorentz_transformation_t) :: l_inv
     integer :: i
     if (phs%p_defined) then
        if (phs%config%cm_frame) then
           phs%q = q
        else
-          l_inv = inverse (phs%lt_cm_to_lab)
-          do i = 1, size (q)
-             phs%q(i) = l_inv * q(i)
-          end do
-          !!! phs%q = inverse (phs%lt_cm_to_lab) * q
+          phs%q = inverse (phs%lt_cm_to_lab) * q
        end if
        phs%q_defined = .true.
     end if
