@@ -1,19 +1,12 @@
-! WHIZARD 2.4.1 Mar 24 2017
+! WHIZARD 2.6.4 Aug 23 2018
 !
-! Copyright (C) 1999-2017 by
+! Copyright (C) 1999-2018 by
 !     Wolfgang Kilian <kilian@physik.uni-siegen.de>
 !     Thorsten Ohl <ohl@physik.uni-wuerzburg.de>
 !     Juergen Reuter <juergen.reuter@desy.de>
 !
 !     with contributions from
-!     Fabian Bach <fabian.bach@t-online.de>
-!     Bijan Chokoufe <bijan.chokoufe@desy.de>
-!     Christian Speckner <cnspeckn@googlemail.com>
-!     So Young Shim <soyoung.shim@desy.de>
-!     Florian Staub <florian.staub@cern.ch>
-!     Christian Weiss <christian.weiss@desy.de>
-!     and Hans-Werner Boschmann, Felix Braam,
-!     Sebastian Schmidt, So-young Shim, Daniel Wiesler
+!     cf. main AUTHORS file
 !
 ! WHIZARD is free software; you can redistribute it and/or modify it
 ! under the terms of the GNU General Public License as published by
@@ -57,6 +50,7 @@ module phs_fks_uti
   public :: phs_fks_generator_4
   public :: phs_fks_generator_5
   public :: phs_fks_generator_6
+  public :: phs_fks_generator_7
 
 contains
 
@@ -64,7 +58,7 @@ contains
     integer, intent(in) :: u
     type(phs_fks_generator_t) :: generator
     type(vector4_t), dimension(:), allocatable :: p_born
-    type(phs_point_t) :: p_real
+    type(vector4_t), dimension(:), allocatable :: p_real
     integer :: emitter, i_phs
     real(default) :: x1, x2, x3
     real(default), parameter :: sqrts = 250.0_default
@@ -89,6 +83,7 @@ contains
 
     allocate (generator%isr_kinematics)
     generator%n_in = 2
+    generator%isr_kinematics%isr_mode = SQRTS_FIXED
 
     call generator%set_sqrts_hat (sqrts)
 
@@ -132,9 +127,10 @@ contains
     write (u, "(A)") "Produce real momenta: "
     i_phs = 1; emitter = phs_identifiers(i_phs)%emitter
     write (u, "(A,I1)") "emitter: ", emitter
-    p_real = 5
+
+    allocate (p_real (5))
     call generator%generate_fsr (emitter, i_phs, p_born, p_real)
-    call vector4_write_set (p_real%p, u, testflag = .true., ultra = .true.)
+    call vector4_write_set (p_real, u, testflag = .true., ultra = .true.)
     call write_separator (u)
     write (u, "(A)")
     write (u, "(A)") "* Test output end: phs_fks_generator_1"
@@ -145,7 +141,7 @@ contains
     integer, intent(in) :: u
     type(phs_fks_generator_t) :: generator
     type(vector4_t), dimension(:), allocatable :: p_born
-    type(phs_point_t) :: p_real
+    type(vector4_t), dimension(:), allocatable :: p_real
     integer :: emitter, i_phs
     real(default) :: x1, x2, x3
     real(default), parameter :: sqrts_hadronic = 250.0_default
@@ -175,12 +171,13 @@ contains
 
     allocate (generator%emitters (2))
     allocate (generator%isr_kinematics)
-    generator%n_in = 2
     generator%emitters(1) = 1; generator%emitters(2) = 2
     generator%sqrts = sqrts_hadronic
     generator%isr_kinematics%beam_energy = sqrts_hadronic / two
     call generator%set_sqrts_hat (sqrts_hadronic)
     call generator%set_isr_kinematics (p_born)
+    generator%n_in = 2
+    generator%isr_kinematics%isr_mode = SQRTS_VAR
 
     write (u, "(A)") "* Use four-particle phase space containing: "
     call vector4_write_set (p_born, u, testflag = .true., ultra = .true.)
@@ -223,9 +220,9 @@ contains
     write (u, "(A)") "Produce real momenta: "
     i_phs = 1; emitter = phs_identifiers(i_phs)%emitter
     write (u, "(A,I1)") "emitter: ", emitter
-    p_real = 5
+    allocate (p_real(5))
     call generator%generate_isr (i_phs, p_born, p_real)
-    call vector4_write_set (p_real%p, u, testflag = .true., ultra = .true.)
+    call vector4_write_set (p_real, u, testflag = .true., ultra = .true.)
     call write_separator (u)
     write (u, "(A)")
     write (u, "(A)") "* Test output end: phs_fks_generator_2"
@@ -236,7 +233,7 @@ contains
     integer, intent(in) :: u
     type(phs_fks_generator_t) :: generator
     type(vector4_t), dimension(:), allocatable :: p_born
-    type(phs_point_t) :: p_real
+    type(vector4_t), dimension(:), allocatable :: p_real
     real(default) :: x1, x2, x3
     real(default) :: mB, mW, mT
     integer :: i, emitter, i_phs
@@ -259,6 +256,8 @@ contains
     p_born(3)%p(1:3) = -p_born(2)%p(1:3)
 
     generator%n_in = 1
+    allocate (generator%isr_kinematics)
+    generator%isr_kinematics%isr_mode = SQRTS_FIXED
 
     mB = 4.2_default
     mW = 80.376_default
@@ -313,17 +312,17 @@ contains
     write (u, "(A)") "Produce real momenta via initial-state emission: "
     i_phs = 1; emitter = phs_identifiers(i_phs)%emitter
     write (u, "(A,I1)") "emitter: ", emitter
-    p_real = 4
-    call generator%generate_isr_decay (i_phs, p_born, p_real)
-    call pacify (p_real%p, 1E-6_default)
-    call vector4_write_set (p_real%p, u, testflag = .true., ultra = .true.)
+    allocate (p_real (4))
+    call generator%generate_isr_fixed_beam_energy (i_phs, p_born, p_real)
+    call pacify (p_real, 1E-6_default)
+    call vector4_write_set (p_real, u, testflag = .true., ultra = .true.)
     call write_separator(u)
     write (u, "(A)") "Produce real momenta via final-state emisson: "
     i_phs = 2; emitter = phs_identifiers(i_phs)%emitter
     write (u, "(A,I1)") "emitter: ", emitter
     call generator%generate_fsr (emitter, i_phs, p_born, p_real)
-    call pacify (p_real%p, 1E-6_default)
-    call vector4_write_set (p_real%p, u, testflag = .true., ultra = .true.)
+    call pacify (p_real, 1E-6_default)
+    call vector4_write_set (p_real, u, testflag = .true., ultra = .true.)
     write (u, "(A)")
     write (u, "(A)") "* Test output end: phs_fks_generator_3"
 
@@ -333,7 +332,7 @@ contains
     integer, intent(in) :: u
     type(phs_fks_generator_t) :: generator
     type(vector4_t), dimension(:), allocatable :: p_born
-    type(phs_point_t) :: p_real
+    type(vector4_t), dimension(:), allocatable :: p_real
     integer, dimension(:), allocatable :: emitters
     integer, dimension(:,:), allocatable :: resonance_lists
     type(resonance_contributors_t), dimension(2) :: alr_contributors
@@ -379,6 +378,7 @@ contains
 
     allocate (generator%isr_kinematics)
     generator%n_in = 2
+    generator%isr_kinematics%isr_mode = SQRTS_FIXED
 
     call generator%set_sqrts_hat (sqrts)
 
@@ -424,7 +424,7 @@ contains
     call generator%generate_radiation_variables &
        ([x1, x2, x3], p_born, phs_identifiers)
 
-    p_real = nlegborn + 1
+    allocate (p_real(nlegborn + 1))
     call generator%compute_xi_ref_momenta (p_born, alr_contributors)
     !!! Keep the distinction between i_phs and i_con because in general,
     !!! they are not the same.
@@ -432,17 +432,17 @@ contains
        i_con = i_phs
        emitter = phs_identifiers(i_phs)%emitter
        write (u, "(A,I1,1X,A,I1,A,I1,A)") &
-          "* Generate FSR phase space for emitter ", emitter, &
-          "and resonance pair (",  resonance_lists (i_con, 1), ",", &
-          resonance_lists (i_con, 2), ")"
+            "* Generate FSR phase space for emitter ", emitter, &
+            "and resonance pair (",  resonance_lists (i_con, 1), ",", &
+            resonance_lists (i_con, 2), ")"
        call generator%compute_xi_max (emitter, i_phs, p_born, &
             generator%real_kinematics%xi_max(i_phs), i_con = i_con)
        call generator%generate_fsr (emitter, i_phs, i_con, p_born, p_real)
-       call vector4_write_set (p_real%p, u, testflag = .true., ultra = .true.)
+       call vector4_write_set (p_real, u, testflag = .true., ultra = .true.)
        call write_separator(u)
        write (u, "(A)") "* Check if resonance masses are conserved: "
        m_inv_born = compute_resonance_mass (p_born, resonance_lists (i_con,:))
-       m_inv_real = compute_resonance_mass (p_real%p, resonance_lists (i_con,:), 7)
+       m_inv_real = compute_resonance_mass (p_real, resonance_lists (i_con,:), 7)
        write (u, "(A,1X, " // fmt // ")") "m_inv_born = ", m_inv_born
        write (u, "(A,1X, " // fmt // ")") "m_inv_real = ", m_inv_real
        if (abs (m_inv_born - m_inv_real) < tiny_07) then
@@ -453,7 +453,7 @@ contains
        call write_separator(u)
        call write_separator(u)
     end do
-    call p_real%final ()
+    deallocate (p_real)
     write (u, "(A)")
     write (u, "(A)") "* Test output end: phs_fks_generator_4"
   end subroutine phs_fks_generator_4
@@ -464,7 +464,7 @@ contains
     type(phs_fks_generator_t) :: generator
     type(vector4_t), dimension(:), allocatable :: p_born
     type(vector4_t), dimension(:), allocatable :: p_born_onshell
-    type(phs_point_t) :: p_real
+    type(vector4_t), dimension(:), allocatable :: p_real
     real(default) :: x1, x2, x3
     real(default) :: mB, mW, mtop, mcheck
     integer :: i, emitter, i_phs
@@ -505,6 +505,8 @@ contains
     p_born(6)%p(3) = 44.25011295081_default
 
     generator%n_in = 2
+    allocate (generator%isr_kinematics)
+    generator%isr_kinematics%isr_mode = SQRTS_FIXED
 
     mB = 4.2_default
     mW = 80.376_default
@@ -537,7 +539,10 @@ contains
     write (u, "(A)")
 
     write (u, "(A)") "* Perform onshell-projection:"
-    call generator%real_kinematics%apply_threshold_projection_born (mtop, L_to_cms)
+    associate (p_born => generator%real_kinematics%p_born_cms%phs_point(1)%p, &
+         p_born_onshell => generator%real_kinematics%p_born_onshell%phs_point(1)%p)
+       call threshold_projection_born (mtop, L_to_cms, p_born, p_born_onshell)
+    end associate
     call generator%real_kinematics%p_born_onshell%write (1, unit = u, testflag = .true., &
          ultra = .true.)
     associate (p => generator%real_kinematics%p_born_onshell%phs_point(1)%p)
@@ -582,12 +587,12 @@ contains
 
     call write_separator (u)
     write (u, "(A)") "* Produce real momenta from on-shell phase space: "
-    p_real = 7
+    allocate (p_real(7))
     do i_phs = 1, 2
        emitter = phs_identifiers(i_phs)%emitter
        write (u, "(A,I1)") "emitter: ", emitter
        call generator%generate_fsr_threshold (emitter, i_phs, p_born_onshell, p_real)
-       call check_phsp (p_real%p, emitter)
+       call check_phsp (p_real, emitter)
     end do
 
     call write_separator(u)
@@ -643,7 +648,7 @@ contains
     integer, intent(in) :: u
     type(phs_fks_generator_t) :: generator
     type(vector4_t), dimension(:), allocatable :: p_born
-    type(phs_point_t) :: p_real
+    type(vector4_t), dimension(:), allocatable :: p_real
     real(default) :: x1, x2, x3
     real(default) :: mB, mW, mT
     integer :: i, emitter, i_phs
@@ -672,6 +677,8 @@ contains
     p_born(4)%p(3) = -8.440032040958_default
 
     generator%n_in = 1
+    allocate (generator%isr_kinematics)
+    generator%isr_kinematics%isr_mode = SQRTS_FIXED
 
     mB = 4.2_default
     mW = 80.376_default
@@ -698,7 +705,7 @@ contains
     generator%emitters(2) = 2
     allocate (generator%m2 (4), generator%is_massive(4))
     generator%m2(1) = mT**2
-    generator%m2(1) = mB**2
+    generator%m2(2) = mB**2
     generator%m2(3) = zero
     generator%m2(4) = zero
     generator%is_massive(1:2) = .true.
@@ -728,21 +735,117 @@ contains
     write (u, "(A)") "Produce real momenta via initial-state emission: "
     i_phs = 1; emitter = phs_identifiers(i_phs)%emitter
     write (u, "(A,I1)") "emitter: ", emitter
-    p_real = 5
-    call generator%generate_isr_decay (i_phs, p_born, p_real)
-    call pacify (p_real%p, 1E-6_default)
-    call vector4_write_set (p_real%p, u, testflag = .true., ultra = .true.)
+    allocate (p_real(5))
+    call generator%generate_isr_fixed_beam_energy (i_phs, p_born, p_real)
+    call pacify (p_real, 1E-6_default)
+    call vector4_write_set (p_real, u, testflag = .true., ultra = .true.)
     call write_separator(u)
     write (u, "(A)") "Produce real momenta via final-state emisson: "
     i_phs = 2; emitter = phs_identifiers(i_phs)%emitter
     write (u, "(A,I1)") "emitter: ", emitter
     call generator%generate_fsr (emitter, i_phs, p_born, p_real)
-    call pacify (p_real%p, 1E-6_default)
-    call vector4_write_set (p_real%p, u, testflag = .true., ultra = .true.)
+    call pacify (p_real, 1E-6_default)
+    call vector4_write_set (p_real, u, testflag = .true., ultra = .true.)
     write (u, "(A)")
     write (u, "(A)") "* Test output end: phs_fks_generator_6"
 
   end subroutine phs_fks_generator_6
 
+  subroutine phs_fks_generator_7 (u)
+    integer, intent(in) :: u
+    type(phs_fks_generator_t) :: generator
+    type(vector4_t), dimension(:), allocatable :: p_born
+    type(vector4_t), dimension(:), allocatable :: p_real
+    real(default) :: x1, x2, x3
+    integer :: i, emitter, i_phs
+    type(phs_identifier_t), dimension(2) :: phs_identifiers
+    real(default), parameter :: sqrts = 1000.0_default
+
+    write (u, "(A)") "* Test output: phs_fks_generator_7"
+    write (u, "(A)") "* Puropse: Create real phase space for scattering ISR"
+    write (u, "(A)") "*          keeping the beam energy fixed."
+    write (u, "(A)")
+
+    allocate (p_born(4))
+    p_born(1)%p(0) = 500._default
+    p_born(1)%p(1) = 0._default
+    p_born(1)%p(2) = 0._default
+    p_born(1)%p(3) = 500._default
+    p_born(2)%p(0) = 500._default
+    p_born(2)%p(1) = 0._default
+    p_born(2)%p(2) = 0._default
+    p_born(2)%p(3) = -500._default
+    p_born(3)%p(0) = 500._default
+    p_born(3)%p(1) = 11.275563070_default
+    p_born(3)%p(2) = -13.588797663_default
+    p_born(3)%p(3) = 486.93070588_default
+    p_born(4)%p(0) = 500._default
+    p_born(4)%p(1:3) = -p_born(3)%p(1:3)
+
+    phs_identifiers(1)%emitter = 1
+    phs_identifiers(2)%emitter = 2
+
+    allocate (generator%emitters(2))
+    generator%n_in = 2
+    allocate (generator%isr_kinematics)
+    generator%isr_kinematics%isr_mode = SQRTS_FIXED
+    generator%emitters(1) = 1; generator%emitters(2) = 2
+    generator%sqrts = sqrts
+
+    write (u, "(A)") "* Use 2 -> 2 phase space containing: "
+    call vector4_write_set (p_born, u, testflag = .true., ultra = .true.)
+    write (u, "(A)") "**********************"
+    write (u, "(A)")
+
+    x1 = 0.5_default; x2 = 0.25_default; x3 = 0.6_default
+    write (u, "(A)") "* Use random numbers: "
+    write (u, "(A,F3.2,1X,A,F3.2,A,1X,F3.2)") &
+       "x1: ", x1, "x2: ", x2, "x3: ", x3
+
+    allocate (generator%real_kinematics)
+    call generator%real_kinematics%init (4, 2, 2, 1)
+    call generator%real_kinematics%p_born_lab%set_momenta (1, p_born)
+
+    allocate (generator%m2 (4))
+    generator%m2 = 0._default
+    allocate (generator%is_massive(4))
+    generator%is_massive = .false.
+    call generator%generate_radiation_variables ([x1,x2,x3], p_born, phs_identifiers)
+    call generator%compute_xi_ref_momenta (p_born)
+    do i_phs = 1, 2
+       emitter = phs_identifiers(i_phs)%emitter
+       call generator%compute_xi_max (emitter, i_phs, p_born, &
+            generator%real_kinematics%xi_max(i_phs))
+    end do
+
+    write (u, "(A)") &
+       "* With these, the following radiation variables have been produced: "
+    associate (rad_var => generator%real_kinematics)
+       write (u, "(A,F4.2)") "xi_tilde: ", rad_var%xi_tilde
+       do i = 1, 2
+          write (u, "(A,I1,A,F5.2)") "i: ", i, "y: " , rad_var%y(i)
+       end do
+       write (u, "(A,F4.2)") "phi: ", rad_var%phi
+    end associate
+
+    call write_separator (u)
+    write (u, "(A)") "Produce real momenta via initial-state emission: "
+    i_phs = 1; emitter = phs_identifiers(i_phs)%emitter
+    write (u, "(A,I1)") "emitter: ", emitter
+    allocate (p_real(5))
+    call generator%generate_isr_fixed_beam_energy (i_phs, p_born, p_real)
+    call pacify (p_real, 1E-6_default)
+    call vector4_write_set (p_real, u, testflag = .true., ultra = .true.)
+    call write_separator(u)
+    i_phs = 2; emitter = phs_identifiers(i_phs)%emitter
+    write (u, "(A,I1)") "emitter: ", emitter
+    call generator%generate_isr_fixed_beam_energy (i_phs, p_born, p_real)
+    call pacify (p_real, 1E-6_default)
+    call vector4_write_set (p_real, u, testflag = .true., ultra = .true.)
+    write (u, "(A)")
+    write (u, "(A)") "* Test output end: phs_fks_generator_7"
+
+  end subroutine phs_fks_generator_7
+
 
 end module phs_fks_uti
