@@ -1,19 +1,12 @@
-! WHIZARD 2.4.1 Mar 24 2017
+! WHIZARD 2.6.4 Aug 23 2018
 !
-! Copyright (C) 1999-2017 by
+! Copyright (C) 1999-2018 by
 !     Wolfgang Kilian <kilian@physik.uni-siegen.de>
 !     Thorsten Ohl <ohl@physik.uni-wuerzburg.de>
 !     Juergen Reuter <juergen.reuter@desy.de>
 !
 !     with contributions from
-!     Fabian Bach <fabian.bach@t-online.de>
-!     Bijan Chokoufe <bijan.chokoufe@desy.de>
-!     Christian Speckner <cnspeckn@googlemail.com>
-!     So Young Shim <soyoung.shim@desy.de>
-!     Florian Staub <florian.staub@cern.ch>
-!     Christian Weiss <christian.weiss@desy.de>
-!     and Hans-Werner Boschmann, Felix Braam,
-!     Sebastian Schmidt, So-young Shim, Daniel Wiesler
+!     cf. main AUTHORS file
 !
 ! WHIZARD is free software; you can redistribute it and/or modify it
 ! under the terms of the GNU General Public License as published by
@@ -52,9 +45,10 @@ module phs_wood
   use sf_base
   use phs_base
   use mappings
-  use resonances, only: resonance_history_t
+  use resonances, only: resonance_history_set_t
   use phs_forests
   use cascades
+  use cascades2
 
   implicit none
   private
@@ -69,6 +63,7 @@ module phs_wood
 
   type, extends (phs_config_t) :: phs_wood_config_t
      character(32) :: md5sum_forest = ""
+     type(string_t) :: phs_path
      integer :: io_unit = 0
      logical :: io_unit_keep_open = .false.
      logical :: use_equivalences = .false.
@@ -77,6 +72,8 @@ module phs_wood
      type(phs_parameters_t) :: par
      type(string_t) :: run_id
      type(cascade_set_t), allocatable :: cascade_set
+     logical :: use_cascades2 = .false.
+     type(feyngraph_set_t), allocatable :: feyngraph_set
      type(phs_forest_t) :: forest
      type(os_data_t) :: os_data
      integer :: extension_mode = EXTENSION_NONE
@@ -93,14 +90,15 @@ module phs_wood
      procedure :: generate_phase_space => phs_wood_config_generate_phase_space
      procedure :: write_phase_space => phs_wood_config_write_phase_space
      procedure :: clear_phase_space => phs_wood_config_clear_phase_space
-     procedure :: extract_resonance_histories &
-          => phs_wood_config_extract_resonance_histories
+     procedure :: extract_resonance_history_set &
+          => phs_wood_config_extract_resonance_history_set
      procedure :: configure => phs_wood_config_configure
+     procedure :: compute_md5sum_forest => phs_wood_config_compute_md5sum_forest
+     procedure :: make_phs_filename => phs_wood_make_phs_filename
      procedure :: reshuffle_flavors => phs_wood_config_reshuffle_flavors
      procedure :: set_momentum_links => phs_wood_config_set_momentum_links
      procedure :: record_s_mappings => phs_wood_config_record_s_mappings
      procedure :: record_on_shell => phs_wood_config_record_on_shell
-     procedure :: compute_md5sum_forest => phs_wood_config_compute_md5sum_forest
      procedure :: get_md5sum => phs_wood_config_get_md5sum
      procedure :: read_phs_file => phs_wood_read_phs_file
      procedure :: startup_message => phs_wood_config_startup_message
@@ -202,23 +200,61 @@ contains
   subroutine phs_wood_config_generate_phase_space (phs_config)
     class(phs_wood_config_t), intent(inout) :: phs_config
     integer :: off_shell, extra_off_shell
+    logical :: valid
+    integer :: unit_fds
+    type(string_t) :: file_name
+    logical :: file_exists
     call msg_message ("Phase space: generating configuration ...")
     off_shell = phs_config%par%off_shell
-    allocate (phs_config%cascade_set)
-    do extra_off_shell = 0, max (phs_config%n_tot - 3, 0)
-       phs_config%par%off_shell = off_shell + extra_off_shell
-       call cascade_set_generate (phs_config%cascade_set, &
-            phs_config%model, phs_config%n_in, phs_config%n_out, &
-            phs_config%flv, &
-            phs_config%par, phs_config%fatal_beam_decay)
-       if (cascade_set_is_valid (phs_config%cascade_set)) then
-          exit
-       else
-          call msg_message ("Phase space: ... failed.  &
-               &Increasing phs_off_shell ...")
-       end if
-    end do
-    if (cascade_set_is_valid (phs_config%cascade_set)) then
+    if (phs_config%use_cascades2) then
+       file_name = char (phs_config%id) // ".fds"
+       inquire (file=char (file_name), exist=file_exists)
+       if (.not. file_exists) call msg_fatal &
+            ("The O'Mega input file " // char (file_name) // &
+            " does not exist. " // "Please make sure that the " // &
+            "variable ?omega_write_phs_output has been set correctly.")
+       unit_fds = free_unit ()
+       open (unit=unit_fds, file=char(file_name), status='old', action='read')
+       do extra_off_shell = 0, max (phs_config%n_tot - 3, 0)
+          phs_config%par%off_shell = off_shell + extra_off_shell
+          allocate (phs_config%feyngraph_set)
+          call feyngraph_set_generate (phs_config%feyngraph_set, &
+               phs_config%model, phs_config%n_in, phs_config%n_out, &
+               phs_config%flv, &
+               phs_config%par, phs_config%fatal_beam_decay, unit_fds, &
+               phs_config%vis_channels)
+          if (feyngraph_set_is_valid (phs_config%feyngraph_set)) then
+             exit
+          else
+             call msg_message ("Phase space: ... failed.  &
+                  &Increasing phs_off_shell ...")
+             call phs_config%feyngraph_set%final ()
+             deallocate (phs_config%feyngraph_set)
+          end if
+       end do
+       close (unit_fds)
+    else
+       allocate (phs_config%cascade_set)
+       do extra_off_shell = 0, max (phs_config%n_tot - 3, 0)
+          phs_config%par%off_shell = off_shell + extra_off_shell
+          call cascade_set_generate (phs_config%cascade_set, &
+               phs_config%model, phs_config%n_in, phs_config%n_out, &
+               phs_config%flv, &
+               phs_config%par, phs_config%fatal_beam_decay)
+          if (cascade_set_is_valid (phs_config%cascade_set)) then
+             exit
+          else
+             call msg_message ("Phase space: ... failed.  &
+                  &Increasing phs_off_shell ...")
+          end if
+       end do
+    end if
+    if (phs_config%use_cascades2) then
+       valid = feyngraph_set_is_valid (phs_config%feyngraph_set)
+    else
+       valid = cascade_set_is_valid (phs_config%cascade_set)
+    end if
+    if (valid) then
        call msg_message ("Phase space: ... success.")
     else
        call msg_fatal ("Phase-space: generation failed")
@@ -232,7 +268,7 @@ contains
     type(string_t), intent(in), optional :: filename_vis
     type(string_t) :: setenv_tex, setenv_mp, pipe, pipe_dvi
     integer :: u, unit_tex, unit_dev, status
-    if (allocated (phs_config%cascade_set)) then
+    if (allocated (phs_config%cascade_set) .or. allocated (phs_config%feyngraph_set)) then
        if (present (unit)) then
           u = unit
        else
@@ -240,7 +276,11 @@ contains
        end if
        write (u, "(1x,A,A)") "process ", char (phs_config%id)
        write (u, "(A)")
-       call cascade_set_write_process_bincode_format (phs_config%cascade_set, u)
+       if (phs_config%use_cascades2) then
+          call feyngraph_set_write_process_bincode_format (phs_config%feyngraph_set, u)
+       else
+          call cascade_set_write_process_bincode_format (phs_config%cascade_set, u)
+       end if
        write (u, "(A)")
        write (u, "(3x,A,A,A32,A)") "md5sum_process    = ", &
             '"', phs_config%md5sum_process, '"'
@@ -249,13 +289,22 @@ contains
        write (u, "(3x,A,A,A32,A)") "md5sum_phs_config = ", &
             '"', phs_config%md5sum_phs_config, '"'
        call phs_parameters_write (phs_config%par, u)
-       call cascade_set_write_file_format (phs_config%cascade_set, u)
+       if (phs_config%use_cascades2) then
+          call feyngraph_set_write_file_format (phs_config%feyngraph_set, u)
+       else
+          call cascade_set_write_file_format (phs_config%cascade_set, u)
+       end if
        if (phs_config%vis_channels) then
           unit_tex = free_unit ()
           open (unit=unit_tex, file=char(filename_vis // ".tex"), &
                action="write", status="replace")
-          call cascade_set_write_graph_format (phs_config%cascade_set, &
-               filename_vis // "-graphs", phs_config%id, unit_tex)
+          if (phs_config%use_cascades2) then
+             call feyngraph_set_write_graph_format (phs_config%feyngraph_set, &
+                  filename_vis // "-graphs", phs_config%id, unit_tex)
+          else
+             call cascade_set_write_graph_format (phs_config%cascade_set, &
+                  filename_vis // "-graphs", phs_config%id, unit_tex)
+          end if
           close (unit_tex)
           call msg_message ("Phase space: visualizing channels in file " &
                // char(trim(filename_vis)) // "...")
@@ -325,17 +374,24 @@ contains
        call cascade_set_final (phs_config%cascade_set)
        deallocate (phs_config%cascade_set)
     end if
+    if (allocated (phs_config%feyngraph_set)) then
+       call phs_config%feyngraph_set%final ()
+       deallocate (phs_config%feyngraph_set)
+    end if
   end subroutine phs_wood_config_clear_phase_space
 
-  subroutine phs_wood_config_extract_resonance_histories (phs_config, res_hist)
+  subroutine phs_wood_config_extract_resonance_history_set &
+       (phs_config, res_set, include_trivial)
     class(phs_wood_config_t), intent(in) :: phs_config
-    type(resonance_history_t), dimension(:), allocatable, intent(out) :: res_hist
-    call phs_config%forest%extract_resonance_histories (res_hist)
-  end subroutine phs_wood_config_extract_resonance_histories
+    type(resonance_history_set_t), intent(out) :: res_set
+    logical, intent(in), optional :: include_trivial
+    call phs_config%forest%extract_resonance_history_set &
+         (res_set, include_trivial)
+  end subroutine phs_wood_config_extract_resonance_history_set
 
   subroutine phs_wood_config_configure (phs_config, sqrts, &
        sqrts_fixed, cm_frame, azimuthal_dependence, rebuild, ignore_mismatch, &
-       nlo_type)
+       nlo_type, subdir)
     class(phs_wood_config_t), intent(inout) :: phs_config
     real(default), intent(in) :: sqrts
     logical, intent(in), optional :: sqrts_fixed
@@ -344,6 +400,7 @@ contains
     logical, intent(in), optional :: rebuild
     logical, intent(in), optional :: ignore_mismatch
     integer, intent(in), optional :: nlo_type
+    type(string_t), intent(in), optional :: subdir
     type(string_t) :: filename, filename_vis
     logical :: variable_limits
     logical :: ok, exist, found, check, match, rebuild_phs
@@ -376,22 +433,21 @@ contains
     phs_config%md5sum_forest = ""
     call phs_config%compute_md5sum (include_id = .false.)
     if (phs_config%io_unit == 0) then
-       if (phs_config%run_id /= "") then
-          filename = phs_config%id // "." // phs_config%run_id // ".phs"
-          filename_vis = phs_config%id // "." // phs_config%run_id // "_phs"
-       else
-          filename = phs_config%id // ".phs"
-          filename_vis = phs_config%id // "_phs"
-       end if
+       filename = phs_config%make_phs_filename (subdir)
+       filename_vis = phs_config%make_phs_filename (subdir) // "-vis"
        if (.not. rebuild_phs) then
           if (check) then
-             call phs_config%read_phs_file (exist, found, match)
+             call phs_config%read_phs_file (exist, found, match, subdir=subdir)
              rebuild_phs = .not. (exist .and. found .and. match)
           else
-             call phs_config%read_phs_file (exist, found)
+             call phs_config%read_phs_file (exist, found, subdir=subdir)
              rebuild_phs = .not. (exist .and. found)
           end if
        end if
+       if (.not. mpi_is_comm_master ()) then
+          rebuild_phs = .false.
+          call msg_message ("MPI: Workers do not build phase space configuration.")
+       end if
        if (rebuild_phs) then
           call phs_config%generate_phase_space ()
           phs_config%io_unit = free_unit ()
@@ -456,6 +512,42 @@ contains
     end if
   end subroutine phs_wood_config_configure
 
+  subroutine phs_wood_config_compute_md5sum_forest (phs_config)
+    class(phs_wood_config_t), intent(inout) :: phs_config
+    integer :: u
+    u = free_unit ()
+    open (u, status = "scratch", action = "readwrite")
+    call phs_config%write_forest (u)
+    rewind (u)
+    phs_config%md5sum_forest = md5sum (u)
+    close (u)
+  end subroutine phs_wood_config_compute_md5sum_forest
+
+  function phs_wood_make_phs_filename (phs_config, subdir) result (filename)
+    class(phs_wood_config_t), intent(in) :: phs_config
+    type(string_t), intent(in), optional :: subdir
+    type(string_t) :: filename
+    type(string_t) :: basename, suffix, comp_code, comp_index
+    basename = phs_config%id
+    call split (basename, suffix, "_", back=.true.)
+    comp_code = extract (suffix, 1, 1)
+    comp_index = extract (suffix, 2)
+    if (comp_code == "i" .and. verify (comp_index, "1234567890") == 0) then
+       suffix = "." // comp_code // comp_index
+    else
+       basename = phs_config%id
+       suffix = ""
+    end if
+    if (phs_config%run_id /= "") then
+       filename = basename // "." // phs_config%run_id // suffix // ".phs"
+    else
+       filename = basename // suffix // ".phs"
+    end if
+    if (present (subdir)) then
+       filename = subdir // "/" // filename
+    end if
+  end function phs_wood_make_phs_filename
+    
   subroutine phs_wood_config_reshuffle_flavors (phs_config, reshuffle, flv_extra)
     class(phs_wood_config_t), intent(inout) :: phs_config
     integer, intent(in), dimension(:), allocatable :: reshuffle
@@ -503,17 +595,6 @@ contains
     end do
   end subroutine phs_wood_config_record_on_shell
 
-  subroutine phs_wood_config_compute_md5sum_forest (phs_config)
-    class(phs_wood_config_t), intent(inout) :: phs_config
-    integer :: u
-    u = free_unit ()
-    open (u, status = "scratch", action = "readwrite")
-    call phs_config%write_forest (u)
-    rewind (u)
-    phs_config%md5sum_forest = md5sum (u)
-    close (u)
-  end subroutine phs_wood_config_compute_md5sum_forest
-
   function phs_wood_config_get_md5sum (phs_config) result (md5sum)
     class(phs_wood_config_t), intent(in) :: phs_config
     character(32) :: md5sum
@@ -524,14 +605,15 @@ contains
     end if
   end function phs_wood_config_get_md5sum
 
-  subroutine phs_wood_read_phs_file (phs_config, exist, found, match)
+  subroutine phs_wood_read_phs_file (phs_config, exist, found, match, subdir)
     class(phs_wood_config_t), intent(inout) :: phs_config
     logical, intent(out) :: exist
     logical, intent(out) :: found
     logical, intent(out), optional :: match
+    type(string_t), intent(in), optional :: subdir
     type(string_t) :: filename
     integer :: u
-    filename = phs_config%id // ".phs"
+    filename = phs_config%make_phs_filename (subdir)
     inquire (file = char (filename), exist = exist)
     if (exist) then
        u = free_unit ()
