@@ -1,19 +1,12 @@
-! WHIZARD 2.4.1 Mar 24 2017
+! WHIZARD 2.6.4 Aug 23 2018
 !
-! Copyright (C) 1999-2017 by
+! Copyright (C) 1999-2018 by
 !     Wolfgang Kilian <kilian@physik.uni-siegen.de>
 !     Thorsten Ohl <ohl@physik.uni-wuerzburg.de>
 !     Juergen Reuter <juergen.reuter@desy.de>
 !
 !     with contributions from
-!     Fabian Bach <fabian.bach@t-online.de>
-!     Bijan Chokoufe <bijan.chokoufe@desy.de>
-!     Christian Speckner <cnspeckn@googlemail.com>
-!     So Young Shim <soyoung.shim@desy.de>
-!     Florian Staub <florian.staub@cern.ch>
-!     Christian Weiss <christian.weiss@desy.de>
-!     and Hans-Werner Boschmann, Felix Braam,
-!     Sebastian Schmidt, So-young Shim, Daniel Wiesler
+!     cf. main AUTHORS file
 !
 ! WHIZARD is free software; you can redistribute it and/or modify it
 ! under the terms of the GNU General Public License as published by
@@ -60,7 +53,6 @@ module process_config
   use prc_core
   use prc_user_defined
   use prc_openloops, only: prc_openloops_t
-  use prc_recola, only: prc_recola_t
   use prc_threshold, only: prc_threshold_t
   use beams
   use mci_base
@@ -69,6 +61,9 @@ module process_config
   use variables
   use expr_base
 
+  use pcm_base, only: pcm_t
+  use pcm, only: pcm_nlo_t
+
   implicit none
   private
 
@@ -162,6 +157,7 @@ module process_config
      procedure :: get_beam_file => process_beam_config_get_beam_file
      procedure :: compute_md5sum => process_beam_config_compute_md5sum
      procedure :: get_md5sum => process_beam_config_get_md5sum
+     procedure :: has_structure_function => process_beam_config_has_structure_function
   end type process_beam_config_t
 
   type :: process_component_t
@@ -203,8 +199,10 @@ module process_config
      type(process_constants_t) :: data
      real(default) :: alpha_s = 0
      integer, dimension(:), allocatable :: flv, hel, col
+     integer :: n_sub, n_sub_color, n_sub_spin
      type(interaction_t) :: int
      type(interaction_t), pointer :: int_eff => null ()
+     class(pcm_t), pointer :: pcm => null ()
    contains
      procedure :: write => process_term_write
      procedure :: write_state_summary => process_term_write_state_summary
@@ -737,6 +735,12 @@ contains
     md5 = beam_config%md5sum
   end function process_beam_config_get_md5sum
 
+  pure function process_beam_config_has_structure_function (beam_config) result (has_sf)
+    logical :: has_sf
+    class(process_beam_config_t), intent(in) :: beam_config
+    has_sf = beam_config%n_strfun > 0
+  end function process_beam_config_has_structure_function
+
   subroutine process_component_final (object)
     class(process_component_t), intent(inout) :: object
     if (allocated (object%mci_template)) then
@@ -805,7 +809,7 @@ contains
           allocate (component%mci_template, source = mci_template)
        allocate (component%phs_config, source = phs_config_template)
        call component%phs_config%init (data, config%model)
-       call component%phs_config%set_component_index (component%index)
+       !!! call component%phs_config%set_component_index (component%index)
     end if
   end subroutine process_component_init
 
@@ -832,12 +836,13 @@ contains
 
   subroutine process_component_configure_phs &
        (component, sqrts, beam_config, rebuild, &
-        ignore_mismatch)
+        ignore_mismatch, subdir)
     class(process_component_t), intent(inout) :: component
     real(default), intent(in) :: sqrts
     type(process_beam_config_t), intent(in) :: beam_config
     logical, intent(in), optional :: rebuild
     logical, intent(in), optional :: ignore_mismatch
+    type(string_t), intent(in), optional :: subdir
     logical :: no_strfun
     integer :: nlo_type
     no_strfun = beam_config%n_strfun == 0
@@ -847,7 +852,8 @@ contains
          sqrts_fixed = no_strfun, &
          cm_frame = beam_config%lab_is_cm_frame .and. no_strfun, &
          rebuild = rebuild, ignore_mismatch = ignore_mismatch, &
-         nlo_type = nlo_type)
+         nlo_type = nlo_type, &
+         subdir = subdir)
   end subroutine process_component_configure_phs
 
   subroutine process_component_compute_md5sum (component)
@@ -1053,23 +1059,54 @@ contains
     end subroutine count_number_of_states
 
     subroutine compute_n_sub ()
-      if (nlo_t == NLO_VIRTUAL) then
-         use_color = .false.; if (present (use_internal_color)) &
-              use_color = use_internal_color
-         n_sub = 1
-         if (.not. use_color) n_sub = n_sub + n_tot * (n_tot - 1) / 2
-      else
-         n_sub = 0
+      logical :: can_have_sub
+      integer :: n_sub_color, n_sub_spin
+      use_color = .false.; if (present (use_internal_color)) &
+           use_color = use_internal_color
+      can_have_sub = nlo_t == NLO_VIRTUAL .or. &
+           (nlo_t == NLO_REAL .and. term%i_term_global == term%i_sub) .or. &
+           nlo_t == NLO_MISMATCH
+      n_sub_color = 0; n_sub_spin = 0
+      if (can_have_sub) then
+         if (.not. use_color) n_sub_color = n_tot * (n_tot - 1) / 2
+         if (nlo_t == NLO_REAL) then
+            select type (pcm_nlo => term%pcm)
+            class is (pcm_nlo_t)
+               if (pcm_nlo%region_data%requires_spin_correlations ()) &
+                    n_sub_spin = 16 * pcm_nlo%region_data%n_emitters
+            end select
+         end if
       end if
+      n_sub = n_sub_color + n_sub_spin
+      !!! For the virtual subtraction we also need the finite virtual contribution
+      !!! corresponding to the $\epsilon^0$-pole
+      if (nlo_t == NLO_VIRTUAL)  n_sub = n_sub + 1
+      if (associated (term%pcm)) then
+         if (term%pcm%has_pdfs .and. ((nlo_t == NLO_REAL .and. can_have_sub) &
+              .or. nlo_t == NLO_DGLAP)) n_sub = n_sub + n_beam_structure_int
+      end if
+      term%n_sub = n_sub
+      term%n_sub_color = n_sub_color
+      term%n_sub_spin = n_sub_spin
     end subroutine compute_n_sub
 
     subroutine fill_quantum_numbers ()
-      if (nlo_t == NLO_VIRTUAL) then
-         allocate (term%flv ((n_sub + 1) * n), &
-              term%col ((n_sub + 1) * n), term%hel ((n_sub + 1) * n))
-      else
-         allocate (term%flv (n), term%col (n), term%hel (n))
-      end if
+      integer :: nn
+      logical :: can_have_sub
+      select type (core)
+      class is (prc_user_defined_base_t)
+         can_have_sub = nlo_t == NLO_VIRTUAL .or. &
+              (nlo_t == NLO_REAL .and. term%i_term_global == term%i_sub) .or. &
+              nlo_t == NLO_MISMATCH .or. nlo_t == NLO_DGLAP
+         if (can_have_sub) then
+            nn = (n_sub + 1) * n
+         else
+            nn = n
+         end if
+      class default
+         nn = n
+      end select
+      allocate (term%flv (nn), term%col (nn), term%hel (nn))
       allocate (flv (n_tot), col (n_tot), hel (n_tot))
       allocate (qn (n_tot))
     end subroutine fill_quantum_numbers
@@ -1090,13 +1127,14 @@ contains
                       call flv%init (data%flv_state (:,f), model)
                       call color_init_from_array (col, &
                            data%col_state(:,:,c), data%ghost_flag(:,c))
-                      call col(:data%n_in)%invert ()
+                      call col(1:data%n_in)%invert ()
                       if (is_pol) then
                          call hel%init (data%hel_state (:,h))
                          call qn%init (flv, hel, col, s)
                       else
                          call qn%init (flv, col, s)
                       end if
+                      call qn%tag_hard_process ()
                       call term%int%add_state (qn)
                   end do
                end do
@@ -1107,8 +1145,6 @@ contains
 
     subroutine setup_states_threshold ()
       integer :: s, f, c, h, i
-      if (is_pol) &
-           call msg_fatal ("Polarized beams only supported by OpenLoops")
       i = 0
       n_sub = 0; if (nlo_t == NLO_VIRTUAL) n_sub = 1
       associate (data => term%data)
@@ -1118,12 +1154,17 @@ contains
                   do c = 1, data%n_col
                      i = i + 1
                      term%flv(i) = f
-                     !!! Dumy initialization of helicity
-                     term%hel(i) = 1
+                     term%hel(i) = h
                      !!! Dummy-initialization of color
                      term%col(i) = 1
                      call flv%init (term%data%flv_state (:,f), model)
-                     call qn%init (flv, s)
+                     if (is_pol) then
+                        call hel%init (data%hel_state (:,h))
+                        call qn%init (flv, hel, s)
+                     else
+                        call qn%init (flv, s)
+                     end if
+                     call qn%tag_hard_process ()
                      call term%int%add_state (qn)
                   end do
                end do
@@ -1147,6 +1188,7 @@ contains
             term%col(i) = 1
             call flv%init (term%data%flv_state (:,f), model)
             call qn%init (flv, s)
+            call qn%tag_hard_process ()
             call term%int%add_state (qn)
          end do
       end do
@@ -1171,6 +1213,7 @@ contains
                     call col(:data%n_in)%invert ()
                     call hel%init (data%hel_state(:,h))
                     call qn%init (flv, col, hel)
+                    call qn%tag_hard_process ()
                     call term%int%add_state (qn)
                  end if
               end do
