@@ -1,19 +1,12 @@
-! WHIZARD 2.4.1 Mar 24 2017
+! WHIZARD 2.6.4 Aug 23 2018
 !
-! Copyright (C) 1999-2017 by
+! Copyright (C) 1999-2018 by
 !     Wolfgang Kilian <kilian@physik.uni-siegen.de>
 !     Thorsten Ohl <ohl@physik.uni-wuerzburg.de>
 !     Juergen Reuter <juergen.reuter@desy.de>
 !
 !     with contributions from
-!     Fabian Bach <fabian.bach@t-online.de>
-!     Bijan Chokoufe <bijan.chokoufe@desy.de>
-!     Christian Speckner <cnspeckn@googlemail.com>
-!     So Young Shim <soyoung.shim@desy.de>
-!     Florian Staub <florian.staub@cern.ch>
-!     Christian Weiss <christian.weiss@desy.de>
-!     and Hans-Werner Boschmann, Felix Braam,
-!     Sebastian Schmidt, So-young Shim, Daniel Wiesler
+!     cf. main AUTHORS file
 !
 ! WHIZARD is free software; you can redistribute it and/or modify it
 ! under the terms of the GNU General Public License as published by
@@ -64,6 +57,7 @@ module subevt_expr
      integer :: n_tot = 0
      logical :: has_selection = .false.
      class(expr_t), allocatable :: selection
+     logical :: colorize_subevt = .false.
    contains
      procedure :: base_write => subevt_expr_write
      procedure (subevt_expr_final), deferred :: final
@@ -73,8 +67,9 @@ module subevt_expr
      procedure :: setup_var_self => subevt_expr_setup_var_self
      procedure :: link_var_list => subevt_expr_link_var_list
      procedure :: setup_selection => subevt_expr_setup_selection
-     procedure :: reset => subevt_expr_reset
-     procedure :: base_reset => subevt_expr_reset
+     procedure :: colorize => subevt_expr_colorize
+     procedure :: reset_contents => subevt_expr_reset_contents
+     procedure :: base_reset_contents => subevt_expr_reset_contents
      procedure :: base_evaluate => subevt_expr_evaluate
   end type subevt_expr_t
 
@@ -140,8 +135,13 @@ module subevt_expr
      procedure :: setup_reweight => event_expr_setup_reweight
      procedure :: set_process_id => event_expr_set_process_id
      procedure :: set_process_num_id => event_expr_set_process_num_id
-     procedure :: reset => event_expr_reset
+     procedure :: reset_contents => event_expr_reset_contents
      procedure :: set => event_expr_set
+     procedure :: has_event_index => event_expr_has_event_index
+     procedure :: get_event_index => event_expr_get_event_index
+     procedure :: set_event_index => event_expr_set_event_index
+     procedure :: reset_event_index => event_expr_reset_event_index
+     procedure :: increment_event_index => event_expr_increment_event_index
      procedure :: fill_subevt => event_expr_fill_subevt
      procedure :: evaluate => event_expr_evaluate
   end type event_expr_t
@@ -240,10 +240,16 @@ contains
     end if
   end subroutine subevt_expr_setup_selection
 
-  subroutine subevt_expr_reset (expr)
+  subroutine subevt_expr_colorize (expr, colorize_subevt)
+    class(subevt_expr_t), intent(inout), target :: expr
+    logical, intent(in) :: colorize_subevt
+    expr%colorize_subevt = colorize_subevt
+  end subroutine subevt_expr_colorize
+    
+  subroutine subevt_expr_reset_contents (expr)
     class(subevt_expr_t), intent(inout) :: expr
     expr%subevt_filled = .false.
-  end subroutine subevt_expr_reset
+  end subroutine subevt_expr_reset_contents
 
   subroutine subevt_expr_evaluate (expr, passed)
     class(subevt_expr_t), intent(inout) :: expr
@@ -660,9 +666,9 @@ contains
     expr%has_num_id = .true.
   end subroutine event_expr_set_process_num_id
 
-  subroutine event_expr_reset (expr)
+  subroutine event_expr_reset_contents (expr)
     class(event_expr_t), intent(inout) :: expr
-    call expr%base_reset ()
+    call expr%base_reset_contents ()
     expr%has_sqme_ref = .false.
     expr%has_sqme_prc = .false.
     expr%has_sqme_alt = .false.
@@ -670,7 +676,7 @@ contains
     expr%has_weight_prc = .false.
     expr%has_weight_alt = .false.
     expr%has_excess_prc = .false.
-  end subroutine event_expr_reset
+  end subroutine event_expr_reset_contents
 
   subroutine event_expr_set (expr, &
        weight_ref, weight_prc, weight_alt, &
@@ -711,21 +717,55 @@ contains
     end if
   end subroutine event_expr_set
 
+  function event_expr_has_event_index (expr) result (flag)
+    class(event_expr_t), intent(in) :: expr
+    logical :: flag
+    flag = expr%has_index
+  end function event_expr_has_event_index
+
+  function event_expr_get_event_index (expr) result (index)
+    class(event_expr_t), intent(in) :: expr
+    integer :: index
+    if (expr%has_index) then
+       index = expr%index
+    else
+       index = 0
+    end if
+  end function event_expr_get_event_index
+  
+  subroutine event_expr_set_event_index (expr, index)
+    class(event_expr_t), intent(inout) :: expr
+    integer, intent(in) :: index
+    expr%index = index
+    expr%has_index = .true.
+  end subroutine event_expr_set_event_index
+
+  subroutine event_expr_reset_event_index (expr)
+    class(event_expr_t), intent(inout) :: expr
+    expr%has_index = .false.
+  end subroutine event_expr_reset_event_index
+
+  subroutine event_expr_increment_event_index (expr, offset)
+    class(event_expr_t), intent(inout) :: expr
+    integer, intent(in), optional :: offset
+    if (expr%has_index) then
+       expr%index = expr%index + 1
+    else if (present (offset)) then
+       call expr%set_event_index (offset + 1)
+    else
+       call expr%set_event_index (1)
+    end if
+  end subroutine event_expr_increment_event_index
+
   subroutine event_expr_fill_subevt (expr, particle_set)
     class(event_expr_t), intent(inout) :: expr
     type(particle_set_t), intent(in) :: particle_set
-    call particle_set%to_subevt (expr%subevt_t)
+    call particle_set%to_subevt (expr%subevt_t, expr%colorize_subevt)
     expr%sqrts_hat = subevt_get_sqrts_hat (expr%subevt_t)
     expr%n_in  = subevt_get_n_in  (expr%subevt_t)
     expr%n_out = subevt_get_n_out (expr%subevt_t)
     expr%n_tot = expr%n_in + expr%n_out
     expr%subevt_filled = .true.
-    if (expr%has_index) then
-       expr%index = expr%index + 1
-    else
-       expr%index = 1
-       expr%has_index = .true.
-    end if
   end subroutine event_expr_fill_subevt
 
   subroutine event_expr_evaluate (expr, passed, reweight, analysis_flag)
