@@ -1,19 +1,12 @@
-! WHIZARD 2.4.1 Mar 24 2017
+! WHIZARD 2.6.4 Aug 23 2018
 !
-! Copyright (C) 1999-2017 by
+! Copyright (C) 1999-2018 by
 !     Wolfgang Kilian <kilian@physik.uni-siegen.de>
 !     Thorsten Ohl <ohl@physik.uni-wuerzburg.de>
 !     Juergen Reuter <juergen.reuter@desy.de>
 !
 !     with contributions from
-!     Fabian Bach <fabian.bach@t-online.de>
-!     Bijan Chokoufe <bijan.chokoufe@desy.de>
-!     Christian Speckner <cnspeckn@googlemail.com>
-!     So Young Shim <soyoung.shim@desy.de>
-!     Florian Staub <florian.staub@cern.ch>
-!     Christian Weiss <christian.weiss@desy.de>
-!     and Hans-Werner Boschmann, Felix Braam,
-!     Sebastian Schmidt, So-young Shim, Daniel Wiesler
+!     cf. main AUTHORS file
 !
 ! WHIZARD is free software; you can redistribute it and/or modify it
 ! under the terms of the GNU General Public License as published by
@@ -59,6 +52,7 @@ module flavors
   type :: flavor_t
      private
      integer :: f = UNDEFINED
+     logical :: hard_process = .false.
      logical :: radiated = .false.
      type(field_data_t), pointer :: field_data => null ()
    contains
@@ -76,6 +70,7 @@ module flavors
      procedure, private :: flavor_init_model_alt
      procedure, private :: flavor_init_name_model
      procedure :: tag_radiated => flavor_tag_radiated
+     procedure :: tag_hard_process => flavor_tag_hard_process
      procedure :: undefine => flavor_undefine
      procedure :: write => flavor_write
      procedure :: write_raw => flavor_write_raw
@@ -85,6 +80,7 @@ module flavors
      procedure :: is_valid => flavor_is_valid
      procedure :: is_associated => flavor_is_associated
      procedure :: is_radiated => flavor_is_radiated
+     procedure :: is_hard_process => flavor_is_hard_process
      procedure :: get_pdg => flavor_get_pdg
      procedure :: get_pdg_anti => flavor_get_pdg_anti
      procedure :: get_pdg_abs => flavor_get_pdg_abs
@@ -139,6 +135,7 @@ contains
   elemental subroutine flavor_init_empty (flv)
     class(flavor_t), intent(inout) :: flv
     flv%f = UNDEFINED
+    flv%hard_process = .false.
     flv%radiated = .false.
     flv%field_data => null ()
   end subroutine flavor_init_empty
@@ -147,6 +144,7 @@ contains
     class(flavor_t), intent(inout) :: flv
     integer, intent(in) :: f
     flv%f = f
+    flv%hard_process = .false.
     flv%radiated = .false.
     flv%field_data => null ()
   end subroutine flavor_init
@@ -155,6 +153,7 @@ contains
     class(flavor_t), intent(inout) :: flv
     type(field_data_t), intent(in), target :: field_data
     flv%f = field_data%get_pdg ()
+    flv%hard_process = .false.
     flv%radiated = .false.
     flv%field_data => field_data
   end subroutine flavor_init_field_data
@@ -164,6 +163,7 @@ contains
     integer, intent(in) :: f
     class(model_data_t), intent(in), target :: model
     flv%f = f
+    flv%hard_process = .false.
     flv%radiated = .false.
     flv%field_data => model%get_field_ptr (f, check=.true.)
   end subroutine flavor_init_model
@@ -173,6 +173,7 @@ contains
     integer, intent(in) :: f
     class(model_data_t), intent(in), target :: model, alt_model
     flv%f = f
+    flv%hard_process = .false.
     flv%radiated = .false.
     flv%field_data => model%get_field_ptr (f, check=.false.)
     if (.not. associated (flv%field_data)) then
@@ -192,6 +193,7 @@ contains
     type(string_t), intent(in) :: name
     class(model_data_t), intent(in), target :: model
     flv%f = model%get_pdg (name)
+    flv%hard_process = .false.
     flv%radiated = .false.
     flv%field_data => model%get_field_ptr (name, check=.true.)
   end subroutine flavor_init_name_model
@@ -201,6 +203,11 @@ contains
     flv%radiated = .true.
   end subroutine flavor_tag_radiated
 
+  elemental subroutine flavor_tag_hard_process (flv)
+    class(flavor_t), intent(inout) :: flv
+    flv%hard_process = .true.
+  end subroutine flavor_tag_hard_process
+
   elemental subroutine flavor_undefine (flv)
     class(flavor_t), intent(inout) :: flv
     flv%f = UNDEFINED
@@ -285,6 +292,12 @@ contains
     flag = flv%radiated
   end function flavor_is_radiated
 
+  elemental function flavor_is_hard_process (flv) result (flag)
+    class(flavor_t), intent(in) :: flv
+    logical :: flag
+    flag = flv%hard_process
+  end function flavor_is_hard_process
+
   elemental function flavor_get_pdg (flv) result (f)
     integer :: f
     class(flavor_t), intent(in) :: flv
