@@ -1,19 +1,12 @@
-! WHIZARD 2.4.1 Mar 24 2017
+! WHIZARD 2.6.4 Aug 23 2018
 !
-! Copyright (C) 1999-2017 by
+! Copyright (C) 1999-2018 by
 !     Wolfgang Kilian <kilian@physik.uni-siegen.de>
 !     Thorsten Ohl <ohl@physik.uni-wuerzburg.de>
 !     Juergen Reuter <juergen.reuter@desy.de>
 !
 !     with contributions from
-!     Fabian Bach <fabian.bach@t-online.de>
-!     Bijan Chokoufe <bijan.chokoufe@desy.de>
-!     Christian Speckner <cnspeckn@googlemail.com>
-!     So Young Shim <soyoung.shim@desy.de>
-!     Florian Staub <florian.staub@cern.ch>
-!     Christian Weiss <christian.weiss@desy.de>
-!     and Hans-Werner Boschmann, Felix Braam,
-!     Sebastian Schmidt, So-young Shim, Daniel Wiesler
+!     cf. main AUTHORS file
 !
 ! WHIZARD is free software; you can redistribute it and/or modify it
 ! under the terms of the GNU General Public License as published by
@@ -54,6 +47,7 @@ module helicities
      procedure, private :: helicity_init_different
      procedure :: undefine => helicity_undefine
      procedure :: diagonalize => helicity_diagonalize
+     procedure :: flip => helicity_flip
      procedure :: get_indices => helicity_get_indices
      procedure :: write => helicity_write
      procedure :: write_raw => helicity_write_raw
@@ -128,6 +122,12 @@ contains
     hel%h2 = hel%h1
   end subroutine helicity_diagonalize
 
+  elemental subroutine helicity_flip (hel)
+    class(helicity_t), intent(inout) :: hel
+    hel%h1 = - hel%h1
+    hel%h2 = - hel%h2
+  end subroutine helicity_flip
+
   subroutine helicity_get_indices (hel, h1, h2)
     class(helicity_t), intent(in) :: hel
     integer, intent(out) :: h1, h2
