@@ -1,19 +1,12 @@
-! WHIZARD 2.4.1 Mar 24 2017
+! WHIZARD 2.6.4 Aug 23 2018
 !
-! Copyright (C) 1999-2017 by
+! Copyright (C) 1999-2018 by
 !     Wolfgang Kilian <kilian@physik.uni-siegen.de>
 !     Thorsten Ohl <ohl@physik.uni-wuerzburg.de>
 !     Juergen Reuter <juergen.reuter@desy.de>
 !
 !     with contributions from
-!     Fabian Bach <fabian.bach@t-online.de>
-!     Bijan Chokoufe <bijan.chokoufe@desy.de>
-!     Christian Speckner <cnspeckn@googlemail.com>
-!     So Young Shim <soyoung.shim@desy.de>
-!     Florian Staub <florian.staub@cern.ch>
-!     Christian Weiss <christian.weiss@desy.de>
-!     and Hans-Werner Boschmann, Felix Braam,
-!     Sebastian Schmidt, So-young Shim, Daniel Wiesler
+!     cf. main AUTHORS file
 !
 ! WHIZARD is free software; you can redistribute it and/or modify it
 ! under the terms of the GNU General Public License as published by
@@ -49,6 +42,7 @@ module interactions
   implicit none
   private
 
+  public :: qn_index_map_t
   public :: external_link_get_ptr
   public :: external_link_get_index
   public :: interaction_t
@@ -69,9 +63,28 @@ module interactions
   public :: interaction_exchange_mask
   public :: interaction_send_momenta
   public :: interaction_pacify_momenta
-  public :: interaction_declare_subtraction
   public :: find_connections
 
+  type :: qn_index_map_t
+     private
+     type(quantum_numbers_t), dimension(:, :), allocatable :: qn_flv
+     type(quantum_numbers_t), dimension(:, :), allocatable :: qn_hel
+     logical :: flip_hel = .false.
+     integer :: n_flv = 0, n_hel = 0, n_sub = 0
+     integer, dimension(:, :, :), allocatable :: index
+   contains
+     generic :: init => qn_index_map_init
+     procedure, private :: qn_index_map_init
+     generic :: init => qn_index_map_init_trivial
+     procedure, private :: qn_index_map_init_trivial
+     procedure :: write => qn_index_map_write
+     procedure :: set_helicity_flip => qn_index_map_set_helicity_flip
+     procedure :: get_index => qn_index_map_get_index
+     procedure :: get_n_flv => qn_index_map_get_n_flv
+     procedure :: get_n_hel => qn_index_map_get_n_hel
+     procedure :: get_n_sub => qn_index_map_get_n_sub
+  end type qn_index_map_t
+
   type :: external_link_t
      private
      type(interaction_t), pointer :: int => null ()
@@ -119,6 +132,7 @@ module interactions
      procedure :: is_empty => interaction_is_empty
      procedure :: get_n_matrix_elements => &
           interaction_get_n_matrix_elements
+     procedure :: get_state_depth => interaction_get_state_depth
      procedure :: get_n_in_helicities => interaction_get_n_in_helicities
      procedure :: get_me_size => interaction_get_me_size
      procedure :: get_norm => interaction_get_norm
@@ -133,6 +147,7 @@ module interactions
      procedure :: get_quantum_numbers_all_qn_mask => &
         interaction_get_quantum_numbers_all_qn_mask
      procedure :: get_quantum_numbers_all_sub => interaction_get_quantum_numbers_all_sub
+     procedure :: get_flavors => interaction_get_flavors
      procedure :: get_quantum_numbers_mask => interaction_get_quantum_numbers_mask
      generic :: get_matrix_element => get_matrix_element_single
      generic :: get_matrix_element => get_matrix_element_array
@@ -168,6 +183,7 @@ module interactions
      procedure :: evaluate_square_c => interaction_evaluate_square_c
      procedure :: evaluate_sum => interaction_evaluate_sum
      procedure :: evaluate_me_sum => interaction_evaluate_me_sum
+     procedure :: tag_hard_process => interaction_tag_hard_process
      procedure :: get_tag => interaction_get_tag
      procedure :: get_n_tot => interaction_get_n_tot
      procedure :: get_n_in => interaction_get_n_in
@@ -194,6 +210,7 @@ module interactions
      procedure :: set_source_link => interaction_set_source_link
      procedure :: find_source => interaction_find_source
      procedure :: receive_momenta => interaction_receive_momenta
+     procedure :: declare_subtraction => interaction_declare_subtraction
   end type interaction_t
 
 
@@ -204,6 +221,229 @@ module interactions
 
 contains
 
+  subroutine qn_index_map_init (self, int, qn_flv, n_sub, qn_hel) 
+    class(qn_index_map_t), intent(out) :: self
+    class(interaction_t), intent(in) :: int
+    type(quantum_numbers_t), dimension(:, :), intent(in) :: qn_flv
+    integer, intent(in) :: n_sub
+    type(quantum_numbers_t), dimension(:, :), intent(in), optional :: qn_hel
+    type(quantum_numbers_t), dimension(:), allocatable :: qn, qn_int
+    integer :: i, i_flv, i_hel, i_sub
+    self%qn_flv = qn_flv
+    self%n_flv = size (qn_flv, dim=2)
+    self%n_sub = n_sub
+    if (present (qn_hel)) then
+       if (size (qn_flv, dim=1) /= size (qn_hel, dim=1)) then
+          call msg_bug ("[qn_index_map_init] number of particles does not match.")
+       end if
+       self%qn_hel = qn_hel
+       self%n_hel = size (qn_hel, dim=2)
+    else
+       self%n_hel = 1
+    end if
+    allocate (self%index (self%n_flv, self%n_hel, 0:self%n_sub), source=0)
+    associate (n_me => int%get_n_matrix_elements ())
+       do i = 1, n_me
+          qn_int = int%get_quantum_numbers (i, by_me_index = .true.)
+          qn = pack (qn_int, qn_int%are_hard_process ())
+          i_flv = find_flv_index (self, qn)
+          i_hel = 1; if (allocated (self%qn_hel)) &
+               i_hel = find_hel_index (self, qn)
+          i_sub = find_sub_index (self, qn)
+          self%index(i_flv, i_hel, i_sub) = i
+       end do
+    end associate
+  contains
+    integer function find_flv_index (self, qn) result (i_flv)
+      type(qn_index_map_t), intent(in) :: self
+      type(quantum_numbers_t), dimension(:), intent(in) :: qn
+      integer :: j
+      i_flv = 0
+      do j = 1, self%n_flv
+         if (.not. all (qn .fmatch. self%qn_flv(:, j))) cycle
+         i_flv = j
+         exit
+      end do
+      if (i_flv < 1) then
+         call msg_message ("QN:")
+         call quantum_numbers_write (qn)
+         call msg_message ("")
+         call msg_message ("QN_FLV:")
+         do j = 1, self%n_flv
+            call quantum_numbers_write (self%qn_flv(:, j))
+            call msg_message ("")
+         end do
+         call msg_bug ("[find_flv_index] could not find flv in qn_flv.")
+      end if
+    end function find_flv_index
+
+    integer function find_hel_index (self, qn) result (i_hel)
+      type(qn_index_map_t), intent(in) :: self
+      type(quantum_numbers_t), dimension(:), intent(in) :: qn
+      integer :: j
+      i_hel = 0
+      do j = 1, self%n_hel
+         if (.not. all (qn .hmatch. self%qn_hel(:, j))) cycle
+         i_hel = j
+         exit
+      end do
+      if (i_hel < 1) then
+         call msg_message ("QN:")
+         call quantum_numbers_write (qn)
+         call msg_message ("")
+         call msg_message ("QN_HEL:")
+         do j = 1, self%n_hel
+            call quantum_numbers_write (self%qn_hel(:, j))
+            call msg_message ("")
+         end do
+         call msg_bug ("[find_hel_index] could not find hel in qn_hel.")
+      end if
+    end function find_hel_index
+
+    integer function find_sub_index (self, qn) result (i_sub)
+      type(qn_index_map_t), intent(in) :: self
+      type(quantum_numbers_t), dimension(:), intent(in) :: qn
+      integer :: s
+      i_sub = -1
+      do s = 0, self%n_sub
+         if ((all (pack(qn%get_sub (), qn%get_sub () > 0) == s)) &
+              .or. (all (qn%get_sub () == 0) .and. s == 0)) then
+            i_sub = s
+            exit
+         end if
+      end do
+      if (i_sub < 0) then
+         call msg_message ("QN:")
+         call quantum_numbers_write (qn)
+         call msg_bug ("[find_sub_index] could not find sub in qn.")
+      end if
+    end function find_sub_index
+  end subroutine qn_index_map_init
+
+  subroutine qn_index_map_init_trivial (self, int)
+    class(qn_index_map_t), intent(out) :: self
+    class(interaction_t), intent(in) :: int
+    integer :: qn
+    self%n_flv = int%get_n_matrix_elements ()
+    self%n_hel = 1
+    self%n_sub = 0
+    allocate (self%index(self%n_flv, self%n_hel, 0:self%n_sub), source = 0)
+    do qn = 1, self%n_flv
+       self%index(qn, 1, 0) = qn
+    end do
+  end subroutine qn_index_map_init_trivial
+
+  subroutine qn_index_map_write (self, unit)
+    class(qn_index_map_t), intent(in) :: self
+    integer, intent(in), optional :: unit
+    integer :: u, i_flv, i_hel, i_sub
+    u = given_output_unit (unit); if (u < 0) return
+    write (u, *) "flip_hel: ", self%flip_hel
+    do i_flv = 1, self%n_flv
+       if (allocated (self%qn_flv)) &
+            call quantum_numbers_write (self%qn_flv(:, i_flv))
+       write (u, *)
+       do i_hel = 1, self%n_hel
+          if (allocated (self%qn_hel)) then
+             call quantum_numbers_write (self%qn_hel(:, i_hel))
+             write (u, *)
+          end if
+          do i_sub = 0, self%n_sub
+             write (u, *) &
+                  "(", i_flv, ",", i_hel, ",", i_sub, ") => ", self%index(i_flv, i_hel, i_sub)
+          end do
+       end do
+    end do
+  end subroutine qn_index_map_write
+
+  subroutine qn_index_map_set_helicity_flip (self, yorn)
+    class(qn_index_map_t), intent(inout) :: self
+    logical, intent(in) :: yorn
+    integer :: i, i_flv, i_hel, i_hel_new
+    type(quantum_numbers_t), dimension(:, :), allocatable :: qn_hel_flip
+    integer, dimension(:, :, :), allocatable :: index
+    if (.not. allocated (self%qn_hel)) then
+       call msg_bug ("[qn_index_map_set_helicity_flip] &
+            &cannot flip not-given helicity.")
+    end if
+    ! Workaround for ifort (allocate-on-assignmet)
+    allocate (qn_hel_flip (size (self%qn_hel, dim=1),&
+         size (self%qn_hel, dim=2)))
+    allocate (index (self%n_flv, self%n_hel, 0:self%n_sub),&
+         source=self%index)
+    self%flip_hel = yorn
+    if (self%flip_hel) then
+       do i_flv = 1, self%n_flv
+          qn_hel_flip = self%qn_hel
+          do i_hel = 1, self%n_hel
+             do i = 1, size (self%qn_flv, dim=1)
+                if (is_anti_particle (self%qn_flv(i, i_flv))) then
+                   call qn_hel_flip(i, i_hel)%flip_helicity ()
+                end if
+             end do
+          end do
+          do i_hel = 1, self%n_hel
+             i_hel_new = find_hel_index (qn_hel_flip, self%qn_hel(:, i_hel))
+             self%index(i_flv, i_hel_new, :) = index(i_flv, i_hel, :)
+          end do
+       end do
+    end if
+  contains
+    logical function is_anti_particle (qn) result (yorn)
+      type(quantum_numbers_t), intent(in) :: qn
+      type(flavor_t) :: flv
+      flv = qn%get_flavor ()
+      yorn = flv%get_pdg () < 0
+    end function is_anti_particle
+
+    integer function find_hel_index (qn_sort, qn) result (i_hel)
+      type(quantum_numbers_t), dimension(:, :), intent(in) :: qn_sort
+      type(quantum_numbers_t), dimension(:), intent(in) :: qn
+      integer :: j
+      do j = 1, size(qn_sort, dim=2)
+         if (.not. all (qn .hmatch. qn_sort(:, j))) cycle
+         i_hel = j
+         exit
+      end do
+    end function find_hel_index
+  end subroutine qn_index_map_set_helicity_flip
+
+  integer function qn_index_map_get_index (self, i_flv, i_hel, i_sub) result (index)
+    class(qn_index_map_t), intent(in) :: self
+    integer, intent(in) :: i_flv
+    integer, intent(in), optional :: i_hel
+    integer, intent(in), optional :: i_sub
+    integer :: i_sub_opt, i_hel_opt
+    i_sub_opt = 0; if (present (i_sub)) &
+         i_sub_opt = i_sub
+    i_hel_opt = 1; if (present (i_hel)) &
+         i_hel_opt = i_hel
+    index = 0
+    if (.not. allocated (self%index)) then
+       call msg_bug ("[qn_index_map_get_index] The index map is not allocated.")
+    end if
+    index = self%index(i_flv, i_hel_opt, i_sub_opt)
+    if (index <= 0) then
+       call self%write ()
+       call msg_bug ("[qn_index_map_get_index] The index for the given quantum numbers could not be retrieved.")
+    end if
+  end function qn_index_map_get_index
+
+  integer function qn_index_map_get_n_flv (self) result (n_flv)
+    class(qn_index_map_t), intent(in) :: self
+    n_flv = self%n_flv
+  end function qn_index_map_get_n_flv
+
+  integer function qn_index_map_get_n_hel (self) result (n_hel)
+    class(qn_index_map_t), intent(in) :: self
+    n_hel = self%n_hel
+  end function qn_index_map_get_n_hel
+
+  integer function qn_index_map_get_n_sub (self) result (n_sub)
+    class(qn_index_map_t), intent(in) :: self
+    n_sub = self%n_sub
+  end function qn_index_map_get_n_sub
+
   subroutine external_link_set (link, int, i)
     type(external_link_t), intent(out) :: link
     type(interaction_t), target, intent(in) :: int
@@ -497,12 +737,20 @@ contains
        testflag = testflag)
   end subroutine interaction_write_state_matrix
 
-  subroutine interaction_reduce_state_matrix (int, qn_mask)
+  subroutine interaction_reduce_state_matrix (int, qn_mask, keep_order)
     class(interaction_t), intent(inout) :: int
     type(quantum_numbers_mask_t), intent(in), dimension(:) :: qn_mask
+    logical, optional, intent(in) :: keep_order
     type(state_matrix_t) :: state
-    call int%state_matrix%reduce (qn_mask, state)
+    logical :: opt_keep_order
+    opt_keep_order = .false.
+    if (present (keep_order)) opt_keep_order = keep_order
+    call int%state_matrix%reduce (qn_mask, state, keep_me_index = keep_order)
     int%state_matrix = state
+    if (opt_keep_order) then
+       call int%state_matrix%reorder_me (state)
+       int%state_matrix = state
+    end if
   end subroutine interaction_reduce_state_matrix
 
   subroutine interaction_assign (int_out, int_in)
@@ -551,19 +799,20 @@ contains
   end subroutine interaction_assign
 
   subroutine interaction_add_state &
-       (int, qn, index, value, sum_values, counter_index, me_index)
+       (int, qn, index, value, sum_values, counter_index, ignore_sub, me_index)
     class(interaction_t), intent(inout) :: int
     type(quantum_numbers_t), dimension(:), intent(in) :: qn
     integer, intent(in), optional :: index
     complex(default), intent(in), optional :: value
     logical, intent(in), optional :: sum_values
     integer, intent(in), optional :: counter_index
+    logical, intent(in), optional :: ignore_sub
     integer, intent(out), optional :: me_index
     type(quantum_numbers_t), dimension(size(qn)) :: qn_tmp
     qn_tmp = qn
     call qn_tmp%undefine (int%mask)
     call int%state_matrix%add_state (qn_tmp, index, value, sum_values, &
-         counter_index, me_index)
+         counter_index, ignore_sub, me_index)
     int%update_values = .true.
   end subroutine interaction_add_state
 
@@ -592,6 +841,12 @@ contains
     n = int%state_matrix%get_n_matrix_elements ()
   end function interaction_get_n_matrix_elements
 
+  function interaction_get_state_depth (int) result (n)
+    integer :: n
+    class(interaction_t), intent(in) :: int
+    n = int%state_matrix%get_depth ()
+  end function interaction_get_state_depth
+
   function interaction_get_n_in_helicities (int) result (n_hel)
     integer :: n_hel
     class(interaction_t), intent(in) :: int
@@ -634,18 +889,19 @@ contains
     norm = int%state_matrix%get_norm ()
   end function interaction_get_norm
 
-  pure function interaction_get_n_sub (int) result (n_sub)
+  function interaction_get_n_sub (int) result (n_sub)
     integer :: n_sub
     class(interaction_t), intent(in) :: int
     n_sub = int%state_matrix%get_n_sub ()
   end function interaction_get_n_sub
 
-  function interaction_get_quantum_numbers_single (int, i) result (qn)
+  function interaction_get_quantum_numbers_single (int, i, by_me_index) result (qn)
     type(quantum_numbers_t), dimension(:), allocatable :: qn
     class(interaction_t), intent(in), target :: int
     integer, intent(in) :: i
+    logical, intent(in), optional :: by_me_index
     allocate (qn (int%state_matrix%get_depth ()))
-    qn = int%state_matrix%get_quantum_number (i)
+    qn = int%state_matrix%get_quantum_number (i, by_me_index)
   end function interaction_get_quantum_numbers_single
 
   function interaction_get_quantum_numbers_all (int) result (qn)
@@ -690,6 +946,14 @@ contains
     end do
   end subroutine interaction_get_quantum_numbers_all_sub
 
+  subroutine interaction_get_flavors (int, only_elementary, qn_mask, flv)
+    class(interaction_t), intent(in), target :: int
+    logical, intent(in) :: only_elementary
+    type(quantum_numbers_mask_t), intent(in), dimension(:), optional :: qn_mask
+    integer, intent(out), dimension(:,:), allocatable :: flv
+    call int%state_matrix%get_flavors (only_elementary, qn_mask, flv)
+  end subroutine interaction_get_flavors
+
   subroutine interaction_get_quantum_numbers_mask (int, qn_mask, qn)
     class(interaction_t), intent(in) :: int
     type(quantum_numbers_mask_t), intent(in) :: qn_mask
@@ -736,10 +1000,11 @@ contains
     call int%state_matrix%set_matrix_element (value)
   end subroutine interaction_set_matrix_element_all
 
-  subroutine interaction_set_matrix_element_array (int, value)
+  subroutine interaction_set_matrix_element_array (int, value, range)
     class(interaction_t), intent(inout) :: int
-    complex(default), dimension(:), intent(in) :: value
-    call int%state_matrix%set_matrix_element (value)
+    complex(default), intent(in), dimension(:) :: value
+    integer, intent(in), dimension(:), optional :: range
+    call int%state_matrix%set_matrix_element (value, range)
   end subroutine interaction_set_matrix_element_array
 
   pure subroutine interaction_set_matrix_element_single (int, i, value)
@@ -877,6 +1142,15 @@ contains
     call int%state_matrix%evaluate_me_sum (i, int1%state_matrix, index1)
   end subroutine interaction_evaluate_me_sum
 
+  subroutine interaction_tag_hard_process (int, tag)
+    class(interaction_t), intent(inout) :: int
+    integer, dimension(:), intent(in), optional :: tag
+    type(state_matrix_t) :: state
+    call int%state_matrix%tag_hard_process (state, tag)
+    call int%state_matrix%final ()
+    int%state_matrix = state
+  end subroutine interaction_tag_hard_process
+
   function interaction_get_tag (int) result (tag)
     class(interaction_t), intent(in) :: int
     integer :: tag
@@ -1388,32 +1662,27 @@ contains
   end subroutine interaction_pacify_momenta
 
   subroutine interaction_declare_subtraction (int, n_sub)
-    type(interaction_t), intent(inout), target :: int
+    class(interaction_t), intent(inout), target :: int
     integer, intent(in) :: n_sub
+    integer :: i_sub
     type(state_iterator_t) :: it
     type(quantum_numbers_t), dimension(:), allocatable :: qn
-    integer :: i, s
-    integer :: n_me_orig
-    complex(default), dimension(:), allocatable :: me_orig
-    call it%init (int%state_matrix)
-    i = 1; n_me_orig = int%state_matrix%get_n_matrix_elements ()
-    allocate (me_orig (n_me_orig))
-    allocate (qn (it%get_depth ()))
-    do while (it%is_valid () .and. i <= n_me_orig)
-       qn = it%get_quantum_numbers ()
-       me_orig (i) = it%get_matrix_element ()
-       do s = 1, n_sub
-          call qn%set_subtraction_index (s)
-          call int%state_matrix%add_state (qn)
+    type(state_matrix_t) :: state_matrix
+    call state_matrix%init (store_values = .true.)
+    allocate (qn (int%get_state_depth ()))
+    do i_sub = 0, n_sub
+       call it%init (int%state_matrix)
+       do while (it%is_valid ())
+          qn = it%get_quantum_numbers ()
+          call qn%set_subtraction_index (i_sub)
+          call state_matrix%add_state (qn, value = it%get_matrix_element ())
+          call it%advance ()
        end do
-       call it%advance ()
-       i = i + 1
-    end do
-    call int%state_matrix%freeze()
-    do i = 1, n_me_orig
-       call int%state_matrix%set_matrix_element (i, me_orig(i))
-       call int%state_matrix%set_matrix_element (i + n_me_orig, me_orig(i))
     end do
+    call state_matrix%freeze ()
+    call state_matrix%set_n_sub ()
+    call int%state_matrix%final ()
+    int%state_matrix = state_matrix
   end subroutine interaction_declare_subtraction
 
   subroutine find_connections (int1, int2, n, connection_index)
@@ -1423,14 +1692,14 @@ contains
     integer, dimension(:,:), allocatable :: conn_index_tmp
     integer, dimension(:), allocatable :: ordering
     integer :: i, j, k
-    type(external_link_t) :: link2, link1
-    type(interaction_t), pointer :: int_link, int_link1
+    type(external_link_t) :: link1, link2
+    type(interaction_t), pointer :: int_link1, int_link2
     n = 0
     do i = 1, size (int2%source)
        link2 = interaction_get_ultimate_source (int2, i)
        if (external_link_is_set (link2)) then
-          int_link => external_link_get_ptr (link2)
-          if (int_link%tag == int1%tag) then
+          int_link2 => external_link_get_ptr (link2)
+          if (int_link2%tag == int1%tag) then
              n = n + 1
           else
              k = external_link_get_index (link2)
@@ -1438,10 +1707,9 @@ contains
                 link1 = interaction_get_ultimate_source (int1, j)
                 if (external_link_is_set (link1)) then
                    int_link1 => external_link_get_ptr (link1)
-                   if (int_link1%tag == int_link%tag) then
-                      if (external_link_get_index (link1) == k) then
-                         n = n + 1
-                      end if
+                   if (int_link1%tag == int_link2%tag) then
+                      if (external_link_get_index (link1) == k) &
+                           n = n + 1
                    end if
                 end if
              end do
@@ -1453,8 +1721,8 @@ contains
     do i = 1, size (int2%source)
        link2 = interaction_get_ultimate_source (int2, i)
        if (external_link_is_set (link2)) then
-          int_link => external_link_get_ptr (link2)
-          if (int_link%tag == int1%tag) then
+          int_link2 => external_link_get_ptr (link2)
+          if (int_link2%tag == int1%tag) then
              n = n + 1
              conn_index_tmp(n,1) = external_link_get_index (int2%source(i))
              conn_index_tmp(n,2) = i
@@ -1464,7 +1732,7 @@ contains
                 link1 = interaction_get_ultimate_source (int1, j)
                 if (external_link_is_set (link1)) then
                    int_link1 => external_link_get_ptr (link1)
-                   if (int_link1%tag == int_link%tag) then
+                   if (int_link1%tag == int_link2%tag) then
                       if (external_link_get_index (link1) == k) then
                          n = n + 1
                          conn_index_tmp(n,1) = j
