@@ -1,19 +1,12 @@
-! WHIZARD 2.4.1 Mar 24 2017
+! WHIZARD 2.6.4 Aug 23 2018
 !
-! Copyright (C) 1999-2017 by
+! Copyright (C) 1999-2018 by
 !     Wolfgang Kilian <kilian@physik.uni-siegen.de>
 !     Thorsten Ohl <ohl@physik.uni-wuerzburg.de>
 !     Juergen Reuter <juergen.reuter@desy.de>
 !
 !     with contributions from
-!     Fabian Bach <fabian.bach@t-online.de>
-!     Bijan Chokoufe <bijan.chokoufe@desy.de>
-!     Christian Speckner <cnspeckn@googlemail.com>
-!     So Young Shim <soyoung.shim@desy.de>
-!     Florian Staub <florian.staub@cern.ch>
-!     Christian Weiss <christian.weiss@desy.de>
-!     and Hans-Werner Boschmann, Felix Braam,
-!     Sebastian Schmidt, So-young Shim, Daniel Wiesler
+!     cf. main AUTHORS file
 !
 ! WHIZARD is free software; you can redistribute it and/or modify it
 ! under the terms of the GNU General Public License as published by
@@ -1394,8 +1387,12 @@ contains
 
   subroutine model_data_final (model)
     class(model_data_t), intent(inout) :: model
-    deallocate (model%par_real)
-    deallocate (model%par_complex)
+    if (associated (model%par_real)) then
+       deallocate (model%par_real)
+    end if
+    if (associated (model%par_complex)) then
+       deallocate (model%par_complex)
+    end if
   end subroutine model_data_final
 
   subroutine model_data_write (model, unit, verbose, &
