@@ -1,19 +1,12 @@
-! WHIZARD 2.4.1 Mar 24 2017
+! WHIZARD 2.6.4 Aug 23 2018
 !
-! Copyright (C) 1999-2017 by
+! Copyright (C) 1999-2018 by
 !     Wolfgang Kilian <kilian@physik.uni-siegen.de>
 !     Thorsten Ohl <ohl@physik.uni-wuerzburg.de>
 !     Juergen Reuter <juergen.reuter@desy.de>
 !
 !     with contributions from
-!     Fabian Bach <fabian.bach@t-online.de>
-!     Bijan Chokoufe <bijan.chokoufe@desy.de>
-!     Christian Speckner <cnspeckn@googlemail.com>
-!     So Young Shim <soyoung.shim@desy.de>
-!     Florian Staub <florian.staub@cern.ch>
-!     Christian Weiss <christian.weiss@desy.de>
-!     and Hans-Werner Boschmann, Felix Braam,
-!     Sebastian Schmidt, So-young Shim, Daniel Wiesler
+!     cf. main AUTHORS file
 !
 ! WHIZARD is free software; you can redistribute it and/or modify it
 ! under the terms of the GNU General Public License as published by
@@ -50,6 +43,7 @@ module quantum_numbers
   public :: quantum_numbers_get_color
   public :: quantum_numbers_get_helicity
   public :: quantum_numbers_get_color_type
+  public :: quantum_numbers_eq_wo_sub
   public :: assignment(=)
   public :: quantum_numbers_are_compatible
   public :: quantum_numbers_are_physical
@@ -105,6 +99,7 @@ module quantum_numbers
      procedure :: set_color_ghost => quantum_numbers_set_color_ghost
      procedure :: set_model => quantum_numbers_set_model
      procedure :: tag_radiated => quantum_numbers_tag_radiated
+     procedure :: tag_hard_process => quantum_numbers_tag_hard_process
      procedure :: set_subtraction_index => quantum_numbers_set_subtraction_index
      procedure :: get_subtraction_index => quantum_numbers_get_subtraction_index
      procedure :: get_color_type => quantum_numbers_get_color_type
@@ -112,20 +107,24 @@ module quantum_numbers
      procedure :: are_associated => quantum_numbers_are_associated
      procedure :: are_diagonal => quantum_numbers_are_diagonal
      procedure :: is_color_ghost => quantum_numbers_is_color_ghost
+     procedure :: are_hard_process => quantum_numbers_are_hard_process
      generic :: operator(.match.) => quantum_numbers_match
      generic :: operator(.fmatch.) => quantum_numbers_match_f
+     generic :: operator(.hmatch.) => quantum_numbers_match_h
      generic :: operator(.fhmatch.) => quantum_numbers_match_fh
      generic :: operator(.dhmatch.) => quantum_numbers_match_hel_diag
      generic :: operator(==) => quantum_numbers_eq
      generic :: operator(/=) => quantum_numbers_neq
      procedure, private :: quantum_numbers_match
      procedure, private :: quantum_numbers_match_f
+     procedure, private :: quantum_numbers_match_h
      procedure, private :: quantum_numbers_match_fh
      procedure, private :: quantum_numbers_match_hel_diag
      procedure, private :: quantum_numbers_eq
      procedure, private :: quantum_numbers_neq
      procedure :: add_color_offset => quantum_numbers_add_color_offset
      procedure :: invert_color => quantum_numbers_invert_color
+     procedure :: flip_helicity => quantum_numbers_flip_helicity
      procedure :: undefine => quantum_numbers_undefine
      procedure :: undefined => quantum_numbers_undefined0
      procedure :: are_redundant => quantum_numbers_are_redundant
@@ -330,7 +329,7 @@ contains
        call qn%h%write (u)
     end if
     if (qn%sub > 0) &
-       write (u, "(A,I1)", advance = "no") " SUB = ", qn%sub
+       write (u, "(A,I0)", advance = "no") " SUB = ", qn%sub
     write (u, "(A)", advance="no")  "]"
   end subroutine quantum_numbers_write_single
 
@@ -366,7 +365,7 @@ contains
           call qn(i)%h%write (u)
        end if
        if (qn(i)%sub > 0) &
-          write (u, "(A,I1)", advance = "no") " SUB = ", qn(i)%sub
+          write (u, "(A,I2)", advance = "no") " SUB = ", qn(i)%sub
     end do
     write (u, "(A)", advance = "no")  "]"
   end subroutine quantum_numbers_write_array
@@ -429,6 +428,11 @@ contains
     call qn%f%tag_radiated ()
   end subroutine quantum_numbers_tag_radiated
 
+  elemental subroutine quantum_numbers_tag_hard_process (qn)
+    class(quantum_numbers_t), intent(inout) :: qn
+    call qn%f%tag_hard_process ()
+  end subroutine quantum_numbers_tag_hard_process
+
   elemental subroutine quantum_numbers_set_subtraction_index (qn, i)
     class(quantum_numbers_t), intent(inout) :: qn
     integer, intent(in) :: i
@@ -471,6 +475,12 @@ contains
     ghost = qn%c%is_ghost ()
   end function quantum_numbers_is_color_ghost
 
+  elemental function quantum_numbers_are_hard_process (qn) result (hard_process)
+    logical :: hard_process
+    class(quantum_numbers_t), intent(in) :: qn
+    hard_process = qn%f%is_hard_process ()
+  end function quantum_numbers_are_hard_process
+
   elemental function quantum_numbers_match (qn1, qn2) result (match)
     logical :: match
     class(quantum_numbers_t), intent(in) :: qn1, qn2
@@ -485,6 +495,12 @@ contains
     match = (qn1%f .match. qn2%f)
   end function quantum_numbers_match_f
 
+  elemental function quantum_numbers_match_h (qn1, qn2) result (match)
+    logical :: match
+    class(quantum_numbers_t), intent(in) :: qn1, qn2
+    match = (qn1%h .match. qn2%h)
+  end function quantum_numbers_match_h
+
   elemental function quantum_numbers_match_fh (qn1, qn2) result (match)
     logical :: match
     class(quantum_numbers_t), intent(in) :: qn1, qn2
@@ -500,6 +516,14 @@ contains
          (qn1%h .dmatch. qn2%h)
   end function quantum_numbers_match_hel_diag
 
+  elemental function quantum_numbers_eq_wo_sub (qn1, qn2) result (eq)
+    logical :: eq
+    type(quantum_numbers_t), intent(in) :: qn1, qn2
+    eq = (qn1%f == qn2%f) .and. &
+         (qn1%c == qn2%c) .and. &
+         (qn1%h == qn2%h)
+  end function quantum_numbers_eq_wo_sub
+
   elemental function quantum_numbers_eq (qn1, qn2) result (eq)
     logical :: eq
     class(quantum_numbers_t), intent(in) :: qn1, qn2
@@ -625,6 +649,11 @@ contains
     call qn%c%invert ()
   end subroutine quantum_numbers_invert_color
 
+  elemental subroutine quantum_numbers_flip_helicity (qn)
+    class(quantum_numbers_t), intent(inout) :: qn
+    call qn%h%flip ()
+  end subroutine quantum_numbers_flip_helicity
+
   function merge_quantum_numbers0 (qn1, qn2) result (qn3)
     type(quantum_numbers_t) :: qn3
     type(quantum_numbers_t), intent(in) :: qn1, qn2
@@ -828,7 +857,7 @@ contains
           call qn%h%diagonalize ()
        end if
     end if
-    if (mask%sub > 0) qn%sub = 0
+    if (mask%sub > 0)  qn%sub = 0
   end subroutine quantum_numbers_undefine
 
   function quantum_numbers_undefined0 (qn, mask) result (qn_new)
