@@ -1,19 +1,12 @@
-! WHIZARD 2.4.1 Mar 24 2017
+! WHIZARD 2.6.4 Aug 23 2018
 !
-! Copyright (C) 1999-2017 by
+! Copyright (C) 1999-2018 by
 !     Wolfgang Kilian <kilian@physik.uni-siegen.de>
 !     Thorsten Ohl <ohl@physik.uni-wuerzburg.de>
 !     Juergen Reuter <juergen.reuter@desy.de>
 !
 !     with contributions from
-!     Fabian Bach <fabian.bach@t-online.de>
-!     Bijan Chokoufe <bijan.chokoufe@desy.de>
-!     Christian Speckner <cnspeckn@googlemail.com>
-!     So Young Shim <soyoung.shim@desy.de>
-!     Florian Staub <florian.staub@cern.ch>
-!     Christian Weiss <christian.weiss@desy.de>
-!     and Hans-Werner Boschmann, Felix Braam,
-!     Sebastian Schmidt, So-young Shim, Daniel Wiesler
+!     cf. main AUTHORS file
 !
 ! WHIZARD is free software; you can redistribute it and/or modify it
 ! under the terms of the GNU General Public License as published by
@@ -85,7 +78,7 @@ module state_matrices
      integer :: n_counters = 0
      complex(default), dimension(:), allocatable :: me
      real(default) :: norm = 1
-     integer :: n_sub = 0
+     integer :: n_sub = -1
    contains
      procedure :: init => state_matrix_init
      procedure :: final => state_matrix_final
@@ -93,19 +86,25 @@ module state_matrices
      procedure :: write_raw => state_matrix_write_raw
      procedure :: read_raw => state_matrix_read_raw
      procedure :: set_model => state_matrix_set_model
+     procedure :: tag_hard_process => state_matrix_tag_hard_process
      procedure :: is_defined => state_matrix_is_defined
      procedure :: is_empty => state_matrix_is_empty
-     procedure :: get_n_matrix_elements => state_matrix_get_n_matrix_elements
+     generic :: get_n_matrix_elements => get_n_matrix_elements_all, get_n_matrix_elements_mask
+     procedure :: get_n_matrix_elements_all => state_matrix_get_n_matrix_elements_all
+     procedure :: get_n_matrix_elements_mask => state_matrix_get_n_matrix_elements_mask
      procedure :: get_me_size => state_matrix_get_me_size
      procedure :: compute_n_sub => state_matrix_compute_n_sub
+     procedure :: set_n_sub => state_matrix_set_n_sub
      procedure :: get_n_sub => state_matrix_get_n_sub
      procedure :: get_n_leaves => state_matrix_get_n_leaves
      procedure :: get_depth => state_matrix_get_depth
      procedure :: get_norm => state_matrix_get_norm
      procedure :: get_quantum_number => &
         state_matrix_get_quantum_number
-     procedure :: get_quantum_numbers => &
-        state_matrix_get_quantum_numbers
+     generic :: get_quantum_numbers => get_quantum_numbers_all, get_quantum_numbers_mask
+     procedure :: get_quantum_numbers_all => state_matrix_get_quantum_numbers_all
+     procedure :: get_quantum_numbers_mask => state_matrix_get_quantum_numbers_mask
+     procedure :: get_flavors => state_matrix_get_flavors
      generic :: get_matrix_element => get_matrix_element_single
      generic :: get_matrix_element => get_matrix_element_array
      procedure :: get_matrix_element_single => &
@@ -116,6 +115,7 @@ module state_matrices
      procedure :: add_state => state_matrix_add_state
      procedure :: collapse => state_matrix_collapse
      procedure :: reduce => state_matrix_reduce
+     procedure :: reorder_me => state_matrix_reorder_me
      procedure :: freeze => state_matrix_freeze
      generic :: set_matrix_element => set_matrix_element_qn
      generic :: set_matrix_element => set_matrix_element_all
@@ -318,9 +318,9 @@ contains
     do while (associated (current))
        write (u, "(A)", advance="no")  repeat (" ", i)
        call node_write (current, me_array, verbose = verb, &
-          unit = u, col_verbose = col_verbose, testflag = testflag)
+            unit = u, col_verbose = col_verbose, testflag = testflag)
        call node_write_rec (current, me_array, verbose = verb, &
-          indent = i + 2, unit = u, col_verbose = col_verbose, testflag = testflag)
+            indent = i + 2, unit = u, col_verbose = col_verbose, testflag = testflag)
        current => current%next
     end do
   end subroutine node_write_rec
@@ -410,11 +410,11 @@ contains
     if (associated (state%root)) then
        if (allocated (state%me)) then
           call node_write_rec (state%root, state%me, verbose = verbose, &
-             indent = 1, unit = u, col_verbose = col_verbose, &
-             testflag = testflag)
+               indent = 1, unit = u, col_verbose = col_verbose, &
+               testflag = testflag)
        else
           call node_write_rec (state%root, verbose = verbose, indent = 1, &
-             unit = u, col_verbose = col_verbose, testflag = testflag)
+               unit = u, col_verbose = col_verbose, testflag = testflag)
        end if
     end if
     if (present (write_value_list)) then
@@ -518,6 +518,32 @@ contains
     end do
   end subroutine state_matrix_set_model
 
+  subroutine state_matrix_tag_hard_process (state, tagged_state, tag)
+    class(state_matrix_t), intent(in), target :: state
+    type(state_matrix_t), intent(out) :: tagged_state
+    integer, dimension(:), intent(in), optional :: tag
+    type(state_iterator_t) :: it
+    type(quantum_numbers_t), dimension(:), allocatable :: qn
+    complex(default) :: value
+    integer :: i
+    call tagged_state%init (store_values = .true.)
+    call it%init (state)
+    do while (it%is_valid ())
+       qn = it%get_quantum_numbers ()
+       value = it%get_matrix_element ()
+       if (present (tag)) then
+          do i = 1, size (tag)
+             call qn(tag(i))%tag_hard_process ()
+          end do
+       else
+          call qn%tag_hard_process ()
+       end if
+       call tagged_state%add_state (qn, index = it%get_me_index (), value = value)
+       call it%advance ()
+    end do
+    call tagged_state%freeze ()
+  end subroutine state_matrix_tag_hard_process
+
   elemental function state_matrix_is_defined (state) result (defined)
     logical :: defined
     class(state_matrix_t), intent(in) :: state
@@ -530,11 +556,30 @@ contains
     flag = state%depth == 0
   end function state_matrix_is_empty
 
-  pure function state_matrix_get_n_matrix_elements (state) result (n)
+  pure function state_matrix_get_n_matrix_elements_all (state) result (n)
     integer :: n
     class(state_matrix_t), intent(in) :: state
     n = state%n_matrix_elements
-  end function state_matrix_get_n_matrix_elements
+  end function state_matrix_get_n_matrix_elements_all
+
+  function state_matrix_get_n_matrix_elements_mask (state, qn_mask) result (n)
+    integer :: n
+    class(state_matrix_t), intent(in) :: state
+    type(quantum_numbers_mask_t), intent(in), dimension(:) :: qn_mask
+    type(state_iterator_t) :: it
+    type(quantum_numbers_t), dimension(size(qn_mask)) :: qn
+    type(state_matrix_t) :: state_tmp
+    call state_tmp%init ()
+    call it%init (state)
+    do while (it%is_valid ())
+       qn = it%get_quantum_numbers ()
+       call qn%undefine (qn_mask)
+       call state_tmp%add_state (qn)
+       call it%advance ()
+    end do
+    n = state_tmp%n_matrix_elements
+    call state_tmp%final ()
+  end function state_matrix_get_n_matrix_elements_mask
 
   pure function state_matrix_get_me_size (state) result (n)
     integer :: n
@@ -546,24 +591,48 @@ contains
     end if
   end function state_matrix_get_me_size
 
-  subroutine state_matrix_compute_n_sub (state)
-    class(state_matrix_t), intent(inout) :: state
+  function state_matrix_compute_n_sub (state) result (n_sub)
+    integer :: n_sub
+    class(state_matrix_t), intent(in) :: state
     type(state_iterator_t) :: it
     type(quantum_numbers_t), dimension(state%depth) :: qn
-    integer :: sub
+    integer :: sub, sub_pos
+    n_sub = 0
     call it%init (state)
     do while (it%is_valid ())
        qn = it%get_quantum_numbers ()
-       sub = qn(1)%get_sub ()
-       if (sub > state%n_sub) state%n_sub = sub
+       sub = 0
+       sub_pos = qn_array_sub_pos ()
+       if (sub_pos > 0)  sub = qn(sub_pos)%get_sub ()
+       if (sub > n_sub)  n_sub = sub
        call it%advance ()
     end do
-  end subroutine state_matrix_compute_n_sub
+  contains
+    function qn_array_sub_pos () result (pos)
+      integer :: pos
+      integer :: i
+      pos = 0
+      do i = 1, state%depth
+         if (qn(i)%get_sub () > 0) then
+            pos = i
+            exit
+         end if
+      end do
+    end function qn_array_sub_pos
+  end function state_matrix_compute_n_sub
 
-  pure function state_matrix_get_n_sub (state) result (n)
-    integer :: n
+  subroutine state_matrix_set_n_sub (state)
+    class(state_matrix_t), intent(inout) :: state
+    state%n_sub = state%compute_n_sub ()
+  end subroutine state_matrix_set_n_sub
+
+  function state_matrix_get_n_sub (state) result (n_sub)
+    integer :: n_sub
     class(state_matrix_t), intent(in) :: state
-    n = state%n_sub
+    if (state%n_sub < 0) then
+       call msg_bug ("[state_matrix_get_n_sub] number of subtractions not set.")
+    end if
+    n_sub = state%n_sub
   end function state_matrix_get_n_sub
 
   function state_matrix_get_n_leaves (state) result (n)
@@ -590,34 +659,100 @@ contains
     norm = state%norm
   end function state_matrix_get_norm
 
-  function state_matrix_get_quantum_number (state, i) result (qn)
+  function state_matrix_get_quantum_number (state, i, by_me_index) result (qn)
     class(state_matrix_t), intent(in), target :: state
     integer, intent(in) :: i
+    logical, intent(in), optional :: by_me_index
+    logical :: opt_by_me_index
     type(quantum_numbers_t), dimension(state%depth) :: qn
     type(state_iterator_t) :: it
     integer :: k
+    opt_by_me_index = .false.
+    if (present (by_me_index)) opt_by_me_index = by_me_index
     k = 0
     call it%init (state)
     do while (it%is_valid ())
-       k = k + 1
+       if (opt_by_me_index) then
+          k = it%get_me_index ()
+       else
+          k = k + 1
+       end if
        if (k == i) then
           qn = it%get_quantum_numbers ()
-          return
+          exit
        end if
        call it%advance ()
     end do
   end function state_matrix_get_quantum_number
 
-  subroutine state_matrix_get_quantum_numbers (state, qn)
-    type(quantum_numbers_t), dimension(:,:), allocatable, intent(out) :: qn
+  subroutine state_matrix_get_quantum_numbers_all (state, qn)
     class(state_matrix_t), intent(in), target :: state
+    type(quantum_numbers_t), intent(out), dimension(:,:), allocatable :: qn
     integer :: i
     allocate (qn (state%get_n_matrix_elements (), &
        state%get_depth()))
     do i = 1, state%get_n_matrix_elements ()
        qn (i, :) = state%get_quantum_number (i)
     end do
-  end subroutine state_matrix_get_quantum_numbers
+  end subroutine state_matrix_get_quantum_numbers_all
+
+  subroutine state_matrix_get_quantum_numbers_mask (state, qn_mask, qn)
+    class(state_matrix_t), intent(in), target :: state
+    type(quantum_numbers_mask_t), intent(in), dimension(:) :: qn_mask
+    type(quantum_numbers_t), intent(out), dimension(:,:), allocatable :: qn
+    type(quantum_numbers_t), dimension(:), allocatable :: qn_tmp
+    type(state_matrix_t) :: state_tmp
+    type(state_iterator_t) :: it
+    integer :: i, n
+    n = state%get_n_matrix_elements (qn_mask)
+    allocate (qn (n, state%get_depth ()))
+    allocate (qn_tmp (state%get_depth ()))
+    call it%init (state)
+    call state_tmp%init ()
+    do while (it%is_valid ())
+       qn_tmp = it%get_quantum_numbers ()
+       call qn_tmp%undefine (qn_mask)
+       call state_tmp%add_state (qn_tmp)
+       call it%advance ()
+    end do
+    do i = 1, n
+       qn (i, :) = state_tmp%get_quantum_number (i)
+    end do
+    call state_tmp%final ()
+  end subroutine state_matrix_get_quantum_numbers_mask
+
+  subroutine state_matrix_get_flavors (state, only_elementary, qn_mask, flv)
+    class(state_matrix_t), intent(in), target :: state
+    logical, intent(in) :: only_elementary
+    type(quantum_numbers_mask_t), intent(in), dimension(:), optional :: qn_mask
+    integer, intent(out), dimension(:,:), allocatable :: flv
+    type(quantum_numbers_t), dimension(:,:), allocatable :: qn
+    integer :: i_flv, n_partons
+    type(flavor_t), dimension(:), allocatable :: flv_flv
+    if (present (qn_mask)) then
+       call state%get_quantum_numbers (qn_mask, qn)
+    else
+       call state%get_quantum_numbers (qn)
+    end if
+    allocate (flv_flv (size (qn, dim=2)))
+    if (only_elementary) then
+       flv_flv = qn(1, :)%get_flavor ()
+       n_partons = count (is_elementary (flv_flv%get_pdg ()))
+    end if
+    allocate (flv (n_partons, size (qn, dim=1)))
+    associate (n_flv => size (qn, dim=1))
+      do i_flv = 1, size (qn, dim=1)
+         flv_flv = qn(i_flv, :)%get_flavor ()
+         flv(:, i_flv) = pack (flv_flv%get_pdg (), is_elementary(flv_flv%get_pdg()))
+      end do
+    end associate
+  contains
+    elemental function is_elementary (pdg)
+      logical :: is_elementary
+      integer, intent(in) :: pdg
+      is_elementary = abs(pdg) /= 2212 .and. abs(pdg) /= 92 .and. abs(pdg) /= 93
+    end function is_elementary
+  end subroutine state_matrix_get_flavors
 
   elemental function state_matrix_get_matrix_element_single (state, i) result (me)
     complex(default) :: me
@@ -663,14 +798,15 @@ contains
     end function node_get_max_color_value
   end function state_matrix_get_max_color_value
 
-  subroutine state_matrix_add_state &
-       (state, qn, index, value, sum_values, counter_index, me_index)
+  subroutine state_matrix_add_state (state, qn, index, value, &
+         sum_values, counter_index, ignore_sub, me_index)
     class(state_matrix_t), intent(inout) :: state
     type(quantum_numbers_t), dimension(:), intent(in) :: qn
     integer, intent(in), optional :: index
     complex(default), intent(in), optional :: value
     logical, intent(in), optional :: sum_values
     integer, intent(in), optional :: counter_index
+    logical, intent(in), optional :: ignore_sub
     integer, intent(out), optional :: me_index
     logical :: set_index, get_index, add
     set_index = present (index)
@@ -692,7 +828,15 @@ contains
        match = .false.
        child => parent%child_first
        SCAN_CHILDREN: do while (associated (child))
-          match = child%qn == qn(1)
+          if (present (ignore_sub)) then
+             if (ignore_sub) then
+                match = quantum_numbers_eq_wo_sub (child%qn, qn(1))
+             else
+                match = child%qn == qn(1)
+             end if
+          else
+             match = child%qn == qn(1)
+          end if
           if (match)  exit SCAN_CHILDREN
           child => child%next
        end do SCAN_CHILDREN
@@ -743,23 +887,80 @@ contains
     end if
    end subroutine state_matrix_collapse
 
-  subroutine state_matrix_reduce (state, mask, red_state)
+  subroutine state_matrix_reduce (state, mask, red_state, keep_me_index)
     class(state_matrix_t), intent(in), target :: state
     type(quantum_numbers_mask_t), dimension(:), intent(in) :: mask
     type(state_matrix_t), intent(out) :: red_state
+    logical, optional, intent(in)  :: keep_me_index
+    logical :: opt_keep_me_index
     type(state_iterator_t) :: it
     type(quantum_numbers_t), dimension(size(mask)) :: qn
+    opt_keep_me_index = .false.
+    if (present (keep_me_index)) opt_keep_me_index = keep_me_index
     call red_state%init ()
     call it%init (state)
     do while (it%is_valid ())
        qn = it%get_quantum_numbers ()
        call qn%undefine (mask)
-       call red_state%add_state (qn)
+       if (opt_keep_me_index) then
+          call red_state%add_state (qn, index = it%get_me_index ())
+       else
+          call red_state%add_state (qn)
+       end if
        call it%advance ()
     end do
-    call red_state%freeze ()
+    if (.not. opt_keep_me_index) then
+       call red_state%freeze ()
+    end if
   end subroutine state_matrix_reduce
 
+  subroutine state_matrix_reorder_me (state, ordered_state)
+    class(state_matrix_t), intent(in), target :: state
+    type(state_matrix_t), intent(out) :: ordered_state
+    type(state_iterator_t) :: it
+    type(quantum_numbers_t), dimension(state%depth) :: qn
+    integer,  dimension(:), allocatable :: me_index
+    integer :: i
+    call ordered_state%init ()
+    call get_me_index_sorted (state, me_index)
+    i = 1; call it%init (state)
+    do while (it%is_valid ())
+       qn = it%get_quantum_numbers ()
+       call ordered_state%add_state (qn, index = me_index(i))
+       i = i + 1; call it%advance ()
+    end do
+    call ordered_state%freeze ()
+  contains
+    subroutine get_me_index_sorted (state, me_index)
+      class(state_matrix_t), intent(in), target :: state
+      integer, dimension(:), allocatable, intent(out) :: me_index
+      type(state_iterator_t) :: it
+      integer :: i, j
+      integer, dimension(:), allocatable :: me_index_unsorted, me_index_sorted
+      associate (n_matrix_elements => state%get_n_matrix_elements ())
+        allocate (me_index(n_matrix_elements), source = 0)
+        allocate (me_index_sorted(n_matrix_elements), source = 0)
+        allocate (me_index_unsorted(n_matrix_elements), source = 0)
+        i = 1; call it%init (state)
+        do while (it%is_valid ())
+           me_index_unsorted(i) = it%get_me_index ()
+           i = i + 1
+           call it%advance ()
+        end do
+        me_index_sorted = sort (me_index_unsorted)
+        ! We do not care about efficiency at this point.
+        UNSORTED: do i = 1, n_matrix_elements
+           SORTED: do j = 1, n_matrix_elements
+              if (me_index_unsorted(i) == me_index_sorted(j)) then
+                 me_index(i) = j
+                 cycle UNSORTED
+              end if
+           end do SORTED
+        end do UNSORTED
+      end associate
+    end subroutine get_me_index_sorted
+  end subroutine state_matrix_reorder_me
+
   subroutine state_matrix_freeze (state)
     class(state_matrix_t), intent(inout), target :: state
     type(state_iterator_t) :: it
@@ -767,7 +968,7 @@ contains
        if (allocated (state%me))  deallocate (state%me)
        allocate (state%me (state%n_matrix_elements))
        state%me = 0
-       call state%compute_n_sub ()
+       call state%set_n_sub ()
     end if
     if (state%leaf_nodes_store_values) then
        call it%init (state)
@@ -801,13 +1002,18 @@ contains
     state%me = value
   end subroutine state_matrix_set_matrix_element_all
 
-  subroutine state_matrix_set_matrix_element_array (state, value)
+  subroutine state_matrix_set_matrix_element_array (state, value, range)
     class(state_matrix_t), intent(inout) :: state
-    complex(default), dimension(:), intent(in) :: value
-    if (.not. allocated (state%me)) then
-       allocate (state%me (size (value)))
+    complex(default), intent(in), dimension(:) :: value
+    integer, intent(in), dimension(:), optional :: range
+    integer :: i, n_me, n_val, i_first, i_last
+    if (present (range)) then
+       state%me(range) = value
+    else
+       if (.not. allocated (state%me)) &
+            allocate (state%me (size (value)))
+       state%me(:) = value
     end if
-    state%me = value
   end subroutine state_matrix_set_matrix_element_array
 
   pure subroutine state_matrix_set_matrix_element_single (state, i, value)
@@ -1189,7 +1395,7 @@ contains
     state%norm = norm
   end subroutine state_matrix_set_norm
 
-  function state_matrix_sum (state) result (value)
+  pure function state_matrix_sum (state) result (value)
     complex(default) :: value
     class(state_matrix_t), intent(in) :: state
     value = sum (state%me)
@@ -1207,7 +1413,7 @@ contains
     do while (it%is_valid ())
        qn = it%get_quantum_numbers ()
        if (present (qn_in)) then
-          if (.not. all (qn .match. qn_in)) then
+          if (.not. all (qn .fhmatch. qn_in)) then
              call it%advance ();  cycle
           end if
        end if
@@ -1328,7 +1534,7 @@ contains
     type(quantum_numbers_t), dimension(state2%depth) :: qn2
     type(quantum_numbers_t), dimension(state1%depth+state2%depth) :: qn3
     complex(default) :: val1, val2
-    call state3%init (store_values=.true.)
+    call state3%init (store_values = .true.)
     call it1%init (state1)
     do while (it1%is_valid ())
        qn1 = it1%get_quantum_numbers ()
@@ -1421,10 +1627,10 @@ contains
     end if
     allocate (single_state (depth))
     do i = 1, depth
-       call single_state(i)%init (store_values=.true.)
+       call single_state(i)%init (store_values = .true.)
     end do
     if (present (correlated_state)) &
-         call correlated_state%init (store_values=.true.)
+         call correlated_state%init (store_values = .true.)
     qn = it%get_quantum_numbers ()
     select case (mode)
     case (FM_SELECT_HELICITY)  ! single branch selected; shortcut
@@ -1451,22 +1657,18 @@ contains
     do while (it%is_valid ())
        qn1 = it%get_quantum_numbers ()
        if (all (qn .match. qn1)) then
-          !!! !!! !!! Workaround for ifort 16.0 standard-semantics bug
-          !!! diagonal = qn1%are_diagonal ()
-          do i = 1, depth
-             diagonal(i) = qn1(i)%are_diagonal ()
-          end do
+          diagonal = qn1%are_diagonal ()
           value = it%get_matrix_element ()
           select case (mode)
           case (FM_IGNORE_HELICITY, FM_CORRELATED_HELICITY)
-             ! trace over diagonal states that match qn
+             !!! trace over diagonal states that match qn
              if (all (diagonal)) then
                 do i = 1, depth
                    call single_state(i)%add_state &
                         ([qn(i)], value=value, sum_values=.true.)
                 end do
              end if
-          case (FM_FACTOR_HELICITY)  ! trace over all other particles
+          case (FM_FACTOR_HELICITY)  !!! trace over all other particles
              do i = 1, depth
                 if (all (diagonal .or. mask(:,i))) then
                    call single_state(i)%add_state &
@@ -1576,11 +1778,7 @@ contains
     do while (it%is_valid ())
        i = i + 1
        flv = it%get_flavor ()
-       !!! !!! !!! Workaround for ifort 16.0 standard-semantics bug
-       !!! pdg = flv%get_pdg ()
-       do j = 1, d
-          pdg(j) = flv(j)%get_pdg ()
-       end do
+       pdg = flv%get_pdg ()
        idx_subset = pack (idx, mask)
        pdg_subset = pack (pdg, mask)
        map_subset = order_abs (pdg_subset)
