@@ -1,19 +1,12 @@
-! WHIZARD 2.4.1 Mar 24 2017
+! WHIZARD 2.6.4 Aug 23 2018
 !
-! Copyright (C) 1999-2017 by
+! Copyright (C) 1999-2018 by
 !     Wolfgang Kilian <kilian@physik.uni-siegen.de>
 !     Thorsten Ohl <ohl@physik.uni-wuerzburg.de>
 !     Juergen Reuter <juergen.reuter@desy.de>
 !
 !     with contributions from
-!     Fabian Bach <fabian.bach@t-online.de>
-!     Bijan Chokoufe <bijan.chokoufe@desy.de>
-!     Christian Speckner <cnspeckn@googlemail.com>
-!     So Young Shim <soyoung.shim@desy.de>
-!     Florian Staub <florian.staub@cern.ch>
-!     Christian Weiss <christian.weiss@desy.de>
-!     and Hans-Werner Boschmann, Felix Braam,
-!     Sebastian Schmidt, So-young Shim, Daniel Wiesler
+!     cf. main AUTHORS file
 !
 ! WHIZARD is free software; you can redistribute it and/or modify it
 ! under the terms of the GNU General Public License as published by
@@ -40,13 +33,17 @@ module recola_wrapper
   use kinds
   use iso_varying_string, string_t => varying_string
   use constants, only: zero
-  use diagnostics, only: msg_fatal
+  use diagnostics, only: msg_fatal, msg_message, msg_debug, msg_debug2, D_ME_METHODS
+  use io_units, only: given_output_unit
 
   implicit none
   private
 
   public :: get_recola_particle_string
   public :: rclwrap_define_process
+  public :: rclwrap_get_new_recola_id
+  public :: rclwrap_get_n_processes
+  public :: rclwrap_request_generate_processes
   public :: rclwrap_generate_processes
   public :: rclwrap_compute_process
   public :: rclwrap_get_amplitude
@@ -99,7 +96,7 @@ module recola_wrapper
   public :: rclwrap_unselect_all_gs_powers_born_amp
   public :: rclwrap_set_resonant_squared_momentum
   public :: rclwrap_compute_running_alpha_s
-  !public :: rclwrap_set_dynamic_settings
+  public :: rclwrap_set_dynamic_settings
   public :: rclwrap_rescale_process
   public :: rclwrap_get_polarized_squared_amplitude
   public :: rclwrap_compute_color_correlation
@@ -120,6 +117,24 @@ module recola_wrapper
   logical, parameter :: rclwrap_is_active = .true.
 
 
+  type :: rcl_controller_t
+     private
+     logical :: active = .false.
+     logical :: done = .false.
+     integer :: recola_id = 0
+   contains
+     procedure :: reset => rcl_controller_reset
+     procedure :: write => rcl_controller_write
+     procedure :: get_new_id => rcl_controller_get_new_id
+     procedure :: get_current_id => rcl_controller_get_current_id
+     procedure :: activate => rcl_controller_activate
+     procedure :: generate_processes => rcl_controller_generate_processes
+  end type rcl_controller_t
+  
+
+  type(rcl_controller_t) :: rcl_controller
+  
+
 contains
 
   elemental function get_recola_particle_string (pdg) result (name)
@@ -192,12 +207,83 @@ contains
   subroutine rclwrap_define_process (id, process_string, order)
     integer, intent(in) :: id
     type(string_t), intent(in) :: process_string
-    character(len=*), intent(in) :: order
-    call define_process_rcl (id, char (process_string), order)
+    type(string_t), intent(in) :: order
+    call msg_debug2 (D_ME_METHODS, "define_process_rcl")
+    call define_process_rcl (id, char (process_string), char (order))
   end subroutine rclwrap_define_process
 
+  subroutine rcl_controller_reset (rcl)
+    class(rcl_controller_t), intent(inout) :: rcl
+    if (rcl%active .or. rcl%done) then
+       call msg_debug2 (D_ME_METHODS, "reset_recola_rcl")
+       call reset_recola_rcl ()
+    end if
+    rcl%active = .false.
+    rcl%done = .false.
+    rcl%recola_id = 0
+  end subroutine rcl_controller_reset
+    
+  subroutine rcl_controller_write (object, unit)
+    class(rcl_controller_t), intent(in) :: object
+    integer, intent(in), optional :: unit
+    integer :: u
+    u = given_output_unit (unit)
+    write (u, "(1x,A,2(1x,A,L1),1(1x,A,I0))")  "RECOLA controller:", &
+         "active=", object%active, "done=", object%done, &
+         "id=", object%recola_id
+  end subroutine rcl_controller_write
+    
+  subroutine rcl_controller_get_new_id (object, id)
+    class(rcl_controller_t), intent(inout) :: object
+    integer, intent(out) :: id
+    object%recola_id = object%recola_id + 1
+    id = object%recola_id
+  end subroutine rcl_controller_get_new_id
+    
+  subroutine rcl_controller_get_current_id (object, id)
+    class(rcl_controller_t), intent(inout) :: object
+    integer, intent(out) :: id
+    id = object%recola_id
+  end subroutine rcl_controller_get_current_id
+    
+  subroutine rcl_controller_activate (rcl)
+    class(rcl_controller_t), intent(inout) :: rcl
+    if (rcl_controller%done) then
+       call msg_fatal ("Recola interface: attempt at adding processes after initialization")
+    else
+       rcl_controller%active = .true.
+    end if
+  end subroutine rcl_controller_activate
+  
+  subroutine rcl_controller_generate_processes (rcl)
+    class(rcl_controller_t), intent(inout) :: rcl
+    if (rcl_controller%active) then
+       if (.not. rcl_controller%done) then
+          call msg_message ("Recola: preparing processes for integration")
+          call generate_processes_rcl ()
+          rcl_controller%done = .true.
+       end if
+    end if
+  end subroutine rcl_controller_generate_processes
+       
+  subroutine rclwrap_get_new_recola_id (id)
+    integer, intent(out) :: id
+    call rcl_controller%get_new_id (id)
+  end subroutine rclwrap_get_new_recola_id
+    
+  function rclwrap_get_n_processes () result (n)
+    integer :: n
+    call rcl_controller%get_current_id (n)
+  end function rclwrap_get_n_processes
+    
+  subroutine rclwrap_request_generate_processes ()
+    call msg_debug2 (D_ME_METHODS, "request_generate_processes_rcl")
+    call rcl_controller%activate ()
+  end subroutine rclwrap_request_generate_processes
+
   subroutine rclwrap_generate_processes ()
-    call generate_processes_rcl ()
+    call msg_debug2 (D_ME_METHODS, "generate_processes_rcl")
+    call rcl_controller%generate_processes ()
   end subroutine rclwrap_generate_processes
 
   subroutine rclwrap_compute_process (id, p, order, sqme)
@@ -205,6 +291,7 @@ contains
     real(double), intent(in), dimension(:,:) :: p
     character(len=*), intent(in) :: order
     real(double), intent(out), dimension(0:1), optional :: sqme
+    call msg_debug2 (D_ME_METHODS, "compute_process_rcl")
     call compute_process_rcl (id, p, order, sqme)
   end subroutine rclwrap_compute_process
 
@@ -213,19 +300,22 @@ contains
     character(len=*), intent(in) :: order
     integer, dimension(:), intent(in) :: col, hel
     complex(double), intent(out) :: amp
+    call msg_debug2 (D_ME_METHODS, "get_amplitude_rcl")
     call get_amplitude_rcl (id, g_power, order, col, hel, amp)
   end subroutine rclwrap_get_amplitude
 
-  subroutine rclwrap_get_squared_amplitude (id, alpha_power, order, sqme)
-    integer, intent(in) :: id, alpha_power
+  subroutine rclwrap_get_squared_amplitude (id, alphas_power, order, sqme)
+    integer, intent(in) :: id, alphas_power
     character(len=*), intent(in) :: order
     real(double), intent(out) :: sqme
-    call get_squared_amplitude_rcl (id, alpha_power, order, sqme)
+    call msg_debug2 (D_ME_METHODS, "get_squared_amplitude_rcl")
+    call get_squared_amplitude_rcl (id, alphas_power, order, sqme)
   end subroutine rclwrap_get_squared_amplitude
 
   subroutine rclwrap_set_pole_mass (pdg_id, mass, width)
     integer, intent(in) :: pdg_id
     real(double), intent(in) :: mass, width
+    call msg_debug2 (D_ME_METHODS, "rclwrap_set_pole_mass of ", pdg_id)
     select case (abs(pdg_id))
     case (11)
        if (width > zero) &
@@ -267,6 +357,7 @@ contains
   subroutine rclwrap_set_onshell_mass (pdg_id, mass, width)
     integer, intent(in) :: pdg_id
     real(double), intent(in) :: mass, width
+    call msg_debug2 (D_ME_METHODS, "rclwrap_set_onshell_mass of ", pdg_id)
     select case (abs(pdg_id))
     case (23)
        call set_onshell_mass_z_rcl (mass, width)
@@ -279,16 +370,21 @@ contains
 
   subroutine rclwrap_use_gfermi_scheme (gf)
     real(double), intent(in), optional :: gf
+    call msg_debug2 (D_ME_METHODS, "use_gfermi_scheme_rcl", &
+         real(gf, kind=default))
     call use_gfermi_scheme_rcl (gf)
   end subroutine rclwrap_use_gfermi_scheme
 
   subroutine rclwrap_set_light_fermions (m)
     real(double), intent(in) :: m
+    call msg_debug2 (D_ME_METHODS, "set_light_fermions_rcl", &
+         real(m, kind=default))
     call set_light_fermions_rcl (m)
   end subroutine rclwrap_set_light_fermions
 
   subroutine rclwrap_set_light_fermion (pdg_id)
     integer, intent(in) :: pdg_id
+    call msg_debug2 (D_ME_METHODS, "rclwrap_set_light_fermion", pdg_id)
     select case (abs(pdg_id))
     case (1)
        call set_light_down_rcl ()
@@ -313,6 +409,7 @@ contains
 
   subroutine rclwrap_unset_light_fermion (pdg_id)
     integer, intent(in) :: pdg_id
+    call msg_debug2 (D_ME_METHODS, "rclwrap_unset_light_fermion", pdg_id)
     select case (abs(pdg_id))
     case (1)
        call unset_light_down_rcl ()
@@ -336,128 +433,152 @@ contains
   end subroutine rclwrap_unset_light_fermion
 
   subroutine rclwrap_set_onshell_scheme
+    call msg_debug2 (D_ME_METHODS, "set_on_shell_scheme_rcl")
     call set_on_shell_scheme_rcl ()
   end subroutine rclwrap_set_onshell_scheme
 
   subroutine rclwrap_set_alpha_s (alpha_s, mu, nf)
     real(double), intent(in) :: alpha_s, mu
     integer, intent(in) :: nf
+    call msg_debug2 (D_ME_METHODS, "set_alphas_rcl")
     call set_alphas_rcl (alpha_s, mu, nf)
   end subroutine rclwrap_set_alpha_s
 
   function rclwrap_get_alpha_s () result (alpha_s)
     real(double) :: alpha_s
+    call msg_debug2 (D_ME_METHODS, "get_alphas_rcl")
     call get_alphas_rcl (alpha_s)
   end function rclwrap_get_alpha_s
 
   subroutine rclwrap_get_helicity_configurations (id, hel)
     integer, intent(in) :: id
     integer, intent(inout), dimension(:,:), allocatable :: hel
-    !!! Not in the official RECOLA release yet
-    ! call get_helicity_configurations_rcl (id, hel)
+    call get_helicity_configurations_rcl (id, hel)
   end subroutine rclwrap_get_helicity_configurations
 
   subroutine rclwrap_get_color_configurations (id, col)
     integer, intent(in) :: id
     integer, intent(out), dimension(:,:), allocatable :: col
-    !!! Not in the official RECOLA release yet
-    ! call get_colour_configurations_rcl (id, col)
+    call get_colour_configurations_rcl (id, col)
   end subroutine rclwrap_get_color_configurations
 
   subroutine rclwrap_use_dim_reg_soft ()
-     call use_dim_reg_soft_rcl ()
+    call msg_debug2 (D_ME_METHODS, "use_dim_reg_soft_rcl")
+    call use_dim_reg_soft_rcl ()
   end subroutine rclwrap_use_dim_reg_soft
 
   subroutine rclwrap_use_mass_reg_soft (m)
     real(double), intent(in) :: m
+    call msg_debug2 (D_ME_METHODS, "use_mass_reg_soft_rcl")
     call use_mass_reg_soft_rcl (m)
   end subroutine rclwrap_use_mass_reg_soft
 
   subroutine rclwrap_set_delta_uv (d)
     real(double), intent(in) :: d
+    call msg_debug2 (D_ME_METHODS, "set_delta_uv_rcl")
     call set_delta_uv_rcl (d)
   end subroutine rclwrap_set_delta_uv
 
   subroutine rclwrap_set_mu_uv (mu)
     real(double), intent(in) :: mu
+    call msg_debug2 (D_ME_METHODS, "set_mu_uv_rcl")
     call set_mu_uv_rcl (mu)
   end subroutine rclwrap_set_mu_uv
 
   subroutine rclwrap_set_delta_ir (d, d2)
     real(double), intent(in) :: d, d2
+    call msg_debug2 (D_ME_METHODS, "set_delta_ir_rcl", &
+         real(d, kind=default))
+    call msg_debug2 (D_ME_METHODS, "set_delta_ir_rcl", &
+         real(d2, kind=default))
     call set_delta_ir_rcl (d, d2)
   end subroutine rclwrap_set_delta_ir
 
   subroutine rclwrap_set_mu_ir (mu)
     real(double), intent(in) :: mu
+    call msg_debug2 (D_ME_METHODS, "set_mu_ir_rcl")
     call set_mu_ir_rcl (mu)
   end subroutine rclwrap_set_mu_ir
 
   function rclwrap_get_renormalization_scale () result (mu)
     real(double) :: mu
+    call msg_debug2 (D_ME_METHODS, "get_renormalization_scale_rcl")
     call get_renormalization_scale_rcl (mu)
   end function rclwrap_get_renormalization_scale
 
   subroutine rclwrap_get_flavor_scheme (nf)
     integer, intent(out) :: nf
+    call msg_debug2 (D_ME_METHODS, "get_flavour_scheme_rcl")
     call get_flavour_scheme_rcl (nf)
   end subroutine rclwrap_get_flavor_scheme
 
   subroutine rclwrap_use_alpha0_scheme (al0)
     real(double), intent(in), optional :: al0
+    call msg_debug2 (D_ME_METHODS, "use_alpha0_scheme_rcl")
     call use_alpha0_scheme_rcl (al0)
   end subroutine rclwrap_use_alpha0_scheme
 
   subroutine rclwrap_use_alphaz_scheme (alz)
     real(double), intent(in), optional :: alz
+    call msg_debug2 (D_ME_METHODS, "use_alphaz_scheme_rcl")
     call use_alphaz_scheme_rcl (alz)
   end subroutine rclwrap_use_alphaz_scheme
 
   subroutine rclwrap_set_complex_mass_scheme ()
+    call msg_debug2 (D_ME_METHODS, "set_complex_mass_scheme_rcl")
     call set_complex_mass_scheme_rcl ()
   end subroutine rclwrap_set_complex_mass_scheme
 
   subroutine rclwrap_set_resonant_particle (pdg_id)
     integer, intent(in) :: pdg_id
+    call msg_debug2 (D_ME_METHODS, "set_resonant_particle_rcl")
     call set_resonant_particle_rcl (char(get_recola_particle_string (pdg_id)))
   end subroutine rclwrap_set_resonant_particle
 
   subroutine rclwrap_switch_on_resonant_self_energies ()
+    call msg_debug2 (D_ME_METHODS, "switchon_resonant_selfenergies_rcl")
     call switchon_resonant_selfenergies_rcl ()
   end subroutine rclwrap_switch_on_resonant_self_energies
 
   subroutine rclwrap_switch_off_resonant_self_energies ()
+    call msg_debug2 (D_ME_METHODS, "switchoff_resonant_selfenergies_rcl")
     call switchoff_resonant_selfenergies_rcl ()
   end subroutine rclwrap_switch_off_resonant_self_energies
 
   subroutine rclwrap_set_draw_level_branches (n)
     integer, intent(in) :: n
+    call msg_debug2 (D_ME_METHODS, "set_draw_level_branches_rcl")
     call set_draw_level_branches_rcl (n)
   end subroutine rclwrap_set_draw_level_branches
 
   subroutine rclwrap_set_print_level_amplitude (n)
     integer, intent(in) :: n
+    call msg_debug2 (D_ME_METHODS, "set_print_level_amplitude_rcl")
     call set_print_level_amplitude_rcl (n)
   end subroutine rclwrap_set_print_level_amplitude
 
   subroutine rclwrap_set_print_level_squared_amplitude (n)
     integer, intent(in) :: n
+    call msg_debug2 (D_ME_METHODS, "set_print_level_squared_amplitude_rcl")
     call set_print_level_squared_amplitude_rcl (n)
   end subroutine rclwrap_set_print_level_squared_amplitude
 
   subroutine rclwrap_set_print_level_correlations (n)
     integer, intent(in) :: n
+    call msg_debug2 (D_ME_METHODS, "set_print_level_correlations_rcl")
     call set_print_level_correlations_rcl (n)
   end subroutine rclwrap_set_print_level_correlations
 
   subroutine rclwrap_set_print_level_RAM (n)
     integer, intent(in) :: n
+    call msg_debug2 (D_ME_METHODS, "set_print_level_RAM_rcl")
     call set_print_level_RAM_rcl (n)
   end subroutine rclwrap_set_print_level_RAM
 
   subroutine rclwrap_scale_coupling3 (pdg_id1, pdg_id2, pdg_id3, factor)
     integer, intent(in) :: pdg_id1, pdg_id2, pdg_id3
     complex(double), intent(in) :: factor
+    call msg_debug2 (D_ME_METHODS, "scale_coupling3_rcl")
     call scale_coupling3_rcl (factor, char(get_recola_particle_string (pdg_id1)), &
          char(get_recola_particle_string (pdg_id2)), char(get_recola_particle_string (pdg_id3)))
   end subroutine rclwrap_scale_coupling3
@@ -465,6 +586,7 @@ contains
   subroutine rclwrap_scale_coupling4 (pdg_id1, pdg_id2, pdg_id3, pdg_id4, factor)
     integer, intent(in) :: pdg_id1, pdg_id2, pdg_id3, pdg_id4
     complex(double), intent(in) :: factor
+    call msg_debug2 (D_ME_METHODS, "scale_coupling4_rcl")
     call scale_coupling4_rcl (factor, char(get_recola_particle_string (pdg_id1)), &
          char(get_recola_particle_string (pdg_id2)), char(get_recola_particle_string (pdg_id3)), &
          char(get_recola_particle_string (pdg_id4)))
@@ -472,99 +594,118 @@ contains
 
   subroutine rclwrap_switch_off_coupling3 (pdg_id1, pdg_id2, pdg_id3)
     integer, intent(in) :: pdg_id1, pdg_id2, pdg_id3
+    call msg_debug2 (D_ME_METHODS, "switchoff_coupling3_rcl")
     call switchoff_coupling3_rcl (char(get_recola_particle_string (pdg_id1)), &
          char(get_recola_particle_string (pdg_id2)), char(get_recola_particle_string (pdg_id3)))
   end subroutine rclwrap_switch_off_coupling3
 
   subroutine rclwrap_switch_off_coupling4 (pdg_id1, pdg_id2, pdg_id3, pdg_id4)
     integer, intent(in) :: pdg_id1, pdg_id2, pdg_id3, pdg_id4
-    call switchoff_coupling4_rcl (char(get_recola_particle_string (pdg_id1)), &
-         char(get_recola_particle_string (pdg_id2)), char(get_recola_particle_string (pdg_id3)), &
-         char(get_recola_particle_string (pdg_id4)))
+    call msg_debug2 (D_ME_METHODS, "switchoff_coupling4_rcl")
+    call switchoff_coupling4_rcl &
+         (char(get_recola_particle_string (pdg_id1)), &
+          char(get_recola_particle_string (pdg_id2)), &
+          char(get_recola_particle_string (pdg_id3)), &
+          char(get_recola_particle_string (pdg_id4)))
   end subroutine rclwrap_switch_off_coupling4
 
   subroutine rclwrap_set_ifail (i)
     integer, intent(in) :: i
+    call msg_debug2 (D_ME_METHODS, "set_ifail_rcl")
     call set_ifail_rcl (i)
   end subroutine rclwrap_set_ifail
 
   subroutine rclwrap_get_ifail (i)
     integer, intent(out) :: i
+    call msg_debug2 (D_ME_METHODS, "get_ifail_rcl")
     call get_ifail_rcl (i)
   end subroutine rclwrap_get_ifail
 
   subroutine rclwrap_set_output_file (filename)
     character(len=*), intent(in) :: filename
+    call msg_debug2 (D_ME_METHODS, "set_output_file_rcl")
     call set_output_file_rcl (filename)
   end subroutine rclwrap_set_output_file
 
   subroutine rclwrap_set_gs_power (id, gs_array)
     integer, intent(in) :: id
     integer, dimension(:,:), intent(in) :: gs_array
+    call msg_debug2 (D_ME_METHODS, "set_gs_power_rcl")
     call set_gs_power_rcl (id, gs_array)
   end subroutine rclwrap_set_gs_power
 
   subroutine rclwrap_select_gs_power_born_amp (id, gs_power)
     integer, intent(in) :: id, gs_power
+    call msg_debug2 (D_ME_METHODS, "select_gs_power_BornAmpl_rcl")
     call select_gs_power_BornAmpl_rcl (id, gs_power)
  end subroutine rclwrap_select_gs_power_born_amp
 
   subroutine rclwrap_unselect_gs_power_born_amp (id, gs_power)
     integer, intent(in) :: id, gs_power
+    call msg_debug2 (D_ME_METHODS, "unselect_gs_power_BornAmpl_rcl")
     call unselect_gs_power_BornAmpl_rcl (id, gs_power)
  end subroutine rclwrap_unselect_gs_power_born_amp
 
   subroutine rclwrap_select_gs_power_loop_amp (id, gs_power)
     integer, intent(in) :: id, gs_power
+    call msg_debug2 (D_ME_METHODS, "select_gs_power_LoopAmpl_rcl")
     call select_gs_power_LoopAmpl_rcl (id, gs_power)
  end subroutine rclwrap_select_gs_power_loop_amp
 
   subroutine rclwrap_unselect_gs_power_loop_amp (id, gs_power)
     integer, intent(in) :: id, gs_power
+    call msg_debug2 (D_ME_METHODS, "unselect_gs_power_LoopAmpl_rcl")
     call unselect_gs_power_LoopAmpl_rcl (id, gs_power)
  end subroutine rclwrap_unselect_gs_power_loop_amp
 
   subroutine rclwrap_select_all_gs_powers_born_amp (id)
     integer, intent(in) :: id
+    call msg_debug2 (D_ME_METHODS, "select_all_gs_powers_BornAmpl_rcl")
     call select_all_gs_powers_BornAmpl_rcl (id)
   end subroutine rclwrap_select_all_gs_powers_born_amp
 
   subroutine rclwrap_unselect_all_gs_powers_loop_amp (id)
     integer, intent(in) :: id
+    call msg_debug2 (D_ME_METHODS, "unselect_all_gs_powers_BornAmpl_rcl")
     call unselect_all_gs_powers_BornAmpl_rcl (id)
   end subroutine rclwrap_unselect_all_gs_powers_loop_amp
 
   subroutine rclwrap_select_all_gs_powers_loop_amp (id)
     integer, intent(in) :: id
+    call msg_debug2 (D_ME_METHODS, "select_all_gs_powers_LoopAmpl_rcl")
     call select_all_gs_powers_LoopAmpl_rcl (id)
   end subroutine rclwrap_select_all_gs_powers_loop_amp
 
   subroutine rclwrap_unselect_all_gs_powers_born_amp (id)
     integer, intent(in) :: id
+    call msg_debug2 (D_ME_METHODS, "unselect_all_gs_powers_LoopAmpl_rcl")
     call unselect_all_gs_powers_LoopAmpl_rcl (id)
   end subroutine rclwrap_unselect_all_gs_powers_born_amp
 
   subroutine rclwrap_set_resonant_squared_momentum (id, i_res, p2)
     integer, intent(in) :: id, i_res
     real(double), intent(in) :: p2
+    call msg_debug2 (D_ME_METHODS, "set_resonant_squared_momentum_rcl")
     call set_resonant_squared_momentum_rcl (id, i_res, p2)
   end subroutine rclwrap_set_resonant_squared_momentum
 
   subroutine rclwrap_compute_running_alpha_s (Q, nf, n_loops)
     real(double), intent(in) :: Q
     integer, intent(in) :: nf, n_loops
+    call msg_debug2 (D_ME_METHODS, "compute_running_alphas_rcl")
     call compute_running_alphas_rcl (Q, nf, n_loops)
   end subroutine rclwrap_compute_running_alpha_s
 
-  !!! Recola-1.1 feature, not yet active
-  !!! subroutine rclwrap_set_dynamic_settings ()
-  !!!   call set_dynamic_settings_rcl (1)
-  !!! end subroutine rclwrap_set_dynamic_settings
+  subroutine rclwrap_set_dynamic_settings ()
+    call msg_debug2 (D_ME_METHODS, "set_dynamic_settings_rcl")
+    call set_dynamic_settings_rcl (1)
+  end subroutine rclwrap_set_dynamic_settings
 
   subroutine rclwrap_rescale_process (id, order, sqme)
     integer, intent(in) :: id
     character(len=*), intent(in) :: order
     real(double), dimension(0:1), intent(out), optional :: sqme
+    call msg_debug2 (D_ME_METHODS, "rescale_process_rcl")
     call rescale_process_rcl (id, order, sqme)
   end subroutine rclwrap_rescale_process
 
@@ -574,6 +715,7 @@ contains
     character(len=*), intent(in) :: order
     integer, dimension(:), intent(in) :: hel
     real(double), intent(out) :: sqme
+    call msg_debug2 (D_ME_METHODS, "get_polarized_squared_amplitude_rcl")
     call get_polarized_squared_amplitude_rcl (id, alphas_power, &
          order, hel, sqme)
   end subroutine rclwrap_get_polarized_squared_amplitude
@@ -584,29 +726,34 @@ contains
     real(double), dimension(:,:), intent(in) :: p
     integer, intent(in) :: i1, i2
     real(double), intent(out), optional :: sqme
+    call msg_debug2 (D_ME_METHODS, "compute_colour_correlation_rcl")
     call compute_colour_correlation_rcl (id, p, i1, i2, sqme)
   end subroutine rclwrap_compute_color_correlation
 
   subroutine rclwrap_compute_all_color_correlations (id, p)
     integer, intent(in) :: id
     real(double), dimension(:,:), intent(in) :: p
+    call msg_debug2 (D_ME_METHODS, "compute_all_colour_correlations_rcl")
     call compute_all_colour_correlations_rcl (id, p)
   end subroutine rclwrap_compute_all_color_correlations
 
   subroutine rclwrap_rescale_color_correlation (id, i1, i2, sqme)
     integer, intent(in) :: id, i1, i2
     real(double), intent(out), optional :: sqme
+    call msg_debug2 (D_ME_METHODS, "rescale_colour_correlation_rcl")
     call rescale_colour_correlation_rcl (id, i1, i2, sqme)
   end subroutine rclwrap_rescale_color_correlation
 
   subroutine rclwrap_rescale_all_color_correlations (id)
     integer, intent(in) :: id
+    call msg_debug2 (D_ME_METHODS, "rescale_all_colour_correlations_rcl")
     call rescale_all_colour_correlations_rcl (id)
   end subroutine rclwrap_rescale_all_color_correlations
 
   subroutine rclwrap_get_color_correlation (id, alphas_power, i1, i2, sqme)
     integer, intent(in) :: id, alphas_power, i1, i2
     real(double), intent(out) :: sqme
+    call msg_debug2 (D_ME_METHODS, "get_colour_correlation_rcl")
     call get_colour_correlation_rcl (id, alphas_power, i1, i2, sqme)
   end subroutine rclwrap_get_color_correlation
 
@@ -616,6 +763,7 @@ contains
     integer, intent(in) :: i_photon
     complex(double), dimension(:), intent(in) :: pol
     real(double), intent(out), optional :: sqme
+    call msg_debug2 (D_ME_METHODS, "compute_spin_correlation_rcl")
     call compute_spin_correlation_rcl (id, p, i_photon, pol, sqme)
   end subroutine rclwrap_compute_spin_correlation
 
@@ -623,12 +771,14 @@ contains
     integer, intent(in) :: id, i_photon
     complex(double), dimension(:), intent(in) :: pol
     real(double), intent(out), optional :: sqme
+    call msg_debug2 (D_ME_METHODS, "rescale_spin_correlation_rcl")
     call rescale_spin_correlation_rcl (id, i_photon, pol, sqme)
   end subroutine rclwrap_rescale_spin_correlation
 
   subroutine rclwrap_get_spin_correlation (id, alphas_power, sqme)
     integer, intent(in) :: id, alphas_power
     real(double), intent(out) :: sqme
+    call msg_debug2 (D_ME_METHODS, "get_spin_correlation_rcl")
     call get_spin_correlation_rcl (id, alphas_power, sqme)
   end subroutine rclwrap_get_spin_correlation
 
@@ -639,6 +789,7 @@ contains
     integer, intent(in) :: i_gluon, i_spectator
     complex(double), dimension(:), intent(in) :: pol
     real(double), intent(out), optional :: sqme
+    call msg_debug2 (D_ME_METHODS, "compute_spin_colour_correlation_rcl")
     call compute_spin_colour_correlation_rcl (id, p, &
          i_gluon, i_spectator, pol, sqme)
   end subroutine rclwrap_compute_spin_color_correlation
@@ -648,6 +799,7 @@ contains
     integer, intent(in) :: id, i_gluon, i_spectator
     complex(double), dimension(:), intent(in) :: pol
     real(double), intent(out), optional :: sqme
+    call msg_debug2 (D_ME_METHODS, "rescale_spin_colour_correlation_rcl")
     call rescale_spin_colour_correlation_rcl (id, i_gluon, &
          i_spectator, pol, sqme)
   end subroutine rclwrap_rescale_spin_color_correlation
@@ -656,6 +808,7 @@ contains
        i_gluon, i_spectator, sqme)
     integer, intent(in) :: id, alphas_power, i_gluon, i_spectator
     real(double), intent(out) :: sqme
+    call msg_debug2 (D_ME_METHODS, "get_spin_colour_correlation_rcl")
     call get_spin_colour_correlation_rcl (id, alphas_power, &
          i_gluon, i_spectator, sqme)
   end subroutine rclwrap_get_spin_color_correlation
@@ -663,11 +816,13 @@ contains
   subroutine rclwrap_get_momenta (id, p)
     integer, intent(in) :: id
     real(double), dimension(:,:), intent(out) :: p
+    call msg_debug2 (D_ME_METHODS, "get_momenta_rcl")
     call get_momenta_rcl (id, p)
   end subroutine rclwrap_get_momenta
 
   subroutine rclwrap_reset_recola
-    call reset_recola_rcl ()
+    call msg_debug (D_ME_METHODS, "rclwrap_reset_recola")
+    call rcl_controller%reset ()
   end subroutine rclwrap_reset_recola
 
 
