@@ -1,19 +1,12 @@
-! WHIZARD 2.4.1 Mar 24 2017
+! WHIZARD 2.6.4 Aug 23 2018
 !
-! Copyright (C) 1999-2017 by
+! Copyright (C) 1999-2018 by
 !     Wolfgang Kilian <kilian@physik.uni-siegen.de>
 !     Thorsten Ohl <ohl@physik.uni-wuerzburg.de>
 !     Juergen Reuter <juergen.reuter@desy.de>
 !
 !     with contributions from
-!     Fabian Bach <fabian.bach@t-online.de>
-!     Bijan Chokoufe <bijan.chokoufe@desy.de>
-!     Christian Speckner <cnspeckn@googlemail.com>
-!     So Young Shim <soyoung.shim@desy.de>
-!     Florian Staub <florian.staub@cern.ch>
-!     Christian Weiss <christian.weiss@desy.de>
-!     and Hans-Werner Boschmann, Felix Braam,
-!     Sebastian Schmidt, So-young Shim, Daniel Wiesler
+!     cf. main AUTHORS file
 !
 ! WHIZARD is free software; you can redistribute it and/or modify it
 ! under the terms of the GNU General Public License as published by
@@ -35,10 +28,7 @@
 
 module unit_tests
 
-  use kinds, only: default
-  use constants, only: zero
   use iso_varying_string, string_t => varying_string
-  use format_defs
   use io_units
 
   implicit none
@@ -181,6 +171,7 @@ contains
     write (u_log, "(A)")  "Test: " // name
     u_test = free_unit ()
     open (u_test, status="scratch", action="readwrite")
+    call fatal_force_crash ()
     call test_proc (u_test)
     rewind (u_test)
     inquire (file=ref_prefix//name//ref, exist=exist)
