@@ -1,19 +1,12 @@
-! WHIZARD 2.4.1 Mar 24 2017
+! WHIZARD 2.6.4 Aug 23 2018
 !
-! Copyright (C) 1999-2017 by
+! Copyright (C) 1999-2018 by
 !     Wolfgang Kilian <kilian@physik.uni-siegen.de>
 !     Thorsten Ohl <ohl@physik.uni-wuerzburg.de>
 !     Juergen Reuter <juergen.reuter@desy.de>
 !
 !     with contributions from
-!     Fabian Bach <fabian.bach@t-online.de>
-!     Bijan Chokoufe <bijan.chokoufe@desy.de>
-!     Christian Speckner <cnspeckn@googlemail.com>
-!     So Young Shim <soyoung.shim@desy.de>
-!     Florian Staub <florian.staub@cern.ch>
-!     Christian Weiss <christian.weiss@desy.de>
-!     and Hans-Werner Boschmann, Felix Braam,
-!     Sebastian Schmidt, So-young Shim, Daniel Wiesler
+!     cf. main AUTHORS file
 !
 ! WHIZARD is free software; you can redistribute it and/or modify it
 ! under the terms of the GNU General Public License as published by
@@ -49,7 +42,6 @@ module ttv_formfactors
   use system_dependencies
   use, intrinsic :: iso_fortran_env !NODEP!
   use diagnostics
-  use solver, only: solver_function_t, solve_qgaus
   implicit none
   private
   save
@@ -185,15 +177,6 @@ module ttv_formfactors
     procedure :: write => phase_space_point_write
   end type phase_space_point_t
 
-  type, extends (solver_function_t) :: p0_q_integrand_t
-    real(default) :: a = 0
-    type(phase_space_point_t) :: ps
-    integer :: i = 0
-  contains
-    procedure :: update => p0_q_integrand_update
-    procedure :: evaluate => p0_q_integrand_evaluate
-  end type p0_q_integrand_t
-
 
   type(threshold_t) :: threshold
   real(default) :: M1S, GAM, GAM_M1S
@@ -444,8 +427,8 @@ contains
        alphas = alphas_notsohard (sqrts)
     end if
     if (threshold%settings%nlo) then
-       gamma = top_width_sm_qcd_nlo_jk (width%aem, width%sw, top_mass, &
-            width%mw, width%mb, alphas) + width%gam_inv
+       gamma = top_width_sm_qcd_nlo_jk (width%aem, width%sw, width%vtb, &
+            top_mass, width%mw, width%mb, alphas) + width%gam_inv
     else
        gamma = top_width_sm_lo (width%aem, width%sw, width%vtb, top_mass, &
             width%mw, width%mb) + width%gam_inv
@@ -691,7 +674,11 @@ contains
     complex(default) :: c
     c = one
     if (.not. threshold%settings%initialized_ff .or. .not. ps%inside_grid) return
-    call interpolate_linear (sq_grid, p_grid, ff_grid(:,:,1,vec_type), ps%sqrts, ps%p, c)
+    if (POINTS_SQ > 1) then
+       call interpolate_linear (sq_grid, p_grid, ff_grid(:,:,1,vec_type), ps%sqrts, ps%p, c)
+    else
+       call interpolate_linear (p_grid, ff_grid(1,:,1,vec_type), ps%p, c)
+    end if
   end function resummed_formfactor
 
   !!! leading nonrelativistic O(alphas^1) contribution (-> expansion of resummation)
@@ -703,20 +690,34 @@ contains
     real(default) :: shift_from_hard_current
     complex(default) :: v, contrib_from_potential
     FF = one
-    if (.not. threshold%settings%initialized_parameters .or. vec_type == AXIAL) return
+    if (.not. threshold%settings%initialized_parameters) return
     call update_global_sqrts_dependent_variables (ps%sqrts)
     v = sqrts_to_v (ps%sqrts, GAM)
     if (NRQCD_ORDER == 1) then
-       shift_from_hard_current = - two * CF / pi
+       if (vec_type == AXIAL) then
+          shift_from_hard_current = - CF / pi
+       else
+          shift_from_hard_current = - two * CF / pi
+       end if
     else
        shift_from_hard_current = zero
     end if
     if (ps%onshell) then
        contrib_from_potential = CF * ps%mpole * Pi / (4 * ps%p)
     else
-       contrib_from_potential = imago * CF * ps%mpole * &
-            log ((ps%p + ps%mpole * v) / &
-                 (-ps%p + ps%mpole * v) + ieps) / (two * ps%p)
+       if (vec_type == AXIAL) then
+          contrib_from_potential = - CF * ps%mpole / (two * ps%p) * &
+               (imago * ps%mpole * v / ps%p + &
+               (ps%mpole**2 * v**2 + (ps%p)**2 / (4 *Pi * (ps%p)**2) * ( &
+               (log (- ps%mpole * v - ps%p))**2 - &
+               (log (- ps%mpole * v + ps%p))**2 + &
+               (log (ps%mpole * v - ps%p))**2 - &
+               (log (ps%mpole * v + ps%p))**2 )))
+       else
+          contrib_from_potential = imago * CF * ps%mpole * &
+               log ((ps%p + ps%mpole * v) / &
+               (-ps%p + ps%mpole * v) + ieps) / (two * ps%p)
+       end if
     end if
     FF = one + alphas_soft * contrib_from_potential + &
          alphas_hard * shift_from_hard_current
@@ -1088,9 +1089,13 @@ contains
   pure function sqrts_iter (i_sq) result (sqrts)
     integer, intent(in) :: i_sq
     real(default) :: sqrts
-    sqrts = sqrts_min - sqrts_it + &
-            (sqrts_max - sqrts_min + two * sqrts_it) * &
-            real(i_sq - 1) / real(POINTS_SQ - 1)
+    if (POINTS_SQ > 1) then
+       sqrts = sqrts_min - sqrts_it + &
+               (sqrts_max - sqrts_min + two * sqrts_it) * &
+               real(i_sq - 1) / real(POINTS_SQ - 1)
+    else
+       sqrts = sqrts_min
+    end if
   end function sqrts_iter
 
   function scan_formfactor_over_p_LL_analytic (a_soft, sqrts, vec_type) result (ff_analytic)
@@ -1117,7 +1122,7 @@ contains
     real*8 :: xenergy, xtm, xtg, xalphas, xscale, xc0, xc1, xc2, xim, xdi, &
         xcutn, xcutv, xkincm, xkinca, xkincv, xcdeltc, &
         xcdeltl, xcfullc, xcfulll, xcrm2
-    integer, parameter :: nmax=400
+    integer, parameter :: nmax=900
     real*8 :: xdsdp(nmax), xpp(nmax), xww(nmax)
     complex*16 :: zff(nmax)
     integer :: np, jknflg, jgcflg, jvflg
@@ -1264,14 +1269,18 @@ contains
   subroutine init_threshold_phase_space_grid ()
     integer :: i_sq
     call msg_debug (D_THRESHOLD, "init_threshold_phase_space_grid")
-    POINTS_SQ = int ((sqrts_max - sqrts_min) / sqrts_it + tiny_07) + 3
+    if (sqrts_it > tiny_07) then
+       POINTS_SQ = int ((sqrts_max - sqrts_min) / sqrts_it + tiny_07) + 3
+    else
+       POINTS_SQ = 1
+    end if
     call msg_debug (D_THRESHOLD, "Number of sqrts grid points: POINTS_SQ", POINTS_SQ)
     call msg_debug (D_THRESHOLD, "sqrts_max", sqrts_max)
     call msg_debug (D_THRESHOLD, "sqrts_min", sqrts_min)
     call msg_debug (D_THRESHOLD, "sqrts_it", sqrts_it)
     allocate (sq_grid(POINTS_SQ))
     sq_grid = [(sqrts_iter (i_sq), i_sq=1, POINTS_SQ)]
-    POINTS_P = 360
+    POINTS_P = 600
     allocate (p_grid(POINTS_P))
     p_grid = p_grid_from_TOPPIK ()
     POINTS_P0 = 1
@@ -1614,25 +1623,6 @@ contains
     write (c, '(l1)') l
   end function logical_to_char
 
-  subroutine p0_q_integrand_update (solver_f, a, ps, i)
-    class(p0_q_integrand_t), intent(inout) :: solver_f
-    real(default), intent(in) :: a
-    type(phase_space_point_t), intent(in) :: ps
-    integer, intent(in) :: i
-    solver_f%a = a
-    solver_f%ps = ps
-    solver_f%i = i
-  end subroutine p0_q_integrand_update
-
-  function p0_q_integrand_evaluate (solver_f, x) result (f)
-    complex(default) :: f
-    class(p0_q_integrand_t), intent(in) :: solver_f
-    real(default), intent(in) :: x
-    f = G0p_tree (solver_f%ps%en, x, solver_f%ps%mpole, GAM) &
-          * minus_q2_V (solver_f%a, x, solver_f%ps%p, solver_f%ps%p0, solver_f%i) &
-          * ff_p_spline%interpolate (x)
-  end function p0_q_integrand_evaluate
-
   subroutine get_rest_frame (p1_in, p2_in, p1_out, p2_out)
     type(vector4_t), intent(in) :: p1_in, p2_in
     type(vector4_t), intent(out) :: p1_out, p2_out
