@@ -1,19 +1,12 @@
-! WHIZARD 2.4.1 Mar 24 2017
+! WHIZARD 2.6.4 Aug 23 2018
 !
-! Copyright (C) 1999-2017 by
+! Copyright (C) 1999-2018 by
 !     Wolfgang Kilian <kilian@physik.uni-siegen.de>
 !     Thorsten Ohl <ohl@physik.uni-wuerzburg.de>
 !     Juergen Reuter <juergen.reuter@desy.de>
 !
 !     with contributions from
-!     Fabian Bach <fabian.bach@t-online.de>
-!     Bijan Chokoufe <bijan.chokoufe@desy.de>
-!     Christian Speckner <cnspeckn@googlemail.com>
-!     So Young Shim <soyoung.shim@desy.de>
-!     Florian Staub <florian.staub@cern.ch>
-!     Christian Weiss <christian.weiss@desy.de>
-!     and Hans-Werner Boschmann, Felix Braam,
-!     Sebastian Schmidt, So-young Shim, Daniel Wiesler
+!     cf. main AUTHORS file
 !
 ! WHIZARD is free software; you can redistribute it and/or modify it
 ! under the terms of the GNU General Public License as published by
@@ -35,6 +28,7 @@
 
 module dispatch_transforms
 
+  use kinds, only: default
   use iso_varying_string, string_t => varying_string
   use process
   use variables
@@ -45,6 +39,8 @@ module dispatch_transforms
   use models
   use os_interface
   use beam_structures
+  use resonances, only: resonance_history_set_t
+  use instances, only: process_instance_t, process_instance_hook_t
 
   use event_base, only: event_callback_t, event_callback_nop_t
   use eio_base
@@ -60,6 +56,8 @@ module dispatch_transforms
   use eio_dump
 
   use event_transforms
+  use resonance_insertion
+  use isr_epa_handler
   use decays
   use shower_base
   use shower_core
@@ -76,8 +74,11 @@ module dispatch_transforms
   private
 
   public :: dispatch_evt_nlo
+  public :: dispatch_evt_resonance
+  public :: dispatch_evt_isr_epa_handler
   public :: dispatch_evt_decay
   public :: dispatch_evt_shower
+  public :: dispatch_evt_shower_hook
   public :: dispatch_matching
   public :: dispatch_evt_hadrons
   public :: dispatch_eio
@@ -87,7 +88,7 @@ contains
   subroutine dispatch_evt_nlo (evt, keep_failed_events)
     class(evt_t), intent(out), pointer :: evt
     logical, intent(in) :: keep_failed_events
-    call msg_message ("Simuate: activating fixed-order NLO events")
+    call msg_message ("Simulate: activating fixed-order NLO events")
     allocate (evt_nlo_t :: evt)
     evt%only_weighted_events = .true.
     select type (evt)
@@ -97,14 +98,105 @@ contains
     end select
   end subroutine dispatch_evt_nlo
 
-  subroutine dispatch_evt_decay (evt, var_list)
+  subroutine dispatch_evt_resonance (evt, var_list, res_history_set, libname)
     class(evt_t), intent(out), pointer :: evt
     type(var_list_t), intent(in) :: var_list
+    type(resonance_history_set_t), dimension(:), intent(in) :: res_history_set
+    type(string_t), intent(in) :: libname
+    logical :: resonance_history
+    resonance_history = var_list%get_lval (var_str ("?resonance_history"))
+    if (resonance_history) then
+       allocate (evt_resonance_t :: evt)
+       call msg_message ("Simulate: activating resonance insertion")
+       select type (evt)
+       type is (evt_resonance_t)
+          call evt%set_resonance_data (res_history_set)
+          call evt%set_library (libname)
+       end select
+    else
+       evt => null ()
+    end if
+  end subroutine dispatch_evt_resonance
+
+  subroutine dispatch_evt_isr_epa_handler (evt, var_list)
+    class(evt_t), intent(out), pointer :: evt
+    type(var_list_t), intent(in) :: var_list
+    logical :: isr_recoil
+    logical :: epa_recoil
+    logical :: isr_handler_active
+    logical :: epa_handler_active
+    type(string_t) :: isr_handler_mode
+    type(string_t) :: epa_handler_mode
+    real(default) :: sqrts
+    real(default) :: isr_q_max
+    real(default) :: epa_q_max
+    real(default) :: isr_mass
+    real(default) :: epa_mass
+    isr_handler_active = var_list%get_lval (var_str ("?isr_handler"))
+    if (isr_handler_active) then
+       call msg_message ("Simulate: activating ISR handler")
+       isr_recoil = var_list%get_lval (var_str ("?isr_recoil"))
+       isr_handler_mode = var_list%get_sval (var_str ("$isr_handler_mode"))
+       if (isr_recoil) then
+          call msg_fatal ("Simulate: ISR handler is incompatible &
+               &with ?isr_recoil=true")
+       end if
+    end if
+    epa_handler_active = var_list%get_lval (var_str ("?epa_handler"))
+    if (epa_handler_active) then
+       call msg_message ("Simulate: activating EPA handler")
+       epa_recoil = var_list%get_lval (var_str ("?epa_recoil"))
+       epa_handler_mode = var_list%get_sval (var_str ("$epa_handler_mode"))
+       if (epa_recoil) then
+          call msg_fatal ("Simulate: EPA handler is incompatible &
+               &with ?epa_recoil=true")
+       end if
+    end if
+    if (isr_handler_active .and. epa_handler_active) then
+       if (isr_handler_mode /= epa_handler_mode) then
+          call msg_fatal ("Simulate: ISR/EPA handler: modes must coincide")
+       end if
+    end if
+    if (isr_handler_active .or. epa_handler_active) then
+       allocate (evt_isr_epa_t :: evt)
+       select type (evt)
+       type is (evt_isr_epa_t)
+          if (isr_handler_active) then
+             call evt%set_mode_string (isr_handler_mode)
+          else
+             call evt%set_mode_string (epa_handler_mode)
+          end if
+          sqrts = var_list%get_rval (var_str ("sqrts"))
+          if (isr_handler_active) then
+             isr_q_max = var_list%get_rval (var_str ("isr_q_max"))
+             isr_mass = var_list%get_rval (var_str ("isr_mass"))
+             call evt%set_data_isr (sqrts, isr_q_max, isr_mass)
+          end if
+          if (epa_handler_active) then
+             epa_q_max = var_list%get_rval (var_str ("epa_q_max"))
+             epa_mass = var_list%get_rval (var_str ("epa_mass"))
+             call evt%set_data_epa (sqrts, epa_q_max, epa_mass)
+          end if
+          call msg_message ("Simulate: ISR/EPA handler mode: " &
+               // char (evt%get_mode_string ()))
+       end select
+    else
+       evt => null ()
+    end if
+  end subroutine dispatch_evt_isr_epa_handler
+
+  subroutine dispatch_evt_decay (evt, var_list)
+    class(evt_t), intent(out), pointer :: evt
+    type(var_list_t), intent(in), target :: var_list
     logical :: allow_decays
     allow_decays = var_list%get_lval (var_str ("?allow_decays"))
     if (allow_decays) then
        allocate (evt_decay_t :: evt)
        call msg_message ("Simulate: activating decays")
+       select type (evt)
+       type is (evt_decay_t)
+          call evt%set_var_list (var_list)
+       end select
     else
        evt => null ()
     end if
@@ -165,6 +257,19 @@ contains
     call dispatch_matching (evt, settings, var_list, process_name)
   end subroutine dispatch_evt_shower
 
+  subroutine dispatch_evt_shower_hook (hook, var_list, process_instance)
+    class(process_instance_hook_t), pointer, intent(out) :: hook
+    type(var_list_t), intent(in) :: var_list
+    class(process_instance_t), intent(in), target :: process_instance
+    if (var_list%get_lval (var_str ('?powheg_matching'))) then
+       call msg_message ("Integration hook: add POWHEG hook")
+       allocate (powheg_matching_hook_t :: hook)
+       call hook%init (var_list, process_instance)
+    else
+       hook => null ()
+    end if
+  end subroutine dispatch_evt_shower_hook
+
   subroutine dispatch_matching (evt, settings, var_list, process_name)
     class(evt_t), intent(inout) :: evt
     type(var_list_t), intent(in) :: var_list
@@ -230,8 +335,10 @@ contains
     type(var_list_t), intent(in) :: var_list
     type(model_t), target, intent(in) :: fallback_model
     class(event_callback_t), allocatable, intent(in) :: event_callback
+    !!! !!! !!! Workaround for ifort v18(beta) bug
+    type(event_callback_nop_t) :: event_callback_tmp
     logical :: check, keep_beams, keep_remnants, recover_beams
-    logical :: use_alpha_s_from_file, use_scale_from_file
+    logical :: use_alphas_from_file, use_scale_from_file
     logical :: write_sqme_prc, write_sqme_ref, write_sqme_alt
     logical :: output_cross_section, ensure_order
     type(string_t) :: lhef_version, lhef_extension, raw_version
@@ -253,8 +360,8 @@ contains
          var_list%get_lval (var_str ("?hepevt_ensure_order"))
     recover_beams = &
          var_list%get_lval (var_str ("?recover_beams"))
-    use_alpha_s_from_file = &
-         var_list%get_lval (var_str ("?use_alpha_s_from_file"))
+    use_alphas_from_file = &
+         var_list%get_lval (var_str ("?use_alphas_from_file"))
     use_scale_from_file = &
          var_list%get_lval (var_str ("?use_scale_from_file"))
     select case (char (method))
@@ -289,7 +396,9 @@ contains
           if (allocated (event_callback)) then
              call eio%set_parameters (event_callback, checkpoint)
           else
-             call eio%set_parameters (event_callback_nop_t (), 0)
+             !!! !!! !!! Workaround for ifort v18(beta) bug
+             ! call eio%set_parameters (event_callback_nop_t (), 0)
+             call eio%set_parameters (event_callback_tmp, 0)
           end if
        end select
     case ("lhef")
@@ -308,7 +417,7 @@ contains
                var_list%get_lval (var_str ("?lhef_write_sqme_alt"))
           call eio%set_parameters ( &
                keep_beams, keep_remnants, recover_beams, &
-               use_alpha_s_from_file, use_scale_from_file, &
+               use_alphas_from_file, use_scale_from_file, &
                char (lhef_version), lhef_extension, &
                write_sqme_ref, write_sqme_prc, write_sqme_alt)
        end select
@@ -321,7 +430,7 @@ contains
           extension_hepmc = &
                var_list%get_sval (var_str ("$extension_hepmc"))
           call eio%set_parameters (recover_beams, &
-               use_alpha_s_from_file, use_scale_from_file, &
+               use_alphas_from_file, use_scale_from_file, &
                extension_hepmc, output_cross_section)
        end select
     case ("lcio")
@@ -331,7 +440,7 @@ contains
           extension_lcio = &
                var_list%get_sval (var_str ("$extension_lcio"))
           call eio%set_parameters (recover_beams, &
-               use_alpha_s_from_file, use_scale_from_file, &
+               use_alphas_from_file, use_scale_from_file, &
                extension_lcio)
        end select
     case ("stdhep")
@@ -342,7 +451,7 @@ contains
                var_list%get_sval (var_str ("$extension_stdhep"))
           call eio%set_parameters &
                (keep_beams, keep_remnants, ensure_order, recover_beams, &
-                use_alpha_s_from_file, use_scale_from_file, extension_stdhep)
+                use_alphas_from_file, use_scale_from_file, extension_stdhep)
        end select
     case ("stdhep_up")
        allocate (eio_stdhep_hepeup_t :: eio)
@@ -351,7 +460,7 @@ contains
           extension_stdhep_up = &
                var_list%get_sval (var_str ("$extension_stdhep_up"))
           call eio%set_parameters (keep_beams, keep_remnants, ensure_order, &
-               recover_beams, use_alpha_s_from_file, &
+               recover_beams, use_alphas_from_file, &
                use_scale_from_file, extension_stdhep_up)
        end select
     case ("stdhep_ev4")
@@ -362,7 +471,7 @@ contains
                var_list%get_sval (var_str ("$extension_stdhep_ev4"))
           call eio%set_parameters &
                (keep_beams, keep_remnants, ensure_order, recover_beams, &
-                use_alpha_s_from_file, use_scale_from_file, extension_stdhep_ev4)
+                use_alphas_from_file, use_scale_from_file, extension_stdhep_ev4)
        end select
     case ("ascii")
        allocate (eio_ascii_ascii_t :: eio)
