@@ -1,19 +1,12 @@
-! WHIZARD 2.4.1 Mar 24 2017
+! WHIZARD 2.6.4 Aug 23 2018
 !
-! Copyright (C) 1999-2017 by
+! Copyright (C) 1999-2018 by
 !     Wolfgang Kilian <kilian@physik.uni-siegen.de>
 !     Thorsten Ohl <ohl@physik.uni-wuerzburg.de>
 !     Juergen Reuter <juergen.reuter@desy.de>
 !
 !     with contributions from
-!     Fabian Bach <fabian.bach@t-online.de>
-!     Bijan Chokoufe <bijan.chokoufe@desy.de>
-!     Christian Speckner <cnspeckn@googlemail.com>
-!     So Young Shim <soyoung.shim@desy.de>
-!     Florian Staub <florian.staub@cern.ch>
-!     Christian Weiss <christian.weiss@desy.de>
-!     and Hans-Werner Boschmann, Felix Braam,
-!     Sebastian Schmidt, So-young Shim, Daniel Wiesler
+!     cf. main AUTHORS file
 !
 ! WHIZARD is free software; you can redistribute it and/or modify it
 ! under the terms of the GNU General Public License as published by
@@ -289,6 +282,7 @@ contains
           select type (event)
           type is (event_t)
              write (eio%unit)  i_prc
+             write (eio%unit)  event%get_index ()
              write (eio%unit)  event%get_i_mci ()
              write (eio%unit)  event%get_i_term ()
              write (eio%unit)  event%get_channel ()
@@ -342,7 +336,7 @@ contains
     class(eio_raw_t), intent(inout) :: eio
     class(generic_event_t), intent(inout), target :: event
     integer, intent(out) :: iostat
-    integer :: i_mci, i_term, channel, i
+    integer :: event_index, i_mci, i_term, channel, i
     real(default) :: weight, excess, sqme
     real(default), dimension(:), allocatable :: weight_alt, sqme_alt
     logical :: has_transform
@@ -351,6 +345,8 @@ contains
     if (eio%reading) then
        select type (event)
        type is (event_t)
+          read (eio%unit, iostat = iostat)  event_index
+          if (iostat /= 0)  return
           read (eio%unit, iostat = iostat)  i_mci
           if (iostat /= 0)  return
           read (eio%unit, iostat = iostat)  i_term
@@ -363,7 +359,8 @@ contains
           if (iostat /= 0)  return
           read (eio%unit, iostat = iostat)  sqme
           if (iostat /= 0)  return
-          call event%reset ()
+          call event%reset_contents ()
+          call event%set_index (event_index)
           call event%select (i_mci, i_term, channel)
           if (eio%norm_mode /= NORM_UNDEFINED) then
              call event_normalization_update (weight, &
