@@ -1,19 +1,12 @@
-! WHIZARD 2.4.1 Mar 24 2017
+! WHIZARD 2.6.4 Aug 23 2018
 !
-! Copyright (C) 1999-2017 by
+! Copyright (C) 1999-2018 by
 !     Wolfgang Kilian <kilian@physik.uni-siegen.de>
 !     Thorsten Ohl <ohl@physik.uni-wuerzburg.de>
 !     Juergen Reuter <juergen.reuter@desy.de>
 !
 !     with contributions from
-!     Fabian Bach <fabian.bach@t-online.de>
-!     Bijan Chokoufe <bijan.chokoufe@desy.de>
-!     Christian Speckner <cnspeckn@googlemail.com>
-!     So Young Shim <soyoung.shim@desy.de>
-!     Florian Staub <florian.staub@cern.ch>
-!     Christian Weiss <christian.weiss@desy.de>
-!     and Hans-Werner Boschmann, Felix Braam,
-!     Sebastian Schmidt, So-young Shim, Daniel Wiesler
+!     cf. main AUTHORS file
 !
 ! WHIZARD is free software; you can redistribute it and/or modify it
 ! under the terms of the GNU General Public License as published by
@@ -71,6 +64,7 @@ module events
      integer :: norm_mode = NORM_UNDEFINED
      integer :: factorization_mode = FM_IGNORE_HELICITY
      logical :: keep_correlations = .false.
+     logical :: colorize_subevt = .false.
      real(default) :: sigma = 1
      integer :: n = 1
      real(default) :: safety_factor = 1
@@ -114,11 +108,14 @@ module events
      procedure :: set_analysis => event_set_analysis
      procedure :: setup_expressions => event_setup_expressions
      procedure :: evaluate_transforms => event_evaluate_transforms
+     procedure :: set_index => event_set_index
+     procedure :: increment_index => event_increment_index
      procedure :: evaluate_expressions => event_evaluate_expressions
      procedure :: passed_selection => event_passed_selection
      procedure :: store_alt_values => event_store_alt_values
      procedure :: is_nlo => event_is_nlo
-     procedure :: reset => event_reset
+     procedure :: reset_contents => event_reset_contents
+     procedure :: reset_index => event_reset_index
      procedure :: import_instance_results => event_import_instance_results
      procedure :: accept_sqme_ref => event_accept_sqme_ref
      procedure :: accept_sqme_prc => event_accept_sqme_prc
@@ -142,7 +139,8 @@ module events
      procedure :: has_transform => event_has_transform
      procedure :: get_norm_mode => event_get_norm_mode
      procedure :: get_kinematical_weight => event_get_kinematical_weight
-     procedure :: get_index => event_get_index
+     procedure :: has_index => event_has_index 
+     procedure :: get_index => event_get_index 
      procedure :: get_fac_scale => event_get_fac_scale
      procedure :: get_alpha_s => event_get_alpha_s
      procedure :: get_sqrts => event_get_sqrts
@@ -178,6 +176,9 @@ contains
        write (u, "(A)")  "factorize"
     end select
     write (u, "(3x,A,L1)")  "Keep correlations  = ", object%keep_correlations
+    if (object%colorize_subevt) then
+       write (u, "(3x,A,L1)")  "Colorize subevent  = ", object%colorize_subevt
+    end if
     if (.not. nearly_equal (object%safety_factor, one)) then
        write (u, "(3x,A," // FMT_12 // ")")  &
             "Safety factor      = ", object%safety_factor
@@ -267,10 +268,14 @@ contains
     dec = .true.;  if (present (show_decay))  dec = show_decay
     verb = .false.;  if (present (verbose))  verb = verbose
     call write_separator (u, 2)
+    write (u, "(1x,A)", advance="no")  "Event"
+    if (object%has_index ()) then
+       write (u, "(1x,'#',I0)", advance="no")  object%get_index ()
+    end if
     if (object%is_complete) then
-       write (u, "(1x,A)")  "Event"
+       write (u, *)
     else
-       write (u, "(1x,A)")  "Event [incomplete]"
+       write (u, "(1x,A)")  "[incomplete]"
     end if
     call write_separator (u)
     call object%config%write (u)
@@ -396,6 +401,8 @@ contains
        else
           event%config%factorization_mode = FM_IGNORE_HELICITY
        end if
+       event%config%colorize_subevt = &
+            var_list%get_lval (var_str ("?colorize_subevt"))
        if (event%config%unweighted) then
           event%config%safety_factor = var_list%get_rval (&
                var_str ("safety_factor"))
@@ -475,6 +482,7 @@ contains
     call event%expr%setup_selection (event%config%ef_selection)
     call event%expr%setup_analysis (event%config%ef_analysis)
     call event%expr%setup_reweight (event%config%ef_reweight)
+    call event%expr%colorize (event%config%colorize_subevt)
   end subroutine event_setup_expressions
 
   subroutine event_evaluate_transforms (event, r)
@@ -483,7 +491,8 @@ contains
     class(evt_t), pointer :: evt
     real(default) :: sigma_over_sqme
     integer :: i_term
-    logical :: failed_but_keep = .false.
+    logical :: failed_but_keep
+    failed_but_keep = .false.
     call msg_debug (D_TRANSFORMS, "event_evaluate_transforms")
     call event%discard_particle_set ()
     call event%check ()
@@ -558,6 +567,18 @@ contains
     end subroutine print_transform_name_if_debug
   end subroutine event_evaluate_transforms
 
+  subroutine event_set_index (event, index)
+    class(event_t), intent(inout) :: event
+    integer, intent(in) :: index
+    call event%expr%set_event_index (index)
+  end subroutine event_set_index
+  
+  subroutine event_increment_index (event, offset)
+    class(event_t), intent(inout) :: event
+    integer, intent(in), optional :: offset
+    call event%expr%increment_event_index (offset)
+  end subroutine event_increment_index
+  
   subroutine event_evaluate_expressions (event)
     class(event_t), intent(inout) :: event
     type(particle_set_t), pointer :: particle_set
@@ -618,15 +639,15 @@ contains
     end if
   end function event_is_nlo
 
-  subroutine event_reset (event)
+  subroutine event_reset_contents (event)
     class(event_t), intent(inout) :: event
     class(evt_t), pointer :: evt
-    call event%base_reset ()
+    call event%base_reset_contents ()
     event%selected_i_mci = 0
     event%selected_i_term = 0
     event%selected_channel = 0
     event%is_complete = .false.
-    call event%expr%reset ()
+    call event%expr%reset_contents ()
     event%selection_evaluated = .false.
     event%passed = .false.
     event%analysis_flag = .false.
@@ -640,7 +661,12 @@ contains
        call evt%reset ()
        evt => evt%next
     end do
-  end subroutine event_reset
+  end subroutine event_reset_contents
+
+  subroutine event_reset_index (event)
+    class(event_t), intent(inout) :: event
+    call event%expr%reset_event_index ()
+  end subroutine event_reset_index
 
   subroutine event_import_instance_results (event)
     class(event_t), intent(inout) :: event
@@ -728,9 +754,10 @@ contains
     integer, intent(in) :: i_mci
     real(default), dimension(:), intent(in), optional :: r
     integer, intent(in), optional :: i_nlo
-    logical :: generate_new = .true.
+    logical :: generate_new
+    generate_new = .true.
     if (present (i_nlo)) generate_new = (i_nlo == 1)
-    if (generate_new) call event%reset ()
+    if (generate_new) call event%reset_contents ()
     event%selected_i_mci = i_mci
     if (event%config%unweighted) then
        call event%instance%generate_unweighted_event (i_mci)
@@ -811,13 +838,16 @@ contains
   end subroutine event_set_scale_forced
 
   subroutine event_recalculate &
-       (event, update_sqme, weight_factor, recover_beams)
+       (event, update_sqme, weight_factor, recover_beams, recover_phs)
     class(event_t), intent(inout) :: event
     logical, intent(in) :: update_sqme
     real(default), intent(in), optional :: weight_factor
     logical, intent(in), optional :: recover_beams
+    logical, intent(in), optional :: recover_phs
     type(particle_set_t), pointer :: particle_set
     integer :: i_mci, i_term, channel
+    logical :: rec_phs_mci
+    rec_phs_mci = .true.;  if (present (recover_phs))  rec_phs_mci = recover_phs
     if (event%has_valid_particle_set ()) then
        particle_set => event%get_particle_set_ptr ()
        i_mci = event%selected_i_mci
@@ -832,8 +862,8 @@ contains
           call event%instance%set_alpha_qcd_forced &
                (i_term, event%alpha_qcd_forced)
        end if
-       call event%instance%recover (channel, i_term, update_sqme, &
-            event%scale_forced)
+       call event%instance%recover (channel, i_term, &
+            update_sqme, rec_phs_mci, event%scale_forced)
        if (signal_is_pending ())  return
        if (update_sqme .and. present (weight_factor)) then
           call event%instance%evaluate_event_data &
@@ -841,7 +871,7 @@ contains
        else if (event%weight_ref_is_known ()) then
           call event%instance%evaluate_event_data &
                (weight = event%get_weight_ref ())
-       else
+       else if (rec_phs_mci) then
           call event%instance%recover_event ()
           if (signal_is_pending ())  return
           call event%instance%evaluate_event_data ()
@@ -928,10 +958,16 @@ contains
     end if
   end function event_get_kinematical_weight
 
+  function event_has_index (event) result (flag)
+    class(event_t), intent(in) :: event
+    logical :: flag
+    flag = event%expr%has_event_index ()
+  end function event_has_index
+
   function event_get_index (event) result (index)
     class(event_t), intent(in) :: event
     integer :: index
-    index = event%expr%index
+    index = event%expr%get_event_index ()
   end function event_get_index
 
   function event_get_fac_scale (event) result (fac_scale)
