@@ -1,19 +1,12 @@
-! WHIZARD 2.4.1 Mar 24 2017
+! WHIZARD 2.6.4 Aug 23 2018
 !
-! Copyright (C) 1999-2017 by
+! Copyright (C) 1999-2018 by
 !     Wolfgang Kilian <kilian@physik.uni-siegen.de>
 !     Thorsten Ohl <ohl@physik.uni-wuerzburg.de>
 !     Juergen Reuter <juergen.reuter@desy.de>
 !
 !     with contributions from
-!     Fabian Bach <fabian.bach@t-online.de>
-!     Bijan Chokoufe <bijan.chokoufe@desy.de>
-!     Christian Speckner <cnspeckn@googlemail.com>
-!     So Young Shim <soyoung.shim@desy.de>
-!     Florian Staub <florian.staub@cern.ch>
-!     Christian Weiss <christian.weiss@desy.de>
-!     and Hans-Werner Boschmann, Felix Braam,
-!     Sebastian Schmidt, So-young Shim, Daniel Wiesler
+!     cf. main AUTHORS file
 !
 ! WHIZARD is free software; you can redistribute it and/or modify it
 ! under the terms of the GNU General Public License as published by
@@ -57,6 +50,10 @@ module subevents
   public :: prt_get_msq
   public :: prt_is_polarized
   public :: prt_get_helicity
+  public :: prt_is_colorized
+  public :: prt_get_n_col
+  public :: prt_get_n_acl
+  public :: prt_get_color_indices
   public :: c_prt
   public :: prt_write
   public :: are_disjoint
@@ -78,6 +75,7 @@ module subevents
   public :: subevt_set_p2_incoming
   public :: subevt_set_p2_outgoing
   public :: subevt_polarize
+  public :: subevt_colorize
   public :: subevt_is_nonempty
   public :: subevt_get_length
   public :: subevt_get_prt
@@ -109,10 +107,13 @@ module subevents
      integer :: type = PRT_UNDEFINED
      integer :: pdg
      logical :: polarized = .false.
+     logical :: colorized = .false.
      integer :: h
      type(vector4_t) :: p
      real(default) :: p2
      integer, dimension(:), allocatable :: src
+     integer, dimension(:), allocatable :: col
+     integer, dimension(:), allocatable :: acl
   end type prt_t
 
   type :: subevt_t
@@ -245,6 +246,50 @@ contains
     h = prt%h
   end function prt_get_helicity
 
+  elemental function prt_is_colorized (prt) result (flag)
+    logical :: flag
+    type(prt_t), intent(in) :: prt
+    flag = prt%colorized
+  end function prt_is_colorized
+
+  elemental function prt_get_n_col (prt) result (n)
+    integer :: n
+    type(prt_t), intent(in) :: prt
+    integer, dimension(:), allocatable :: col, acl
+    integer :: i
+    n = 0
+    if (prt%colorized) then
+       do i = 1, size (prt%col)
+          if (all (prt%col(i) /= prt%acl)) n = n + 1
+       end do
+    end if
+  end function prt_get_n_col
+
+  elemental function prt_get_n_acl (prt) result (n)
+    integer :: n
+    type(prt_t), intent(in) :: prt
+    integer, dimension(:), allocatable :: col, acl
+    integer :: i
+    n = 0
+    if (prt%colorized) then
+       do i = 1, size (prt%acl)
+          if (all (prt%acl(i) /= prt%col)) n = n + 1
+       end do
+    end if
+  end function prt_get_n_acl
+
+  subroutine prt_get_color_indices (prt, col, acl)
+    type(prt_t), intent(in) :: prt
+    integer, dimension(:), allocatable, intent(out) :: col, acl
+    if (prt%colorized) then
+       col = prt%col
+       acl = prt%acl
+    else
+       col = [integer::]
+       acl = [integer::]
+    end if
+  end subroutine prt_get_color_indices
+
   subroutine prt_set (prt, pdg, p, p2, src)
     type(prt_t), intent(inout) :: prt
     integer, intent(in) :: pdg
@@ -290,6 +335,14 @@ contains
     prt%h = h
   end subroutine prt_polarize
 
+  subroutine prt_colorize (prt, col, acl)
+    type(prt_t), intent(inout) :: prt
+    integer, dimension(:), intent(in) :: col, acl
+    prt%colorized = .true.
+    prt%col = col
+    prt%acl = acl
+  end subroutine prt_colorize
+
   elemental function c_prt_from_prt (prt) result (c_prt)
     type(c_prt_t) :: c_prt
     type(prt_t), intent(in) :: prt
@@ -334,6 +387,15 @@ contains
        end if
     end select
     select case (prt%type)
+    case (PRT_BEAM, PRT_INCOMING, PRT_OUTGOING, PRT_COMPOSITE)
+       if (prt%colorized) then
+          write (u, "(*(I0,:,','))", advance="no")  prt%col
+          write (u, "('/')", advance="no")
+          write (u, "(*(I0,:,','))", advance="no")  prt%acl
+          write (u, "('|')", advance="no")
+       end if
+    end select
+    select case (prt%type)
     case (PRT_BEAM, PRT_INCOMING, PRT_OUTGOING, PRT_COMPOSITE)
        write (u, "(" // FMT_14 // ",';'," // FMT_14 // ",','," // &
             FMT_14 // ",','," // FMT_14 // ")", advance="no") tmp%p
@@ -363,9 +425,27 @@ contains
     type(prt_t), intent(in) :: prt_in1, prt_in2
     logical :: ok
     integer, dimension(:), allocatable :: src
+    integer, dimension(:), allocatable :: col1, acl1, col2, acl2
     call combine_index_lists (src, prt_in1%src, prt_in2%src)
     ok = allocated (src)
-    if (ok)  call prt_init_composite (prt, prt_in1%p + prt_in2%p, src)
+    if (ok) then
+       call prt_init_composite (prt, prt_in1%p + prt_in2%p, src)
+       if (prt_in1%colorized .or. prt_in2%colorized) then
+          select case (prt_in1%type)
+          case default
+             call prt_get_color_indices (prt_in1, col1, acl1)
+          case (PRT_BEAM, PRT_INCOMING)
+             call prt_get_color_indices (prt_in1, acl1, col1)
+          end select
+          select case (prt_in2%type)
+          case default
+             call prt_get_color_indices (prt_in2, col2, acl2)
+          case (PRT_BEAM, PRT_INCOMING)
+             call prt_get_color_indices (prt_in2, acl2, col2)
+          end select
+          call prt_colorize (prt, [col1, col2], [acl1, acl2])
+       end if
+    end if
   end subroutine prt_combine
 
   function are_disjoint (prt_in1, prt_in2) result (flag)
@@ -658,6 +738,20 @@ contains
     call prt_polarize (subevt%prt(i), h)
   end subroutine subevt_polarize
 
+  subroutine subevt_colorize (subevt, i, col, acl)
+    type(subevt_t), intent(inout) :: subevt
+    integer, intent(in) :: i, col, acl
+    if (col > 0 .and. acl > 0) then
+       call prt_colorize (subevt%prt(i), [col], [acl])
+    else if (col > 0) then
+       call prt_colorize (subevt%prt(i), [col], [integer ::])
+    else if (acl > 0) then
+       call prt_colorize (subevt%prt(i), [integer ::], [acl])
+    else
+       call prt_colorize (subevt%prt(i), [integer ::], [integer ::])
+    end if
+  end subroutine subevt_colorize
+
   function subevt_is_nonempty (subevt) result (flag)
     logical :: flag
     type(subevt_t), intent(in) :: subevt
@@ -818,65 +912,25 @@ contains
     logical, dimension(:), intent(in) :: mask1
     type(jet_definition_t), intent(in) :: jet_def
     logical, intent(in) :: keep_jets
-    integer, dimension(:), allocatable :: src, src_tmp
-    integer, dimension(:), allocatable :: map, jet_idx
+    integer, dimension(:), allocatable :: map, jet_index
     type(pseudojet_t), dimension(:), allocatable :: jet_in, jet_out
     type(pseudojet_vector_t) :: jv_in, jv_out
     type(cluster_sequence_t) :: cs
-    integer :: i, prt_idx, k, n_src, n_active, this_jet, combined_pdg, pdg, n_quarks
-    n_active = 0
-    allocate (map (pl1%n_active), source = 0)
-    allocate (src (0))
-    do i = 1, pl1%n_active
-       if (mask1(i)) then
-          call combine_index_lists (src_tmp, src, pl1%prt(i)%src)
-          if (allocated (src_tmp)) then
-             call move_alloc (from=src_tmp, to=src)
-             n_active = n_active + 1
-             map(n_active) = i
-          end if
-       end if
-    end do
-    allocate (jet_in (count (map /= 0)))
-    do i = 1, size (jet_in)
+    integer :: i, n_src, n_active
+    call map_prt_index (pl1, mask1, n_src, map)
+    n_active = count (map /= 0)
+    allocate (jet_in (n_active))
+    allocate (jet_index (n_active))
+    do i = 1, n_active
        call jet_in(i)%init (prt_get_momentum (pl1%prt(map(i))))
     end do
     call jv_in%init (jet_in)
     call cs%init (jv_in, jet_def)
     jv_out = cs%inclusive_jets ()
-    allocate (jet_idx (size (jet_in)))
-    call cs%assign_jet_indices (jv_out, jet_idx)
+    call cs%assign_jet_indices (jv_out, jet_index)
     allocate (jet_out (jv_out%size ()))
     jet_out = jv_out
-    call subevt_reset (subevt, size (jet_out))
-    do this_jet = 1, size (jet_out)
-       src = 0
-       n_src = 0
-       combined_pdg = 0
-       n_quarks = 0
-       do prt_idx = 1, size (jet_idx)
-          if (jet_idx(prt_idx) == this_jet) then
-             associate (prt => pl1%prt(map(prt_idx)))
-               do k = 1, size (prt%src)
-                  src(n_src + k) = prt%src(k)
-               end do
-               n_src = n_src + size (prt%src)
-               if (is_quark (prt%pdg)) then
-                  n_quarks = n_quarks + 1
-                  if (combined_pdg == 0) then
-                     combined_pdg = prt%pdg
-                  end if
-               end if
-             end associate
-          end if
-       end do
-       if (keep_jets .and. n_quarks == 1) then
-          pdg = combined_pdg
-       else
-          pdg = 0
-       end if
-       call prt_init_pseudojet (subevt%prt(this_jet), jet_out(this_jet), src(:n_src), pdg)
-    end do
+    call fill_pseudojet (subevt, pl1, jet_out, jet_index, n_src, map)
     do i = 1, size (jet_out)
        call jet_out(i)%final ()
     end do
@@ -886,6 +940,58 @@ contains
     do i = 1, size (jet_in)
        call jet_in(i)%final ()
     end do
+  contains
+    ! Uniquely combine sources and add map those new indices to the old ones
+    subroutine map_prt_index (pl1, mask1, n_src, map)
+      type(subevt_t), intent(in) :: pl1
+      logical, dimension(:), intent(in) :: mask1
+      integer, intent(out) :: n_src
+      integer, dimension(:), allocatable, intent(out) :: map
+      integer, dimension(:), allocatable :: src, src_tmp
+      integer :: i
+      allocate (src(0))
+      allocate (map (pl1%n_active), source = 0)
+      n_active = 0
+      do i = 1, pl1%n_active
+         if (.not. mask1(i)) cycle
+         call combine_index_lists (src_tmp, src, pl1%prt(i)%src)
+         if (.not. allocated (src_tmp)) cycle
+         call move_alloc (from=src_tmp, to=src)
+         n_active = n_active + 1
+         map(n_active) = i
+      end do
+      n_src = size (src)
+    end subroutine map_prt_index
+    ! Retrieve source(s) of a jet and fill corresponding subevent
+    subroutine fill_pseudojet (subevt, pl1, jet_out, jet_index, n_src, map)
+      type(subevt_t), intent(inout) :: subevt
+      type(subevt_t), intent(in) :: pl1
+      type(pseudojet_t), dimension(:), intent(in) :: jet_out
+      integer, dimension(:), intent(in) :: jet_index
+      integer, dimension(:), intent(in) :: map
+      integer, intent(in) :: n_src
+      integer, dimension(n_src) :: src_fill
+      integer :: i, jet, k, combined_pdg, pdg, n_quarks, n_src_fill
+      call subevt_reset (subevt, size (jet_out))
+      do jet = 1, size (jet_out)
+         pdg = 0; src_fill = 0; n_src_fill = 0; combined_pdg = 0; n_quarks = 0
+         PARTICLE: do i = 1, size (jet_index)
+            if (jet_index(i) /= jet) cycle PARTICLE
+            associate (prt => pl1%prt(map(i)), n_src_prt => size(pl1%prt(map(i))%src))
+              do k = 1, n_src_prt
+                 src_fill(n_src_fill + k) = prt%src(k)
+              end do
+              n_src_fill = n_src_fill + n_src_prt
+              if (is_quark (prt%pdg)) then
+                 n_quarks = n_quarks + 1
+                 if (combined_pdg == 0) combined_pdg = prt%pdg
+              end if
+            end associate
+         end do PARTICLE
+         if (keep_jets .and. n_quarks == 1) pdg = combined_pdg
+         call prt_init_pseudojet (subevt%prt(jet), jet_out(jet), src_fill(:n_src_fill), pdg)
+      end do
+    end subroutine fill_pseudojet
   end subroutine subevt_cluster
 
   subroutine subevt_select (subevt, pl, mask1)
@@ -948,17 +1054,15 @@ contains
     type(subevt_t), intent(inout) :: subevt
     type(subevt_t), intent(in) :: pl
     real(default), dimension(:), intent(in) :: rval
-    integer, dimension(size(rval)) :: idx
     integer :: i
+    integer, dimension(size(rval)) :: idx
     call subevt_reset (subevt, pl%n_active)
     subevt%n_active = pl%n_active
     if (allocated (subevt%prt))  deallocate (subevt%prt)
     allocate (subevt%prt (size(pl%prt)))
-    !!! !!! !!! Workaround for the ifort 16.0/1
-    !subevt%prt = pl%prt(order (rval))
     idx = order (rval)
-    do i = 1, size(idx)
-       subevt%prt(i) = pl%prt(idx(i))
+    do i = 1, size (idx)
+       subevt%prt(i) = pl%prt (idx(i))
     end do
   end subroutine subevt_sort_real
 
