@@ -1,19 +1,12 @@
-! WHIZARD 2.4.1 Mar 24 2017
+! WHIZARD 2.6.4 Aug 23 2018
 !
-! Copyright (C) 1999-2017 by
+! Copyright (C) 1999-2018 by
 !     Wolfgang Kilian <kilian@physik.uni-siegen.de>
 !     Thorsten Ohl <ohl@physik.uni-wuerzburg.de>
 !     Juergen Reuter <juergen.reuter@desy.de>
 !
 !     with contributions from
-!     Fabian Bach <fabian.bach@t-online.de>
-!     Bijan Chokoufe <bijan.chokoufe@desy.de>
-!     Christian Speckner <cnspeckn@googlemail.com>
-!     So Young Shim <soyoung.shim@desy.de>
-!     Florian Staub <florian.staub@cern.ch>
-!     Christian Weiss <christian.weiss@desy.de>
-!     and Hans-Werner Boschmann, Felix Braam,
-!     Sebastian Schmidt, So-young Shim, Daniel Wiesler
+!     cf. main AUTHORS file
 !
 ! WHIZARD is free software; you can redistribute it and/or modify it
 ! under the terms of the GNU General Public License as published by
@@ -60,6 +53,9 @@ module compilations
      type(string_t) :: static_external_tag
      type(process_library_t), pointer :: lib => null ()
      logical :: recompile_library = .false.
+     logical :: verbose = .false.
+     logical :: use_workspace = .false.
+     type(string_t) :: workspace
    contains
      procedure :: init => compilation_item_init
      procedure :: compile => compilation_item_compile
@@ -82,6 +78,12 @@ module compilations
   end type compilation_t
 
 
+  character(*), parameter :: ALLOWED_IN_DIRNAME = &
+       "abcdefghijklmnopqrstuvwxyz&
+       &ABCDEFGHIJKLMNOPQRSTUVWXYZ&
+       &1234567890&
+       &.,_-+="
+
 contains
 
   subroutine compilation_item_init (comp, libname, stack, var_list)
@@ -97,6 +99,17 @@ contains
     end if
     comp%recompile_library = &
          var_list%get_lval (var_str ("?recompile_library"))
+    comp%verbose = &
+         var_list%get_lval (var_str ("?me_verbose"))
+    comp%use_workspace = &
+         var_list%is_known (var_str ("$compile_workspace"))
+    if (comp%use_workspace) then
+       comp%workspace = &
+            var_list%get_sval (var_str ("$compile_workspace"))
+       if (comp%workspace == "")  comp%use_workspace = .false.
+    else
+       comp%workspace = ""
+    end if
   end subroutine compilation_item_init
 
   subroutine compilation_item_compile (comp, model, os_data, force, recompile)
@@ -105,32 +118,49 @@ contains
     type(os_data_t), intent(in) :: os_data
     logical, intent(in) :: force, recompile
     if (associated (comp%lib)) then
+       if (comp%use_workspace)  call setup_workspace (comp%workspace, os_data)
        call msg_message ("Process library '" &
             // char (comp%libname) // "': compiling ...")
        call comp%lib%configure (os_data)
        if (signal_is_pending ())  return
        call comp%lib%compute_md5sum (model)
-       call comp%lib%write_makefile (os_data, force)
+       call comp%lib%write_makefile &
+            (os_data, force, verbose=comp%verbose, workspace=comp%workspace)
        if (signal_is_pending ())  return
        if (force) then
-          call comp%lib%clean (os_data, distclean = .false.)
+          call comp%lib%clean &
+               (os_data, distclean = .false., workspace=comp%workspace)
           if (signal_is_pending ())  return
        end if
-       call comp%lib%write_driver (force)
+       call comp%lib%write_driver (force, workspace=comp%workspace)
        if (signal_is_pending ())  return
        if (recompile) then
-          call comp%lib%load (os_data, keep_old_source = .true.)
+          call comp%lib%load &
+               (os_data, keep_old_source = .true., workspace=comp%workspace)
           if (signal_is_pending ())  return
        end if
-       call comp%lib%update_status (os_data)
+       call comp%lib%update_status (os_data, workspace=comp%workspace)
     end if
   end subroutine compilation_item_compile
 
+  subroutine setup_workspace (workspace, os_data)
+    type(string_t), intent(in) :: workspace
+    type(os_data_t), intent(in) :: os_data
+    if (verify (workspace, ALLOWED_IN_DIRNAME) == 0) then
+       call msg_message ("Compile: preparing workspace directory '" &
+            // char (workspace) // "'")
+       call os_system_call ("mkdir -p '" // workspace // "'")
+    else
+       call msg_fatal ("compile: workspace name '" &
+            // char (workspace) // "' contains illegal characters")
+    end if
+  end subroutine setup_workspace
+
   subroutine compilation_item_load (comp, os_data)
     class(compilation_item_t), intent(inout) :: comp
     type(os_data_t), intent(in) :: os_data
     if (associated (comp%lib)) then
-       call comp%lib%load (os_data)
+       call comp%lib%load (os_data, workspace=comp%workspace)
     end if
   end subroutine compilation_item_load
 
@@ -246,9 +276,11 @@ contains
     close (u)
   end subroutine compilation_write_dispatcher
 
-  subroutine compilation_write_makefile (compilation, os_data, ext_libtag)
+  subroutine compilation_write_makefile &
+       (compilation, os_data, ext_libtag, verbose)
     class(compilation_t), intent(in) :: compilation
     type(os_data_t), intent(in) :: os_data
+    logical, intent(in) :: verbose
     type(string_t), intent(in), optional :: ext_libtag
     type(string_t) :: file, ext_tag
     integer :: u, i
@@ -287,8 +319,13 @@ contains
     write (u, "(A)") ""
     write (u, "(A)") "# Libtool"
     write (u, "(A)") "LIBTOOL = " // char (os_data%whizard_libtool)
-    write (u, "(A)") "FCOMPILE = $(LIBTOOL) --tag=FC --mode=compile"
-    write (u, "(A)") "LINK = $(LIBTOOL) --tag=FC --mode=link"
+    if (verbose) then
+       write (u, "(A)") "FCOMPILE = $(LIBTOOL) --tag=FC --mode=compile"
+       write (u, "(A)") "LINK = $(LIBTOOL) --tag=FC --mode=link"
+    else
+       write (u, "(A)") "FCOMPILE = @$(LIBTOOL) --silent --tag=FC --mode=compile"
+       write (u, "(A)") "LINK = @$(LIBTOOL) --silent --tag=FC --mode=link"
+    end if
     write (u, "(A)") ""
     write (u, "(A)") "# Compile commands (default)"
     write (u, "(A)") "LTFCOMPILE = $(FCOMPILE) $(FC) -c $(FCINCL) $(FCFLAGS)"
@@ -308,10 +345,16 @@ contains
     write (u, "(A)") "# Library dispatcher"
     write (u, "(A)") "DISP = $(EXE)_prclib_dispatcher"
     write (u, "(A)") "$(DISP).lo: $(DISP).f90 $(LIBRARIES)"
+    if (.not. verbose) then
+       write (u, "(A)")  TAB // '@echo  "  FC       " $@'
+    end if
     write (u, "(A)") TAB // "$(LTFCOMPILE) $<"
     write (u, "(A)") ""
     write (u, "(A)") "# Executable"
     write (u, "(A)") "$(EXE): $(DISP).lo $(LIBRARIES)"
+    if (.not. verbose) then
+       write (u, "(A)")  TAB // '@echo  "  FCLD     " $@'
+    end if
     write (u, "(A)") TAB // "$(LINK) $(FC) -static-libtool-libs $(FCFLAGS) \"
     write (u, "(A)") TAB // "   $(LDWHIZARD) $(LDFLAGS) \"
     write (u, "(A)") TAB // "   -o $(EXE) $^ \"
@@ -367,13 +410,15 @@ contains
     type(compilation_t) :: compilation
     type(compilation_item_t) :: item
     type(string_t) :: ext_libtag
-    logical :: force, recompile
+    logical :: force, recompile, verbose
     integer :: i
     ext_libtag = ""
     force = &
          global%var_list%get_lval (var_str ("?rebuild_library"))
     recompile = &
          global%var_list%get_lval (var_str ("?recompile_library"))
+    verbose = &
+         global%var_list%get_lval (var_str ("?me_verbose"))
     call compilation%init (exename, [libname])
     if (signal_is_pending ())  return
     call compilation%write_dispatcher ()
@@ -386,7 +431,8 @@ contains
        if (signal_is_pending ())  return
        call item%success ()
     end do
-    call compilation%write_makefile (global%os_data, ext_libtag)
+    call compilation%write_makefile &
+         (global%os_data, ext_libtag=ext_libtag, verbose=verbose)
     if (signal_is_pending ())  return
     call compilation%make_compile (global%os_data)
     if (signal_is_pending ())  return
