@@ -1,19 +1,12 @@
-! WHIZARD 2.4.1 Mar 24 2017
+! WHIZARD 2.6.4 Aug 23 2018
 !
-! Copyright (C) 1999-2017 by
+! Copyright (C) 1999-2018 by
 !     Wolfgang Kilian <kilian@physik.uni-siegen.de>
 !     Thorsten Ohl <ohl@physik.uni-wuerzburg.de>
 !     Juergen Reuter <juergen.reuter@desy.de>
 !
 !     with contributions from
-!     Fabian Bach <fabian.bach@t-online.de>
-!     Bijan Chokoufe <bijan.chokoufe@desy.de>
-!     Christian Speckner <cnspeckn@googlemail.com>
-!     So Young Shim <soyoung.shim@desy.de>
-!     Florian Staub <florian.staub@cern.ch>
-!     Christian Weiss <christian.weiss@desy.de>
-!     and Hans-Werner Boschmann, Felix Braam,
-!     Sebastian Schmidt, So-young Shim, Daniel Wiesler
+!     cf. main AUTHORS file
 !
 ! WHIZARD is free software; you can redistribute it and/or modify it
 ! under the terms of the GNU General Public License as published by
@@ -62,6 +55,9 @@ module rt_data
   private
 
   public :: rt_data_t
+  public :: fix_system_dependencies
+  public :: show_description_of_string
+  public :: show_tex_descriptions
 
   type :: rt_parse_nodes_t
      type(parse_node_t), pointer :: cuts_lexpr => null ()
@@ -82,6 +78,7 @@ module rt_data
   type :: rt_data_t
      type(lexer_t), pointer :: lexer => null ()
      type(rt_data_t), pointer :: context => null ()
+     type(string_t), dimension(:), allocatable :: export
      type(var_list_t) :: var_list
      type(iterations_list_t) :: it_list
      type(os_data_t) :: os_data
@@ -103,6 +100,7 @@ module rt_data
      type(string_t) :: logfile
      logical :: nlo_fixed_order = .false.
      logical, dimension(0:5) :: selected_nlo_parts = .false.
+     integer, dimension(:), allocatable :: nlo_component
    contains
      procedure :: write => rt_data_write
      procedure :: write_vars => rt_data_write_vars
@@ -111,6 +109,8 @@ module rt_data
      procedure :: write_beams => rt_data_write_beams
      procedure :: write_expr => rt_data_write_expr
      procedure :: write_process_stack => rt_data_write_process_stack
+     procedure :: write_var_descriptions => rt_data_write_var_descriptions
+     procedure :: show_description_of_string => rt_data_show_description_of_string
      procedure :: clear_beams => rt_data_clear_beams
      procedure :: global_init => rt_data_global_init
      procedure :: local_init => rt_data_local_init
@@ -119,9 +119,15 @@ module rt_data
      procedure :: deactivate => rt_data_deactivate
      procedure :: copy_globals => rt_data_copy_globals
      procedure :: restore_globals => rt_data_restore_globals
+     procedure :: write_exports => rt_data_write_exports
+     procedure :: get_n_export => rt_data_get_n_export
+     procedure :: append_exports => rt_data_append_exports
+     procedure :: handle_exports => rt_data_handle_exports
+     procedure :: transfer_process_stack => rt_data_transfer_process_stack
      procedure :: final => rt_data_global_final
      procedure :: local_final => rt_data_local_final
      procedure :: read_model => rt_data_read_model
+     procedure :: read_ufo_model => rt_data_read_ufo_model
      procedure :: init_fallback_model => rt_data_init_fallback_model
      procedure :: select_model => rt_data_select_model
      procedure :: unselect_model => rt_data_unselect_model
@@ -274,6 +280,12 @@ contains
     u = given_output_unit (unit)
     call write_separator (u, 2)
     write (u, "(1x,A)")  "Runtime data:"
+    if (object%get_n_export () > 0) then
+       call write_separator (u, 2)
+       write (u, "(1x,A)")  "Exported objects and variables:"
+       call write_separator (u)
+       call object%write_exports (u)
+    end if
     if (present (vars)) then
        if (size (vars) /= 0) then
           call write_separator (u, 2)
@@ -329,21 +341,19 @@ contains
   subroutine rt_data_write_vars (object, unit, vars)
     class(rt_data_t), intent(in), target :: object
     integer, intent(in), optional :: unit
-    type(string_t), dimension(:), intent(in), optional :: vars
+    type(string_t), dimension(:), intent(in) :: vars
     type(var_list_t), pointer :: var_list
     integer :: u, i
     u = given_output_unit (unit)
-    if (present (vars)) then
-       var_list => object%get_var_list_ptr ()
-       do i = 1, size (vars)
-          associate (var => vars(i))
-            if (var_list%contains (var, follow_link=.true.)) then
-               call var_list%write_var (var, unit = u, &
-                    follow_link = .true.)
-            end if
-          end associate
-       end do
-    end if
+    var_list => object%get_var_list_ptr ()
+    do i = 1, size (vars)
+       associate (var => vars(i))
+         if (var_list%contains (var, follow_link=.true.)) then
+            call var_list%write_var (var, unit = u, &
+                 follow_link = .true., defined=.true.)
+         end if
+       end associate
+    end do
   end subroutine rt_data_write_vars
 
   subroutine rt_data_write_model_list (object, unit)
@@ -389,6 +399,32 @@ contains
     call object%process_stack%write (unit)
   end subroutine rt_data_write_process_stack
 
+  subroutine rt_data_write_var_descriptions (rt_data, unit, ascii_output)
+    class(rt_data_t), intent(in) :: rt_data
+    integer, intent(in), optional :: unit
+    logical, intent(in), optional :: ascii_output
+    integer :: u
+    logical :: ao
+    u = given_output_unit (unit)
+    ao = .false.;  if (present (ascii_output))  ao = ascii_output
+    call rt_data%var_list%write (u, follow_link=.true., &
+         descriptions=.true., ascii_output=ao)
+  end subroutine rt_data_write_var_descriptions
+
+  subroutine rt_data_show_description_of_string (rt_data, string, &
+         unit, ascii_output)
+    class(rt_data_t), intent(in) :: rt_data
+    type(string_t), intent(in) :: string
+    integer, intent(in), optional :: unit
+    logical, intent(in), optional :: ascii_output
+    integer :: u
+    logical :: ao
+    u = given_output_unit (unit)
+    ao = .false.;  if (present (ascii_output))  ao = ascii_output
+    call rt_data%var_list%write_var (string, unit=u, follow_link=.true., &
+         defined=.false., descriptions=.true., ascii_output=ao)
+  end subroutine rt_data_show_description_of_string
+
   subroutine rt_data_clear_beams (global)
     class(rt_data_t), intent(inout) :: global
     call global%beam_structure%final_sf ()
@@ -442,9 +478,17 @@ contains
     class(rt_data_t), intent(inout), target :: local
     logical, target, save :: known = .true.
     call local%var_list%append_string_ptr (var_str ("$fc"), &
-         local%os_data%fc, known, intrinsic=.true.)
+         local%os_data%fc, known, intrinsic=.true., &
+         description=var_str('This string variable gives the ' // &
+         '\ttt{Fortran} compiler used within \whizard. It can ' // &
+         'only be accessed, not set by the user. (cf. also ' // &
+         '\ttt{\$fcflags})'))
     call local%var_list%append_string_ptr (var_str ("$fcflags"), &
-         local%os_data%fcflags, known, intrinsic=.true.)
+         local%os_data%fcflags, known, intrinsic=.true., &
+         description=var_str('This string variable gives the ' // &
+         'compiler flags for the \ttt{Fortran} compiler used ' // &
+         'within \whizard. It can only be accessed, not set by ' // &
+         'the user. (cf. also \ttt{\$fc})'))
   end subroutine rt_data_init_pointer_variables
 
   subroutine rt_data_activate (local)
@@ -541,10 +585,95 @@ contains
 
   subroutine rt_data_restore_globals (global, local)
     class(rt_data_t), intent(inout) :: global
-    class(rt_data_t), intent(in) :: local
+    class(rt_data_t), intent(inout) :: local
     global%prclib_stack = local%prclib_stack
+    call local%handle_exports (global)
   end subroutine rt_data_restore_globals
 
+  subroutine rt_data_write_exports (rt_data, unit)
+    class(rt_data_t), intent(in) :: rt_data
+    integer, intent(in), optional :: unit
+    integer :: u, i
+    u = given_output_unit (unit)
+    do i = 1, rt_data%get_n_export ()
+       write (u, "(A)")  char (rt_data%export(i))
+    end do
+  end subroutine rt_data_write_exports
+
+  function rt_data_get_n_export (rt_data) result (n)
+    class(rt_data_t), intent(in) :: rt_data
+    integer :: n
+    if (allocated (rt_data%export)) then
+       n = size (rt_data%export)
+    else
+       n = 0
+    end if
+  end function rt_data_get_n_export
+
+  subroutine rt_data_append_exports (rt_data, export)
+    class(rt_data_t), intent(inout) :: rt_data
+    type(string_t), dimension(:), intent(in) :: export
+    logical, dimension(:), allocatable :: mask
+    type(string_t), dimension(:), allocatable :: tmp
+    integer :: i, j, n
+    if (.not. allocated (rt_data%export))  allocate (rt_data%export (0))
+    n = size (rt_data%export)
+    allocate (mask (size (export)), source=.false.)
+    do i = 1, size (export)
+       mask(i) = all (export(i) /= rt_data%export) &
+            .and. all (export(i) /= export(:i-1))
+    end do
+    if (count (mask) > 0) then
+       allocate (tmp (n + count (mask)))
+       tmp(1:n) = rt_data%export(:)
+       j = n
+       do i = 1, size (export)
+          if (mask(i)) then
+             j = j + 1
+             tmp(j) = export(i)
+          end if
+       end do
+       call move_alloc (from=tmp, to=rt_data%export)
+    end if
+  end subroutine rt_data_append_exports
+
+  subroutine rt_data_handle_exports (local, global)
+    class(rt_data_t), intent(inout), target :: local
+    class(rt_data_t), intent(inout), target :: global
+    type(string_t) :: export
+    integer :: i
+    if (local%get_n_export () > 0) then
+       do i = 1, local%get_n_export ()
+          export = local%export(i)
+          select case (char (export))
+          case ("results")
+             call msg_message ("Exporting integration results &
+                  &to outer environment")
+             call local%transfer_process_stack (global)
+          case default
+             call msg_bug ("handle exports: '" &
+                  // char (export) // "' unsupported")
+          end select
+       end do
+    end if
+  end subroutine rt_data_handle_exports
+
+  subroutine rt_data_transfer_process_stack (local, global)
+    class(rt_data_t), intent(inout), target :: local
+    class(rt_data_t), intent(inout), target :: global
+    type(process_entry_t), pointer :: process
+    type(string_t) :: process_id
+    do
+       call local%process_stack%pop_last (process)
+       if (.not. associated (process))  exit
+       process_id = process%get_id ()
+       call global%process_stack%push (process)
+       call global%process_stack%fill_result_vars (process_id)
+       call global%process_stack%update_result_vars &
+            (process_id, global%var_list)
+    end do
+  end subroutine rt_data_transfer_process_stack
+    
   subroutine rt_data_global_final (global)
     class(rt_data_t), intent(inout) :: global
     call global%process_stack%final ()
@@ -575,6 +704,17 @@ contains
          (name, filename, global%os_data, model, scheme)
   end subroutine rt_data_read_model
 
+  subroutine rt_data_read_ufo_model (global, name, model, ufo_path)
+    class(rt_data_t), intent(inout) :: global
+    type(string_t), intent(in) :: name
+    type(model_t), pointer, intent(out) :: model
+    type(string_t), intent(in), optional :: ufo_path
+    type(string_t) :: filename
+    filename = name // ".ufo.mdl"
+    call global%model_list%read_model &
+         (name, filename, global%os_data, model, ufo=.true., ufo_path=ufo_path)
+  end subroutine rt_data_read_ufo_model
+
   subroutine rt_data_init_fallback_model (global, name, filename)
     class(rt_data_t), intent(inout) :: global
     type(string_t), intent(in) :: name, filename
@@ -582,24 +722,31 @@ contains
          (name, filename, global%os_data, global%fallback_model)
   end subroutine rt_data_init_fallback_model
 
-  subroutine rt_data_select_model (global, name, scheme)
+  subroutine rt_data_select_model (global, name, scheme, ufo, ufo_path)
     class(rt_data_t), intent(inout), target :: global
     type(string_t), intent(in) :: name
     type(string_t), intent(in), optional :: scheme
-    logical :: same_model
+    logical, intent(in), optional :: ufo
+    type(string_t), intent(in), optional :: ufo_path
+    logical :: same_model, ufo_model
+    ufo_model = .false.;  if (present (ufo))  ufo_model = ufo
     if (associated (global%model)) then
-       same_model = global%model%matches (name, scheme)
+       same_model = global%model%matches (name, scheme, ufo)
     else
        same_model = .false.
     end if
     if (.not. same_model) then
-       global%model => global%model_list%get_model_ptr (name, scheme)
+       global%model => global%model_list%get_model_ptr (name, scheme, ufo)
        if (.not. associated (global%model)) then
-          call global%read_model (name, global%model)
+          if (ufo_model) then
+             call global%read_ufo_model (name, global%model, ufo_path)
+          else
+             call global%read_model (name, global%model)
+          end if
           global%model_is_copy = .false.
        else if (associated (global%context)) then
           global%model_is_copy = &
-               global%model_list%model_exists (name, scheme, &
+               global%model_list%model_exists (name, scheme, ufo, &
                follow_link=.false.)
        else
           global%model_is_copy = .false.
@@ -609,7 +756,10 @@ contains
        call global%model%link_var_list (global%var_list)
        call global%var_list%set_string (var_str ("$model_name"), &
             name, is_known = .true.)
-       if (global%model%has_schemes ()) then
+       if (global%model%is_ufo_model ()) then
+          call msg_message ("Switching to model '" // char (name) // "' " &
+               // "(generated from UFO source)")
+       else if (global%model%has_schemes ()) then
           call msg_message ("Switching to model '" // char (name) // "', " &
                // "scheme '" // char (global%model%get_scheme ()) // "'")
        else
@@ -767,74 +917,81 @@ contains
     call global%var_list%undefine (follow_link=.false.)
   end subroutine rt_data_unset_values
 
-  subroutine rt_data_set_log (global, name, lval, is_known, verbose)
+  subroutine rt_data_set_log &
+       (global, name, lval, is_known, force, verbose)
     class(rt_data_t), intent(inout) :: global
     type(string_t), intent(in) :: name
     logical, intent(in) :: lval
     logical, intent(in) :: is_known
-    logical, intent(in), optional :: verbose
+    logical, intent(in), optional :: force, verbose
     call global%var_list%set_log (name, lval, is_known, &
-         verbose=verbose)
+         force=force, verbose=verbose)
   end subroutine rt_data_set_log
 
-  subroutine rt_data_set_int (global, name, ival, is_known, verbose)
+  subroutine rt_data_set_int &
+       (global, name, ival, is_known, force, verbose)
     class(rt_data_t), intent(inout) :: global
     type(string_t), intent(in) :: name
     integer, intent(in) :: ival
     logical, intent(in) :: is_known
-    logical, intent(in), optional :: verbose
+    logical, intent(in), optional :: force, verbose
     call global%var_list%set_int (name, ival, is_known, &
-         verbose=verbose)
+         force=force, verbose=verbose)
   end subroutine rt_data_set_int
 
-  subroutine rt_data_set_real (global, name, rval, is_known, verbose, pacified)
+  subroutine rt_data_set_real &
+       (global, name, rval, is_known, force, verbose, pacified)
     class(rt_data_t), intent(inout) :: global
     type(string_t), intent(in) :: name
     real(default), intent(in) :: rval
     logical, intent(in) :: is_known
-    logical, intent(in), optional :: verbose, pacified
+    logical, intent(in), optional :: force, verbose, pacified
     call global%var_list%set_real (name, rval, is_known, &
-         verbose=verbose, pacified=pacified)
+         force=force, verbose=verbose, pacified=pacified)
   end subroutine rt_data_set_real
 
-  subroutine rt_data_set_cmplx (global, name, cval, is_known, verbose, pacified)
+  subroutine rt_data_set_cmplx &
+       (global, name, cval, is_known, force, verbose, pacified)
     class(rt_data_t), intent(inout) :: global
     type(string_t), intent(in) :: name
     complex(default), intent(in) :: cval
     logical, intent(in) :: is_known
-    logical, intent(in), optional :: verbose, pacified
+    logical, intent(in), optional :: force, verbose, pacified
     call global%var_list%set_cmplx (name, cval, is_known, &
-         verbose=verbose, pacified=pacified)
+         force=force, verbose=verbose, pacified=pacified)
   end subroutine rt_data_set_cmplx
 
-  subroutine rt_data_set_subevt (global, name, pval, is_known, verbose)
+  subroutine rt_data_set_subevt &
+       (global, name, pval, is_known, force, verbose)
     class(rt_data_t), intent(inout) :: global
     type(string_t), intent(in) :: name
     type(subevt_t), intent(in) :: pval
     logical, intent(in) :: is_known
-    logical, intent(in), optional :: verbose
+    logical, intent(in), optional :: force, verbose
     call global%var_list%set_subevt (name, pval, is_known, &
-         verbose=verbose)
+         force=force, verbose=verbose)
   end subroutine rt_data_set_subevt
 
-  subroutine rt_data_set_pdg_array (global, name, aval, is_known, verbose)
+  subroutine rt_data_set_pdg_array &
+       (global, name, aval, is_known, force, verbose)
     class(rt_data_t), intent(inout) :: global
     type(string_t), intent(in) :: name
     type(pdg_array_t), intent(in) :: aval
     logical, intent(in) :: is_known
-    logical, intent(in), optional :: verbose
+    logical, intent(in), optional :: force, verbose
     call global%var_list%set_pdg_array (name, aval, is_known, &
-         verbose=verbose)
+         force=force, verbose=verbose)
   end subroutine rt_data_set_pdg_array
 
-  subroutine rt_data_set_string (global, name, sval, is_known, verbose)
+  subroutine rt_data_set_string &
+       (global, name, sval, is_known, force, verbose)
     class(rt_data_t), intent(inout) :: global
     type(string_t), intent(in) :: name
     type(string_t), intent(in) :: sval
     logical, intent(in) :: is_known
-    logical, intent(in), optional :: verbose
+    logical, intent(in), optional :: force, verbose
     call global%var_list%set_string (name, sval, is_known, &
-         verbose=verbose)
+         force=force, verbose=verbose)
   end subroutine rt_data_set_string
 
   function rt_data_get_lval (global, name) result (lval)
@@ -1126,5 +1283,48 @@ contains
     end if
   end function rt_data_get_event_callback
 
+  subroutine fix_system_dependencies (global)
+    class(rt_data_t), intent(inout), target :: global
+    type(var_list_t), pointer :: var_list
+
+    var_list => global%get_var_list_ptr ()
+    call var_list%set_log (var_str ("?omega_openmp"), &
+         .false., is_known = .true., force=.true.)
+    call var_list%set_log (var_str ("?openmp_is_active"), &
+         .false., is_known = .true., force=.true.)
+    call var_list%set_int (var_str ("openmp_num_threads_default"), &
+         1, is_known = .true., force=.true.)
+    call var_list%set_int (var_str ("openmp_num_threads"), &
+         1, is_known = .true., force=.true.)
+    call var_list%set_int (var_str ("real_range"), &
+         307, is_known = .true., force=.true.)
+    call var_list%set_int (var_str ("real_precision"), &
+         15, is_known = .true., force=.true.)
+    call var_list%set_real (var_str ("real_epsilon"), &
+         1.e-16_default, is_known = .true., force=.true.)
+    call var_list%set_real (var_str ("real_tiny"), &
+         1.e-300_default, is_known = .true., force=.true.)
+
+    global%os_data%fc = "Fortran-compiler"
+    global%os_data%fcflags = "Fortran-flags"
+
+  end subroutine fix_system_dependencies
+
+  subroutine show_description_of_string (string)
+    type(string_t), intent(in) :: string
+    type(rt_data_t), target :: global
+    call global%global_init ()
+    call global%show_description_of_string (string, ascii_output=.true.)
+  end subroutine show_description_of_string
+
+  subroutine show_tex_descriptions ()
+    type(rt_data_t), target :: global
+    call global%global_init ()
+    call fix_system_dependencies (global)
+    call global%set_int (var_str ("seed"), 0, is_known=.true.)
+    call global%var_list%sort ()
+    call global%write_var_descriptions ()
+  end subroutine show_tex_descriptions
+
 
 end module rt_data
