@@ -1,19 +1,12 @@
-! WHIZARD 2.4.1 Mar 24 2017
+! WHIZARD 2.6.4 Aug 23 2018
 !
-! Copyright (C) 1999-2017 by
+! Copyright (C) 1999-2018 by
 !     Wolfgang Kilian <kilian@physik.uni-siegen.de>
 !     Thorsten Ohl <ohl@physik.uni-wuerzburg.de>
 !     Juergen Reuter <juergen.reuter@desy.de>
 !
 !     with contributions from
-!     Fabian Bach <fabian.bach@t-online.de>
-!     Bijan Chokoufe <bijan.chokoufe@desy.de>
-!     Christian Speckner <cnspeckn@googlemail.com>
-!     So Young Shim <soyoung.shim@desy.de>
-!     Florian Staub <florian.staub@cern.ch>
-!     Christian Weiss <christian.weiss@desy.de>
-!     and Hans-Werner Boschmann, Felix Braam,
-!     Sebastian Schmidt, So-young Shim, Daniel Wiesler
+!     cf. main AUTHORS file
 !
 ! WHIZARD is free software; you can redistribute it and/or modify it
 ! under the terms of the GNU General Public License as published by
@@ -47,6 +40,7 @@ module user_files
   public :: file_list_t
   public :: file_list_final
   public :: file_list_is_open
+  public :: file_list_get_unit
   public :: file_list_open
   public :: file_list_close
   public :: file_list_write
@@ -117,6 +111,12 @@ contains
     end select
   end function file_is_open
 
+  function file_get_unit (file) result (unit)
+    integer :: unit
+    type(file_t), intent(in) :: file
+    unit = file%unit
+  end function file_get_unit
+
   subroutine file_write_string (file, string, advancing)
     type(file_t), intent(in) :: file
     type(string_t), intent(in), optional :: string
@@ -203,6 +203,19 @@ contains
     end if
   end function file_list_is_open
 
+  function file_list_get_unit (file_list, name) result (unit)
+    integer :: unit
+    type(file_list_t), intent(in) :: file_list
+    type(string_t), intent(in) :: name
+    type(file_t), pointer :: current
+    current => file_list_get_file_ptr (file_list, name)
+    if (associated (current)) then
+       unit = file_get_unit (current)
+    else
+       unit = -1
+    end if
+  end function file_list_get_unit
+  
   subroutine file_list_open (file_list, name, action, status, position)
     type(file_list_t), intent(inout) :: file_list
     type(string_t), intent(in) :: name
